﻿using System.Collections.Generic;
using System.IO;
using System.Text;
using Bitrix.Install;
using Bitrix.IO;
using Bitrix.UI.Wizards;
using SiteUpdater;
using System;
using Bitrix.DataTypes;

namespace Bitrix.Wizards.Install
{
	public partial class InstallUpdaterWizardStep : BXWizardStepStandardHtmlControl
	{
		protected override void OnWizardInit()
		{
			UI.SetProgressBarMaxValue("Installer.ProgressBar", "Updater", 3);
		}
		
		List<string> errors;
		private List<string> Errors
		{
			get { return errors ?? (errors = new List<string>()); }
		}

		protected string htmlMessage;
		private BXWizardResult ShowStatus(string htmlMessage, string nextStep, string nextAction)
		{
			this.htmlMessage = htmlMessage;
			BXWizardResultView view = Result.Render(GetMessage("Title"), errors);
			if ((errors == null || errors.Count == 0) && (!string.IsNullOrEmpty(nextStep) || !string.IsNullOrEmpty(nextAction)))
			{
				view.AutoRedirect = true;
				view.RedirectStep = nextStep;
				view.RedirectAction = nextAction;
			}
			WizardContext.Navigation.Selected = "install";
			return view;
		}

		protected override BXWizardResult OnWizardAction(string action, BXCommonBag parameters)
		{
			switch (action)
			{
				case "":
					return ShowStatus(GetMessage("SubTitle"), null, "installfiles");
				case "installfiles":
					return InstallFiles();
				case "installconfig":
					return InstallConfig();
				case "installkey":
					return InstallKey();
				case "finalize":
					return Finalize();
				default:
					return base.OnWizardAction(action, parameters);
			}
		}

		private void CopyFiles(string sourcePath, string destinationPath)
		{
			DirectoryInfo sourceDir = new DirectoryInfo(sourcePath);

			if (!sourceDir.Exists)
				return;

			foreach (FileInfo file in sourceDir.GetFiles("*", SearchOption.AllDirectories))
			{
				string relativeDestPath = file.Directory.FullName.Substring(sourceDir.FullName.Length).Trim(new char[] { '\\', '/' });
				string fullDestPath = Path.Combine(destinationPath, relativeDestPath);
				Directory.CreateDirectory(fullDestPath);
				file.CopyTo(Path.Combine(fullDestPath, file.Name), true);
			}
		}
		private BXWizardResult InstallFiles()
		{
			CopyFiles(BXPath.MapPath("~/bitrix/install/updater/admin"), BXPath.MapPath("~/bitrix/admin"));
			CopyFiles(BXPath.MapPath("~/bitrix/install/updater/lang"), BXPath.MapPath("~/bitrix/lang"));
			UI.SetProgressBarValue("Installer.ProgressBar", "Updater", 1);
			return ShowStatus(GetMessage("SubTitle"), null, "installconfig");
		}

		private BXWizardResult InstallConfig()
		{
			Directory.CreateDirectory(BXPath.MapPath("~/bitrix/updates"));
			File.WriteAllText(BXPath.MapPath("~/bitrix/updates/updater.config"), GetMessage("Updater.Config"), Encoding.UTF8);
			UI.SetProgressBarValue("Installer.ProgressBar", "Updater", 2);
			return ShowStatus(GetMessage("SubTitle"), null, "installkey");
		}
		private BXWizardResult InstallKey()
		{
			string key = (string)WizardContext.State["Install.LicenseKey"];
			if (BXInstaller.CheckKey(key))
			{
				BXSiteUpdater siteUpdater = BXSiteUpdater.GetUpdater();
				siteUpdater.Config.Key = key;
				siteUpdater.Config.Update();
			}
			UI.SetProgressBarValue("Installer.ProgressBar", "Updater", 3);
			return ShowStatus(GetMessage("SubTitle"), null, "finalize");
		}
		private BXWizardResult Finalize()
		{
			return Result.Next();
		}
	}
}