﻿using Bitrix.IO;
using Bitrix.UI.Wizards;
using Bitrix.DataTypes;
using System.Web;
using Bitrix.IBlock;
using Bitrix.Configuration;
using System;
using Bitrix.Catalog;
using Bitrix.DataLayer;
using System.Web.Script.Serialization;
using System.Collections.Generic;
using System.Collections;
using Bitrix.Services;
using Bitrix.Services.Text;
using System.Web.Hosting;
using System.IO;
using System.Text;
using Bitrix.Security;
using Bitrix.CommerceSite;

namespace Bitrix.Wizards.Solutions
{
	public partial class Finish : BXWizardStepStandardHtmlControl
	{
		private BXCustomField CreateCustomField(string entityId, string name, string customTypeId, string labelId, bool isMultiple)
		{
			BXCustomField field = new BXCustomField(entityId, name, customTypeId);
			field.Multiple = isMultiple;
			if (!string.IsNullOrEmpty(labelId))
				foreach (string langId in BXLoc.Locales)
				{
					string msg = BXLoc.GetMessage(langId, this, labelId, true);
					if (!string.IsNullOrEmpty(msg))
						(field.Localization[langId] = new BXCustomFieldLocalization()).EditFormLabel = msg;
				}
			return field;
		}

		protected override BXWizardResult OnActionShow(BXCommonBag parameters)
		{
			WizardContext.Navigation.Selected = "finish";

			List<string> errorLst = new List<string>();

			BXSite site = null;
			string siteId = WizardContext.State.GetString("Installer.SiteId");
			if (string.IsNullOrEmpty(siteId) || (site = BXSite.GetById(siteId)) == null)
				errorLst.Add(GetMessage("Error.SiteIdNotFound"));

			string siteRoot = VirtualPathUtility.AppendTrailingSlash(site.DirectoryVirtualPath);

			int blockTypeId = BXOptionManager.GetOptionInt("Bitrix.CommerceSite", "CatalogIBlockTypeId", 0, siteId);
			if (blockTypeId <= 0)
				errorLst.Add(GetMessage("Error.CatalogIBlockTypeIdNotFound"));

			string catalogName = WizardContext.State.GetString("CatalogName", string.Empty);
			if (string.IsNullOrEmpty(catalogName))
				errorLst.Add(GetMessage("Error.CatalogNameNotFound"));

			string catalogDirectoryName = WizardContext.State.GetString("CatalogDirectoryName", string.Empty);
			if (string.IsNullOrEmpty(catalogDirectoryName))
				errorLst.Add(GetMessage("Error.CatalogDirectoryNameNotFound"));

			BXIBlockCollection blocks = BXIBlock.GetList(
				new BXFilter(new BXFilterItem(BXIBlock.Fields.Code, BXSqlFilterOperators.Equal, catalogDirectoryName)),
				null,
				new BXSelect(BXSelectFieldPreparationMode.Normal, BXIBlock.Fields.ID),
				new BXQueryParams(new BXPagingOptions(0, 1)));

			if (blocks.Count > 0)
				errorLst.Add(string.Format(GetMessage("Error.IBlockAlreadyExists"), catalogDirectoryName));

			string catalogPath = VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine(siteRoot, "catalog")),
				directoryPath = VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine(catalogPath, catalogDirectoryName)),
				filePath = VirtualPathUtility.ToAbsolute(VirtualPathUtility.Combine(directoryPath, "default.aspx"));

			DirectoryInfo di = new DirectoryInfo(HostingEnvironment.MapPath(directoryPath));
			if (di.Exists)
			{
				if (!WizardContext.State.GetBool("CommerceSite.Overwrite", false))
					errorLst.Add(string.Format(GetMessage("Error.DirectoryAlreadyExists"), directoryPath));
			}
			else
				di.Create();

			if (errorLst.Count > 0)
				return Result.Render(GetMessage("Title"), errorLst);

			try
			{
				//Создание инфоблока
				BXIBlock block = new BXIBlock();
				block.Active = true;
				block.Sites.Add(siteId);
				block.TypeId = blockTypeId;
				block.Name = catalogName;
				block.Code = catalogDirectoryName;
				block.Sort = 500;
				block.IndexContent = true;
				block.XmlId = WizardContext.State.GetString("CatalogXmlId", string.Empty);

				block.Create();

				//Сохранение ИД инфоблока в опциях
				List<string> blockIdList = BXStringUtility.CsvToList(BXOptionManager.GetOptionString(BXCommerceSiteModuleConfiguration.Id, "Catalogue.IBlockIds", string.Empty, siteId));
				blockIdList.Add(block.Id.ToString());
				BXOptionManager.SetOptionString(BXCommerceSiteModuleConfiguration.Id, "Catalogue.IBlockIds", BXStringUtility.ListToCsv(blockIdList), siteId);

				//Права на чтение
				BXRoleManager.AddRolesToOperations(new string[] { "Guest" }, new string[] { "IBlockRead" }, "iblock", block.Id.ToString());

				//Создание свойств
				string entityId = BXIBlockElement.GetCustomFieldsKey(block.Id);
				//системные
				//SPECIALOFFER
				CreateCustomField(entityId, "SPECIALOFFER", "Bitrix.System.Boolean", "PropertyLabel.SpecialOffer", false).Create();
				//NEWPRODUCT
				CreateCustomField(entityId, "NEWPRODUCT", "Bitrix.System.Boolean", "PropertyLabel.NewProduct", false).Create();
				//SALELEADER
				CreateCustomField(entityId, "SALELEADER", "Bitrix.System.Boolean", "PropertyLabel.SaleLeader", false).Create();
				//MORE_PHOTO
				CreateCustomField(entityId, "MORE_PHOTO", "Bitrix.System.File", "PropertyLabel.MorePhoto", true).Create();

				//пользовательские
				StringBuilder comparisonSb = new StringBuilder(),
					propSb = new StringBuilder();
				string propertyData = WizardContext.State.GetString("ProductPropertyData");
				if (!string.IsNullOrEmpty(propertyData))
				{
					JavaScriptSerializer serializer = new JavaScriptSerializer();
					serializer.RegisterConverters(new JavaScriptConverter[] { new CatalogProductPropertyInfoJsonConverter() });
					CatalogProductPropertyInfo[] properties = serializer.Deserialize<CatalogProductPropertyInfo[]>(propertyData);

					for (int i = 0; i < properties.Length; i++)
					{
						CatalogProductPropertyInfo property = properties[i];
						if (property.Name.Length == 0)
							continue;

						BXCustomField field = CreateCustomField(entityId, string.Concat("PROPERTY", (i + 1).ToString()), property.Type, null, false);
						(field.Localization[BXLoc.CurrentLocale] = new BXCustomFieldLocalization()).EditFormLabel = property.Name;

						if (property.ViewMode == CatalogProductPropertyListViewMode.List)
							field.Settings.Add("ViewMode", "list");

						field.Multiple = property.IsMultiple;
						field.Create();

						for (int j = 0; j < property.Values.Count; j++)
						{
							BXCustomFieldEnum e = new BXCustomFieldEnum();
							e.FieldId = field.Id;
							e.FieldType = field.FieldType;
							e.Value = property.Values[j];
							e.Sort = 10 * j;
							//e.Default = j == 0;
							e.Save();
						}

						if (comparisonSb.Length > 0)
							comparisonSb.Append(";");
						comparisonSb.Append("-").Append(field.CorrectedName);

						if (propSb.Length > 0)
							propSb.Append(";");
						propSb.Append(field.CorrectedName);
					}
				}

				if (comparisonSb.Length > 0)
					comparisonSb.Insert(0, ";");

				if (propSb.Length > 0)
					propSb.Insert(0, ";");

				//Привязка к каталогу
				BXCatalog catalog = new BXCatalog();
				catalog.Id = block.Id;

				BXCatalogVATRateCollection vatRates = BXCatalogVATRate.GetList(
					new BXFilter(new BXFilterItem(BXCatalogVATRate.Fields.Value, BXSqlFilterOperators.Equal, 0m)),
					null,
					null,
					new BXQueryParams(new BXPagingOptions(0, 1))
					);

				if (vatRates.Count > 0)
					catalog.VATRateId = vatRates[0].Id;

				catalog.CheckStockAvailability = true;
				catalog.Create();

				//Создание страницы главной страницы с каталогом			
				BXCatalogPriceTypeCollection priceTypes = BXCatalogPriceType.GetList(
					new BXFilter(new BXFilterItem(BXCatalogPriceType.Fields.IsBase, BXSqlFilterOperators.Equal, true)),
					null,
					new BXSelect(BXSelectFieldPreparationMode.Normal, BXCatalogPriceType.Fields.Id),
					new BXQueryParams(new BXPagingOptions(0, 1)));

				this.catalogUrl = VirtualPathUtility.ToAbsolute(directoryPath);

				string componentData =
					string.Format(
	@"<bx:IncludeComponent ID=""catalog"" runat=""server"" PagingTemplate=""catalog"" GenerateMenu=""True"" GenerateMenuType=""left""
	ComponentName=""bitrix:catalogue"" Template=""main"" IBlockTypeId=""{0}"" IBlockId=""{1}"" 
	SetPageTitle=""True"" EnableSEF=""True"" SEFFolder=""{2}""  
	DisplayStockCatalogData=""True"" DisplayAllPriceTiers=""{3}"" PriceTypes=""{4}"" 
	InitQuantity=""1"" AcceptQuantity=""False"" IncludeVATInPrice=""True"" DisplayVAT=""False"" 
	ShowCatalogItemProperties=""MORE_PHOTO;SPECIALOFFER;NEWPRODUCT;SALELEADER"" DisplaySearchResults=""True"" UserCartUrlTemplate=""{5}"" 
	AllowComparison=""{6}"" ComparisonSelectedFields=""-SPECIALOFFER;-NEWPRODUCT{7}"" 
	ListProperties=""SPECIALOFFER;NEWPRODUCT;MORE_PHOTO{8}"" DetailProperties=""NEWPRODUCT;MORE_PHOTO{8}""
	ElementDetailTemplate=""/#SectionId#/item-#ElementId#/"" ElementListTemplate=""/#SectionId#/"" 
	CommonListPageTemplate=""/page-#pageId#"" CommonListShowAllTemplate=""/all"" 
	SectionListPageTemplate=""/#SectionId#/page-#pageId#"" SectionListShowAllTemplate=""/#SectionId#/all""
	ElementDetailCommentReadTemplate=""/#SectionId#/item-#ElementId#/comment-#CommentId#/"" ElementDetailCommentOperationTemplate=""/#SectionId#/item-#ElementId#/comment-#CommentId#/act-#Operation#/"" ElementDetailCommentPageTemplate=""/#SectionId#/item-#ElementId#/page-#PageID#/"" 
	ComparisonResultTemplate=""/compare"" />",
					blockTypeId.ToString(),
					block.Id.ToString(),
					directoryPath.Substring(1),
					(priceTypes.Count == 0).ToString(),
					priceTypes.Count > 0 ? priceTypes[0].Id.ToString() : string.Empty,
					BXOptionManager.GetOptionString("Bitrix.CommerceSite", "UserCartUrl", string.Empty),
					WizardContext.State.GetBool("DisplayComparisonTable", true).ToString(),
					comparisonSb.ToString(),
					propSb.ToString());

				string pageData = BXDefaultFiles.BuildAspx(catalogName, filePath, string.Empty, componentData);
				BXSecureIO.SaveAspx(filePath, pageData, site);

				//Создание пункта меню
				try
				{
					BXPublicMenu lMenu = BXPublicMenu.Menu;
					BXPublicMenuItemCollection items = lMenu.Load(catalogPath, "left");
					BXPublicMenuItem item = items.Find(x => string.Equals(x.Href, BXUri.ToAbsoluteUri(directoryPath), StringComparison.InvariantCultureIgnoreCase));
					if (item == null)
					{
						item = new BXPublicMenuItem();
						item.Link = directoryPath;
						item.Title = catalogName;

						int maxSort = 0;
						foreach (BXPublicMenuItem presentItem in items)
							if (maxSort < presentItem.Sort)
								maxSort = presentItem.Sort;

						item.Sort = maxSort + 10;
						items.Add(item);
						lMenu.Save(catalogPath, "left", items);
					}
				}
				catch
				{
				}
			}
			catch (Exception exc)
			{
				errorLst.Add(exc.Message);
			}

			if (WizardContext.State.GetBool("CommerceSite.IsRunning", false))
				return new BXWizardResultFinish();

			return Result.Render(GetMessage("Title"), errorLst);
		}

		private string catalogUrl = string.Empty;
		public string CatalogUrl
		{
			get { return this.catalogUrl; }
		}
	}

	internal enum CatalogProductPropertyListViewMode
	{
		List = 1,
		Flags
	}

	internal class CatalogProductPropertyInfo
	{
		private string name = string.Empty;
		public string Name
		{
			get { return this.name; }
			set { this.name = value ?? string.Empty; }
		}

		private string type = string.Empty;
		public string Type
		{
			get { return this.type; }
			set { this.type = value ?? string.Empty; }
		}

		private List<string> values = null;
		public IList<string> Values
		{
			get { return this.values ?? (this.values = new List<string>()); }
		}

		private CatalogProductPropertyListViewMode viewMode = CatalogProductPropertyListViewMode.Flags;
		public CatalogProductPropertyListViewMode ViewMode
		{
			get { return this.viewMode; }
			set { this.viewMode = value; }
		}

		private bool isMultiple = false;
		public bool IsMultiple
		{
			get { return this.isMultiple; }
			set { this.isMultiple = value; }
		}
	}

	internal class CatalogProductPropertyInfoJsonConverter : JavaScriptConverter
	{
		public override object Deserialize(IDictionary<string, object> dictionary, Type type, JavaScriptSerializer serializer)
		{
			if (type != typeof(CatalogProductPropertyInfo))
				return null;

			CatalogProductPropertyInfo r = new CatalogProductPropertyInfo();

			object o;
			if (dictionary.TryGetValue("name", out o))
				r.Name = (string)o;

			if (dictionary.TryGetValue("type", out o))
			{
				string s = (string)o;
				if (string.Equals(s, "list", StringComparison.OrdinalIgnoreCase) || string.Equals(s, "flag", StringComparison.OrdinalIgnoreCase))
				{
					r.Type = "Bitrix.System.Enumeration";
					if(string.Equals(s, "list", StringComparison.OrdinalIgnoreCase))
					{
						r.ViewMode = CatalogProductPropertyListViewMode.List;
						r.IsMultiple = false;
					}
					else
					{
						r.ViewMode = CatalogProductPropertyListViewMode.Flags;
						r.IsMultiple = true;
					}
				}
				//else if(string.Equals(s, "flag", StringComparison.OrdinalIgnoreCase))
				//	r.Type = "Bitrix.System.Boolean";
				else
				{
					r.Type = "Bitrix.System.Text";
					r.IsMultiple = false;
				}
			}

			if (dictionary.TryGetValue("values", out o))
				foreach (object item in ((IEnumerable)o))
					r.Values.Add((string)item);

			return r;
		}

		public override IDictionary<string, object> Serialize(object obj, JavaScriptSerializer serializer)
		{
			throw new NotSupportedException();
		}

		public override IEnumerable<Type> SupportedTypes
		{
			get { return new Type[] { typeof(CatalogProductPropertyInfo) }; }
		}
	}
}