﻿using Bitrix.IO;
using Bitrix.UI.Wizards;
using Bitrix.DataTypes;
using Bitrix.UI;
using System.Web;
using System.Web.UI;
using Bitrix.Services.Text;
using System.Web.Hosting;
using System.IO;
using System.Text;

namespace Bitrix.Wizards.Solutions
{
    public partial class CommerceSiteDeliveryPaymentSettingsStep : BXWizardStepStandardHtmlControl
	{
		private BXSite site = null;
		private BXSite Site
		{
			get { return this.site ?? (this.site = BXSite.GetById(WizardContext.State.GetString("Installer.SiteId"))); }
		}

		private string PrepareFilePath(string directoryPath, string fileName)
		{
			return VirtualPathUtility.Combine(VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine("~/", directoryPath)), fileName);
		}

		private string GetPersistentString(string key, string messageId, string fn)
		{
			string s = WizardContext.State.GetString(key, null);
			if (s == null)
			{
				string path = HostingEnvironment.MapPath(PrepareFilePath(Site.DirectoryVirtualPath, fn));
				if (File.Exists(path))
					s = BXStringUtility.HtmlToText(File.ReadAllText(path, Encoding.UTF8));
			}
			if (s == null)
				s = GetMessage(messageId);

			return s;
		}

		protected override void OnWizardInit()
		{
			UI.Load("CommerceSiteDeliveryPaymentSettings");

			UI.Data["EnableCashPayment"] = WizardContext.State.GetBool("EnableCashPayment",  true);
			UI.Data["EnableSberbankPayment"] = WizardContext.State.GetBool("EnableSberbankPayment",  true);
			UI.Data["EnableBillPayment"] = WizardContext.State.GetBool("EnableBillPayment",  true);
			UI.Data["CourierDelivery"] = WizardContext.State.GetBool("CourierDelivery",  true);
			UI.Data["PickupDelivery"] = WizardContext.State.GetBool("PickupDelivery",  true);
			UI.Data["RussianPostDelivery"] = WizardContext.State.GetBool("RussianPostDelivery", false);
			UI.Data["EmsRussianPostDelivery"] = WizardContext.State.GetBool("EmsRussianPostDelivery", false);
			UI.Data["DeliveryServiceContactInfo"] = (GetPersistentString("DeliveryServiceContactInfo", "Defaults.DeliveryServiceContactInfo", "delivery.html") ?? string.Empty).Trim();

			UI.Overwrite("CommerceSiteDeliveryPaymentSettings");
		}

		protected override BXWizardResult OnActionShow(BXCommonBag parameters)
		{
			UI.Load("CommerceSiteDeliveryPaymentSettings");

            WizardContext.Navigation.Selected = "delivery_payment_settings";
			BXWizardResultView view = Result.Render(GetMessage("Title"), null);

            view.Buttons.Add("prev", null);
            view.Buttons.Add("next", null);
			return view;
		}

        protected override BXWizardResult OnActionPrevious(BXCommonBag parameters)
        {
            return Result.Previous();
        }

		protected override BXWizardResult OnActionNext(BXCommonBag parameters)
		{
			UI.LoadValues(parameters);

			WizardContext.State["EnableCashPayment"] = UI.Data.GetBool("EnableCashPayment", false);
			WizardContext.State["EnableSberbankPayment"] = UI.Data.GetBool("EnableSberbankPayment", false);
			WizardContext.State["EnableBillPayment"] = UI.Data.GetBool("EnableBillPayment", false);
			WizardContext.State["CourierDelivery"] = UI.Data.GetBool("CourierDelivery", false);
			WizardContext.State["PickupDelivery"] = UI.Data.GetBool("PickupDelivery", false);
			WizardContext.State["RussianPostDelivery"] = UI.Data.GetBool("RussianPostDelivery", false);
			WizardContext.State["EmsRussianPostDelivery"] = UI.Data.GetBool("EmsRussianPostDelivery", false);
			WizardContext.State["DeliveryServiceContactInfo"] = UI.Data.GetString("DeliveryServiceContactInfo", string.Empty);

			UI.Overwrite("CommerceSiteDeliveryPaymentSettings");
			UI.SetProgressBarMaxValue("Installer.ProgressBar", "Bitrix.CommerceSite", 7);
			return new BXWizardResultFinish();
		}
	}
}