﻿using Bitrix.IO;
using Bitrix.UI.Wizards;
using Bitrix.DataTypes;
using Bitrix.UI;
using System.Web;
using System.Web.UI;
using Bitrix.Services.Text;
using System.IO;
using System.Web.Hosting;
using System.Text;
using System.Collections.Generic;
using System;
using System.Text.RegularExpressions;
using Bitrix.Configuration;


namespace Bitrix.Wizards.Solutions
{
	public partial class CommerceSiteSettingsStep : BXWizardStepStandardHtmlControl
	{
		//private string siteLogoImageUrl = string.Empty;
		//public string SiteLogoImageUrl
		//{
		//    get { return HttpUtility.HtmlAttributeEncode(this.siteLogoImageUrl); }
		//}

		private BXSite site = null;
		private BXSite Site
		{
			get { return this.site ?? (this.site = BXSite.GetById(WizardContext.State.GetString("Installer.SiteId"))); }
		}

		private string PrepareFilePath(string directoryPath, string fileName)
		{
			return VirtualPathUtility.Combine(VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine("~/", directoryPath)), fileName);
		}

		private string GetPersistentString(string key, string messageId, string fn)
		{
			string s = null;

			if (!string.IsNullOrEmpty(key))
				s = WizardContext.State.GetString(key, null);

			if (s == null)
			{
				string path = HostingEnvironment.MapPath(PrepareFilePath(Site.DirectoryVirtualPath, fn));
				if (File.Exists(path))
					s = File.ReadAllText(path, Encoding.UTF8);
			}
			if (s == null && !string.IsNullOrEmpty(messageId))
				s = GetMessage(messageId);

			return s ?? string.Empty;
		}

		protected override void OnWizardInit()
		{
			UI.Load("CommerceSiteSettings");

			UI.Data["InstallDemo"] = UI.Data["Overwrite"] = !string.Equals(BXOptionManager.GetOptionString("main", "InstalledSolution", null, WizardContext.State.GetString("Installer.SiteId")), "Bitrix.CommerceSite");

			UI.Data["SiteName"] = GetPersistentString("SiteName", "Defaults.SiteName", "src/logotext.html");
			UI.Data["SitePhone"] = GetPersistentString("SitePhone", "Defaults.SitePhone", "src/phone.html");
			UI.Data["SiteWebmasterEmail"] = GetPersistentString("SiteWebmasterEmail", "Defaults.SiteWebmasterEmail", "src/contactemail.html");
			UI.Data["SiteSchedule"] = GetPersistentString("SiteSchedule", "Defaults.SiteSchedule", "src/schedule.html");
			UI.Data["SiteCopyRight"] = GetPersistentString("SiteCopyRight", "Defaults.SiteCopyRight", "src/copyright.html");

			UI.Overwrite("CommerceSiteSettings");

			var installStep = WizardContext.Navigation["install"];
			if(installStep != null)
			{
				WizardContext.Navigation.Remove(installStep);
				WizardContext.Navigation.Insert(WizardContext.Navigation.IndexOf(WizardContext.Navigation["delivery_payment_settings"]) + 1, installStep);
			}
				
		}

		protected override BXWizardResult OnActionShow(BXCommonBag parameters)
		{
			UI.Load("CommerceSiteSettings");

			WizardContext.Navigation.Selected = "site_settings";
			return PrepareView(null);
		}

		protected override BXWizardResult OnActionPrevious(BXCommonBag parameters)
		{
			return new BXWizardResultCancel();
		}

		protected override BXWizardResult OnActionNext(BXCommonBag parameters)
		{
			UI.LoadValues(parameters);

			WizardContext.State["CommerceSite.InstallDemo"] = UI.Data.GetBool("InstallDemo", false);
			WizardContext.State["CommerceSite.Overwrite"] = UI.Data.GetBool("Overwrite", false);

			WizardContext.State["SiteName"] = UI.Data.GetString("SiteName", string.Empty);
			WizardContext.State["SitePhone"] = UI.Data.GetString("SitePhone", string.Empty);
			WizardContext.State["SiteWebmasterEmail"] = UI.Data.GetString("SiteWebmasterEmail", string.Empty);
			WizardContext.State["SiteSchedule"] = UI.Data.GetString("SiteSchedule", string.Empty);
			WizardContext.State["SiteCopyRight"] = UI.Data.GetString("SiteCopyRight", string.Empty);

			UI.Overwrite("CommerceSiteSettings");
			return Result.Next();
		}

		//protected override BXWizardResult OnWizardAction(string action, BXCommonBag parameters)
		//{
		//    if (action == "upload")
		//    {
		//        UI.LoadValues(parameters);

		//        int fileId = 0;
		//        List<string> errors = new List<string>();
		//        try
		//        {
		//            //BXParamsBag<object> settings = new BXParamsBag<object>();
		//            //settings["maxFileSize"] = 1024 * 1024;
		//            //settings["maxWidth"] = 500;
		//            //settings["maxHeight"] = 105;

		//            fileId = UI.SaveImage("SiteLogoImage", null, errors);
		//        }
		//        catch (Exception ex)
		//        {
		//            errors = new List<string>() { { ex.Message } };
		//        }

		//        if (fileId > 0)
		//            WizardContext.State["SiteLogoImage"] = fileId;

		//        UI.Overwrite("CommerceSiteSettings");

		//        return PrepareView(errors);
		//    }
		//    return base.OnWizardAction(action, parameters);
		//}

		private BXWizardResult PrepareView(IEnumerable<string> errors)
		{
			#region LogoImage
			//int logoFileId = WizardContext.State.GetInt("SiteLogoImage", 0);
			//if (logoFileId > 0)
			//{
			//    BXFile logoFile = BXFile.GetById(logoFileId, BXTextEncoder.EmptyTextEncoder);
			//    if (logoFile != null)
			//        this.siteLogoImageUrl = HttpUtility.HtmlAttributeEncode(logoFile.FilePath);
			//}

			//if (string.IsNullOrEmpty(this.siteLogoImageUrl))
			//{
			//    string content = GetPersistentString(string.Empty, string.Empty, "src/logo.html");
			//    if (!string.IsNullOrEmpty(content))
			//    {
			//        Match m = (new Regex(@"src=[""']([^""']*)[""']", RegexOptions.IgnoreCase)).Match(content);
			//        string urlStr = m.Success ? HttpUtility.HtmlDecode(m.Groups[1].Captures[0].Value) : string.Empty;
			//        Uri url = null;
			//        if (!string.IsNullOrEmpty(urlStr) && Uri.TryCreate(HttpContext.Current.Request.Url, urlStr, out url))
			//        {
			//            try
			//            {
			//                if (File.Exists(HostingEnvironment.MapPath(HttpUtility.UrlDecode(url.AbsolutePath))))
			//                    this.siteLogoImageUrl = urlStr;
			//            }
			//            catch
			//            {
			//            }
			//        }
			//    }
			//}
			#endregion
			BXWizardResultView view = Result.Render(GetMessage("Title"), errors);
			view.Buttons.Add("prev", null);
			view.Buttons.Add("next", null);
			return view;
		}
	}
}