﻿using Bitrix.IO;
using Bitrix.UI.Wizards;
using Bitrix.DataTypes;
using Bitrix.UI;
using System.Web;
using System.Web.UI;
using Bitrix.Services.Text;
using System;
using System.IO;
using System.Web.Hosting;
using Bitrix.Configuration;
using Bitrix.Sale;

namespace Bitrix.Wizards.Solutions
{
	public partial class CommerceSiteStoreSettingsStep : BXWizardStepStandardHtmlControl
	{
		private BXSite site = null;
		private BXSite Site
		{
			get { return this.site ?? (this.site = BXSite.GetById(WizardContext.State.GetString("Installer.SiteId"))); }
		}

		protected override void OnWizardInit()
		{
			bool isInstalled = string.Equals(BXOptionManager.GetOptionString("main", "InstalledSolution", null, WizardContext.State.GetString("Installer.SiteId")), "Bitrix.CommerceSite");

			UI.Load("CommerceSiteStoreSettings");
			if (!isInstalled)
			{
				UI.Data["StoreName"] = WizardContext.State.GetString("StoreName", null) ?? GetMessage("Defaults.StoreName");
				UI.Data["StorePostalCode"] = WizardContext.State.GetString("StorePostalCode", null) ?? GetMessage("Defaults.StorePostalCode");
				UI.Data["StoreAddress"] = WizardContext.State.GetString("StoreAddress", null) ?? GetMessage("Defaults.StoreAddress");
				UI.Data["StoreInn"] = WizardContext.State.GetString("StoreInn", null) ?? GetMessage("Defaults.StoreInn");
				UI.Data["StoreKpp"] = WizardContext.State.GetString("StoreKpp", null) ?? GetMessage("Defaults.StoreKpp");
				UI.Data["StoreSettlementAccount"] = WizardContext.State.GetString("StoreSettlementAccount", null) ?? GetMessage("Defaults.StoreSettlementAccount");
				UI.Data["StoreCorrespondentAccount"] = WizardContext.State.GetString("StoreCorrespondentAccount", null) ?? GetMessage("Defaults.StoreCorrespondentAccount");
				UI.Data["StoreBankName"] = WizardContext.State.GetString("StoreBankName", null) ?? GetMessage("Defaults.StoreBankName");
				UI.Data["StoreBankBic"] = WizardContext.State.GetString("StoreBankBic", null) ?? GetMessage("Defaults.StoreBankBic");
			}
			else
			{
				BXSale config = BXSale.GetBySiteId(Site.Id);

				UI.Data["StoreName"] = WizardContext.State.GetString("StoreName", null) ?? config.Name;
				UI.Data["StorePostalCode"] = WizardContext.State.GetString("StorePostalCode", null) ?? config.PostalCode;
				UI.Data["StoreAddress"] = WizardContext.State.GetString("StoreAddress", null) ?? config.Address;
				UI.Data["StoreInn"] = WizardContext.State.GetString("StoreInn", null) ?? config.Inn;
				UI.Data["StoreKpp"] = WizardContext.State.GetString("StoreKpp", null) ?? config.Kpp;
				UI.Data["StoreSettlementAccount"] = WizardContext.State.GetString("StoreSettlementAccount", null) ?? config.SettlementAccount;
				UI.Data["StoreCorrespondentAccount"] = WizardContext.State.GetString("StoreCorrespondentAccount", null) ?? config.CorrespondentAccount;
				UI.Data["StoreBankName"] = WizardContext.State.GetString("StoreBankName", null) ?? config.BankName;
				UI.Data["StoreBankBic"] = WizardContext.State.GetString("StoreBankBic", null) ?? config.BankBic;
			}
			UI.Overwrite("CommerceSiteStoreSettings");
		}

		protected override BXWizardResult OnActionShow(BXCommonBag parameters)
		{
			UI.Load("CommerceSiteStoreSettings");

			WizardContext.Navigation.Selected = "store_settings";
			BXWizardResultView view = Result.Render(GetMessage("Title"), null);

			view.Buttons.Add("prev", null);
			view.Buttons.Add("next", null);
			return view;
		}

		protected override BXWizardResult OnActionPrevious(BXCommonBag parameters)
		{
			return Result.Previous();
		}

		protected override BXWizardResult OnActionNext(BXCommonBag parameters)
		{
			UI.LoadValues(parameters);

			WizardContext.State["StoreName"] = UI.Data.GetString("StoreName", string.Empty);
			//WizardContext.State["StoreSettlement"] = UI.Data.GetString("StoreSettlement", string.Empty);
			WizardContext.State["StorePostalCode"] = UI.Data.GetString("StorePostalCode", string.Empty);
			WizardContext.State["StoreAddress"] = UI.Data.GetString("StoreAddress", string.Empty);
			WizardContext.State["StoreInn"] = UI.Data.GetString("StoreInn", string.Empty);
			WizardContext.State["StoreKpp"] = UI.Data.GetString("StoreKpp", string.Empty);
			WizardContext.State["StoreSettlementAccount"] = UI.Data.GetString("StoreSettlementAccount", string.Empty);
			WizardContext.State["StoreCorrespondentAccount"] = UI.Data.GetString("StoreCorrespondentAccount", string.Empty);
			WizardContext.State["StoreBankName"] = UI.Data.GetString("StoreBankName", string.Empty);
			WizardContext.State["StoreBankBic"] = UI.Data.GetString("StoreBankBic", string.Empty);

			HttpPostedFile postFile = UI.Data.Get<HttpPostedFile>("StoreStamp", null);
			if (postFile != null && postFile.ContentLength > 0)
			{
				string nameAndExt = postFile.FileName;
				int lastBackSlashInd = nameAndExt.LastIndexOf('\\'); //FOR IE
				if (lastBackSlashInd >= 0)
					nameAndExt = nameAndExt.Substring(lastBackSlashInd + 1);

				string mimeType = postFile.ContentType.ToUpperInvariant();
				string name = string.Empty, ext = string.Empty;
				int delimeterIndex = !string.IsNullOrEmpty(nameAndExt) ? nameAndExt.LastIndexOf('.') : -1;
				if (delimeterIndex >= 0)
				{
					name = nameAndExt.Substring(0, delimeterIndex);
					ext = nameAndExt.Substring(delimeterIndex + 1).ToUpperInvariant();
				}

				if (ext.Length > 0
					&& (string.Equals(ext, "JPG", StringComparison.Ordinal)
					|| string.Equals(ext, "JPEG", StringComparison.Ordinal)
					|| string.Equals(ext, "GIF", StringComparison.Ordinal)
					|| string.Equals(ext, "PNG", StringComparison.Ordinal))
					&& (string.Equals(mimeType, "IMAGE/JPEG", StringComparison.Ordinal)
					|| string.Equals(mimeType, "IMAGE/PJPEG", StringComparison.Ordinal) //FOR IE
					|| string.Equals(mimeType, "IMAGE/JPG", StringComparison.Ordinal)
					|| string.Equals(mimeType, "IMAGE/GIF", StringComparison.Ordinal)
					|| string.Equals(mimeType, "IMAGE/PNG", StringComparison.Ordinal)))
				{

					string dPath = VirtualPathUtility.AppendTrailingSlash(
										VirtualPathUtility.Combine("~/", BXConfigurationUtility.Constants.FileCachePath));
					DirectoryInfo di = new DirectoryInfo(HostingEnvironment.MapPath(dPath));
					if (!di.Exists)
						di.Create();

					string fPath = VirtualPathUtility.Combine(dPath, Guid.NewGuid().ToString("N") + ".temp.cs");
					postFile.SaveAs(HostingEnvironment.MapPath(fPath));
					WizardContext.State["StoreStampFilePath"] = fPath;
					WizardContext.State["StoreStampFileName"] = string.Concat(name.ToLowerInvariant(), ".", ext.ToLowerInvariant());
					WizardContext.State["StoreStampFileMimeType"] = mimeType;
				}
			}
			UI.Overwrite("CommerceSiteStoreSettings");
			return Result.Next();
		}
	}
}