﻿using Bitrix.IO;
using Bitrix.UI.Wizards;
using Bitrix.DataTypes;
using Bitrix.IBlock;
using Bitrix.Configuration;
using Bitrix.Services;
using Bitrix.DataLayer;
using System.Web;
using System.IO;
using System.Web.Hosting;
using System.Collections.Generic;
using Bitrix.Services.Text;
using Bitrix.CommerceSite;
using Bitrix.Security;
using System.Text;
using Bitrix.Catalog;
using Bitrix.Install.Internal;
using System;
using Bitrix.Currency;

namespace Bitrix.Wizards.Solutions
{
	public partial class CommerceSiteCatalogStep : BXWizardStepStandardHtmlControl
	{
		private BXSite site = null;
		private BXSite Site
		{
			get { return this.site ?? (this.site = BXSite.GetById(WizardContext.State.GetString("Installer.SiteId"))); }
		}

		protected override BXWizardResult OnActionShow(BXCommonBag parameters)
		{
			BXWizardResultView view = Result.Render(GetMessage("Setup"));
			view.AutoRedirect = true;
			view.RedirectAction = "next";
			return view;
		}

		protected override BXWizardResult OnActionNext(BXCommonBag parameters)
		{
			bool installDemo = WizardContext.State.GetBool("CommerceSite.InstallDemo", false);
			string solutionPath = VirtualPathUtility.AppendTrailingSlash(WizardContext.State.GetString("Installer.SolutionPath")),
				//catalogXmlId = string.Concat("Bitrix.CommerceSite.Catalog.", Site.Id),
				//catalogTypeXmlId = "Bitrix.CommerceSite.Catalog";
				catalogTypeXmlId = "type_iblock_commerce_catalog";

			IBlockXmlImporter importer = new IBlockXmlImporter(Site.Id, "siteEntityId", "dirToStoreFilesTemplate");
			importer.LoadInfoBlocks(HostingEnvironment.MapPath(VirtualPathUtility.Combine(VirtualPathUtility.Combine(solutionPath, "data/"), "iblock.xml")));
			if (installDemo)
				importer.LoadInfoBlocks(HostingEnvironment.MapPath(VirtualPathUtility.Combine(VirtualPathUtility.Combine(solutionPath, "data/"), "iblock_data.xml")));

			int blockTypeId = 0;
			if(installDemo)
			{
				BXIBlockTypeCollection blockTypeCol = BXIBlockType.GetList(
					new BXFilter(new BXFilterItem(BXIBlockType.Fields.XmlId, BXSqlFilterOperators.Equal, catalogTypeXmlId)),
					null);

				BXIBlockType blockType = blockTypeCol.Count > 0 ? blockTypeCol[0] : null;
				if(blockType == null)
				{
					blockType = new BXIBlockType(GetMessage("BlockTypeName"), true, 500);
					blockType.XmlId = catalogTypeXmlId;
					blockType.Create();
				}

				blockTypeId = blockType.Id;

				BXCmlImport import = new BXCmlImport();
				BXCmlImportSettings s = new BXCmlImportSettings();
				s.SiteList.Add(Site.Id);
				s.BlockTypeId = blockTypeId;
				s.DirectoryPath = VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine(solutionPath, "data"));
				s.CreatePreviewImage = true;
				s.DefaultDecimalSeparator = ".";
				s.DefaultVATInPrice = true;
				s.IgnoredCmlProperties.Add("CML2_ARTICLE");
				s.FileNames = new string[] 
				{
					"catalog.xml",
					"catalog_prices.xml" 
				};
				import.Process(s);	

				BXCatalogPriceTypeCollection priceTypeCol = BXCatalogPriceType.GetList(
					new BXFilter(new BXFilterItem(BXCatalogPriceType.Fields.XmlId, BXSqlFilterOperators.Equal, "BASE")),
					null,
					null,
					new BXQueryParams(BXPagingOptions.Top(1)));

				if(priceTypeCol.Count > 0)
				{
					BXCatalogPriceType basePriceType = priceTypeCol[0];
					if(!basePriceType.IsBase)
					{
						basePriceType.IsBase = true;
						basePriceType.Update();
					}
				}
			}

			BXIBlockCollection blockCol = BXIBlock.GetList(
				new BXFilter(
					new BXFilterItem(BXIBlock.Fields.Type.ID, BXSqlFilterOperators.Equal, blockTypeId),
					new BXFilterItem(BXIBlock.Fields.Sites.SiteId, BXSqlFilterOperators.Equal, Site.Id)
					),
				new BXOrderBy(new BXOrderByPair(BXIBlock.Fields.UpdateDate, BXOrderByDirection.Desc)),
				null,
				new BXQueryParams(BXPagingOptions.Top(1)));

			if (blockCol.Count > 0)
			{
				BXIBlock block = blockCol[0];
				string siteRoot = VirtualPathUtility.AppendTrailingSlash(Site.DirectoryVirtualPath),
					catalogDirectoryName = block.Code;

				string catalogPath = VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine(siteRoot, "catalog")),
					directoryPath = VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine(catalogPath, catalogDirectoryName)),
					filePath = VirtualPathUtility.ToAbsolute(VirtualPathUtility.Combine(directoryPath, "default.aspx"));

				//создание директории
				DirectoryInfo di = new DirectoryInfo(HostingEnvironment.MapPath(directoryPath));
				if (!di.Exists)
					di.Create();

				//Привязка к каталогу
				BXCatalog catalog = BXCatalog.GetById(block.Id);
				if (catalog == null)
				{
					catalog = new BXCatalog();
					catalog.Id = block.Id;
				}

				BXCatalogVATRateCollection vatRates = BXCatalogVATRate.GetList(
					new BXFilter(new BXFilterItem(BXCatalogVATRate.Fields.Value, BXSqlFilterOperators.Equal, 0m)),
					null,
					null,
					new BXQueryParams(new BXPagingOptions(0, 1))
					);

				if (vatRates.Count > 0)
					catalog.VATRateId = vatRates[0].Id;

				catalog.CheckStockAvailability = true;
				catalog.Save();

				//Создание страницы главной страницы с каталогом			
				BXCatalogPriceTypeCollection priceTypes = BXCatalogPriceType.GetList(
					new BXFilter(new BXFilterItem(BXCatalogPriceType.Fields.IsBase, BXSqlFilterOperators.Equal, true)),
					null,
					new BXSelect(BXSelectFieldPreparationMode.Normal, BXCatalogPriceType.Fields.Id),
					new BXQueryParams(BXPagingOptions.Top(1)));

				string componentData =
					string.Format(
	@"<bx:IncludeComponent ID=""catalog"" runat=""server"" PagingTemplate=""catalog"" GenerateMenu=""True"" GenerateMenuType=""left"" 
	ComponentName=""bitrix:catalogue"" Template=""main"" IBlockTypeId=""{0}"" IBlockId=""{1}"" 
	SetPageTitle=""True"" EnableSEF=""True"" SEFFolder=""{2}""  
	DisplayStockCatalogData=""True"" DisplayAllPriceTiers=""{3}"" PriceTypes=""{4}""  UserCartUrlTemplate=""{5}""
	InitQuantity=""1"" AcceptQuantity=""False"" IncludeVATInPrice=""True"" DisplayVAT=""False"" 
	ShowCatalogItemProperties=""MORE_PHOTO;SPECIALOFFER;NEWPRODUCT;SALELEADER"" DisplaySearchResults=""True"" 
	ListProperties=""SPECIALOFFER;SIZE;S_SIZE;ARTNUMBER;MATERIAL;MANUFACTURER;MORE_PHOTO;NEWPRODUCT"" 
	DetailProperties=""SIZE;S_SIZE;ARTNUMBER;MATERIAL;MANUFACTURER;NEWPRODUCT;MORE_PHOTO""
	AllowComparison=""True"" ComparisonSelectedFields=""-SPECIALOFFER;-NEWPRODUCT;-SIZE;-ARTNUMBER;-MATERIAL;-MANUFACTURER""
	ElementDetailTemplate=""/#SectionId#/item-#ElementId#/"" ElementListTemplate=""/#SectionId#/"" 
	CommonListPageTemplate=""/page-#pageId#"" CommonListShowAllTemplate=""/all"" 
	SectionListPageTemplate=""/#SectionId#/page-#pageId#"" SectionListShowAllTemplate=""/#SectionId#/all""
	ElementDetailCommentReadTemplate=""/#SectionId#/item-#ElementId#/comment-#CommentId#/"" ElementDetailCommentOperationTemplate=""/#SectionId#/item-#ElementId#/comment-#CommentId#/act-#Operation#/"" ElementDetailCommentPageTemplate=""/#SectionId#/item-#ElementId#/page-#PageID#/"" 
	ComparisonResultTemplate=""/compare"" />",
					block.TypeId.ToString(),
					block.Id.ToString(),
					directoryPath.Substring(1),
					(priceTypes.Count == 0).ToString(),
					priceTypes.Count > 0 ? priceTypes[0].Id.ToString() : string.Empty,
					BXOptionManager.GetOptionString("Bitrix.CommerceSite", "UserCartUrl", string.Empty));

				string pageData = BXDefaultFiles.BuildAspx(block.Name, filePath, string.Empty, componentData);
				BXSecureIO.SaveAspx(filePath, pageData, Site);

				//Создание пункта меню
				try
				{
					BXPublicMenu lMenu = BXPublicMenu.Menu;
					BXPublicMenuItemCollection items = lMenu.Load(catalogPath, "left");
					BXPublicMenuItem item = items.Find(x => string.Equals(x.Href, BXUri.ToAbsoluteUri(directoryPath), StringComparison.InvariantCultureIgnoreCase));
					if (item == null)
					{
						item = new BXPublicMenuItem();
						item.Link = directoryPath;
						item.Title = block.Name;

						int maxSort = 0;
						foreach (BXPublicMenuItem presentItem in items)
							if (maxSort < presentItem.Sort)
								maxSort = presentItem.Sort;

						item.Sort = maxSort + 10;
						items.Add(item);
						lMenu.Save(catalogPath, "left", items);
					}
				}
				catch
				{
				}
			}

			if (blockCol.Count > 0 && installDemo)
			{
				BXIBlock block = blockCol[0];

				//Права на чтение
				try
				{
					BXRoleManager.AddRolesToOperations(new string[] { "Guest" }, new string[]{ "IBlockRead" }, "iblock", block.Id.ToString());
				}
				catch(System.Configuration.Provider.ProviderException)
				{
				}

				BXOptionManager.DeleteOption("Bitrix.CommerceSite", "DemoDeleted", Site.Id);
				BXOptionManager.SetOptionInt("Bitrix.CommerceSite", "DemoCatalogId", block.Id, Site.Id);

				int stdVatRateId = 0;

				BXCatalogVATRateCollection vatRateCol = BXCatalogVATRate.GetList(
					new BXFilter(new BXFilterItem(BXCatalogVATRate.Fields.Value, BXSqlFilterOperators.Equal, 18m)),
					null,
					new BXSelect(
						BXSelectFieldPreparationMode.Normal, BXCatalogVATRate.Fields.Id),
					new BXQueryParams(new BXPagingOptions(0, 1)));

				if (vatRateCol.Count > 0)
					stdVatRateId = vatRateCol[0].Id;

				BXCatalogItemCollection catalogItemCol = BXCatalogItem.GetList(
					new BXFilter(new BXFilterItem(BXCatalogItem.Fields.CatalogId, BXSqlFilterOperators.Equal, block.Id)),
					null);

				Random rQty = new Random();
				foreach (BXCatalogItem catalogItem in catalogItemCol)
				{
					catalogItem.VATRateId = stdVatRateId;
					catalogItem.Quantity = rQty.Next(10, 101);
					catalogItem.Save();
				}

				BXCatalogPriceTypeCollection priceTypeCol = BXCatalogPriceType.GetList(
					new BXFilter(new BXFilterItem(BXCatalogPriceType.Fields.XmlId, BXSqlFilterOperators.Equal, "BASE")),
					null,
					null,
					new BXQueryParams(BXPagingOptions.Top(1)));
			}
			#region OLD
			/*
			if (blockCol.Count > 0 && installDemo)
			{
				int blockId = blockCol[0].Id;
				int basePriceTypeId = BXCatalogPriceType.BasePriceType.Id;

				BXOptionManager.DeleteOption("Bitrix.CommerceSite", "DemoDeleted", Site.Id);
				BXOptionManager.SetOptionInt("Bitrix.CommerceSite", "DemoCatalogId", blockCol[0].Id, Site.Id);

				int stdVatRateId = 0;

				BXCatalogVATRateCollection vatRateCol = BXCatalogVATRate.GetList(
					new BXFilter(new BXFilterItem(BXCatalogVATRate.Fields.Value, BXSqlFilterOperators.Equal, 18m)),
					null,
					new BXSelect(
						BXSelectFieldPreparationMode.Normal, BXCatalogVATRate.Fields.Id),
					new BXQueryParams(new BXPagingOptions(0, 1)));

				if (vatRateCol.Count > 0)
					stdVatRateId = vatRateCol[0].Id;

				BXCatalogItemCollection presentItems = BXCatalogItem.GetList(
					new BXFilter(new BXFilterItem(BXCatalogItem.Fields.CatalogId, BXSqlFilterOperators.Equal, blockId)),
					null);

				BXCatalogPriceCollection presentPriceTypes = BXCatalogPrice.GetList(
					new BXFilter(
						new BXFilterItem(BXCatalogPrice.Fields.Item.CatalogId, BXSqlFilterOperators.Equal, blockId),
						new BXFilterItem(BXCatalogPrice.Fields.PriceTypeId, BXSqlFilterOperators.Equal, basePriceTypeId)),
						null);

				BXIBlockElementCollection elementCol = BXIBlockElement.GetList(
					new BXFilter(new BXFilterItem(BXIBlockElement.Fields.IBlock.ID, BXSqlFilterOperators.Equal, blockId)),
					null);

				Random rQty = new Random(),
					rWgt = new Random(),
					rPrc = new Random();
				foreach (BXIBlockElement element in elementCol)
				{
					BXCatalogItem item = presentItems.Find(x => x.Id == element.Id);
					if (item == null)
					{
						item = new BXCatalogItem();
						item.Id = element.Id;
						item.CatalogId = element.IBlockId;
					}

					item.VATRateId = stdVatRateId;
					item.Quantity = rQty.Next(10, 101);
					item.Weight = rWgt.Next(1, 10) * 1000; 
					item.Save();

					BXCatalogPrice price = presentPriceTypes.Find(x => x.ItemId == item.Id);
					if(price == null)
					{
						price = new BXCatalogPrice();
						price.ItemId = item.Id;
					}

					price.PriceTypeId = basePriceTypeId;
					price.CurrencyId = BXCurrency.NationalCurrencyId;
					price.VATIncluded = true;
					price.QuantityFrom = 1;
					price.Value = 100 * rPrc.Next(30, 300);
					price.Save();
				}
			}
			*/
			#endregion
			UI.SetProgressBarValue("Installer.ProgressBar", "Bitrix.CommerceSite", 4);
			return Result.Next();
		}
	}
}