﻿using Bitrix.IO;
using Bitrix.UI.Wizards;
using Bitrix.DataTypes;
using Bitrix.Services.Text;
using Bitrix.Install;
using System.Collections.Generic;
using System.IO;
using System.Web;
using Bitrix.Configuration;
using Bitrix.CommerceSite;
using System.Web.Hosting;

namespace Bitrix.Wizards.Solutions
{
	public partial class CommerceSiteFilesStep : BXWizardStepStandardHtmlControl
	{
		private BXSite site = null;
		private BXSite Site
		{
			get { return this.site ?? (this.site = BXSite.GetById(WizardContext.State.GetString("Installer.SiteId"))); }
		}

		private string PrepareFilePath(string directoryPath, string fileName)
		{
			return VirtualPathUtility.Combine(VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine("~/", directoryPath)), fileName);
		}

		protected override BXWizardResult OnActionShow(Bitrix.DataTypes.BXCommonBag parameters)
		{
			BXWizardResultView view = Result.Render(GetMessage("Setup"));
			view.AutoRedirect = true;
			view.RedirectAction = "next";
			return view;
		}

		protected override BXWizardResult OnActionNext(Bitrix.DataTypes.BXCommonBag parameters)
		{
			string solutionPath = WizardContext.State.GetString("Installer.SolutionPath");
			List<BXInstallHelperFileRule> rules = new List<BXInstallHelperFileRule>(
				new[] 
				{ 
					new BXInstallHelperFileRule { Regex = @"^catalog/", Include = false },
					new BXInstallHelperFileRule { Regex = @"^catalog/default.aspx", Include = true },
					new BXInstallHelperFileRule { Regex = @"^src/", Include = false },
					new BXInstallHelperFileRule { Regex = @"^src/welcome.html", Include = true }
				});

			if (!WizardContext.State.GetBool("CommerceSite.Overwrite", false))
				rules.Insert(0, new BXInstallHelperFileRule { Overwrite = false, Include = true });

			BXInstallHelper.CopyDirectory(
				BXPath.Combine(solutionPath, "public"),
				Site.DirectoryVirtualPath,
				rules
			);

			string srcDirPath = BXPath.MapPath(Site.DirectoryVirtualPath + "src");
			Directory.CreateDirectory(srcDirPath);

			int logoFileId = WizardContext.State.GetInt("SiteLogoImage", 0);
			if(logoFileId > 0)
			{
				BXFile logoFile = BXFile.GetById(logoFileId, BXTextEncoder.EmptyTextEncoder);
				if(logoFile != null)
					File.WriteAllText(Path.Combine(srcDirPath, "logo.html"), string.Concat(@"<img alt=""logo"" src=""", HttpUtility.HtmlAttributeEncode(logoFile.FilePath), @""" />"));
			}

			File.WriteAllText(Path.Combine(srcDirPath, "logotext.html"), WizardContext.State.GetString("SiteName", string.Empty));
			File.WriteAllText(Path.Combine(srcDirPath, "phone.html"), WizardContext.State.GetString("SitePhone", string.Empty));
			File.WriteAllText(Path.Combine(srcDirPath, "schedule.html"), WizardContext.State.GetString("SiteSchedule", string.Empty));
			File.WriteAllText(Path.Combine(srcDirPath, "copyright.html"), WizardContext.State.GetString("SiteCopyRight", string.Empty));
			File.WriteAllText(Path.Combine(srcDirPath, "contactemail.html"), WizardContext.State.GetString("SiteWebmasterEmail", string.Empty));
			File.WriteAllText(Path.Combine(srcDirPath, "delivery.html"), WizardContext.State.GetString("DeliveryServiceContactInfo", string.Empty));
			
			string stampFilePath = WizardContext.State.GetString("StoreStampFilePath", string.Empty),
				stampFileName = WizardContext.State.GetString("StoreStampFileName", string.Empty);
				//stampFileMimeType = WizardContext.State.GetString("StoreStampFileMimeType", string.Empty);

			if(!string.IsNullOrEmpty(stampFilePath) 
				&& !string.IsNullOrEmpty(stampFileName))
			{
				FileInfo fi = new FileInfo(HostingEnvironment.MapPath(stampFilePath));
				if(fi.Exists)
				{
					fi.CopyTo(Path.Combine(srcDirPath, stampFileName), true);
					BXOptionManager.SetOptionString(BXCommerceSiteModuleConfiguration.Id, "StoreStampFileUrl", PrepareFilePath(Site.DirectoryVirtualPath, string.Concat("src/", stampFileName)));
				}
			}
			UI.SetProgressBarValue("Installer.ProgressBar", "Bitrix.CommerceSite", 1);
			return Result.Next();
		}
	}
}