﻿using Bitrix.IO;
using Bitrix.UI.Wizards;
using Bitrix.DataTypes;
using System.Collections.Generic;
using Bitrix.Services.Text;
using Bitrix.Install;
using System.IO;
using System;
using System.Text;
using Bitrix.Configuration;

namespace Bitrix.Wizards.Solutions
{
	public partial class CommerceSiteMenuStep : BXWizardStepStandardHtmlControl
	{
		const string LinkPrefix = "estore";
		private KeyValuePair<string, string>[] menuTypes = null;
		KeyValuePair<string, string>[] MenuTypes
		{
			get
			{
				if(menuTypes != null)
					return menuTypes;

				return (menuTypes = new KeyValuePair<string, string>[] {
					new KeyValuePair<string, string>("top", GetMessage("MenuType.Main")),
					new KeyValuePair<string, string>("left", GetMessage("MenuType.Left")),
					new KeyValuePair<string, string>("bottom", GetMessage("MenuType.Bottom")),				
				});
			}
		}

		protected override BXWizardResult OnActionShow(Bitrix.DataTypes.BXCommonBag parameters)
		{
			var view = Result.Render(GetMessage("Setup"));
			view.AutoRedirect = true;
			view.RedirectAction = "next";
			return view;
		}

		protected override BXWizardResult OnActionNext(Bitrix.DataTypes.BXCommonBag parameters)
		{
			var siteId = WizardContext.State.GetString("Installer.SiteId");
			var site = BXSite.GetById(siteId, BXTextEncoder.EmptyTextEncoder);

			BXConfigurationUtility.Options.Site[siteId].SiteMapMenuTypes = "top,left;bottom";

			var menuItems = BXPublicMenu.GetMenuTypes(siteId, true);
			foreach (var p in MenuTypes)
			{
				if (!menuItems.ContainsKey(p.Key))
					menuItems[p.Key] = p.Value;
			}
			BXPublicMenu.SetMenuTypes(siteId, menuItems);

			var solutionPath = WizardContext.State.GetString("Installer.SolutionPath");
			var rootDir = BXPath.Combine(solutionPath, "public");

			try
			{
				BXInstallHelper.ProcessFiles(
					rootDir,
					delegate(FileInfo file, string rel)
					{

						BXPublicMenuItemCollection items;
						using (var stream = file.OpenRead())
						using (var reader = System.Xml.XmlReader.Create(stream))
							items = BXPublicMenu.LoadXml(reader);

						foreach (var item in items)
						{
							for (int i = item.Links.Count - 1; i >= 0; i--)
							{
								var link = item.Links[i];
								if (!link.StartsWith(LinkPrefix, StringComparison.OrdinalIgnoreCase))
									continue;

								if (link.Length == LinkPrefix.Length)
									link = "";
								else
								{
									char c = link[LinkPrefix.Length];
									if (c != '/' && c != '\\')
										continue;

									if (link.Length == LinkPrefix.Length + 1)
										link = "";
									else
										link = link.Substring(LinkPrefix.Length + 1).Replace('\\', '/');
								}
								item.Links[i] = site.DirectoryVirtualPath + link;
							}
						}

						var settings = new System.Xml.XmlWriterSettings();
						settings.Encoding = Encoding.UTF8;
						settings.Indent = true;
						settings.IndentChars = "\t";
						using (var writer = System.Xml.XmlWriter.Create(BXPath.MapPath(site.DirectoryVirtualPath + rel), settings))
						{
							writer.WriteStartDocument();
							BXPublicMenu.SaveXml(writer, items);
						}
					},
					new[]
					{
					new BXInstallHelperFileRule(),
					new BXInstallHelperFileRule { Regex = @"\.menu$", Include = true }//,
					//new BXInstallHelperFileRule { Regex = @"left\.menu$", Include = false }
					}
				);
			}
			catch
			{
			}

			BXPublicMenu.Menu.RefreshSettings();
			UI.SetProgressBarValue("Installer.ProgressBar", "Bitrix.CommerceSite", 3);

			return Result.Next();
		}
	}
}