﻿using Bitrix.IO;
using Bitrix.UI.Wizards;
using Bitrix.DataTypes;
using Bitrix.Services.Text;
using System.Collections.Generic;
using Bitrix.Configuration;
using Bitrix.Catalog;
using Bitrix.CommerceSite;
using Bitrix.IBlock;
using Bitrix.DataLayer;
using Bitrix.Sale;

namespace Bitrix.Wizards.Solutions
{
	public partial class CommerceSiteOptionsStep : BXWizardStepStandardHtmlControl
	{
		private BXSite site = null;
		private BXSite Site
		{
			get { return this.site ?? (this.site = BXSite.GetById(WizardContext.State.GetString("Installer.SiteId"))); }
		}

		protected override BXWizardResult OnActionShow(BXCommonBag parameters)
		{
			var	view = Result.Render(GetMessage("Setup"));
			view.AutoRedirect = true;
			view.RedirectAction = "next";
			return view;
		}

		protected override BXWizardResult OnActionNext(BXCommonBag parameters)
		{
			Dictionary<string, string> options = new Dictionary<string, string>
			{
				{ "AdminEmail", WizardContext.State.GetString("SiteWebmasterEmail", string.Empty) },
				{ "SpecialOffer.IBlockElementUrlTemplate", Site.DirectoryVirtualPath + "catalog/#IBlockCode#/#SectionId#/item-#ElementId#/" },
				{ "News.DetailUrl", Site.DirectoryAbsolutePath + "news/#SectionId#/news-#ElementId#/" },
				{ "News.RssUrl", Site.DirectoryAbsolutePath+"news/rss/" },
				{ "WelcomeIncludeAreaFilePath", Site.DirectoryAbsolutePath + "src/welcome.html" },
				{ "NewProducts.IBlockElementUrlTemplate", Site.DirectoryVirtualPath + "catalog/#IBlockCode#/#SectionId#/item-#ElementId#/" },
				{ "SaleLeaders.IBlockElementUrlTemplate", Site.DirectoryVirtualPath + "catalog/#IBlockCode#/#SectionId#/item-#ElementId#/" },
				{ "Catalogue.BasePriceTypeId", BXCatalogPriceType.BasePriceType != null ? BXCatalogPriceType.BasePriceType.Id.ToString() : string.Empty },
				{ "LogoIncludeAreaFilePath", Site.DirectoryAbsolutePath + "src/logo.html" },
				{ "LogoTextIncludeAreaFilePath", Site.DirectoryAbsolutePath + "src/logotext.html" },
				{ "PhoneIncludeAreaFilePath", Site.DirectoryAbsolutePath + "src/phone.html" },
				{ "ScheduleIncludeAreaFilePath", Site.DirectoryAbsolutePath + "src/schedule.html" },
				{ "CopyrightIncludeAreaFilePath", Site.DirectoryAbsolutePath + "src/copyright.html" },
				{ "DeliveryIncludeAreaFilePath", Site.DirectoryAbsolutePath + "src/delivery.html" },
				{ "ContactsEmailIncludeAreaFilePath", Site.DirectoryAbsolutePath + "src/contactemail.html" },
				{ "ProfilePath", Site.DirectoryAbsolutePath + "personal/profile/" },
				{ "RegisterPath", Site.DirectoryAbsolutePath + "login/register/" },
				{ "LoginPath", Site.DirectoryAbsolutePath + "login/" },
				{ "CartUrlTemplate", Site.DirectoryAbsolutePath + "personal/cart/" },
				{ "SearchForm.SearchUrlTemplate", Site.DirectoryAbsolutePath + "search/?q=#query#" },
				{ "VerticalMenu.RootUrl", Site.DirectoryAbsolutePath + "catalog/" },
				{ "OrdersListUrl", Site.DirectoryAbsolutePath + "personal/orders/" },
				{ "NewOrdersUrl", Site.DirectoryAbsolutePath + "personal/orders/new/" },
				{ "RegisterRedirectUrl", site.DirectoryAbsolutePath },
				{ "NewsSefFolder", (Site.DirectoryVirtualPath + "/news/").Substring(1) },
				{ "LoginSefFolder", (Site.DirectoryVirtualPath + "/login/").Substring(1) },
				{"IBlockList.IBlockUrlTemplate", Site.DirectoryVirtualPath + "catalog/#IBlockCode#/" },
				{"IBlockList.IBlockUrlSectionTemplate", Site.DirectoryAbsolutePath + "catalog/#IBlockCode#/#SectionId#/"}
			};
			
			foreach(KeyValuePair<string, string> kv in options)
				BXOptionManager.SetOptionString(BXCommerceSiteModuleConfiguration.Id, kv.Key, kv.Value, Site.Id);

			//BXOptionManager.SetOptionString(BXCommerceSiteModuleConfiguration.Id, "Catalogue.IBlockIds", BXStringUtility.ListToCsv(blockIdList), Site.Id);
			List<string> blockIdList = new List<string>(); 
			//BXIBlockCollection blockCol = BXIBlock.GetList(
			//    new BXFilter(new BXFilterItem(BXIBlock.Fields.XmlId, BXSqlFilterOperators.Equal, "Bitrix.CommerceSite.Catalog." + Site.Id)),
			//    null);

			BXIBlockCollection blockCol = BXIBlock.GetList(
				new BXFilter(
					new BXFilterItem(BXIBlock.Fields.Type.XmlId, BXSqlFilterOperators.Equal, "type_iblock_commerce_catalog"),
					new BXFilterItem(BXIBlock.Fields.Sites.SiteId, BXSqlFilterOperators.Equal, Site.Id)
					),
				new BXOrderBy(new BXOrderByPair(BXIBlock.Fields.UpdateDate, BXOrderByDirection.Desc)),
				null,
				null);

			foreach(BXIBlock block in blockCol)
				blockIdList.Add(block.Id.ToString());

			BXOptionManager.SetOptionString(BXCommerceSiteModuleConfiguration.Id, "Catalogue.IBlockIds", BXStringUtility.ListToCsv(blockIdList), Site.Id);
			BXOptionManager.SetOptionInt(BXCommerceSiteModuleConfiguration.Id, "CatalogIBlockTypeId", blockCol.Count > 0 ? blockCol[0].TypeId : 0, Site.Id);

			BXSale sale = BXSale.GetBySiteId(Site.Id);
			sale.PostalCode = WizardContext.State.GetString("StorePostalCode");
			sale.CountryNumericCode = 643;
			sale.Name = WizardContext.State.GetString("StoreName", string.Empty);
			sale.Address = WizardContext.State.GetString("StoreAddress", string.Empty);
			sale.Inn = WizardContext.State.GetString("StoreInn", string.Empty);
			sale.Kpp = WizardContext.State.GetString("StoreKpp", string.Empty);
			sale.SettlementAccount = WizardContext.State.GetString("StoreSettlementAccount", string.Empty);
			sale.CorrespondentAccount = WizardContext.State.GetString("StoreCorrespondentAccount", string.Empty);
			sale.BankName = WizardContext.State.GetString("StoreBankName", string.Empty);
			sale.BankBic = WizardContext.State.GetString("StoreBankBic", string.Empty);
			sale.Save();

			blockCol = BXIBlock.GetList(
				new BXFilter(new BXFilterItem(BXIBlock.Fields.XmlId, BXSqlFilterOperators.Equal, "iblock_commerce_news." + Site.Id)),
				null);

			if(blockCol.Count > 0)
				BXOptionManager.SetOptionInt(BXCommerceSiteModuleConfiguration.Id, "NewsIBlockId", blockCol[0].Id, Site.Id);

			blockCol = BXIBlock.GetList(
				new BXFilter(new BXFilterItem(BXIBlock.Fields.XmlId, BXSqlFilterOperators.Equal, "iblock_commerce_messages_feedback." + Site.Id)),
				null);

			if(blockCol.Count > 0)
				BXOptionManager.SetOptionInt(BXCommerceSiteModuleConfiguration.Id, "FeedbackIBlockId", blockCol[0].Id, Site.Id);

			UI.SetProgressBarValue("Installer.ProgressBar", "Bitrix.CommerceSite", 5);
			return Result.Next();
		}
	}
}