﻿using Bitrix.IO;
using Bitrix.UI.Wizards;
using Bitrix.DataTypes;
using Bitrix.Services.Text;
using Bitrix.Sale;
using Bitrix.Currency;
using Bitrix.Configuration;
using Bitrix.CommerceSite;
using Bitrix.DataLayer;
using Bitrix.Catalog;
using System.Collections.Generic;
using Bitrix.Services;
using Bitrix.Security;
using System;
using System.Web;

namespace Bitrix.Wizards.Solutions
{
	public partial class CommerceSiteStructureStep : BXWizardStepStandardHtmlControl
	{
		protected override BXWizardResult OnActionShow(Bitrix.DataTypes.BXCommonBag parameters)
		{
			BXWizardResultView view = Result.Render(GetMessage("Setup"));
			view.AutoRedirect = true;
			view.RedirectAction = "next";
			return view;
		}
		protected override BXWizardResult OnActionNext(Bitrix.DataTypes.BXCommonBag parameters)
		{
			try
			{
				if (BXCatalogPriceType.BasePriceType == null)
				{
					BXCatalogPriceType basePriceType = null;

					BXCatalogPriceTypeCollection priceTypeCol = BXCatalogPriceType.GetList(
						new BXFilter(new BXFilterItem(BXCatalogPriceType.Fields.XmlId, BXSqlFilterOperators.Equal, "BASE")),
						null,
						null,
						new BXQueryParams(BXPagingOptions.Top(1)));

					if(priceTypeCol.Count > 0)
						basePriceType = priceTypeCol[0];
					else
					{
						basePriceType = new BXCatalogPriceType();
						basePriceType.Code = "BASE";
						basePriceType.CurrencyId = BXCurrency.NationalCurrencyId;
						basePriceType.Sort = 100;
					}

					basePriceType.IsBase = true;

					List<BXCatalogPriceTypeLocalizationInfo> locInfoList = new List<BXCatalogPriceTypeLocalizationInfo>();
					foreach (string loc in BXLoc.Locales)
					{
						string msg = BXLoc.GetModuleMessage(BXCatalogModuleConfiguration.Id, "BasePriceTypeName", true);
						if (!string.IsNullOrEmpty(msg))
							locInfoList.Add(new BXCatalogPriceTypeLocalizationInfo(loc, msg));
					}
					basePriceType.LocalizationInfos = locInfoList.ToArray();

					BXRole[] roles = new BXRole[] { BXRoleManager.GetByName("Guest") ?? BXRoleManager.GetByName("User") ?? BXRoleManager.GetByName("Admin") };
					basePriceType.SetAllowedToViewRoles(roles);
					basePriceType.SetAssignedToRoles(roles);
					basePriceType.Create();
				}

				BXCatalogVATRateCollection vatRateCol = BXCatalogVATRate.GetList(
					new BXFilter(new BXFilterItem(BXCatalogVATRate.Fields.Value, BXSqlFilterOperators.Equal, 0m)),
					null,
					new BXSelect(
						BXSelectFieldPreparationMode.Normal, BXCatalogVATRate.Fields.Id),
					new BXQueryParams(new BXPagingOptions(0, 1)));

				if (vatRateCol.Count == 0)
				{
					BXCatalogVATRate zeroVatRate = new BXCatalogVATRate();
					zeroVatRate.Active = true;
					zeroVatRate.Name = BXCatalogModule.GetMessage("VatRateName.NoVat", false);
					zeroVatRate.Value = 0m;
					zeroVatRate.Create();
				}

				vatRateCol = BXCatalogVATRate.GetList(
				   new BXFilter(new BXFilterItem(BXCatalogVATRate.Fields.Value, BXSqlFilterOperators.Equal, 18m)),
				   null,
				   new BXSelect(
					   BXSelectFieldPreparationMode.Normal, BXCatalogVATRate.Fields.Id),
				   new BXQueryParams(new BXPagingOptions(0, 1)));

				if (vatRateCol.Count == 0)
				{
					BXCatalogVATRate stdVatRate = new BXCatalogVATRate();
					stdVatRate.Active = true;
					stdVatRate.Name = BXCatalogModule.GetMessage("VatRateName.18PcVat", false);
					stdVatRate.Value = 18m;
					stdVatRate.Create();
				}

				string siteId = WizardContext.State.GetString("Installer.SiteId");

				BXRole registerUser = BXRoleManager.GetRole("User");

				if (registerUser != null)
				{
					//Edit Profile
					try
					{
						if (!registerUser.GetOperations().Exists
							(
								delegate(BXRoleOperation operation)
								{
									return String.Equals(operation.OperationName, "UserModifySelf", StringComparison.OrdinalIgnoreCase);
								}
							)
						)
							registerUser.AddToOperations(new string[] { "UserModifySelf" });
					}
					catch
					{
					}
				}

				BXSaleCustomerType phy = null, jur = null;
				//типы плательщиков
				BXSaleCustomerTypeCollection customerTypeCol = BXSaleCustomerType.GetList(
					new BXFilter(
						new BXFilterItem(BXSaleCustomerType.Fields.XmlId, BXSqlFilterOperators.Equal, "COMMERCE.SITE.PHYSICAL"),
						new BXFilterItem(BXSaleCustomerType.Fields.Site.ID, BXSqlFilterOperators.Equal, siteId)
						),
					null);
				if (WizardContext.State.GetBool("EnablePhysicalPerson", false) && customerTypeCol.Count == 0)
				{
					phy = new BXSaleCustomerType();
					phy.Name = GetMessage("PhysicalPersonName");
					phy.Sort = 100;
					phy.Code = "PHYSICAL";
					phy.SiteId = siteId;
					phy.XmlId = "COMMERCE.SITE.PHYSICAL";
					phy.Create();
				}
				else if (WizardContext.State.GetBool("EnablePhysicalPerson", false) && customerTypeCol.Count > 0)
					phy = customerTypeCol[0];

				customerTypeCol = BXSaleCustomerType.GetList(
					new BXFilter(
						new BXFilterItem(BXSaleCustomerType.Fields.XmlId, BXSqlFilterOperators.Equal, "COMMERCE.SITE.JURISTIC"),
						new BXFilterItem(BXSaleCustomerType.Fields.Site.ID, BXSqlFilterOperators.Equal, siteId)
						),
					null);
				if (WizardContext.State.GetBool("EnableJuristicPerson", false) && customerTypeCol.Count == 0)
				{
					jur = new BXSaleCustomerType();
					jur.Name = GetMessage("JuristicPersonName");
					jur.Sort = phy != null ? 200 : 100;
					jur.SiteId = siteId;
					jur.Code = "JURISTIC";
					jur.XmlId = "COMMERCE.SITE.JURISTIC";
					jur.Create();
				}
				else if (WizardContext.State.GetBool("EnableJuristicPerson", false) && customerTypeCol.Count > 0)
					jur = customerTypeCol[0];

				//свойства заказа
				if (phy != null)
				{
					BXSaleOrderFieldCollection fCol = BXSaleOrderField.GetList(
						new BXFilter(
							new BXFilterItem(BXSaleOrderField.Fields.Code, BXSqlFilterOperators.Equal, "EMAIL"),
							new BXFilterItem(BXSaleOrderField.Fields.CustomerType.Id, BXSqlFilterOperators.Equal, phy.Id)
							),
						null, null, null);

					if (fCol.Count == 0)
					{
						BXSaleOrderField f = new BXSaleOrderField();
						f.Code = "EMAIL";
						f.CustomerTypeId = phy.Id;
						f.CustomTypeId = "Bitrix.System.Text";
						f.Name = "E-mail";
						f.Settings["ValidationRegex"] = @"^[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+(\.[A-Za-z]+)?$";
						f.UseAsEmail = true;
						f.Required = true;
						f.Save();
					}

					fCol = BXSaleOrderField.GetList(
						new BXFilter(
							new BXFilterItem(BXSaleOrderField.Fields.Code, BXSqlFilterOperators.Equal, "POSTALCODE"),
							new BXFilterItem(BXSaleOrderField.Fields.CustomerType.Id, BXSqlFilterOperators.Equal, phy.Id)
							),
						null, null, null);

					if (fCol.Count == 0)
					{
						BXSaleOrderField f = new BXSaleOrderField();
						f.Code = "POSTALCODE";
						f.CustomerTypeId = phy.Id;
						f.CustomTypeId = "Bitrix.System.Text";
						f.Name =  GetMessage("PostalCode");
						f.UseAsPostalCode = true;
						f.Required = true;
						f.Save();
					}

					fCol = BXSaleOrderField.GetList(
						new BXFilter(
							new BXFilterItem(BXSaleOrderField.Fields.Code, BXSqlFilterOperators.Equal, "CITY"),
							new BXFilterItem(BXSaleOrderField.Fields.CustomerType.Id, BXSqlFilterOperators.Equal, phy.Id)
							),
						null, null, null);

					if (fCol.Count == 0)
					{
						BXSaleOrderField f = new BXSaleOrderField();
						f.Code = "CITY";
						f.CustomerTypeId = phy.Id;
						f.CustomTypeId = "Bitrix.System.Text";
						f.Name =  GetMessage("City");
						f.UseAsLocation = true;
						f.Required = true;
						f.Save();
					}

					fCol = BXSaleOrderField.GetList(
						new BXFilter(
							new BXFilterItem(BXSaleOrderField.Fields.Code, BXSqlFilterOperators.Equal, "ADDRESS"),
							new BXFilterItem(BXSaleOrderField.Fields.CustomerType.Id, BXSqlFilterOperators.Equal, phy.Id)
							),
						null, null, null);

					if (fCol.Count == 0)
					{
						BXSaleOrderField f = new BXSaleOrderField();
						f.Code = "ADDRESS";
						f.CustomerTypeId = phy.Id;
						f.CustomTypeId = "Bitrix.System.Text";
						f.Name = GetMessage("Address");
						f.Settings["RowsCount"] = 3;
						f.Required = true;
						f.Save();
					}

					fCol = BXSaleOrderField.GetList(
						new BXFilter(
							new BXFilterItem(BXSaleOrderField.Fields.Code, BXSqlFilterOperators.Equal, "FULLNAME"),
							new BXFilterItem(BXSaleOrderField.Fields.CustomerType.Id, BXSqlFilterOperators.Equal, phy.Id)
							),
						null, null, null);

					if (fCol.Count == 0)
					{
						BXSaleOrderField f = new BXSaleOrderField();
						f.Code = "FULLNAME";
						f.CustomerTypeId = phy.Id;
						f.CustomTypeId = "Bitrix.System.Text";
						f.Name = GetMessage("FullName");
						f.UseAsProfileName = true;
						f.Required = true;
						f.Save();
					}
				}

				if (jur != null)
				{
					BXSaleOrderFieldCollection fCol = BXSaleOrderField.GetList(
						new BXFilter(
							new BXFilterItem(BXSaleOrderField.Fields.Code, BXSqlFilterOperators.Equal, "INN"),
							new BXFilterItem(BXSaleOrderField.Fields.CustomerType.Id, BXSqlFilterOperators.Equal, jur.Id)
							),
						null, null, null);

					if (fCol.Count == 0)
					{
						BXSaleOrderField f = new BXSaleOrderField();
						f.Code = "INN";
						f.CustomerTypeId = jur.Id;
						f.CustomTypeId = "Bitrix.System.Text";
						f.Name = GetMessage("Inn");
						f.Required = true;
						f.Save();
					}

					fCol = BXSaleOrderField.GetList(
						new BXFilter(
							new BXFilterItem(BXSaleOrderField.Fields.Code, BXSqlFilterOperators.Equal, "KPP"),
							new BXFilterItem(BXSaleOrderField.Fields.CustomerType.Id, BXSqlFilterOperators.Equal, jur.Id)
							),
						null, null, null);

					if (fCol.Count == 0)
					{
						BXSaleOrderField f = new BXSaleOrderField();
						f.Code = "KPP";
						f.CustomerTypeId = jur.Id;
						f.CustomTypeId = "Bitrix.System.Text";
						f.Name = GetMessage("Kpp");
						f.Save();
					}

					fCol = BXSaleOrderField.GetList(
						new BXFilter(
							new BXFilterItem(BXSaleOrderField.Fields.Code, BXSqlFilterOperators.Equal, "COMPANYADDRESS"),
							new BXFilterItem(BXSaleOrderField.Fields.CustomerType.Id, BXSqlFilterOperators.Equal, jur.Id)
							),
						null, null, null);

					if (fCol.Count == 0)
					{
						BXSaleOrderField f = new BXSaleOrderField();
						f.Code = "COMPANYADDRESS";
						f.CustomerTypeId = jur.Id;
						f.CustomTypeId = "Bitrix.System.Text";
						f.Name = GetMessage("JuristicAddress");
						f.Settings["RowsCount"] = 3;
						f.Required = true;
						f.Save();
					}

					fCol = BXSaleOrderField.GetList(
						new BXFilter(
							new BXFilterItem(BXSaleOrderField.Fields.Code, BXSqlFilterOperators.Equal, "COMPANYNAME"),
							new BXFilterItem(BXSaleOrderField.Fields.CustomerType.Id, BXSqlFilterOperators.Equal, jur.Id)
							),
						null, null, null);

					if (fCol.Count == 0)
					{
						BXSaleOrderField f = new BXSaleOrderField();
						f.Code = "COMPANYNAME";
						f.CustomerTypeId = jur.Id;
						f.CustomTypeId = "Bitrix.System.Text";
						f.Name = GetMessage("CompanyName");
						f.UseAsProfileName = true;
						f.Required = true;
						f.Save();
					}
				}

				//службы доставки
				int customDeliverySort = 100;
				BXCustomizableDeliveryServiceCollection dsCol = BXCustomizableDeliveryService.GetList(
					new BXFilter(
						new BXFilterItem(BXCustomizableDeliveryService.Fields.XmlId, BXSqlFilterOperators.Equal, "COMMERCE.SITE.COURIER"),
						new BXFilterItem(BXCustomizableDeliveryService.Fields.SiteId, BXSqlFilterOperators.Equal, siteId)
						),
					null);

				if (WizardContext.State.GetBool("CourierDelivery", false) && dsCol.Count == 0)
				{
					BXCustomizableDeliveryService courier = new BXCustomizableDeliveryService();
					courier.Active = true;
					courier.Code = "COURIER";
					courier.CurrencyId = BXCurrency.NationalCurrencyId;
					courier.DeliveryCost = 500m;
					courier.SiteId = siteId;
					courier.DisplayName = GetMessage("CourierDeliveryName");
					courier.Description = GetMessage("CourierDeliveryDescription");
					courier.Sort = customDeliverySort++;
					courier.XmlId = "COMMERCE.SITE.COURIER";
					courier.Create();
				}
				else if (!WizardContext.State.GetBool("CourierDelivery", false) && dsCol.Count > 0)
					dsCol[0].Active = false;

				dsCol = BXCustomizableDeliveryService.GetList(
					new BXFilter(
						new BXFilterItem(BXCustomizableDeliveryService.Fields.XmlId, BXSqlFilterOperators.Equal, "COMMERCE.SITE.PICKUP"),
						new BXFilterItem(BXCustomizableDeliveryService.Fields.SiteId, BXSqlFilterOperators.Equal, siteId)
						),
					null);

				if (WizardContext.State.GetBool("PickupDelivery", false) && dsCol.Count == 0)
				{
					BXCustomizableDeliveryService pickup = new BXCustomizableDeliveryService();
					pickup.Active = true;
					pickup.Code = "PICKUP";
					pickup.CurrencyId = BXCurrency.NationalCurrencyId;
					pickup.DeliveryCost = 0m;
					pickup.SiteId = siteId;
					pickup.DisplayName = GetMessage("PickupDeliveryName");
					pickup.Description = GetMessage("PickupDeliveryDescription");
					pickup.Sort = customDeliverySort++;
					pickup.XmlId = "COMMERCE.SITE.PICKUP";
					pickup.Create();
				}
				else if (!WizardContext.State.GetBool("PickupDelivery", false) && dsCol.Count > 0)
					dsCol[0].Active = false;

				BXRussianPostDeliveryServiceConfig rusPostConfig = new BXRussianPostDeliveryServiceConfig();
				rusPostConfig.Active = WizardContext.State.GetBool("RussianPostDelivery", false);
				rusPostConfig.Name = GetMessage("RussianPostName");
				rusPostConfig.Description = string.Empty;
				rusPostConfig.PostingType = "0"; //авто
				rusPostConfig.DeliveryTypes = new string[] { "1" }; //наземная
				rusPostConfig.SiteId = siteId;
				rusPostConfig.Sort = 100;
				rusPostConfig.Save();

				BXEmsDeliveryServiceConfig emsConfig = new BXEmsDeliveryServiceConfig();
				emsConfig.Active = WizardContext.State.GetBool("EmsRussianPostDelivery", false);
				emsConfig.Name = GetMessage("EmsRussianPostName");
				emsConfig.Description = string.Empty;
				emsConfig.CargoType =  BXSaleDeliveryCargoType.Commodity; 
				emsConfig.SiteId = siteId;
				emsConfig.Sort = 200;
				emsConfig.Save();

				//системы оплаты
				int paymentSysSort = 100;
				BXSalePaymentSystemCollection psCol = BXSalePaymentSystem.GetList(
					new BXFilter(
						new BXFilterItem(BXSalePaymentSystem.Fields.XmlId, BXSqlFilterOperators.Equal, "COMMERCE.SITE.CASH"),
						new BXFilterItem(BXSalePaymentSystem.Fields.SiteId, BXSqlFilterOperators.Equal, siteId)
						),
					null);

				if (phy != null && WizardContext.State.GetBool("EnableCashPayment", false) && psCol.Count == 0)
				{
					BXSalePaymentSystem cash = new BXSalePaymentSystem();
					cash.Active = true;
					cash.Name = GetMessage("CashPaymentSysName");
					cash.Description = GetMessage("CashPaymentSysDescription");
					cash.SiteId = siteId;
					cash.Sort = paymentSysSort++;
					cash.XmlId = "COMMERCE.SITE.CASH";
					cash.Create();

					BXSalePaymentSystemConfig cashCfg = new BXSalePaymentSystemConfig();
					cashCfg.Name = GetMessage("CashPaymentSysName");
					cashCfg.PaymentSystemId = cash.Id;
					cashCfg.ControllerTypeId = BXCashPaymentControllerInfo.GetControllerTypeId();
					cashCfg.CustomerTypeId = phy.Id;
					cashCfg.Save();
				}
				else if ((phy == null || !WizardContext.State.GetBool("EnableCashPayment", false)) && psCol.Count > 0)
					psCol[0].Active = false;

				psCol = BXSalePaymentSystem.GetList(
					new BXFilter(
						new BXFilterItem(BXSalePaymentSystem.Fields.XmlId, BXSqlFilterOperators.Equal, "COMMERCE.SITE.SBERBANK"),
						new BXFilterItem(BXSalePaymentSystem.Fields.SiteId, BXSqlFilterOperators.Equal, siteId)
						),
					null);

				if (phy != null && WizardContext.State.GetBool("EnableSberbankPayment", false) && psCol.Count == 0)
				{
					BXSalePaymentSystem sberbank = new BXSalePaymentSystem();
					sberbank.Active = true;
					sberbank.Name = GetMessage("SberbankPaymentSysName");
					sberbank.Description = GetMessage("SberbankPaymentSysDescription");
					sberbank.SiteId = siteId;
					sberbank.Sort = paymentSysSort++;
					sberbank.XmlId = "COMMERCE.SITE.SBERBANK";
					sberbank.Create();

					BXSalePaymentSystemConfig sberbankCfg = new BXSalePaymentSystemConfig();
					sberbankCfg.Name = GetMessage("SberbankPaymentSysName");
					sberbankCfg.PaymentSystemId = sberbank.Id;
					sberbankCfg.ControllerTypeId = BXSberbankPaymentControllerInfo.GetControllerTypeId();
					sberbankCfg.CustomerTypeId = phy.Id;
					BXSalePaymentControllerSettings settings = sberbankCfg.ControllerSettings;

					settings.AddParam("companyName", WizardContext.State.GetString("StoreName", string.Empty));
					settings.AddParam("inn", WizardContext.State.GetString("StoreInn", string.Empty));
					settings.AddParam("kpp", WizardContext.State.GetString("StoreKpp", string.Empty));
					settings.AddParam("settlementAccount", WizardContext.State.GetString("StoreSettlementAccount", string.Empty));
					settings.AddParam("correspondentAccount", WizardContext.State.GetString("StoreCorrespondentAccount", string.Empty));
					settings.AddParam("bankName", WizardContext.State.GetString("StoreBankName", string.Empty));
					settings.AddParam("bic", WizardContext.State.GetString("StoreBankBic", string.Empty));
					settings.AddParam("orderId", "O:Id");
					settings.AddParam("orderCreationDate", "O:Created");
					settings.AddParam("customerFullName", "P:FULLNAME");
					settings.AddParam("customerPostalCode", "P:POSTALCODE");
					settings.AddParam("customerAddress", "P:ADDRESS");
					settings.AddParam("sum", "O:Sum");
					sberbankCfg.Save();
				}
				else if ((phy == null || !WizardContext.State.GetBool("EnableSberbankPayment", false)) && psCol.Count > 0)
					psCol[0].Active = false;

				psCol = BXSalePaymentSystem.GetList(
					new BXFilter(
						new BXFilterItem(BXSalePaymentSystem.Fields.XmlId, BXSqlFilterOperators.Equal, "COMMERCE.SITE.BILL"),
						new BXFilterItem(BXSalePaymentSystem.Fields.SiteId, BXSqlFilterOperators.Equal, siteId)
						),
					null);

				if (jur != null && WizardContext.State.GetBool("EnableBillPayment") && psCol.Count == 0)
				{
					BXSalePaymentSystem bill = new BXSalePaymentSystem();
					bill.Active = true;
					bill.Name = GetMessage("BillPaymentSysName");
					bill.Description = GetMessage("BillPaymentSysDescription");
					bill.SiteId = siteId;
					bill.Sort = paymentSysSort++;
					bill.XmlId = "COMMERCE.SITE.BILL";
					bill.Create();

					BXSalePaymentSystemConfig billCfg = new BXSalePaymentSystemConfig();
					billCfg.Name = GetMessage("BillPaymentSysName");
					billCfg.PaymentSystemId = bill.Id;
					billCfg.ControllerTypeId = BXBillPaymentControllerInfo.GetControllerTypeId();
					billCfg.CustomerTypeId = jur.Id;
					BXSalePaymentControllerSettings settings = billCfg.ControllerSettings;

					settings.AddParam("sellerName", WizardContext.State.GetString("StoreName", string.Empty));
					settings.AddParam("orderCreationDate", "O:Created");
					settings.AddParam("sellerAddress", WizardContext.State.GetString("StoreAddress", string.Empty));
					settings.AddParam("sellerPhone", "");
					settings.AddParam("sellerInn", WizardContext.State.GetString("StoreInn", string.Empty));
					settings.AddParam("sellerKpp", WizardContext.State.GetString("StoreKpp", string.Empty));
					settings.AddParam("sellerSettlementAccount", WizardContext.State.GetString("StoreSettlementAccount", string.Empty));
					settings.AddParam("sellerCorrespondentAccount", WizardContext.State.GetString("StoreCorrespondentAccount", string.Empty));
					settings.AddParam("sellerBic", WizardContext.State.GetString("StoreBankBic", string.Empty));
					settings.AddParam("buyerName", "P:COMPANYNAME");
					settings.AddParam("buyerInn", "P:INN");
					settings.AddParam("buyerAddress", "P:COMPANYADDRESS");
					settings.AddParam("buyerPhone", string.Empty);
					settings.AddParam("buyerFax", string.Empty);
					settings.AddParam("buyerContactPerson", "P:COMPANYNAME");

					string stampUrl = BXOptionManager.GetOptionString(BXCommerceSiteModuleConfiguration.Id, "StoreStampFileUrl", string.Empty);
					if (!string.IsNullOrEmpty(stampUrl))
						settings.AddParam("pathToStamp", stampUrl);
					billCfg.Save();
				}
				else if ((jur == null || !WizardContext.State.GetBool("EnableBillPayment", false)) && psCol.Count > 0)
					psCol[0].Active = false;

				//создание почтовых шаблонов
				BXMailerTemplateCollection mtCol = BXMailerTemplate.GetList(
					new BXFilter(
						new BXFilterItem(BXMailerTemplate.Fields.Name, BXSqlFilterOperators.Equal, "saleautoregistermessage")
						),
					null);

				BXMailerTemplate template = mtCol.Count > 0 ? mtCol[0] : new BXMailerTemplate("SaleAutoRegisterMessage");
				if (template.IsNew)
				{
					template.Active = true;
					template.EmailFrom = "#DEFAULT_EMAIL_FROM#";
					template.EmailTo = "#EmailTo#";
					template.Bcc = String.Empty;
					template.BodyType = BXMailerTemplateBodyType.Text;
					template.Subject = BXLoc.GetModuleMessage(BXLoc.CurrentLocale, BXSaleModuleConfiguration.Id, "EventTemplate.SaleAutoRegisterMessage.Subject");
					template.Message = BXLoc.GetModuleMessage(BXLoc.CurrentLocale, BXSaleModuleConfiguration.Id, "EventTemplate.SaleAutoRegisterMessage.Message"); ;
				}
				if(template.Sites.FindIndex(x => string.Equals(x.SiteId, siteId, StringComparison.OrdinalIgnoreCase)) < 0)
					template.Sites.Add(new BXMailerTemplate.BXSite(siteId));
				template.Save();

				//конечный статус заказа

				var statuses = BXSaleOrderStatus.GetList(new BXFilter(new BXFilterItem(BXSaleOrderStatus.Fields.XmlId, BXSqlFilterOperators.Equal, "DEFAULT_FINAL")), null);
				if (statuses.Count == 0)
				{
					var status = new BXSaleOrderStatus();
					var langs = BXLanguage.GetList(null, null);
					foreach ( var lang in langs )
					{
						var loc = new BXSaleOrderStatusLocalizationInfo(BXLoc.GetMessage(this, "FinalStatusName"), BXLoc.GetMessage(this, "FinalStatusDescription"));

						status.LocalizationInfo.Add(lang.Id, loc);
					}
					status.IsFinal = true;
					status.XmlId = "DEFAULT_FINAL";
					status.Save();
				}

				UI.SetProgressBarValue("Installer.ProgressBar", "Bitrix.CommerceSite", 2);
				return Result.Next();
			}
			catch (Exception exc)
			{
				return new BXWizardResultError(HttpUtility.HtmlEncode(exc.Message));
			}
		}
	}
}