﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using Bitrix.Currency;
using Bitrix.Sale;
using Bitrix.Services.Location;
using Bitrix.Sale.Components;

public partial class bitrix_components_bitrix_sale_order_form_templates_default_controls_delivery : BXControl, IPostBackEventHandler
{
	private bool isPrepared = false;
	public bool IsPrepared
	{
		get { return this.isPrepared; }
	}

	public void Prepare(SaleOrderFormDeliveryInfo info)
	{
		if (this.isPrepared)
			return;

		if (info == null)
			return;

		List<SaleOrderFormDeliveryItem> deliveryItemLst = new List<SaleOrderFormDeliveryItem>();
		BXSaleDeliveryContext context = new BXSaleDeliveryContext();
		context.Request.Source.CountryCode = BXCountry.GetByCode(BXSale.Current.CountryNumericCode).Iso3166AlphaCode;
		context.Request.Source.PostalCode = BXSale.Current.PostalCode;
		context.Request.Source.NativeName = BXSale.Current.LocationName;
		context.Request.Destination.CountryCode = BXCountry.GetByCode(info.DestinationCountryNumericCode).Iso3166AlphaCode;
		context.Request.Destination.PostalCode = info.DestinationPostalCode;
		context.Request.Destination.NativeName = info.DestinationLocationName;

		context.Request.CargoType = info.CargoType != BXSaleDeliveryCargoType.Unknown ? info.CargoType : BXSaleDeliveryCargoType.Commodity;
		context.Request.WeightInGrams = Convert.ToInt32(info.Weight);
		context.Request.Cost = info.Cost;

		SaleOrderFormDeliveryItem selectedItem = null;
		BXSaleDeliveryServiceInfo[] serviceAry = BXSaleDeliveryManager.GetDeliveryServiceInfos(context);
		foreach (BXSaleDeliveryServiceInfo service in serviceAry)
		{
			context.Response.ClearVariants();
			service.Process();

			if (context.Response.Error != BXSaleDeliveryResponseError.None)
			{
				if ((context.Response.Error & BXSaleDeliveryResponseError.NotEnabled) != 0)
					continue;

				SaleOrderFormDeliveryItem item = new SaleOrderFormDeliveryItem();
				item.Title = service.DisplayName;

				if ((context.Response.Error & BXSaleDeliveryResponseError.EmptySourcePostalCode) != 0)
					item.AddErrorMessage(GetMessageRaw("EmptySourcePostalCode"));
				if ((context.Response.Error & BXSaleDeliveryResponseError.EmptyDestinationPostalCode) != 0)
					item.AddErrorMessage(GetMessageRaw("EmptyDestinationPostalCode"));
				if ((context.Response.Error & BXSaleDeliveryResponseError.UnsupportedCargoType) != 0)
					item.AddErrorMessage(GetMessageRaw("UnsupportedCargoType"));
				if ((context.Response.Error & BXSaleDeliveryResponseError.InvalidSource) != 0)
					item.AddErrorMessage(GetMessageRaw("InvalidSource"));
				if ((context.Response.Error & BXSaleDeliveryResponseError.EmptySource) != 0)
					item.AddErrorMessage(GetMessageRaw("EmptySource"));
				if ((context.Response.Error & BXSaleDeliveryResponseError.SourceNotSupported) != 0)
					item.AddErrorMessage(GetMessageRaw("SourceNotSupported"));
				if ((context.Response.Error & BXSaleDeliveryResponseError.InvalidDestination) != 0)
					item.AddErrorMessage(GetMessageRaw("InvalidDestination"));
				if ((context.Response.Error & BXSaleDeliveryResponseError.EmptyDestination) != 0)
					item.AddErrorMessage(GetMessageRaw("EmptyDestination"));
				if ((context.Response.Error & BXSaleDeliveryResponseError.DestinationNotSupported) != 0)
					item.AddErrorMessage(GetMessageRaw("DestinationNotSupported"));
				if ((context.Response.Error & BXSaleDeliveryResponseError.NotAvailable) != 0)
					item.AddErrorMessage(GetMessageRaw("NotAvailable"));

				deliveryItemLst.Add(item);
				continue;
			}

			foreach (BXSaleDeliveryVariant variant in context.Response.Variants)
			{
				SaleOrderFormDeliveryItem item = new SaleOrderFormDeliveryItem();
				item.Id = variant.Id;
				item.Title = variant.Title;
				item.Description = variant.Description;
				item.Cost = variant.Cost;
				item.Currency = variant.Currency;
				deliveryItemLst.Add(item);

				if (selectedItem == null
					|| (string.Equals(item.Id, SelectedDeliveryVariant, StringComparison.InvariantCultureIgnoreCase) || (string.IsNullOrEmpty(SelectedDeliveryVariant) && selectedItem.Cost > item.Cost)))
					selectedItem = item;
			}
		}

		if (selectedItem != null)
		{
			selectedItem.IsSelected = true;
			info.DeliveryCost = selectedItem.Cost;
			RecommendedDeliveryVariant = selectedItem.Id;
		}
		else
		{
			info.DeliveryCost = 0m;
		}

		if (deliveryItemLst.Count > 0)
		{
			bool hasValidItems = false;
			foreach (SaleOrderFormDeliveryItem item in deliveryItemLst)
			{
				if (item.HasErrors)
					continue;
				hasValidItems = true;
				break;
			}

			if (!hasValidItems)
				this.error = SaleOrderFormDeliveryError.NoValidItems;
		}
		this.deliveryItems = deliveryItemLst.ToArray();
		this.isPrepared = true;
	}

	protected string GetClientId(string id)
	{
		return ClientID + ClientIDSeparator + id;
	}

	protected override void OnPreRender(EventArgs e)
	{
		base.OnPreRender(e);
		SelectedVariantField.Value = RecommendedDeliveryVariant;
	}

	private string recommendedDeliveryVariant = string.Empty;
	public string RecommendedDeliveryVariant
	{
		get { return this.recommendedDeliveryVariant; }
		private set { this.recommendedDeliveryVariant = value; }
	}

	public string SelectedDeliveryVariant
	{
		get { return SelectedVariantField.Value; }
	}

	private SaleOrderFormDeliveryItem[] deliveryItems = null;
	public SaleOrderFormDeliveryItem[] DeliveryItems
	{
		get { return this.deliveryItems ?? (this.deliveryItems = new SaleOrderFormDeliveryItem[0]); }
	}

	private SaleOrderFormDeliveryError error = SaleOrderFormDeliveryError.None;
	public SaleOrderFormDeliveryError DeliveryError
	{
		get { return this.error; }
	}

	#region IPostBackEventHandler Members
	public void RaisePostBackEvent(string eventArgument)
	{
		if (string.IsNullOrEmpty(eventArgument) || !eventArgument.StartsWith("DELIVERY$", StringComparison.Ordinal))
			return;

		SelectedVariantField.Value = eventArgument.Substring(9);
	}
	#endregion
}

public enum SaleOrderFormDeliveryError
{
	None = 0,
	NoValidItems = 1
}

public class SaleOrderFormDeliveryItem
{
	private string id = string.Empty;
	public string Id
	{
		get { return this.id; }
		set { this.id = value ?? string.Empty; }
	}

	private string title = string.Empty;
	public string Title
	{
		get { return this.title; }
		set { this.title = value ?? string.Empty; }
	}

	public string GetEncodedTitle()
	{
		return HttpUtility.HtmlEncode(Title);
	}

	private decimal cost = 0m;
	public decimal Cost
	{
		get { return this.cost; }
		set { this.cost = value; }
	}

	private BXCurrency currency = null;
	public BXCurrency Currency
	{
		get { return this.currency ?? (this.currency = BXCurrency.NationalCurrency); }
		set { this.currency = value; }
	}

	public string CostToHtml()
	{
		return HttpUtility.HtmlEncode(Currency.MoneyToString(Cost));
	}

	private string description = string.Empty;
	public string Description
	{
		get { return this.description; }
		set { this.description = value ?? string.Empty; }
	}

	public string GetEncodedDescription()
	{
		return HttpUtility.HtmlEncode(Description);
	}

	private bool isSelected = false;
	public bool IsSelected
	{
		get { return this.isSelected; }
		set { this.isSelected = value; }
	}

	private List<string> errorMessages = null;
	private List<string> InnerErrorMessages
	{
		get { return this.errorMessages ?? (this.errorMessages = new List<string>()); }
	}

	public bool HasErrors
	{
		get { return this.errorMessages != null && this.errorMessages.Count > 0; }
	}

	public string[] ErrorMessages
	{
		get { return this.errorMessages != null ? this.errorMessages.ToArray() : new string[0]; }
	}

	public void AddErrorMessage(string msg)
	{
		InnerErrorMessages.Add(msg);
	}
}
