﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using Bitrix.Currency;
using Bitrix.Sale;
using Bitrix.Services.Location;
using Bitrix.Sale.Components;

public partial class bitrix_components_bitrix_sale_order_form_templates_default_controls_payment : BXControl
{
    private bool isPrepared = false;
    public bool IsPrepared
    {
        get { return this.isPrepared; }
    }

    public void Prepare(SaleOrderFormPaymentInfo info)
    {
        if(this.isPrepared)
            return;

        if (info == null)
            return;

        List<SaleOrderFormPaymentItem> paymentItemLst = new List<SaleOrderFormPaymentItem>();
        BXSalePaymentContext context = new BXSalePaymentContext();
        context.Request.CustomerTypeId = info.CustomerTypeId;

        BXSalePaymentManager.PreparePaymentVariants(context);
        foreach(BXSalePaymentVariant v in context.Response.Variants)
        {
            SaleOrderFormPaymentItem item = new SaleOrderFormPaymentItem();
            item.Id = v.Id;
            item.Title = v.Title;
            item.Description = v.Description;
            paymentItemLst.Add(item);
        }

        bool hasValidItems = false;
        foreach(SaleOrderFormPaymentItem item in paymentItemLst)
        {
            if(item.HasErrors)
                continue;

            hasValidItems = true;
            break;
        }

        if(!hasValidItems)
            this.error = SaleOrderFormPaymentError.NoValidItems;
        else
            paymentItemLst[0].IsSelected = true;


        this.paymentItems = paymentItemLst.ToArray();
        this.isPrepared = true;
    }

    protected string GetClientId(string id)
    {
        return ClientID + ClientIDSeparator + id;
    }

    private SaleOrderFormPaymentItem[] paymentItems = null;
    public SaleOrderFormPaymentItem[] PaymentItems
    {
        get { return this.paymentItems ?? (this.paymentItems = new SaleOrderFormPaymentItem[0]); }
    }

    private SaleOrderFormPaymentError error = SaleOrderFormPaymentError.None;
    public SaleOrderFormPaymentError PaymentError 
    {
        get { return this.error; }
    }

    public int SelectedPaymentVariant
    {
        get
        {
            string s = Request.Form["salePaymentItem"];
            int i;
            return !string.IsNullOrEmpty(s) && int.TryParse(s, out i) ? i : 0; 
        }
    }

}

public enum SaleOrderFormPaymentError
{
    None = 0,
    NoValidItems = 1
}

public class SaleOrderFormPaymentItem
{
    private int id = 0;
    public int Id
    {
        get { return this.id; }
        set { this.id = value; }
    }

    private string title = string.Empty;
    public string Title
    {
        get { return this.title; }
        set { this.title = value ?? string.Empty; }
    }

    public string GetEncodedTitle()
    {
        return HttpUtility.HtmlEncode(Title);
    }

    private string description = string.Empty;
    public string Description
    {
        get { return this.description; }
        set { this.description = value ?? string.Empty; }
    }

    public string GetEncodedDescription()
    {
        return HttpUtility.HtmlEncode(Description);
    }

    private bool isSelected = false;
    public bool IsSelected
    {
        get { return this.isSelected; }
        set { this.isSelected = value; }
    }

    private List<string> errorMessages = null;
    private List<string> InnerErrorMessages
    {
        get { return this.errorMessages ?? (this.errorMessages = new List<string>()); }
    }

    public bool HasErrors
    {
        get { return this.errorMessages != null && this.errorMessages.Count > 0; }
    }

    public string[] ErrorMessages
    {
        get { return this.errorMessages != null ? this.errorMessages.ToArray() : new string[0]; }
    }

    public void AddErrorMessage(string msg)
    {
        InnerErrorMessages.Add(msg);
    }
}
