﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.Sale;
using Bitrix;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix.UI;
using Bitrix.Security;

public partial class bitrix_components_bitrix_sale_order_form_templates_default_controls_properties :
	BXControl, IPostBackEventHandler
{
	protected BXSite site;
	protected BXSaleCustomerTypeCollection customerTypes;
	protected BXSaleOrderFieldCollection fieldCollection;
	protected List<FieldInfo> fields;
	protected BXSaleCustomerType currentCustomerType;
	protected BXSaleUserProfile currentUserProfile;
	protected bool validated;
	protected bool showErrors;
	private BXSaleUserProfileCollection userProfiles;
	private bool initFromPost;
	private BXCustomPropertyCollection profileValues;


	public BXSaleOrderFieldCollection Fields
	{
		get
		{
			if (fieldCollection == null)
				BuildFields();
			return fieldCollection;
		}
	}
	public BXSaleCustomerType CustomerType
	{
		get
		{
			return currentCustomerType;
		}
	}
	protected BXSaleUserProfileCollection Profiles
	{
		get
		{
			if (userProfiles != null)
				return userProfiles;

			int userId = BXIdentity.Current.Id;
			userProfiles =
				userId > 0 && currentCustomerType != null
				? BXSaleUserProfile.GetList(
					new BXFilter(
						new BXFilterItem(BXSaleUserProfile.Fields.User.UserId, BXSqlFilterOperators.Equal, userId),
						new BXFilterItem(BXSaleUserProfile.Fields.CustomerType.Id, BXSqlFilterOperators.Equal, currentCustomerType.Id)
					),
					new BXOrderBy(
						new BXOrderByPair(BXSaleUserProfile.Fields.LastUpdatedUtc, BXOrderByDirection.Desc)
					)
				)
				: new BXSaleUserProfileCollection();

			return userProfiles;
		}
	}

	protected override void OnInit(EventArgs e)
	{
		base.OnInit(e);
		site = BXSite.Current;
		customerTypes = BXSaleCustomerType.GetList(
			new BXFilter(new BXFilterItem(BXSaleCustomerType.Fields.Site.ID, BXSqlFilterOperators.Equal, site.TextEncoder.Decode(site.Id))),
			new BXOrderBy(new BXOrderByPair(BXSaleCustomerType.Fields.Sort, BXOrderByDirection.Asc))
		);

		var val = Request.Form[UniqueID + "$ct"];
		int id;
		if (!string.IsNullOrEmpty(val) && int.TryParse(val, out id))
			currentCustomerType = customerTypes.Find(x => x.Id == id);
		if (currentCustomerType == null && customerTypes.Count > 0)
			currentCustomerType = customerTypes[0];


		val = Request.Form[UniqueID + "$profile"];
		if (!string.IsNullOrEmpty(val) && int.TryParse(val, out id))
			currentUserProfile = Profiles.Find(x => x.Id == id);
		else if (!IsPostBack && Profiles.Count > 0)
			currentUserProfile = Profiles[0];

		initFromPost = IsPostBack;
	}
	protected override void OnPreRender(EventArgs e)
	{
		base.OnPreRender(e);

		Validate();
		if (!initFromPost)
			SetProfileValues();
	}

	private void SetProfileValues()
	{
		if (profileValues != null)
			return;

		var userProfileValues =
			currentUserProfile != null
			? currentUserProfile.GetValues(fieldCollection)
			: null;

		profileValues = new BXCustomPropertyCollection();
		foreach (var f in fields)
			f.Editor.SaveDefault(profileValues);

		if (userProfileValues != null)
			foreach (var p in userProfileValues)
				profileValues[p.Key] = p.Value;

		foreach (var f in fields)
		{
			BXCustomProperty p;
			if (profileValues == null || !profileValues.TryGetValue(f.Field.Code, out p) || p == null)
				p = new BXCustomProperty(f.Field.Code, f.Field.Id, f.Field.CustomType.DbType, f.Field.Multiple, null);

			f.Editor.Load(p);
		}
	}

	protected void Validate()
	{
		if (validated)
			return;

		if (fields == null)
			BuildFields();

		if (initFromPost)
		{
			foreach (var f in fields)
			{
				f.Errors = new List<string>();
				f.Valid = f.Editor.DoValidate(f.FormFieldName, f.Errors);
			}
		}

		validated = true;
	}
	private void BuildFields()
	{
		fieldCollection = currentCustomerType != null ? BXSaleOrderField.GetCustomerFields(currentCustomerType.Id, null) : new BXSaleOrderFieldCollection();
		fields = fieldCollection.ConvertAll(x => new FieldInfo { Field = x });

		foreach (var f in fields)
		{
			f.Adapter = f.Field.GetCustomFieldAdapter();
			f.Editor = f.Field.CustomType.CreatePublicEditor();
			f.Editor.Init(f.Adapter);
			f.FormFieldName = UniqueID + "$p$" + f.Field.Code;
			f.ClientId = ClientID + "_p_" + f.Field.Code.ToLowerInvariant();
		}
	}

	public bool Validate(ICollection<string> errors)
	{
		showErrors = true;
		Validate();
		bool valid = true;
		foreach (var f in fields)
		{
			if (f.Valid)
				continue;

			valid = false;

			if (errors != null)
			{
				foreach (var e in f.Errors)
					errors.Add(e);
			}
		}
		return valid;
	}
	public bool Save(BXCustomPropertyCollection storage, ICollection<string> errors)
	{
		if (!initFromPost)
		{
			Validate();
			SetProfileValues();
			foreach(var p in profileValues)
				storage[p.Key] = p.Value;
		}
		else 
		{
			if (!Validate(errors))
				return false;
			foreach (var f in fields)
				f.Editor.DoSave(f.FormFieldName, storage, null);
		}

		return true;
	}
	public void SaveEntities(BXCustomPropertyCollection properties, int userId, BXSqlTransaction transaction)
	{
		if (currentUserProfile != null)
			return;

		var profile = new BXSaleUserProfile
		{
			CustomerTypeId = currentCustomerType.Id,
			UserId = userId,
			Name = ""
		};

		var f = fieldCollection.Find(x => x.UseAsProfileName);
		if (f != null)
		{
			BXCustomProperty p;
			if (properties.TryGetValue(f.Code, out p) && p != null && p.Value != null)
				profile.Name = p.Value.ToString();
		}

		profile.SetValues(fieldCollection, properties);

		profile.Save(transaction.Connection, transaction);
	}

	#region IPostBackEventHandler Members

	public void RaisePostBackEvent(string eventArgument)
	{
		switch (eventArgument)
		{
			case "change_user_profile":
				initFromPost = false;
				break;
			case "change_customer_type":
				initFromPost = false;
				if (currentUserProfile == null && Profiles.Count > 0)
					currentUserProfile = Profiles[0];
				break;
		}
	}

	#endregion

	protected class FieldInfo
	{
		public BXSaleOrderField Field;
		public BXCustomField Adapter;
		public BXCustomTypePublicEdit Editor;
		public string FormFieldName;
		public string ClientId;
		public List<string> Errors;
		public bool Valid = true;
	}
}
