﻿using Bitrix.UI;
using System;
using Bitrix.Sale.Components;
using System.Collections.Generic;
using Bitrix.DataTypes;
using Bitrix.Services;
using Bitrix.Security;
using Bitrix;

public partial class bitrix_components_bitrix_sale_order_form_templates__default_template : Bitrix.Sale.Components.SaleOrderFormTemplate
{
	bool propertyErrorsShown;

	protected override void OnInit(EventArgs e)
	{
		base.OnInit(e);
		Page.LoadComplete += new EventHandler(Page_LoadComplete);
	}

	protected override void OnLoad(EventArgs e)
	{
		base.OnLoad(e);

		if (Component.Error != SaleOrderFormComponent.SaleOrderFormComponentError.None)
		{
			errorMessage.AddErrorMessage(Component.GetErrorMessage());
		}
	}

	private void PrepareDelivery()
	{
		if (Delivery.IsPrepared)
			return;

		var errors = new List<string>();
		var properties = new BXCustomPropertyCollection();
		if (!Properties.Save(properties, errors))
		{
			if (!propertyErrorsShown)
				foreach (var err in errors)
					errorMessage.AddErrorMessage(err);
			propertyErrorsShown = true;
			return;
		}

		var postalCodeField = Properties.Fields.Find(x => x.UseAsPostalCode);
		var postalCodeProp = postalCodeField != null ? properties[postalCodeField.Code] : null;
		var postalCode = postalCodeProp != null ? postalCodeProp.ToString() : string.Empty;
		DeliveryInfo.DestinationCountryNumericCode = 643;
		DeliveryInfo.DestinationPostalCode = postalCode;
		var locationField = Properties.Fields.Find(x => x.UseAsLocation);
		var locationProp = locationField != null ? properties[locationField.Code] : null;
		DeliveryInfo.DestinationLocationName = locationProp != null ? locationProp.ToString() : string.Empty;

		DeliveryInfo.Weight = Component.GetTotalWeight();
		DeliveryInfo.Cost = Component.GetTotalCost();
		DeliveryInfo.CargoType = Component.CargoType;
		Delivery.Prepare(DeliveryInfo);
	}

	private void PreparePayment()
	{
		if (Payment.IsPrepared)
			return;

		var errors = new List<string>();
		var properties = new BXCustomPropertyCollection();
		if (!Properties.Save(properties, errors))
		{
			if (!propertyErrorsShown)
				foreach (var err in errors)
					errorMessage.AddErrorMessage(err);
			propertyErrorsShown = true;
			return;
		}
		if (Properties.CustomerType == null)
		{
			errorMessage.AddErrorMessage(GetMessageRaw("Error.UnableToSetCustomerType"));
			return;
		}
		PaymentInfo.CustomerTypeId = Properties.CustomerType.Id;
		Payment.Prepare(PaymentInfo);
	}

	void Page_LoadComplete(object sender, EventArgs e)
	{
		PrepareDelivery();
		PreparePayment();
	}

	private SaleOrderFormDeliveryInfo deliveryInfo = new SaleOrderFormDeliveryInfo();
	public SaleOrderFormDeliveryInfo DeliveryInfo
	{
		get { return this.deliveryInfo ?? (this.deliveryInfo = new SaleOrderFormDeliveryInfo()); }
	}

	private SaleOrderFormPaymentInfo paymentInfo = new SaleOrderFormPaymentInfo();
	public SaleOrderFormPaymentInfo PaymentInfo
	{
		get { return this.paymentInfo ?? (this.paymentInfo = new SaleOrderFormPaymentInfo()); }
	}

	protected bool orderIsCreated;

	protected void FormOrderClick(object sender, EventArgs e)
	{
		PrepareDelivery();
		PreparePayment();

		if (Delivery.DeliveryError != SaleOrderFormDeliveryError.None)
		{
			if (Delivery.DeliveryError == SaleOrderFormDeliveryError.NoValidItems)
				errorMessage.AddErrorMessage(GetMessageRaw("Error.UnableToCalculateDelivery"));
			else
				errorMessage.AddErrorMessage(GetMessageRaw("Error.GeneralCalculationError"));

			return;
		}

		//if (string.IsNullOrEmpty(Delivery.SelectedDeliveryVariant))
		//{
		//    errorMessage.AddErrorMessage("Пожалуйста, выберите вариант доставки");
		//    return;
		//}

		var propertyErrors = new List<string>();
		var properties = new BXCustomPropertyCollection();

		var success = Properties.Save(properties, propertyErrors);
		if (!success)
		{
			if (!propertyErrorsShown)
				foreach (var err in propertyErrors)
					errorMessage.AddErrorMessage(err);
			propertyErrorsShown = true;
			return;
		}

		var errors = new List<string>();
		var info = new SaleOrderFormOrderInfo
		{
			CustomerTypeId = Properties.CustomerType.Id,
			Fields = Properties.Fields,
			Values = properties,
			DeliveryCost = DeliveryInfo.DeliveryCost,
			DeliveryVariant = Delivery.SelectedDeliveryVariant,
			PaymentSystemId = Payment.SelectedPaymentVariant,
			CustomerComments = CustomerComments.Text
		};
		info.Save += (context) => { Properties.SaveEntities(properties, context.UserId, context.Transaction); };
		success = orderIsCreated = Component.MakeOrder(info, errors);

		if (!success)
		{
			foreach (var err in errors)
				errorMessage.AddErrorMessage(err);
		}
	}
}
