﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.UI;
using Bitrix.Services.Text;
using Bitrix;
using Bitrix.DataLayer;
using Bitrix.Modules;
using System.Collections.Generic;
using Bitrix.Security;
using Bitrix.Configuration;
using Bitrix.Blog;

public enum BlogEditorMode
{
    Creation = 1,
    Modification = 2
}

public enum BlogEditorError
{
    None                = 0,
    BlogIsNotFound      = -1,
    BlogOwnerIsNotFound = -2,
    BlogCreation        = -3,
    BlogModification    = -4,
    BlogDeleting        = -5
}

public partial class bitrix_admin_BlogEdit : BXAdminPage
{
	private int? _blogId = null;
	protected int BlogId
	{
        get 
        {
            if (_blogId.HasValue)
                return _blogId.Value;

            string blogIdStr = Request.QueryString["id"];
            if (string.IsNullOrEmpty(blogIdStr))
                 _blogId = 0;
            else
            {
                try
                {
                    _blogId = Convert.ToInt32(Request.QueryString["id"]);
                }
                catch (Exception /*exc*/)
                {
                    _blogId = 0;
                }
            }
            return _blogId.Value; 
        }
	}

    private BlogEditorMode _editorMode = BlogEditorMode.Creation;
    protected BlogEditorMode EditorMode
    {
        get { return _editorMode; }
    }

    private BXBlog _blog;
    protected BXBlog Blog
    {
        get { return _blog; }
    }

    private BXBlogSyndication _syndication = null;
    protected BXBlogSyndication Syndication
    {
        get
        {
            if (_syndication != null)
                return _syndication;

            if ((_syndication = BXBlogSyndication.Find(BlogId, null)) == null)
                _syndication = new BXBlogSyndication(BlogId);

            return _syndication;
        }
    }

    private string _errorMessage = string.Empty;
    protected string ErrorMessage
    {
        get { return _errorMessage; }
    }

    private BlogEditorError _editorError = BlogEditorError.None;
    protected BlogEditorError EditorError
    {
        get { return _editorError; }
    }

	protected override string BackUrl
	{
		get { return !string.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "BlogList.aspx"; }
	}

	protected void Page_Init(object sender, EventArgs e)
	{
		if (!BXPrincipal.Current.IsCanOperate(BXBlog.Operations.AdminManagement))
			BXAuthentication.AuthenticationRequired();

		BXBlogCategoryCollection categoryList = BXBlogCategory.GetList(
			null,
			null,
			new BXSelect(
				BXSelectFieldPreparationMode.Normal,
				BXBlogCategory.Fields.Id,
				BXBlogCategory.Fields.Name
				),
			null,
			BXTextEncoder.EmptyTextEncoder
			);

		ListItemCollection categoryItems = BlogCategories.Items;
		categoryItems.Clear();

		foreach (BXBlogCategory category in categoryList)
			categoryItems.Add(new ListItem(category.Name, category.Id.ToString()));

		#region OLD
		/*
        BXUserCollection users = Bitrix.Security.BXUser.GetList(
            null,
            new BXOrderBy(
                new BXOrderByPair(Bitrix.Security.BXUser.Fields.UserName, BXOrderByDirection.Asc)
                ),
            new BXSelect(
                Bitrix.Security.BXUser.Fields.UserId,
                Bitrix.Security.BXUser.Fields.UserName,
                Bitrix.Security.BXUser.Fields.FirstName,
                Bitrix.Security.BXUser.Fields.LastName,
                Bitrix.Security.BXUser.Fields.ProviderName
                ),
                null
            );

        ListItemCollection userItems = BlogOwner.Items;
        foreach (Bitrix.Security.BXUser user in users)
        {
            userItems.Add(new ListItem(string.Format("[{0}] {1} {2}", user.UserName, user.FirstName, user.LastName), user.UserId.ToString()));
        }
        */
		#endregion

		#region DEFERRED
		/*
        BlogPostVisibilityMode.Items.Add(new ListItem("(не выбрано)", "_None"));
        BlogPostVisibilityMode.Items.Add(new ListItem(BXBlogEnumHelper.GetPostVisibilityModeName(BXBlogPostVisibilityMode.Everyone), BXBlogPostVisibilityMode.Everyone.ToString("G")));
        BlogPostVisibilityMode.Items.Add(new ListItem(BXBlogEnumHelper.GetPostVisibilityModeName(BXBlogPostVisibilityMode.Registered), BXBlogPostVisibilityMode.Registered.ToString("G")));
        //BlogPostVisibilityMode.Items.Add(new ListItem(BXBlogEnumHelper.GetPostVisibilityModeName(BXBlogPostVisibilityMode.Friends), BXBlogPostVisibilityMode.Friends.ToString("G")));
        BlogPostVisibilityMode.Items.Add(new ListItem(BXBlogEnumHelper.GetPostVisibilityModeName(BXBlogPostVisibilityMode.Author), BXBlogPostVisibilityMode.Author.ToString("G")));

        BlogAddCommentPermission.Items.Add(new ListItem("(не выбрано)", "_None"));
        BlogAddCommentPermission.Items.Add(new ListItem(BXBlogEnumHelper.GetAddCommentPermissionName(BXBlogAddCommentPermission.Everyone), BXBlogAddCommentPermission.Everyone.ToString("G")));
        BlogAddCommentPermission.Items.Add(new ListItem(BXBlogEnumHelper.GetAddCommentPermissionName(BXBlogAddCommentPermission.Registered), BXBlogAddCommentPermission.Registered.ToString("G")));
        //BlogAddCommentPermission.Items.Add(new ListItem(BXBlogEnumHelper.GetAddCommentPermissionName(BXBlogAddCommentPermission.Friends), BXBlogAddCommentPermission.Friends.ToString("G")));
        BlogAddCommentPermission.Items.Add(new ListItem(BXBlogEnumHelper.GetAddCommentPermissionName(BXBlogAddCommentPermission.NoOne), BXBlogAddCommentPermission.NoOne.ToString("G")));

        BlogCommentApproval.Items.Add(new ListItem("(не выбрано)", "_None"));
        BlogCommentApproval.Items.Add(new ListItem(BXBlogEnumHelper.GetCommentApprovalName(BXBlogCommentApproval.None), BXBlogCommentApproval.None.ToString("G")));
        BlogCommentApproval.Items.Add(new ListItem(BXBlogEnumHelper.GetCommentApprovalName(BXBlogCommentApproval.Anonymous), BXBlogCommentApproval.Anonymous.ToString("G")));
        BlogCommentApproval.Items.Add(new ListItem(BXBlogEnumHelper.GetCommentApprovalName(BXBlogCommentApproval.NotFriends), BXBlogCommentApproval.NotFriends.ToString("G")));
        BlogCommentApproval.Items.Add(new ListItem(BXBlogEnumHelper.GetCommentApprovalName(BXBlogCommentApproval.Everything), BXBlogCommentApproval.Everything.ToString("G")));
        */
		#endregion

		TryLoadBlog();

		CustomFieldList1.EntityId = BXBlog.GetCustomFieldsKey();

        #region OLD
        //if (_blogId != null && _blogId != 0)
        //	CustomFieldList1.Load(BXCustomEntityManager.GetProperties(BXBlog.GetCustomFieldsKey(), _blogId.Value));
        #endregion
        if(_blog != null)
            CustomFieldList1.Load(_blog.CustomValues);

    }

	protected void Page_Load(object sender, EventArgs e)
	{
        SyndicationRssFeedUrlRequired.Enabled = SyndicationRssFeedUrlRegex.Enabled = EnableSyndication.Checked;
        MasterTitle = Page.Title = EditorMode == BlogEditorMode.Modification ? string.Format(GetMessage("PageTitle.EditBlog"), Blog != null ? HttpUtility.HtmlEncode(Blog.Name) : "?") : GetMessage("PageTitle.CreateBlog");
	}

	private void TryLoadBlog()
	{
        int id = BlogId;
        if (id <= 0)
        {
            _editorMode = BlogEditorMode.Creation;
            _blog = new BXBlog();
        }
        else
        {
            _editorMode = BlogEditorMode.Modification;
            #region OLD
            //if ((_blog = BXBlog.GetById(id, BXTextEncoder.EmptyTextEncoder)) == null)
            //{
            //    _errorMessage = string.Format(GetMessageRaw("Error.UnableToFindBlog"), id);
            //    _editorError = BlogEditorError.BlogIsNotFound;
            //    return;
            //}
            #endregion
            BXBlogCollection blogCol = BXBlog.GetList(
                new BXFilter(new BXFilterItem(BXBlog.Fields.Id, BXSqlFilterOperators.Equal, id)),
                null,
                new BXSelectAdd(BXBlog.Fields.CustomFields.DefaultFields),
                null);

            if ((_blog = blogCol.Count > 0 ? blogCol[0] : null) == null)
            {
                _errorMessage = string.Format(GetMessageRaw("Error.UnableToFindBlog"), id);
                _editorError = BlogEditorError.BlogIsNotFound;
                return;
            }
        }

		if (_editorMode == BlogEditorMode.Modification)
        {
            BlogActive.Checked = _blog.Active;
            BlogName.Text = _blog.Name;
            BlogSlug.Text = _blog.Slug;
            BlogDescription.Text = _blog.Description;
            BlogSort.Text = _blog.Sort.ToString();
            BlogNotifyOfComments.Checked = _blog.NotifyOfComments;
            BlogXmlId.Text = _blog.XmlId;
			BlogIndexContent.SelectedValue = _blog.IndexContent.ToString();
            ListItemCollection categoryItems = BlogCategories.Items;
            int[] categoryIds = _blog.GetCategoryIds();
            foreach (int categoryId in categoryIds)
            {
                ListItem li = categoryItems.FindByValue(categoryId.ToString());
                if (li != null)
                    li.Selected = true;
            }

            #region OLD
            /*
            if (!_blog.IsNew)
            {
                ListItemCollection userItems = BlogOwner.Items;
                ListItem userItem = userItems.FindByValue(_blog.OwnerId.ToString());
                if (userItem != null)
                    userItem.Selected = true;
            }
            */
            #endregion

            if (!_blog.IsNew)
            {
                BXBlogUser owner = _blog.Owner;
                FindUserAutocomplete.HiddenValue = _blog.OwnerId.ToString();
                string firstName = (String.IsNullOrEmpty(owner.User.FirstName)) ? String.Empty : owner.User.FirstName.Trim();
                string lastName = (String.IsNullOrEmpty(owner.User.LastName)) ? String.Empty :
                                ((firstName == String.Empty) ? owner.User.LastName.Trim()+ " " : " " + owner.User.LastName.Trim() + " ");
                FindUserAutocomplete.TextBoxValue = firstName + lastName + "(" + owner.GetUserName() + ")";
            }

            EnableSyndication.Checked = Syndication.Enabled;
            SyndicationRssFeedUrl.Text = Syndication.FeedUrl;
            SyndicationUpdateable.Checked = Syndication.Updateable;
            SyndicationRedirectToComments.Checked = Syndication.RedirectToComments;

            SyndicationRssFeedUrlRequired.Enabled = Syndication.Enabled;
            SyndicationRssFeedUrlRegex.Enabled = Syndication.Enabled;

            #region DEFERRED
            /*
            ListItem postVisibilityModeItem = BlogPostVisibilityMode.Items.FindByValue(_blog.PostVisibilityMode.ToString("G"));
            if (postVisibilityModeItem != null)
                postVisibilityModeItem.Selected = true;

            ListItem addCommentPermissionItem = BlogAddCommentPermission.Items.FindByValue(_blog.AddCommentPermission.ToString("G"));
            if (addCommentPermissionItem != null)
                addCommentPermissionItem.Selected = true;

            ListItem commentApprovalItem = BlogCommentApproval.Items.FindByValue(_blog.CommentApproval.ToString("G"));
            if (commentApprovalItem != null)
                commentApprovalItem.Selected = true;
            */
            #endregion
        }
        else
        {
			BlogActive.Checked = true;
			BlogSort.Text = "10";
			BlogNotifyOfComments.Checked = true;
			BlogIndexContent.SelectedValue = BXBlogIndexContentMode.All.ToString();
        }
	}

    private void TrySaveBlog()
    {
        if (_blog == null)
            return;
        try
        {
            #region OLD
            //string userName = BlogOwner.Text;
            //if (!string.IsNullOrEmpty(userName))
            //{
            //    BXUserCollection users = Bitrix.Security.BXUser.GetList(
            //        new BXFilter(new BXFilterItem(Bitrix.Security.BXUser.Fields.UserName, BXSqlFilterOperators.Equal, userName)),
            //        null,
            //        null,
            //        null
            //        );
            //    if (users.Count > 0)
            //        _blog.OwnerId = users[0].UserId;
            //    else
            //    {
            //        _errorMessage = string.Format(GetMessageRaw("Error.UnableToFindUser"), userName);
            //        _editorError = BlogEditorError.BlogOwnerIsNotFound;
            //    }
            //}
            //else
            //    _blog.OwnerId = 0;
            #endregion
            int userId;
            if (!int.TryParse(FindUserAutocomplete.HiddenValue, out userId))
            {
                Bitrix.Security.BXUser owner=null;
                if (!Bitrix.Services.Text.BXStringUtility.IsNullOrTrimEmpty(FindUserAutocomplete.TextBoxValue))
                {
                    BXUserCollection owners = 
                        Bitrix.Security.BXUser.GetList(new BXFilter(
                                                            new BXFilterItem(
                                                                    Bitrix.Security.BXUser.Fields.UserName,
                                                                    BXSqlFilterOperators.Equal,
                                                                    FindUserAutocomplete.TextBoxValue
                                                                    )), 
                                                       null);
                    if (owners.Count > 0) owner = owners[0];
                }

                _blog.OwnerId = owner==null ? 0 : owner.UserId;
            }
            else
            {
                BXUser user = Bitrix.Security.BXUser.GetById(userId);
                if (user != null)
                    _blog.OwnerId = userId;
                else
                {
                    _errorMessage = string.Format(GetMessageRaw("Error.UnableToFindUser"), userId);
                    _editorError = BlogEditorError.BlogOwnerIsNotFound;
                }
            }

			_blog.Active = BlogActive.Checked;
			_blog.Name = BlogName.Text;
			_blog.Slug = BlogSlug.Text;
			_blog.Description = BlogDescription.Text;
			if (string.IsNullOrEmpty(BlogSort.Text))
				_blog.Sort = 0;
			else
			{
				try
				{
					_blog.Sort = Convert.ToInt32(BlogSort.Text);
				}
				catch
                {
                }
			}

			_blog.NotifyOfComments = BlogNotifyOfComments.Checked;
			_blog.XmlId = BlogXmlId.Text;

			List<int> categoryIdList = null;
			ListItemCollection categoryItems = BlogCategories.Items;
			foreach (ListItem categoryItem in categoryItems)
			{
				if (!categoryItem.Selected)
					continue;
				try
				{
                    (categoryIdList ?? (categoryIdList = new List<int>())).Add(Convert.ToInt32(categoryItem.Value));
				}
				catch
				{
				}
			}
            _blog.SetCategoryIds(categoryIdList != null ? categoryIdList.ToArray() : new int[0]);
			#region DEFERRED
			/*
            string postVisibilityModeStr = BlogPostVisibilityMode.SelectedValue;
            if (string.IsNullOrEmpty(postVisibilityModeStr) || string.Equals(postVisibilityModeStr, "_None", StringComparison.InvariantCultureIgnoreCase))
                _blog.PostVisibilityMode = BXBlogPostVisibilityMode.Everyone;
            else
            {
                try
                {
                    _blog.PostVisibilityMode = (BXBlogPostVisibilityMode)Enum.Parse(typeof(BXBlogPostVisibilityMode), postVisibilityModeStr);
                }
                catch (Exception exc)
                {
                    _blog.PostVisibilityMode = BXBlogPostVisibilityMode.Everyone;
                }
            }

            string addCommentPermissionStr = BlogAddCommentPermission.SelectedValue;
            if (string.IsNullOrEmpty(addCommentPermissionStr) || string.Equals(addCommentPermissionStr, "_None", StringComparison.InvariantCultureIgnoreCase))
                _blog.AddCommentPermission = BXBlogAddCommentPermission.Everyone;
            else
            {
                try
                {
                    _blog.AddCommentPermission = (BXBlogAddCommentPermission)Enum.Parse(typeof(BXBlogAddCommentPermission), addCommentPermissionStr);
                }
                catch (Exception exc)
                {
                    _blog.AddCommentPermission = BXBlogAddCommentPermission.Everyone;
                }
            }

            string commentApprovalStr = BlogCommentApproval.SelectedValue;
            if (string.IsNullOrEmpty(commentApprovalStr) || string.Equals(commentApprovalStr, "_None", StringComparison.InvariantCultureIgnoreCase))
                _blog.CommentApproval = BXBlogCommentApproval.None;
            else
            {
                try
                {
                    _blog.CommentApproval = (BXBlogCommentApproval)Enum.Parse(typeof(BXBlogCommentApproval), commentApprovalStr);
                }
                catch (Exception exc)
                {
                    _blog.CommentApproval = BXBlogCommentApproval.None;
                }
            }
            */
			#endregion

			if (BXModuleManager.IsModuleInstalled("Search"))
			{
				_blog.IndexContent = BXBlogIndexContentMode.All;
				try
				{
					if (Enum.IsDefined(typeof(BXBlogIndexContentMode), BlogIndexContent.SelectedValue))
						_blog.IndexContent = (BXBlogIndexContentMode)Enum.Parse(typeof(BXBlogIndexContentMode), BlogIndexContent.SelectedValue);
				}
				catch
				{
				}
			}
			if (CustomFieldList1.HasItems)
				_blog.CustomValues.Override(CustomFieldList1.Save());
            _blog.Save();
			_blogId = _blog.Id;

            #region OLD
            //if (CustomFieldList1.HasItems)
            //	BXCustomEntityManager.SaveEntity(BXBlog.GetCustomFieldsKey(), _blogId.Value, CustomFieldList1.Save());
            #endregion
            /*
             * Если синдикация уже есть или пользователь инициировал её создание в этом запросе
             */
            if (!Syndication.IsNew || EnableSyndication.Checked)
            {
                Syndication.Enabled = EnableSyndication.Checked;
                Syndication.FeedUrl = SyndicationRssFeedUrl.Text.TrimStart();
                Syndication.Updateable = SyndicationUpdateable.Checked;
                Syndication.RedirectToComments = SyndicationRedirectToComments.Checked;
                Syndication.Save();
            }

        }
        catch (Exception exc)
        {
            _errorMessage = exc.Message;
            _editorError = EditorMode == BlogEditorMode.Creation ? BlogEditorError.BlogCreation : BlogEditorError.BlogModification; 
        }
    }

    protected override void OnPreRender(EventArgs e)
    {
        if (EditorError != BlogEditorError.None)
        {
            errorMessage.AddErrorMessage(ErrorMessage);
            if (EditorMode == BlogEditorMode.Modification && EditorError == BlogEditorError.BlogIsNotFound)
                TabControl.Visible = false;
        }

        if (EditorMode != BlogEditorMode.Modification)
        {
            AddButton.Visible = false;
            DeleteButton.Visible = false;
        }

        FindUserAutocomplete.AttachScript();
        #region OLD
        /*
        BXBlogUserCollection users = BXBlogUser.GetList(
            null,
            new BXOrderBy(
                new BXOrderByPair(BXBlogUser.Fields.User.UserName, BXOrderByDirection.Asc)
                ),
            new BXSelect(
                BXBlogUser.Fields.User.UserId,
                BXBlogUser.Fields.User.UserName,
                BXBlogUser.Fields.User.FirstName,
                BXBlogUser.Fields.User.LastName,
                BXBlogUser.Fields.User.ProviderName
                ),
                null
            );


        ListItemCollection userItems = BlogOwner.Items;
        foreach (BXBlogUser user in users)
        {
            Bitrix.Security.BXUser sysUser = user.User;
            if (sysUser == null)
                continue;
            userItems.Add(new ListItem(string.Format("[{0}] {1} {2}", sysUser.UserName, sysUser.FirstName, sysUser.LastName), sysUser.UserId.ToString()));
        }
        */
        #endregion
        base.OnPreRender(e);
    }


	protected void OnBlogEdit(object sender, BXTabControlCommandEventArgs e)
	{
		switch (e.CommandName)
		{
            case "save":
                {
                    if (IsValid && EditorError == BlogEditorError.None)
                    {
                        TrySaveBlog();
                        if (EditorError == BlogEditorError.None)
                            GoBack(); 
                    }
                }
                break;
            case "apply":
                {
                    if (IsValid && EditorError == BlogEditorError.None)
                    {
                        TrySaveBlog();
                        if (EditorError == BlogEditorError.None)
                            Response.Redirect(string.Format("BlogEdit.aspx?id={0}&tabindex={1}", BlogId.ToString(), TabControl.SelectedIndex));
                    }
                }
                break;
			case "cancel":
				GoBack();
				break;
		}
	}

	protected void OnToolBarButtonClick(object sender, CommandEventArgs e)
	{
		if (e.CommandName == "delete")
		{
			try
			{
                BXBlog blog = Blog;
                if (blog != null)
                    blog.Delete();

				GoBack();
			}
			catch (Exception ex)
			{
                _errorMessage = ex.Message;
                _editorError = BlogEditorError.BlogDeleting;
			}
		}
	}
}
