﻿<%@ Reference VirtualPath="~/bitrix/components/bitrix/blog.edit/component.ascx" %>
<%@ Control Language="C#" Inherits="Bitrix.Blog.Components.BlogEditTemplate" EnableViewState="false" %>
<%@ Import Namespace="Bitrix.Blog" %>
<%@ Import Namespace="Bitrix.Blog.Components" %>
<%@ Import Namespace="Bitrix.DataLayer" %>
<%@ Import Namespace="Bitrix.Services.Text" %>
<%@ Import Namespace="System.Collections.Generic" %>
<%@ Import Namespace="Bitrix.Services" %>
<%@ Import Namespace="Bitrix.DataTypes" %>

<script runat="server">
	protected override void LoadData(BlogEditComponent.Data data)
	{
		BlogName.Text = data.BlogName;
		BlogSlug.Text = data.BlogSlug;
		BlogDescription.Text = data.BlogDescription;
		NotifyComments.Checked = data.NotifyComments;
		DisableBlog.Checked = !data.BlogActive;

        if (Component.AllowToAjustBlogGroups && data.Categories != null && data.Categories.Length > 0)
        {
            foreach (ListItem listItem in BlogCategories.Items)
            {
                bool selected = Array.Exists<int>(data.Categories, delegate(int id) { return String.Equals(id.ToString(), listItem.Value); });
                listItem.Selected = selected;
                if (selected && BlogCategories.SelectionMode == ListSelectionMode.Single)
                    break;
            }
        }

        if (Component.CanSyndicateContent)
        {
            EnableSyndication.Checked = data.EnableSyndication;
            SyndicationRssFeedUrl.Text = data.SyndicationFeedUrl;
            SyndicationUpdateable.Checked = data.SyndicationUpdateableContent;
            SyndicationRedirectToComments.Checked = data.SyndicationRedirectToComments;
            if (data.EnableSyndication)
            {
                SyndicationRssFeedUrlRequired.Enabled = true;
                SyndicationRssFeedUrlRegex.Enabled = true;
            }
        }        
	}
    
	protected override void SaveData(BlogEditComponent.Data data)
	{
		data.BlogName = BlogName.Text;
		data.BlogDescription = BlogDescription.Text;
		
		if (Component.ComponentMode == BlogEditComponent.Mode.Add)
			data.BlogSlug = BlogSlug.Text;

		if (Component.ComponentMode == BlogEditComponent.Mode.Edit && Component.CanModerate && !Component.IsOwner)
			data.BlogActive = !DisableBlog.Checked;

        if (!Component.AllowToAjustBlogGroups)
        {
            if (Component.ComponentMode == BlogEditComponent.Mode.Add)
                data.Categories = Component.ObligatoryCategoryIds;
        }
        else
        {
            List<int> selectedCategories = new List<int>();
            foreach (ListItem category in BlogCategories.Items)
            {
                int categoryId;
                if (!(category.Selected && int.TryParse(category.Value, out categoryId)))
                    continue;
                selectedCategories.Add(categoryId);
            }
            data.Categories = selectedCategories.ToArray();
        }
		data.NotifyComments = NotifyComments.Checked;

        if (Component.CanSyndicateContent)
        {
            data.EnableSyndication = EnableSyndication.Checked;
            data.SyndicationFeedUrl = SyndicationRssFeedUrl.Text.TrimStart();
            data.SyndicationUpdateableContent = SyndicationUpdateable.Checked;
            data.SyndicationRedirectToComments = SyndicationRedirectToComments.Checked;
        }        
	}  
    
	protected override void OnInit(EventArgs e)
	{
        if (!Component.AllowToAjustBlogGroups)
            BlogCategories.Visible = false;
        else
        {
            if (Component.MaxBlogGroups == 1)
                BlogCategories.Items.Add(new ListItem(GetMessageRaw("Option.SelectCategory"), String.Empty));
            else
            {
                BlogCategories.SelectionMode = ListSelectionMode.Multiple;
                BlogCategories.Rows = Math.Max(2, Component.SiteCategories.Count > 10 || Component.SiteCategories.Count == 0 ? 10 : Component.SiteCategories.Count);
            }

            int[] availableCategoryIds = Component.AvailableCategoryIds;
			BXFilter filter = new BXFilter(new BXFilterItem(BXBlogCategory.Fields.Sites.SiteId, BXSqlFilterOperators.Equal, Component.DesignerSite));
			if (availableCategoryIds.Length > 0)
				filter.Add(new BXFilterItem(BXBlogCategory.Fields.Id, BXSqlFilterOperators.In, availableCategoryIds));
            BXBlogCategoryCollection availableCategories = BXBlogCategory.GetList(
                filter,
                new BXOrderBy(new BXOrderByPair(BXBlogCategory.Fields.Id, BXOrderByDirection.Asc)),
                new BXSelect(BXSelectFieldPreparationMode.Normal, BXBlogCategory.Fields.Id, BXBlogCategory.Fields.Name),
                null,
                BXTextEncoder.EmptyTextEncoder
			);
            foreach (BXBlogCategory category in availableCategories)
                BlogCategories.Items.Add(new ListItem(category.Name, category.Id.ToString()));
        }     
		//foreach (BXBlogCategory category in Component.SiteCategories)
		//	BlogCategories.Items.Add(new ListItem(category.Name, category.Id.ToString
		base.OnInit(e); //important last call
	}
	
	protected override void OnLoad(EventArgs e)
	{
		base.OnLoad(e);

		if (Component.FatalError == BlogEditComponent.ErrorCode.UnauthorizedNotLogIn)
		{
			Bitrix.Security.BXAuthentication.AuthenticationRequired();
			return;
		}
		else if (Component.FatalError != BlogEditComponent.ErrorCode.None)
		{
			BlogSlugEmptyValidator.Enabled = false;
			BlogSlugRegexValidator.Enabled = false;
			BlogSlugLengthValidator.Enabled = false;
			BlogNameValidator.Enabled = false;
			BlogCategoriesValidator.Enabled = false;
            BlogDescriptionValidator.Enabled = false;
            SyndicationRssFeedUrlRequired.Enabled = false;
            SyndicationRssFeedUrlRegex.Enabled = false;
			Errors.Enabled = false;
			return;
		} 
		
		string validationGroup = ClientID;
		Errors.ValidationGroup = validationGroup;
		BlogNameValidator.ValidationGroup = validationGroup;
        BlogDescriptionValidator.ValidationGroup = validationGroup;
        SyndicationRssFeedUrlRequired.ValidationGroup = validationGroup;
        SyndicationRssFeedUrlRegex.ValidationGroup = validationGroup;
        
		if (Component.ComponentMode == BlogEditComponent.Mode.Add)
		{
			BlogSlugEmptyValidator.Enabled = true;
			BlogSlugRegexValidator.Enabled = true;
			BlogSlugEmptyValidator.ValidationGroup = validationGroup;
			BlogSlugRegexValidator.ValidationGroup = validationGroup;
			BlogSlugRegexValidator.ValidationExpression = BXBlog.SlugRegex.ToString();    
			if (Component.MinSlugLength <= Component.MaxSlugLength)
			{
				BlogSlugLengthValidator.ValidationGroup = validationGroup;
				BlogSlugLengthValidator.Enabled = true;
				BlogSlugLengthValidator.ValidationExpression = String.Concat("^.{", Component.MinSlugLength, ",", Component.MaxSlugLength, "}$");
				BlogSlugLengthValidator.ErrorMessage = String.Format(GetMessage("Error.SlugLength"), Component.MinSlugLength, Component.MaxSlugLength);
				BlogSlug.MaxLength = Component.MaxSlugLength;
			}
		}

        if (Component.AllowToAjustBlogGroups)
        {
            BlogCategoriesValidator.Enabled = true;
            BlogCategoriesValidator.ValidationGroup = validationGroup;
        }
        else
            BlogCategoriesValidator.Enabled = false;
        
        if (Component.CanSyndicateContent)
            EnableSyndication.Attributes.Add("onclick", string.Format("var feedUrlEl=document.getElementById(\"syndicationRssFeedUrlContainer\"); var updEl=document.getElementById(\"syndicationUpdateableContainer\"); var redirectEl=document.getElementById(\"syndicationRedirectToCommentsContainer\"); if(this.checked){{feedUrlEl.style.display=\"\";updEl.style.display=\"\";redirectEl.style.display=\"\";document.getElementById(\"{0}\").focus() }}else{{feedUrlEl.style.display=\"none\";updEl.style.display=\"none\";redirectEl.style.display=\"none\";}} ValidatorEnable(document.getElementById(\"{1}\"), this.checked); ValidatorEnable(document.getElementById(\"{2}\"), this.checked);", SyndicationRssFeedUrl.ClientID, SyndicationRssFeedUrlRequired.ClientID, SyndicationRssFeedUrlRegex.ClientID));
        
		Save.ValidationGroup = validationGroup;
	    Save.Text = GetMessageRaw(Component.ComponentMode == BlogEditComponent.Mode.Add ? "ButtonText.CreateBlog" : "ButtonText.EditBlog");
	}

	protected new void SaveClick(object sender, EventArgs e)
	{
		if (!Page.IsValid)
			return;
		
		base.SaveClick(sender, e);
	}
</script>

<% 
	if (Component.FatalError != BlogEditComponent.ErrorCode.None)
	{ 
		%>
		<div class="blog-content">
		<div class="blog-note-box blog-note-error">
			<div class="blog-note-box-text"><%= Component.GetErrorHtml(Component.FatalError) %></div>
		</div>	
		</div>
		<%
		return;
	}
%>

<div class="blog-content">
	<div class="blog-edit-form blog-edit-blog-form">

		<%
			Errors.HeaderText = string.Format(@"<div class=""blog-note-box-text"">{0}</div>", GetMessage("Error.Title"));
			foreach (string s in Component.ErrorSummary)
				Errors.AddErrorMessage(s);
		%>
		
		<bx:BXValidationSummary runat="server" ID="Errors" CssClass="blog-note-box blog-note-error" ForeColor="" /> 

		<div class="blog-edit-form-title"><%= (Component.ComponentMode == BlogEditComponent.Mode.Add ? GetMessage("FormTitle.CreateBlog") : GetMessage("FormTitle.BlogSettings")) %></div>

		<div class="blog-edit-fields">
		
			<div class="blog-edit-field blog-edit-field-title">
				<label for="<%= Encode(BlogName.ClientID) %>" class="blog-edit-field-caption"><%= GetMessage("Label.Name") %><span class="blog-required-field">*</span></label>
				<asp:TextBox ID="BlogName" runat="server" MaxLength="255" TabIndex="1" Columns="60"></asp:TextBox>
				<asp:RequiredFieldValidator runat="server" ID="BlogNameValidator" ErrorMessage="<%$ Loc:Message.NameNotSpecified %>" ControlToValidate="BlogName" Display="None" SetFocusOnError="true" />
			</div>
			
			<% if (Component.ComponentMode == BlogEditComponent.Mode.Add) { %>
			<div class="blog-edit-field blog-edit-field-slug">
				<label for="<%= Encode(BlogSlug.ClientID) %>" class="blog-edit-field-caption"><%= GetMessage("Label.Slug") %><span class="blog-required-field">*</span></label>
				<asp:TextBox ID="BlogSlug" TabIndex="2" Columns="60" runat="server"></asp:TextBox>
				<% if (Parameters.ContainsKey("BlogAbsoluteUrlTemplate") && !String.IsNullOrEmpty(Parameters["BlogAbsoluteUrlTemplate"])) { 
						BXParamsBag<object> replace = new BXParamsBag<object>();
						replace["BlogSlug"] = GetMessageRaw("BlogSlugDummy");
						string url = BXSefUrlUtility.MakeLink(Parameters["BlogAbsoluteUrlTemplate"], replace);
						url = new Uri(BXSefUrlManager.CurrentUrl, url).ToString();
				%>
				<div class="blog-edit-field-slug-desc"><%= String.Format(GetMessage("BlogSlug.Description"), url) %></div>
				<%} %>
				<asp:RequiredFieldValidator runat="server" Enabled="False" ID="BlogSlugEmptyValidator"	ErrorMessage="<%$ Loc:Message.SlugNotSpecified %>" ControlToValidate="BlogSlug" Display="None" SetFocusOnError="true" />
				<asp:RegularExpressionValidator runat="server" Enabled="False" ID="BlogSlugRegexValidator" ErrorMessage="<%$ Loc:Message.InvalidSlug %>" ControlToValidate="BlogSlug" Display="None" SetFocusOnError="true" />
				<asp:RegularExpressionValidator runat="server" Enabled="False" ID="BlogSlugLengthValidator" ControlToValidate="BlogSlug" Display="None" SetFocusOnError="true" />
			</div>
			<%} %>
			
			<% if (Component.AllowToAjustBlogGroups) { %>
			<div class="blog-edit-field blog-edit-field-cats">
				<label for="<%= Encode(BlogCategories.ClientID) %>" class="blog-edit-field-caption"><%= GetMessage("Label.BlogCategory") %><span class="blog-required-field">*</span></label>
				<asp:ListBox ID="BlogCategories" TabIndex="3" Rows="1" runat="server"></asp:ListBox>
				<asp:RequiredFieldValidator runat="server" ID="BlogCategoriesValidator"	ErrorMessage="<%$ Loc:Message.CategoryNotSpecified %>" ControlToValidate="BlogCategories" Display="None" SetFocusOnError="true" />
			</div>
			<%} %>

			<div class="blog-edit-field blog-edit-field-desc">
				<label for="<%= Encode(BlogDescription.ClientID) %>" class="blog-edit-field-caption"><%= GetMessage("Label.Description") %></label>
				<asp:TextBox ID="BlogDescription"  TabIndex="4" TextMode="MultiLine" Rows="7" runat="server"></asp:TextBox>	 
				<asp:RegularExpressionValidator runat="server" ID="BlogDescriptionValidator" ControlToValidate="BlogDescription" Display="None" SetFocusOnError="true" ErrorMessage="<%$ Loc:Error.DescriptionMaxLengthExceeded %>" ValidationExpression="^[\s\S]{0,2048}$" />				
			</div>
			
			<div class="blog-edit-field blog-edit-field-settings">
				<div class="blog-edit-field-setting"><asp:CheckBox ID="NotifyComments" TabIndex="5" runat="server" Text="<%$ Loc:CheckBoxText.SendEmailNotifications %>" /></div>
			</div>
			
			<% if (Component.ComponentMode == BlogEditComponent.Mode.Edit && Component.CanModerate && !Component.IsOwner) { %>
			<div class="blog-edit-field blog-edit-field-settings">
				<div class="blog-edit-field-setting"><asp:CheckBox ID="DisableBlog" TabIndex="6" runat="server" Text="<%$ Loc:CheckBoxText.DisableBlog %>" /></div>
			</div>
			<%} %>
						
			<%if (Component.CanSyndicateContent){ %>
			    <div class="blog-edit-field blog-edit-field-enable-syndication">
			        <asp:CheckBox ID="EnableSyndication" runat="server" TabIndex="7" Text="<%$ Loc:CheckBoxText.EnableSyndication %>" />
			    </div>
			    <div id="syndicationRssFeedUrlContainer" class="blog-edit-field blog-edit-field-syndication-rss-feed-url" style="<%=EnableSyndication.Checked?string.Empty:"display:none;"%>">
			        <label for="<%= Encode(SyndicationRssFeedUrl.ClientID) %>" class="blog-edit-field-caption"><%= GetMessage("Label.SyndicationRssFeedUrl")%><span class="blog-required-field">*</span></label>
                    <asp:TextBox ID="SyndicationRssFeedUrl" runat="server" TabIndex="8" Text="http://"></asp:TextBox>
                    <asp:RequiredFieldValidator ID="SyndicationRssFeedUrlRequired" runat="server" Enabled="False" Display="Dynamic" ControlToValidate="SyndicationRssFeedUrl" ErrorMessage="<%$ Loc:Message.SyndicationRssFeedUrlRequired %>" Text="*" SetFocusOnError="true" />
				    <asp:RegularExpressionValidator ID="SyndicationRssFeedUrlRegex" runat="server" Enabled="False" Display="Dynamic" ControlToValidate="SyndicationRssFeedUrl" ErrorMessage="<%$ Loc:Message.SyndicationRssFeedUrlNotValid %>" ValidationExpression="\s*https?://.+" Text="*" SetFocusOnError="true" />                    
			    </div>
			    <div id="syndicationUpdateableContainer" class="blog-edit-field blog-edit-field-syndication-updateable" style="<%=EnableSyndication.Checked?string.Empty:"display:none;"%>">
			        <asp:CheckBox ID="SyndicationUpdateable" runat="server" TabIndex="9" Text="<%$ Loc:CheckBoxText.SyndicationUpdateable %>" Checked="true" />
			    </div>
			    <div id="syndicationRedirectToCommentsContainer" class="blog-edit-field blog-edit-field-syndication-redirect-comments" style="<%=EnableSyndication.Checked?string.Empty:"display:none;"%>">
			        <asp:CheckBox ID="SyndicationRedirectToComments" runat="server" TabIndex="10" Text="<%$ Loc:CheckBoxText.SyndicationRedirectToComments %>" Checked="true" />
			    </div>				    			
			<%} %>
			
			<% if (Component.BlogCustomFieldEditors.Length > 0){ %>
		        <% foreach (CustomFieldEditor ed in Component.BlogCustomFieldEditors){%>
		            <div class="blog-edit-field blog-edit-custom-property-<%= ed.Field.Name.ToLowerInvariant() %>">
                    <% if(string.Equals(ed.Field.CustomTypeId.ToUpperInvariant(), "BITRIX.SYSTEM.BOOLEAN", StringComparison.Ordinal) && ed.Field.Settings.GetInt("view", 0) == 0){%>
		                <span class="blog-edit-custom-property-single-line-editor"><%= ed.Render() %></span><label for="<%= Encode(ed.ClientID) %>" class="blog-edit-field-caption blog-edit-custom-property-single-line-caption"><%= ed.Caption %>
		                    <% if (ed.IsRequired){ %>
		                       <span class="blog-required-field">*</span>
		                    <%} %>
		                </label>
                     <%} %>
                     <% else { %>
		                <label class="blog-edit-field-caption">
		                    <%= ed.Caption %>
		                    <% if (ed.IsRequired){ %>
		                       <span class="blog-required-field">*</span>
		                    <%} %>
		                </label>
		                <%= ed.Render() %>
		            <%} %>
		            </div>
		         <%} %>		    
		    <%} %>
						
			<div class="blog-edit-buttons">
				<asp:Button runat="server" ID="Save" TabIndex="10" OnClick="SaveClick" /> 
			</div>
		</div>
	</div>
</div>