﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.DataLayer;
using Bitrix.Search;

namespace Bitrix.Blog.Components
{
	public partial class BlogPostsComponentTagFilter : System.Web.UI.UserControl, IBlogPostsComponentTagFilter
	{
		BXSearchQuery q;

		void IBlogPostsComponentTagFilter.InitFilter(string siteId, string tags, string blogSlugs)
		{
			q = new BXSearchQuery();
			q.SiteId = siteId;
			q.LanguageId = "";
			q.FieldsToSelect.Add(BXSearchField.ItemId);

			BXSearchContentGroupFilter f = new BXSearchContentGroupFilter(BXFilterExpressionCombiningLogic.And);
			//discrete timestep for count caching
			DateTime now = DateTime.Now;
			//DateTime nowFloor = new DateTime(now.Year, now.Month, now.Day, now.Hour, now.Minute / 5 * 5, 0);
            f.Add(BXSearchContentFilter.IsActive(now.AddMinutes(5 - (now.Minute % 5)).AddSeconds(-now.Second).AddMilliseconds(-now.Millisecond)));
			f.Add(new BXSearchContentFilterItem(BXSearchField.ModuleId, BXSqlFilterOperators.Equal, "blog"));


            if (!string.IsNullOrEmpty(blogSlugs))
            {
                string[] blogSlugArr = blogSlugs.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                if (blogSlugArr.Length > 0)
                {
                    for (int i = 0; i < blogSlugArr.Length; i++)
                        blogSlugArr[i] = blogSlugArr[i].Trim();

                    BXBlogCollection blogs = BXBlog.GetList(
                        new BXFilter(new BXFilterItem(BXBlog.Fields.Slug, BXSqlFilterOperators.In, blogSlugArr)), 
                        null,
                        new BXSelect(BXSelectFieldPreparationMode.Normal, BXBlog.Fields.Id),
                        null
                        );
                    if (blogs.Count > 0)
                    {
                        List<string> blogIdLst = new List<string>();
                        for (int j = 0; j < blogs.Count; j++)
                            blogIdLst.Add(blogs[j].Id.ToString());
                        f.Add(new BXSearchContentFilterItem(BXSearchField.ItemGroup, Bitrix.DataLayer.BXSqlFilterOperators.In, blogIdLst.ToArray()));
                    }
                }
            }
			q.Filter = f;

			q.CalculateRelevance = false;
			q.AddTags(tags, ',');

            q.OrderBy = new BXSearchOrderBy(
                new BXSearchOrderByPair(BXSearchField.DateFrom, BXOrderByDirection.Desc),
                new BXSearchOrderByPair(BXSearchField.ContentDate, BXOrderByDirection.Desc),
                new BXSearchOrderByPair(BXSearchField.Id, BXOrderByDirection.Asc)
                ); 
    
		}

		int IBlogPostsComponentTagFilter.Count()
		{
			return q.ExecuteCount(true);
		}

        IList<int> IBlogPostsComponentTagFilter.GetPostIdList(BXPagingOptions paging)
		{
			q.PagingOptions = paging;

			List<int> postIds = new List<int>();
			foreach (BXSearchResult result in q.Execute())
			{
				string id = result.Content.ItemId;
				if (!id.StartsWith("p"))
					continue;

				id = id.Substring(1);
				int i;
				if (!int.TryParse(id, out i))
					continue;
				postIds.Add(i);
			}
			return postIds;
		}
	}
}