﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.DataLayer;
using Bitrix.Search;

namespace Bitrix.Blog.Components
{
	public partial class BlogPostsComponentTagFilter : System.Web.UI.UserControl, IBlogPostsComponentTagFilter
	{
		BXSearchQuery q;

		void IBlogPostsComponentTagFilter.InitFilter(string siteId, string tags)
		{
			q = new BXSearchQuery();
			q.SiteId = siteId;
			q.LanguageId = string.Empty;
			q.FieldsToSelect.Add(BXSearchField.ItemId);

			BXSearchContentGroupFilter f = new BXSearchContentGroupFilter(BXFilterExpressionCombiningLogic.And);
			
			//discrete timestep for count caching
			DateTime now = DateTime.Now;
			DateTime nowFloor = new DateTime(now.Year, now.Month, now.Day, now.Hour, now.Minute / 5 * 5, 0);
			f.Add(BXSearchContentFilter.IsActive(nowFloor));
			
			f.Add(new BXSearchContentFilterItem(BXSearchField.ModuleId, BXSqlFilterOperators.Equal, "blog"));
			q.Filter = f;

			q.CalculateRelevance = false;
			q.AddTags(tags, ',');
            q.OrderBy = new BXSearchOrderBy(
                new BXSearchOrderByPair(BXSearchField.DateFrom, BXOrderByDirection.Desc),
                new BXSearchOrderByPair(BXSearchField.ContentDate, BXOrderByDirection.Desc),
                new BXSearchOrderByPair(BXSearchField.Id, BXOrderByDirection.Asc)
                ); 
		}

		int IBlogPostsComponentTagFilter.Count()
		{
			return q.ExecuteCount(true);
		}

		IList<int> IBlogPostsComponentTagFilter.GetPostIdList(BXPagingOptions paging)
		{
			q.PagingOptions = paging;

			List<int> resultLst = new List<int>();
			foreach (BXSearchResult result in q.Execute())
			{
				string id = result.Content.ItemId;
				if (!id.StartsWith("p"))
					continue;

				id = id.Substring(1);
				int i;
				if (!int.TryParse(id, out i))
					continue;
                resultLst.Add(i);
			}
            return resultLst;
		}
	}
}