﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using Bitrix.Security;
using Bitrix.Configuration;
using Bitrix.Catalog;
using Bitrix.Services;
using System.Web.Script.Serialization;
using Bitrix.IBlock;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using System.Text;
using Bitrix.Services.Js;


public partial class bitrix_modules_Catalog_Options : BXControl
{
    private bool? isAuthorized = null;
    public bool IsAuthorized
    {
        get  { return (this.isAuthorized ?? (this.isAuthorized = BXPrincipal.Current.IsCanOperate(BXRoleOperation.Operations.ProductSettingsManage))).Value; }
    }

    private string backUrl = null;
    public string BackUrl
    {
        get { return this.backUrl ?? (this.backUrl = Request.QueryString[BXConfigurationUtility.Constants.BackUrl] ?? BXSefUrlManager.CurrentUrl.ToString()); }
    }

    private CatalogModuleOptionsEditorError editorError = CatalogModuleOptionsEditorError.None;
    public CatalogModuleOptionsEditorError EditorError
    {
        get { return this.editorError; }
        private set { this.editorError = value; }
    }

    protected override void OnInit(EventArgs e)
    {
        base.OnInit(e);
        tabContainer.ShowApplyButton = tabContainer.ShowSaveButton = IsAuthorized;
        tabContainer.ShowCancelButton = BackUrl.Length > 0;
    }

    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);
        if (!IsPostBack)
            LoadData();
        else
            Page.Validate();
    }

    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);
        BXPage.Scripts.RequireUtils();
       
        BXCatalogVATRateCollection c = BXCatalogVATRate.GetList(
            new BXFilter(new BXFilterItem(BXCatalogVATRate.Fields.Active, BXSqlFilterOperators.Equal, true)),
            new BXOrderBy(new BXOrderByPair(BXCatalogVATRate.Fields.Sort, BXOrderByDirection.Asc), new BXOrderByPair(BXCatalogVATRate.Fields.Name, BXOrderByDirection.Asc)),
			null,
			null,
			BXTextEncoder.EmptyTextEncoder
            );

        StringBuilder sb = new StringBuilder();

		sb.Append("Bitrix.Catalog.IBlockAry = [");
		for(int i = 0; i < this.catalogInfos.Count; i++)
		{
			if(i > 0)
				sb.Append(",");

			sb.Append("{");
			sb.Append("\"id\":").Append(this.catalogInfos[i].IBlockId.ToString());
			sb.Append(",\"descr\":\"").Append(BXJSUtility.Encode(this.catalogInfos[i].Description)).Append("\"");
			sb.Append("}");
		}
		sb.Append("]").AppendLine();

        sb.Append("Bitrix.VATRate.create(0,\"").Append(GetMessage("NotSelectedNeuter")).Append("\");");
        foreach(BXCatalogVATRate vatRate in c)
            sb.Append("Bitrix.VATRate.create(").Append(vatRate.Id).Append(",\"").Append(BXJSUtility.Encode(vatRate.Name)).Append("\");");

        sb.Append("Bitrix.Catalog.constructFromString(document.getElementById(\"").Append(InitCatalogState.ClientID).Append("\").value);").AppendLine();

        ClientScriptManager sm = Page.ClientScript;
        sm.RegisterStartupScript(
            GetType(),
            "Initialize",
            sb.ToString(),
            true
            );

        sm.RegisterOnSubmitStatement(
            GetType(),
            "Finalize",
            string.Concat("document.getElementById(\"", FinalCatalogState.ClientID, "\").value = Bitrix.Catalog.saveToString();")
            );
    }
 
    protected void OnTabCommand(object sender, BXTabControlCommandEventArgs e)
    {
        CatalogModuleOptionsEditorCommand? cmd = null;

        if (string.Equals(e.CommandName, "save", StringComparison.Ordinal))
            cmd = CatalogModuleOptionsEditorCommand.Save;
        else if (string.Equals(e.CommandName, "apply", StringComparison.Ordinal))
            cmd = CatalogModuleOptionsEditorCommand.Apply;
		
        if (!cmd.HasValue)
            Response.Redirect(BackUrl);

        if (Page.IsValid && (cmd.Value == CatalogModuleOptionsEditorCommand.Apply || cmd.Value == CatalogModuleOptionsEditorCommand.Save))
            SaveData();

        if (EditorError == CatalogModuleOptionsEditorError.None)
        {
            if (cmd.Value == CatalogModuleOptionsEditorCommand.Save && BackUrl.Length > 0)
                Page.Response.Redirect(BackUrl);
            else
            {
                LoadData();
                successMessage.Visible = true;
            }
        }
    }

    private void SaveData()
    {
        try
        {
            List<CatalogInfo> finalCatalogInfos = null;
            if (!string.IsNullOrEmpty(FinalCatalogState.Value))
            {
                JavaScriptSerializer serializer = new JavaScriptSerializer();
                serializer.RegisterConverters(new JavaScriptConverter[] { new CatalogInfoJsonConv() });
                finalCatalogInfos = serializer.Deserialize<List<CatalogInfo>>(FinalCatalogState.Value);
            }

            if (finalCatalogInfos != null && finalCatalogInfos.Count > 0)
            {
                BXCatalogCollection catalogs = BXCatalog.GetList(null, 
					null, 
					new BXSelect(BXSelectFieldPreparationMode.Normal,  BXCatalog.Fields.Id),
					null);
                foreach (CatalogInfo info in finalCatalogInfos)
                {
                    if (info.CatalogId > 0)
                    {
                        int ind = catalogs != null ? catalogs.FindIndex(delegate(BXCatalog obj) { return obj.Id == info.CatalogId; }) : -1;
                        if (ind < 0)
                            continue;

                        BXCatalog catalog =  BXCatalog.GetById(catalogs[ind].Id);
                        if (info.Enabled)
                        {
                            catalog.CheckStockAvailability = info.CheckStockAvailability;
                            catalog.VATRateId = info.VATRateId;
                            catalog.Update();

							if(info.SkuIBlockId > 0)
							{
								BXCatalog skuCatalog = BXCatalog.GetById(info.SkuIBlockId);
								if(skuCatalog == null)
								{
									skuCatalog = new BXCatalog();
									skuCatalog.Id = info.SkuIBlockId;
									skuCatalog.CheckStockAvailability = info.CheckStockAvailability;
									skuCatalog.VATRateId = info.VATRateId;
									skuCatalog.Create();
								}

								if(skuCatalog.Id != catalog.Id)
									catalog.SetSKUCatalog(skuCatalog.Id, null);
							}
							else
							{
								BXCatalog skuCatalog = catalog.GetSKUCatalog();
								if(skuCatalog != null)
									catalog.SetSKUCatalog(0, null);
							}
                        }
                        else
                            catalog.Delete();
                    }
                    else if (info.Enabled)
                    {
                        BXCatalog catalog = new BXCatalog();
                        catalog.Id = info.IBlockId;
                        catalog.CheckStockAvailability = info.CheckStockAvailability;
                        catalog.VATRateId = info.VATRateId;
                        catalog.Create();
						if(info.SkuIBlockId > 0)
						{
							BXCatalog skuCatalog = new BXCatalog();
							skuCatalog.Id = info.SkuIBlockId;
							skuCatalog.CheckStockAvailability = info.CheckStockAvailability;
							skuCatalog.VATRateId = info.VATRateId;
							skuCatalog.Create();
							catalog.SetSKUCatalog(skuCatalog.Id, null);
						}
                    }
                }
            }
        }
        catch (Exception exc)
        {
            EditorError = CatalogModuleOptionsEditorError.DataSavingFailed;
            errorMessage.AddErrorMessage(Encode(exc.Message)); 
        }
    }

	private List<CatalogInfo> catalogInfos = new List<CatalogInfo>();

    private void LoadData()
    {
        try
        {
			if(this.catalogInfos.Count > 0)
				this.catalogInfos.Clear();

            BXCatalogCollection catalogs = BXCatalog.GetList(null, null);
            BXIBlockCollection iblocks = BXIBlock.GetList(
                new BXFilter(new BXFilterItem(BXIBlock.Fields.Active, BXSqlFilterOperators.Equal, 'Y')),
                null,
                new BXSelect(
                    BXSelectFieldPreparationMode.Normal,
                    BXIBlock.Fields.ID,
                    BXIBlock.Fields.Name,
                    BXIBlock.Fields.Sites.Site.ID,
                    BXIBlock.Fields.Sites.Site.Name,
                    BXIBlock.Fields.Type.Translations),
                null,
                BXTextEncoder.EmptyTextEncoder);

            foreach (BXIBlock ib in iblocks)
            {
                int ind = catalogs.FindIndex(delegate(BXCatalog o) { return o.Id == ib.Id; });
                catalogInfos.Add(new CatalogInfo(ib, ind >= 0 ? catalogs[ind] : null));
            }

            catalogInfos.Sort(delegate(CatalogInfo x, CatalogInfo y) { return string.Compare(x.Description, y.Description); });

            JavaScriptSerializer serializer = new JavaScriptSerializer();
            serializer.RegisterConverters(new JavaScriptConverter[] { new CatalogInfoJsonConv() });

            InitCatalogState.Value = serializer.Serialize(catalogInfos);
        }
        catch (Exception exc)
        {
            EditorError = CatalogModuleOptionsEditorError.DataLoadingFailed;
            errorMessage.AddErrorMessage(Encode(exc.Message)); 
        }
    }
}

public class CatalogInfo
{
    public CatalogInfo()
    { 
    }

    public CatalogInfo(BXIBlock iblock, BXCatalog catalog)
    {
        if (iblock == null)
            throw new ArgumentNullException("iblock");

        this.IBlockId = iblock.Id;

        StringBuilder descrBuilder = new StringBuilder();
        descrBuilder.Append(iblock.Type != null && iblock.Type.Translations != null ? iblock.Type.Translations["ru"].Name : "N/A").Append("/").Append(iblock.Name);

        for (int i = 0; i < iblock.Sites.Count; i++)
        {
            if (i > 0)
                descrBuilder.Append(", ");
            else
                descrBuilder.Append("(");
            descrBuilder.Append(iblock.Sites[i].SiteId);
            if(i == iblock.Sites.Count - 1)
                descrBuilder.Append(")");
        }

        this.description = descrBuilder.ToString();
        this.editUrl = string.Concat(VirtualPathUtility.ToAbsolute("~/bitrix/admin/IBlockEdit.aspx"), "?id=", iblock.Id);
        if (catalog == null)
        {
            this.catalogId = 0;
            this.VATRateId = 0;
            this.enabled = false;
        }
        else
        {
            this.catalogId = catalog.Id;
            this.VATRateId = catalog.VATRateId;
            this.enabled = true;
            this.checkStockAvailability = catalog.CheckStockAvailability;

			BXCatalog skuCatalog = catalog.GetSKUCatalog();
			if(skuCatalog != null)
				this.skuIBockId = skuCatalog.Id;
        }
    }

    private int catalogId = 0;
    public int CatalogId
    {
        get { return this.catalogId; }
        set { this.catalogId = value; }
    }

    private int iblockId = 0;
    public int IBlockId
    {
        get { return this.iblockId; }
        set { this.iblockId = value; }
    }

    private int skuIBockId = 0;
    public int SkuIBlockId
    {
        get { return this.skuIBockId; }
        set { this.skuIBockId = value; }
    }

    private string description = string.Empty;
    public string Description
    {
        get { return this.description; }
        set { this.description = value; }
    }

    private string editUrl = string.Empty;
    public string EditUrl
    {
        get { return this.editUrl; }
        set { this.editUrl = value; }
    }

    private bool enabled = false;
    public bool Enabled
    {
        get { return this.enabled; }
        set { this.enabled = value; }
    }

    private bool checkStockAvailability = true;
    public bool CheckStockAvailability
    {
        get { return this.checkStockAvailability; }
        set { this.checkStockAvailability = value; }
    }

    private int vatRateId = 0;
    public int VATRateId
    {
        get { return this.vatRateId; }
        set { this.vatRateId = value; }
    }
}

public class CatalogInfoJsonConv : JavaScriptConverter
{
    public override IEnumerable<Type> SupportedTypes
    {
        get { return new Type[] { typeof(CatalogInfo) }; }
    }

    public override IDictionary<string, object> Serialize(object obj, JavaScriptSerializer serializer)
    {
        CatalogInfo item = obj as CatalogInfo;
        if (item == null)
            new Dictionary<string, object>();

        Dictionary<string, object> result = new Dictionary<string, object>();
        result.Add("iblockId", item.IBlockId);
        result.Add("catalogId", item.CatalogId);
        result.Add("skuIBlockId", item.SkuIBlockId);
        result.Add("descr", item.Description);
        result.Add("editUrl", item.EditUrl);
        result.Add("enabled", item.Enabled);
        result.Add("checkStockAvailability", item.CheckStockAvailability);
        result.Add("vatRateId", item.VATRateId);

        return result;
    }

    public override object Deserialize(IDictionary<string, object> dictionary, Type type, JavaScriptSerializer serializer)
    {
        if (type != typeof(CatalogInfo))
            return null;

        CatalogInfo result = new CatalogInfo();
        object obj;
        if (dictionary.TryGetValue("iblockId", out obj))
            result.IBlockId = Convert.ToInt32(obj);

        if (dictionary.TryGetValue("catalogId", out obj))
            result.CatalogId = Convert.ToInt32(obj);

        if (dictionary.TryGetValue("skuIBlockId", out obj))
            result.SkuIBlockId = Convert.ToInt32(obj);

        if (dictionary.TryGetValue("enabled", out obj))
            result.Enabled = Convert.ToBoolean(obj);

        if (dictionary.TryGetValue("checkStockAvailability", out obj))
            result.CheckStockAvailability = Convert.ToBoolean(obj);

        if (dictionary.TryGetValue("vatRateId", out obj))
            result.VATRateId = Convert.ToInt32(obj);

        return result;
    }
}

public enum CatalogModuleOptionsEditorError
{
    None = 0,
    DataLoadingFailed = -1,
    DataSavingFailed = -2
}

public enum CatalogModuleOptionsEditorCommand
{
    Save = 1,
    Apply
}