﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI.Wizards;
using Bitrix.DataTypes;
using Bitrix.Services.Text;
using System.Text.RegularExpressions;
using System.Web.Hosting;
using System.IO;
using Bitrix.IBlock;
using Bitrix.DataLayer;
using Bitrix.Configuration;

namespace Bitrix.Wizards.Solutions
{
	public partial class CatalogSettings : BXWizardStepStandardHtmlControl
	{
		protected override BXWizardResult OnActionShow(BXCommonBag parameters)
		{
			WizardContext.Navigation.Selected = "catalog_settings";
			UI.Load("CatalogSettings");

			string catalogName;
			if (!UI.Data.TryGetString("CatalogName", out catalogName))
				UI.Data["CatalogName"] = GetMessage("MyProducts");

			string catalogDirectoryName;
			if (!UI.Data.TryGetString("CatalogDirectoryName", out catalogDirectoryName))
				UI.Data["CatalogDirectoryName"] = "products";

			bool displayComparisonTable;
			if (!UI.Data.TryGetBool("DisplayComparisonTable", out displayComparisonTable))
				UI.Data["DisplayComparisonTable"] = true;

			UI.Update("CatalogSettings");
			return PrepareView(null);
		}

		protected override BXWizardResult OnActionPrevious(BXCommonBag parameters)
		{
			return Result.Previous();
		}

		protected override BXWizardResult OnActionNext(BXCommonBag parameters)
		{
			UI.LoadValues(parameters);

			List<string> errors = new List<string>();
			string catalogName, catalogDirName;
			if(!UI.Data.TryGetString("CatalogName", out catalogName) || BXStringUtility.IsNullOrTrimEmpty(catalogName))
				errors.Add(GetMessage("Error.CatalogNameRequired"));
			if(!UI.Data.TryGetString("CatalogDirectoryName", out catalogDirName) || BXStringUtility.IsNullOrTrimEmpty(catalogDirName))
				errors.Add(GetMessage("Error.CatalogDirectoryName"));			
			else if (!Regex.IsMatch(catalogDirName, @"^[a-z0-9_]+$", RegexOptions.IgnoreCase))
				errors.Add(GetMessage("Error.InvalidCatalogDirectoryName"));
			else if(Directory.Exists(HostingEnvironment.MapPath(VirtualPathUtility.Combine("~/catalog/", catalogDirName))))
				errors.Add(string.Format(GetMessage("Error.CatalogDirectoryAlreadyExists"), catalogDirName));
			else
			{
				int blockTypeId = BXOptionManager.GetOptionInt("Bitrix.CommerceSite", "CatalogIBlockTypeId", 0);
				if (blockTypeId <= 0)
					errors.Add(GetMessage("Error.CatalogIBlockTypeIdNotFound"));
				else
				{
					BXIBlockCollection blocks = BXIBlock.GetList(
						new BXFilter(new BXFilterItem(BXIBlock.Fields.Code, BXSqlFilterOperators.Equal, catalogDirName)),
						null,
						new BXSelect(BXSelectFieldPreparationMode.Normal, BXIBlock.Fields.ID),
						new BXQueryParams(new BXPagingOptions(0, 1)));

					if (blocks.Count > 0)
						errors.Add(string.Format(GetMessage("Error.IBlockAlreadyExists"), catalogDirName));
				}
			}

			if (errors.Count != 0)
				return PrepareView(errors.ToArray());

		    WizardContext.State["CatalogName"] = catalogName;
		    WizardContext.State["CatalogDirectoryName"] = catalogDirName;
		    WizardContext.State["DisplayComparisonTable"] = UI.Data.GetBool("DisplayComparisonTable", true);

			UI.Overwrite("CatalogSettings"); 

            return Result.Action("product_settings", string.Empty, null);
		}

		private BXWizardResult PrepareView(IEnumerable<string> errors)
		{
			BXWizardResultView view = Result.Render(GetMessage("Title"), errors);
			view.Buttons.Add("prev", null);
			view.Buttons.Add("next", null);
			return view;
		}
	}
}
