﻿using Bitrix.IO;
using Bitrix.UI.Wizards;
using Bitrix.DataTypes;
using System.Web;
using Bitrix.IBlock;
using Bitrix.Configuration;
using System;
using Bitrix.Catalog;
using Bitrix.DataLayer;
using System.Web.Script.Serialization;
using System.Collections.Generic;
using System.Collections;
using Bitrix.Services;
using Bitrix.Services.Text;
using System.Web.Hosting;
using System.IO;
using System.Text;
using Bitrix.Security;

namespace Bitrix.Wizards.Solutions
{
	public partial class Finish : BXWizardStepStandardHtmlControl
	{
		private BXCustomField CreateCustomField(string entityId, string name, string customTypeId, string labelId, bool isMultiple)
		{
			BXCustomField field = new BXCustomField(entityId, name, customTypeId);
			field.Multiple = isMultiple;
			if (!string.IsNullOrEmpty(labelId))
				foreach (string langId in BXLoc.Locales)
				{
					string msg = BXLoc.GetMessage(langId, this, labelId, true);
					if (!string.IsNullOrEmpty(msg))
						(field.Localization[langId] = new BXCustomFieldLocalization()).EditFormLabel = msg;
				}
			return field;
		}

		protected override BXWizardResult OnActionShow(BXCommonBag parameters)
		{
			WizardContext.Navigation.Selected = "finish";

			List<string> errorLst = new List<string>();

			string siteId = WizardContext.State.GetString("Installer.SiteId");
			if (string.IsNullOrEmpty(siteId))
				errorLst.Add(GetMessage("Error.SiteIdNotFound"));

			int blockTypeId = BXOptionManager.GetOptionInt("Bitrix.CommerceSite", "CatalogIBlockTypeId", 0);
			if (blockTypeId <= 0)
				errorLst.Add(GetMessage("Error.CatalogIBlockTypeIdNotFound"));

			string catalogName = WizardContext.State.GetString("CatalogName", string.Empty);
			if (string.IsNullOrEmpty(catalogName))
				errorLst.Add(GetMessage("Error.CatalogNameNotFound"));

			string catalogDirectoryName = WizardContext.State.GetString("CatalogDirectoryName", string.Empty);
			if (string.IsNullOrEmpty(catalogDirectoryName))
				errorLst.Add(GetMessage("Error.CatalogDirectoryNameNotFound"));

			BXIBlockCollection blocks = BXIBlock.GetList(
				new BXFilter(new BXFilterItem(BXIBlock.Fields.Code, BXSqlFilterOperators.Equal, catalogDirectoryName)),
				null,
				new BXSelect(BXSelectFieldPreparationMode.Normal, BXIBlock.Fields.ID),
				new BXQueryParams(new BXPagingOptions(0, 1)));

			if (blocks.Count > 0)
				errorLst.Add(string.Format(GetMessage("Error.IBlockAlreadyExists"), catalogDirectoryName));

			string directoryPath = VirtualPathUtility.ToAbsolute(VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine("~/catalog/", catalogDirectoryName))),
				filePath = VirtualPathUtility.ToAbsolute(VirtualPathUtility.Combine(directoryPath, "default.aspx"));

			DirectoryInfo di = new DirectoryInfo(HostingEnvironment.MapPath(directoryPath));
			if (di.Exists)
				errorLst.Add(string.Format(GetMessage("Error.DirectoryAlreadyExists"), directoryPath));				

			di.Create();
				
			if (errorLst.Count > 0)
				return Result.Render(GetMessage("Title"), errorLst);

			try
			{
				//Создание инфоблока
				BXIBlock block = new BXIBlock();
				block.Active = true;
				block.Sites.Add(siteId);
				block.TypeId = blockTypeId;
				block.Name = catalogName;
				block.Code = catalogDirectoryName;
				block.Sort = 500;
				block.IndexContent = true;

				block.Create();

				//Права на чтение
				BXRoleManager.AddRolesToOperations(new string[] { "Guest" }, new string[]{ "IBlockRead" }, "iblock", block.Id.ToString());

				//Создание свойств
				string entityId = BXIBlockElement.GetCustomFieldsKey(block.Id);
				//системные
				//SPECIALOFFER
				CreateCustomField(entityId, "SPECIALOFFER", "Bitrix.System.Boolean", "PropertyLabel.SpecialOffer", false).Create();
				//NEWPRODUCT
				CreateCustomField(entityId, "NEWPRODUCT", "Bitrix.System.Boolean", "PropertyLabel.NewProduct", false).Create();
				//SALELEADER
				CreateCustomField(entityId, "SALELEADER", "Bitrix.System.Boolean", "PropertyLabel.SaleLeader", false).Create();
				//MORE_PHOTO
				CreateCustomField(entityId, "MORE_PHOTO", "Bitrix.System.File", "PropertyLabel.MorePhoto", true).Create();

				//пользовательские
				StringBuilder comparisonSb = new StringBuilder();
				string propertyData = WizardContext.State.GetString("ProductPropertyData");
				if (!string.IsNullOrEmpty(propertyData))
				{
					JavaScriptSerializer serializer = new JavaScriptSerializer();
					serializer.RegisterConverters(new JavaScriptConverter[] { new CatalogProductPropertyInfoJsonConverter() });
					CatalogProductPropertyInfo[] properties = serializer.Deserialize<CatalogProductPropertyInfo[]>(propertyData);

					for (int i = 0; i < properties.Length; i++)
					{
						CatalogProductPropertyInfo property = properties[i];
						if (property.Name.Length == 0)
							continue;

						BXCustomField field = CreateCustomField(entityId, string.Concat("PROPERTY", (i + 1).ToString()), property.Type, null, false);
						(field.Localization[BXLoc.CurrentLocale] = new BXCustomFieldLocalization()).EditFormLabel = property.Name;

						if (property.ViewMode == CatalogProductPropertyListViewMode.List)
							field.Settings.Add("ViewMode", "list");

						field.Create();

						for(int j = 0; j < property.Values.Count; j++)
						{
							BXCustomFieldEnum e = new BXCustomFieldEnum();
							e.FieldId = field.Id;
							e.FieldType = field.FieldType;
							e.Value = property.Values[j];
							e.Sort = 10 * j;
							e.Default = j == 0;
							e.Save();
						}

						if (comparisonSb.Length > 0)
							comparisonSb.Append(";");
						comparisonSb.Append("-").Append(field.CorrectedName);
					}
				}

				if (comparisonSb.Length > 0)
					comparisonSb.Insert(0, ";");

				//Привязка к каталогу
				BXCatalog catalog = new BXCatalog();
				catalog.Id = block.Id;

				BXCatalogVATRateCollection vatRates = BXCatalogVATRate.GetList(
					new BXFilter(new BXFilterItem(BXCatalogVATRate.Fields.Value, BXSqlFilterOperators.Equal, 0m)),
					null,
					null,
					new BXQueryParams(new BXPagingOptions(0, 1))
					);

				if (vatRates.Count > 0)
					catalog.VATRateId = vatRates[0].Id;

				catalog.CheckStockAvailability = true;
				catalog.Create();

				//Создание страницы главной страницы с каталогом			

				BXCatalogPriceTypeCollection priceTypes = BXCatalogPriceType.GetList(
					new BXFilter(new BXFilterItem(BXCatalogPriceType.Fields.IsBase, BXSqlFilterOperators.Equal, true)),
					null,
					new BXSelect(BXSelectFieldPreparationMode.Normal, BXCatalogPriceType.Fields.Id),
					new BXQueryParams(new BXPagingOptions(0, 1)));

				this.catalogUrl = string.Concat("/catalog/", catalogDirectoryName, "/");
				string componentData =
					string.Format(
	@"<bx:IncludeComponent ID=""catalog"" runat=""server"" 
	ComponentName=""bitrix:catalogue"" IBlockTypeId=""{0}"" IBlockId=""{1}"" 
	SetPageTitle=""True"" EnableSEF=""True"" SEFFolder=""{2}""  
	DisplayStockCatalogData=""True"" DisplayAllPriceTiers=""{3}"" PriceTypes=""{4}"" 
	InitQuantity=""1"" AcceptQuantity=""False"" IncludeVATInPrice=""True"" DisplayVAT=""False"" 
	ShowCatalogItemProperties=""SPECIALOFFER;NEWPRODUCT;SALELEADER"" UserCartUrlTemplate=""{5}"" 
	AllowComparison=""{6}"" ComparisonSelectedFields=""PreviewText;DetailImage{7}"" />",
					blockTypeId.ToString(),
					block.Id.ToString(),
					this.catalogUrl,
					(priceTypes.Count == 0).ToString(),
					priceTypes.Count > 0 ? priceTypes[0].Id.ToString() : string.Empty,
					BXOptionManager.GetOptionString("Bitrix.CommerceSite", "UserCartUrl", string.Empty),
					WizardContext.State.GetBool("DisplayComparisonTable", true).ToString(),
					comparisonSb.ToString());

				string pageData = BXDefaultFiles.BuildAspx(catalogName, filePath, string.Empty, componentData);
				BXSecureIO.FileWriteAllText(HostingEnvironment.MapPath(filePath), pageData);

				//Регистрация правила ЧПУ
				BXSefUrlRuleManager.BeginUpdate();

				BXSefUrlRule sefRule = new BXSefUrlRule();
				sefRule.ComponentName = "bitrix:catalogue";
				sefRule.HelperId = string.Concat("bitrix:catalog:", catalogDirectoryName);
				sefRule.MatchExpression = string.Concat(@"^~/catalog/", catalogDirectoryName, @"[\s]*(?:[\\\/\?\#]|$)");
				sefRule.Path = filePath;
				BXSefUrlRuleManager.Add(sefRule);

				BXSefUrlRuleManager.EndUpdate();
				//Сохранение ссылки
				this.catalogUrl = VirtualPathUtility.ToAbsolute(VirtualPathUtility.Combine("~/", catalogUrl));
			}
			catch(Exception exc)
			{
				errorLst.Add(exc.Message);
			}
			return Result.Render(GetMessage("Title"), errorLst);
		}

		private string catalogUrl = VirtualPathUtility.ToAbsolute("~/catalog/");
		public string CatalogUrl
		{
			get { return this.catalogUrl; }
		}
	}

	internal enum CatalogProductPropertyListViewMode
	{
		List = 1,
		Flags
	}

	internal class CatalogProductPropertyInfo
	{
		private string name = string.Empty;
		public string Name
		{
			get { return this.name; }
			set { this.name = value ?? string.Empty; }
		}

		private string type = string.Empty;
		public string Type
		{
			get { return this.type; }
			set { this.type = value ?? string.Empty; }
		}

		private List<string> values = null;
		public IList<string> Values
		{
			get { return this.values ?? (this.values = new List<string>()); }
		}

		private CatalogProductPropertyListViewMode viewMode = CatalogProductPropertyListViewMode.Flags;
		public CatalogProductPropertyListViewMode ViewMode
		{
			get { return this.viewMode; }
			set { this.viewMode = value; }
		}
	}

	internal class CatalogProductPropertyInfoJsonConverter : JavaScriptConverter
	{
		public override object Deserialize(IDictionary<string, object> dictionary, Type type, JavaScriptSerializer serializer)
		{
			if (type != typeof(CatalogProductPropertyInfo))
				return null;

			CatalogProductPropertyInfo r = new CatalogProductPropertyInfo();

			object o;
			if (dictionary.TryGetValue("name", out o))
				r.Name = (string)o;

			if (dictionary.TryGetValue("type", out o))
			{
				string s = (string)o;
				if (string.Equals(s, "list", StringComparison.OrdinalIgnoreCase) || string.Equals(s, "flag", StringComparison.OrdinalIgnoreCase))
				{
					r.Type = "Bitrix.System.Enumeration";
					r.ViewMode = string.Equals(s, "list", StringComparison.OrdinalIgnoreCase) ? CatalogProductPropertyListViewMode.List : CatalogProductPropertyListViewMode.Flags;
				}
				else
					r.Type = "Bitrix.System.Text";
			}

			if (dictionary.TryGetValue("values", out o))
				foreach (object item in ((IEnumerable)o))
					r.Values.Add((string)item);

			return r;
		}

		public override IDictionary<string, object> Serialize(object obj, JavaScriptSerializer serializer)
		{
			throw new NotSupportedException();
		}

		public override IEnumerable<Type> SupportedTypes
		{
			get { return new Type[] { typeof(CatalogProductPropertyInfo) }; }
		}
	}
}