﻿IF OBJECT_ID (N'b_CatalogVat', N'U') IS NULL
BEGIN
	CREATE TABLE b_CatalogVat
	(
		ID INT IDENTITY(1,1) NOT NULL,
		Name NVARCHAR(50) NOT NULL,
		Active BIT NOT NULL,
		Sort INT NOT NULL,
		Value DECIMAL(18,3) NOT NULL,
		CreatedUtc DATETIME NOT NULL,
		LastUpdatedUtc DATETIME NOT NULL,		
		XmlId VARCHAR(256) NULL,
		CONSTRAINT PK_b_CatalogVat PRIMARY KEY CLUSTERED(ID ASC)		
	)
END
GO

IF OBJECT_ID (N'b_Catalog', N'U') IS NULL
BEGIN
	CREATE TABLE b_Catalog
	(
		ID INT NOT NULL,
		ParentId INT NOT NULL,
		CheckStockAvailability BIT NOT NULL,
		VATRateId INT NOT NULL,
		[Enabled] BIT DEFAULT(1) NOT NULL,
		CreatedUtc DATETIME NOT NULL,
		LastUpdatedUtc DATETIME NOT NULL,
		XmlId VARCHAR(256) NULL,
		CONSTRAINT PK_b_Catalog PRIMARY KEY CLUSTERED(ID ASC)			
	)
END
GO

IF OBJECT_ID (N'b_CatalogItem', N'U') IS NULL
BEGIN
	CREATE TABLE b_CatalogItem
	(
		ID INT NOT NULL,
		CatalogId INT NOT NULL,
		VATRateId INT NOT NULL,
		Quantity INT NOT NULL,
		[Weight] DECIMAL(18,2) NOT NULL,
		ProductId INT DEFAULT(0) NOT NULL,
		SKUQuantity INT DEFAULT(0) NOT NULL,
		[Enabled] BIT DEFAULT(1) NOT NULL,
		CreatedUtc DATETIME NOT NULL,
		LastUpdatedUtc DATETIME NOT NULL,
		XmlId VARCHAR(256) NULL,
		CONSTRAINT PK_b_CatalogItem PRIMARY KEY CLUSTERED(ID ASC)		
	)
END
GO

IF OBJECT_ID (N'b_CatalogPriceType', N'U') IS NULL
BEGIN
	CREATE TABLE b_CatalogPriceType
	(
		ID INT IDENTITY(1,1) NOT NULL,
		Code NVARCHAR(100) NOT NULL,
		CurrencyId INT NOT NULL,
		ExtraId INT NOT NULL,
		RoundDecimalPlaces INT NOT NULL,
		IsBase BIT NOT NULL,		
		Sort INT NOT NULL,
		LocalizationXml XML NULL,
		CreatedUtc DATETIME NOT NULL,
		LastUpdatedUtc DATETIME NOT NULL,
		XmlId VARCHAR(256) NULL,
		CONSTRAINT PK_b_CatalogPriceType PRIMARY KEY CLUSTERED(ID ASC)		
	)
END
GO

IF OBJECT_ID (N'b_CatalogPriceType2Role', N'U') IS NULL
BEGIN
	CREATE TABLE b_CatalogPriceType2Role
	(
		PriceTypeId INT NOT NULL,
		RoleId INT NOT NULL,
		RoleName NVARCHAR(128) NOT NULL,
		ViewOnly BIT NOT NULL	
	)
END
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_b_CatalogPriceType2Role')
	CREATE INDEX IX_b_CatalogPriceType2Role ON b_CatalogPriceType2Role(PriceTypeId ASC, RoleId ASC, ViewOnly ASC);
GO


IF OBJECT_ID (N'b_CatalogExtra', N'U') IS NULL
BEGIN
	CREATE TABLE b_CatalogExtra
	(
		ID INT IDENTITY(1,1) NOT NULL,
		Name NVARCHAR(256) NOT NULL,
		Value DECIMAL(18,5) NOT NULL,
		CreatedUtc DATETIME NOT NULL,
		LastUpdatedUtc DATETIME NOT NULL,		
		XmlId VARCHAR(256) NULL,
		CONSTRAINT PK_b_CatalogExtra PRIMARY KEY CLUSTERED(ID ASC)		
	)
END
GO

IF OBJECT_ID (N'b_CatalogPrice', N'U') IS NULL
BEGIN
	CREATE TABLE b_CatalogPrice
	(
		ID INT IDENTITY(1,1) NOT NULL,
		ItemId INT NOT NULL,
		PriceTypeId INT NOT NULL,
		ExtraId INT NOT NULL,
		CurrencyId INT NOT NULL,
		Value DECIMAL(18,4),
		VATIncluded BIT NOT NULL,			
		QuantityFrom INT NOT NULL,
		CreatedUtc DATETIME NOT NULL,
		LastUpdatedUtc DATETIME NOT NULL,
		XmlId VARCHAR(256) NULL,
		CONSTRAINT PK_b_CatalogPrice PRIMARY KEY CLUSTERED(ID ASC)		
	)
END
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_b_CatalogPrice_ItemId_PriceTypeId')
	CREATE INDEX IX_b_CatalogPrice_ItemId_PriceTypeId ON b_CatalogPrice(ItemId ASC, PriceTypeId ASC);
GO

IF OBJECT_ID (N'Catalog_SynchronizePriceType', N'P') IS NOT NULL
	DROP PROCEDURE Catalog_SynchronizePriceType;
GO

CREATE PROCEDURE Catalog_SynchronizePriceType 
	@BasePriceTypeId INT
AS
BEGIN
	SET NOCOUNT ON;
	IF EXISTS(SELECT ID FROM b_CatalogPriceType WHERE ID = @BasePriceTypeId AND IsBase = 1)
	BEGIN
		UPDATE b_CatalogPriceType SET IsBase = 0 WHERE IsBase = 1 AND ID <> @BasePriceTypeId;
	END
END
GO

IF OBJECT_ID(N'Catalog_IsBlockExists', N'P') IS NOT NULL 
	DROP PROCEDURE Catalog_IsBlockExists;
GO
CREATE PROCEDURE Catalog_IsBlockExists
	@IBlockId INT
AS
BEGIN
	SELECT CASE WHEN EXISTS(SELECT TOP 1 ID FROM b_Catalog WHERE ID = @IBlockId) THEN 1 ELSE 0 END;	
END
GO


IF OBJECT_ID(N'CatalogItem_Delete', N'P') IS NOT NULL 
	DROP PROCEDURE CatalogItem_Delete;
GO
CREATE PROCEDURE CatalogItem_Delete
	@CatalogId INT
AS
BEGIN
	DELETE FROM b_CatalogItem WHERE CatalogId = @CatalogId;
END
GO		

IF OBJECT_ID(N'CatalogPrice_Delete', N'P') IS NOT NULL 
	DROP PROCEDURE CatalogPrice_Delete;
GO

CREATE PROCEDURE CatalogPrice_Delete
	@CatalogId INT = 0,
	@ItemId INT
AS
BEGIN
	IF(@CatalogId > 0)
		DELETE FROM b_CatalogPrice WHERE ItemId IN (SELECT ID FROM b_CatalogItem WHERE CatalogId = @CatalogId);
	ELSE
		DELETE FROM b_CatalogPrice WHERE ItemId = @ItemId;		
END
GO

IF OBJECT_ID(N'Catalog_UpdateBalance', N'P') IS NOT NULL 
	DROP PROCEDURE Catalog_UpdateBalance;
GO

CREATE PROCEDURE Catalog_UpdateBalance
@products xml
AS
BEGIN
	SET NOCOUNT ON
	
	CREATE TABLE #products (id INT, quantity INT)
	CREATE TABLE #errors (id INT, quantity INT, name NVARCHAR(255))
	
	insert #products
	select ref.value('@id', 'int'), ref.value('@q', 'int')
	from @products.nodes('//p') as N(ref)
	
	-- проверим что можно изменить количество товаров. 
	-- Изменять его нужно только у каталогов, в которых контролируем остатки.
	
	insert #errors(id, quantity, name)
	SELECT ci.ID, ci.Quantity, ie.Name
	FROM 
		#products p join b_CatalogItem ci on ci.ID = p.id 
		join b_Catalog c on c.ID = ci.CatalogId
		join b_IBlockElement ie on ie.ID = ci.ID
	WHERE
		ci.Quantity < ABS(p.quantity) AND p.quantity < 0 
		AND c.CheckStockAvailability = 1
		
	IF (EXISTS ( SELECT * FROM #errors )) -- если есть товары, у которых количество списать нельзя, 
		SELECT e.name, e.quantity FROM #errors e 		-- выводим ошибки
	
	IF(NOT EXISTS( SELECT * FROM #errors )) BEGIN -- если ошибок нет, изменим количество всем элементам
		UPDATE ci SET Quantity = ci.Quantity + p.quantity
		FROM
			b_CatalogItem ci join #products p on ci.ID = p.id
			join b_Catalog c on c.ID = ci.CatalogId
		WHERE
			c.CheckStockAvailability = 1
			
		UPDATE parents SET 
			SKUQuantity = (SELECT COUNT(ID) FROM b_CatalogItem WHERE ProductId = parents.ID),
			Quantity = ISNULL((SELECT SUM(Quantity) FROM b_CatalogItem WHERE ProductId = parents.ID), 0)
		FROM
			b_CatalogItem ci join #products p on ci.ID = p.id
			join b_CatalogItem parents on parents.ID = ci.ProductId
			join b_Catalog c on c.ID = ci.CatalogId
		
	END
	
	DROP TABLE #products
	DROP TABLE #errors
END
GO
-- BEGINNING OF CML IMPORT
IF OBJECT_ID(N'Catalog_CmlImportPrepareTables', N'P') IS NOT NULL 
	DROP PROCEDURE Catalog_CmlImportPrepareTables;
GO
CREATE PROCEDURE Catalog_CmlImportPrepareTables
AS
BEGIN
	IF OBJECT_ID(N'b_CatalogCmlImport', N'U') IS NULL
	BEGIN
		CREATE TABLE b_CatalogCmlImport
		(
			ID INT IDENTITY(1,1) NOT NULL,
			CONSTRAINT PK_b_CatalogCmlImport PRIMARY KEY CLUSTERED(ID ASC)		
		);	
	END
	
	IF OBJECT_ID(N'b_CatalogCmlImportSection', N'U') IS NULL
	BEGIN
		CREATE TABLE b_CatalogCmlImportSection
		(
			ImportId INT NOT NULL,
			SectionId INT NOT NULL
		);
		CREATE INDEX IX_b_CatalogCmlImportSection ON b_CatalogCmlImportSection (ImportId ASC, SectionId ASC);			
	END
	
	IF OBJECT_ID(N'b_CatalogCmlImportElement', N'U') IS NULL
	BEGIN
		CREATE TABLE b_CatalogCmlImportElement
		(
			ImportId INT NOT NULL,
			ElementId INT NOT NULL
		);
		CREATE INDEX IX_b_CatalogCmlImportElement ON b_CatalogCmlImportElement (ImportId ASC, ElementId ASC);
	END	
END
GO

IF OBJECT_ID(N'Catalog_CmlImportCreate', N'P') IS NOT NULL 
	DROP PROCEDURE Catalog_CmlImportCreate;
GO

CREATE PROCEDURE Catalog_CmlImportCreate
AS
BEGIN
	INSERT INTO b_CatalogCmlImport DEFAULT VALUES;
	SELECT @@IDENTITY;
END
GO

IF OBJECT_ID(N'Catalog_CmlImportDelete', N'P') IS NOT NULL 
	DROP PROCEDURE Catalog_CmlImportDelete;
GO

CREATE PROCEDURE Catalog_CmlImportDelete
	@ImportId INT
AS
BEGIN
	DELETE FROM b_CatalogCmlImportSection WHERE ImportId = @ImportId;
	DELETE FROM b_CatalogCmlImportElement WHERE ImportId = @ImportId;
	DELETE FROM b_CatalogCmlImport WHERE ID = @ImportId;
END
GO


IF OBJECT_ID(N'Catalog_CmlImportAddSection', N'P') IS NOT NULL 
	DROP PROCEDURE Catalog_CmlImportAddSection;
GO

CREATE PROCEDURE Catalog_CmlImportAddSection
	@ImportId INT,
	@SectionId INT
AS
BEGIN
	IF NOT EXISTS(SELECT 'X' FROM b_CatalogCmlImportSection WHERE ImportId = @ImportId AND SectionId = @SectionId)
		INSERT INTO b_CatalogCmlImportSection(ImportId, SectionId) VALUES(@ImportId, @SectionId);
END
GO

IF OBJECT_ID(N'Catalog_CmlImportAddElement', N'P') IS NOT NULL 
	DROP PROCEDURE Catalog_CmlImportAddElement;
GO

CREATE PROCEDURE Catalog_CmlImportAddElement
	@ImportId INT,
	@ElementId INT
AS
BEGIN
	IF NOT EXISTS(SELECT 'X' FROM b_CatalogCmlImportElement WHERE ImportId = @ImportId AND ElementId = @ElementId)
		INSERT INTO b_CatalogCmlImportElement(ImportId, ElementId) VALUES(@ImportId, @ElementId);
END
GO

IF OBJECT_ID(N'Catalog_CmlImportGetOmittedElementIds', N'P') IS NOT NULL 
	DROP PROCEDURE Catalog_CmlImportGetOmittedElementIds;
GO

CREATE PROCEDURE Catalog_CmlImportGetOmittedElementIds
	@ImportId INT,
	@BlockId INT
AS
BEGIN
	SELECT ID FROM b_IBlockElement WHERE IBlockId = @BlockId AND ID NOT IN(SELECT ElementId FROM b_CatalogCmlImportElement WHERE ImportId = @ImportId);	
END
GO

IF OBJECT_ID(N'Catalog_CmlImportGetOmittedSectionIds', N'P') IS NOT NULL 
	DROP PROCEDURE Catalog_CmlImportGetOmittedSectionIds;
GO

CREATE PROCEDURE Catalog_CmlImportGetOmittedSectionIds
	@ImportId INT,
	@BlockId INT
AS
BEGIN
	SELECT ID FROM b_IBlockSection WHERE IBlockId = @BlockId AND ID NOT IN(SELECT SectionId FROM b_CatalogCmlImportSection WHERE ImportId = @ImportId);
END
GO

-- END OF CML IMPORT
-- BEGINNING OF SKU
IF OBJECT_ID (N'Catalog_GetSKUQuantity', N'P') IS NOT NULL
	DROP PROCEDURE Catalog_GetSKUQuantity;
GO

CREATE PROCEDURE Catalog_GetSKUQuantity 
	@Id INT
AS
BEGIN
	SET NOCOUNT ON;
	SELECT SKUQuantity FROM b_CatalogItem WHERE ID = @Id;
END	
GO

IF OBJECT_ID (N'Catalog_SetSkuCatalog', N'P') IS NOT NULL
	DROP PROCEDURE Catalog_SetSkuCatalog;
GO

CREATE PROCEDURE Catalog_SetSkuCatalog 
	@CatalogId INT,
	@SkuCatalogId INT
AS
BEGIN
	SET NOCOUNT ON;
	UPDATE b_CatalogItem SET [Enabled] = 0 WHERE CatalogId IN (SELECT ID FROM b_Catalog WHERE ParentId = @CatalogId AND ID <> @SkuCatalogId);
	UPDATE b_Catalog SET [Enabled] = 0 WHERE ParentId = @CatalogId AND ID <> @SkuCatalogId;	
	UPDATE b_CatalogItem SET [Enabled] = 1 WHERE CatalogId = @SkuCatalogId;
	UPDATE b_Catalog SET ParentId = @CatalogId, [Enabled] = 1 WHERE ID = @SkuCatalogId;		
END
GO

IF OBJECT_ID (N'Catalog_SetProductId', N'P') IS NOT NULL
	DROP PROCEDURE Catalog_SetProductId;
GO

CREATE PROCEDURE Catalog_SetProductId 
	@SkuId INT,
	@ProductId INT
AS
BEGIN
	SET NOCOUNT ON;
	DECLARE @PrevProductId INT;
	IF NOT EXISTS(SELECT ID FROM b_CatalogItem WHERE ID = @SkuId) RETURN;
	SELECT @PrevProductId = ProductId FROM b_CatalogItem WHERE ID = @SkuId;
	
	UPDATE b_CatalogItem SET ProductId = @ProductId WHERE ID = @SkuId;
	IF(@ProductId > 0)
		UPDATE b_CatalogItem SET VATRateId = (SELECT VATRateId FROM b_CatalogItem WHERE ID = @ProductId) WHERE ID = @SkuId;

	IF(@PrevProductId > 0)
		UPDATE b_CatalogItem SET 
			SKUQuantity = (SELECT COUNT(ID) FROM b_CatalogItem WHERE ProductId = @PrevProductId),
			Quantity = ISNULL((SELECT SUM(Quantity) FROM b_CatalogItem WHERE ProductId = @PrevProductId), 0) WHERE ID = @PrevProductId;
	
	IF(@ProductId > 0)
		UPDATE b_CatalogItem SET 
			SKUQuantity = (SELECT COUNT(ID) FROM b_CatalogItem WHERE ProductId = @ProductId),
			Quantity = ISNULL((SELECT SUM(Quantity) FROM b_CatalogItem WHERE ProductId = @ProductId), 0) WHERE ID = @ProductId;	
END
GO

IF OBJECT_ID (N'Catalog_SynchronizeProduct', N'P') IS NOT NULL
	DROP PROCEDURE Catalog_SynchronizeProduct;
GO

CREATE PROCEDURE Catalog_SynchronizeProduct
	@ProductId INT
AS
BEGIN
	UPDATE b_CatalogItem SET 
		SKUQuantity = (SELECT COUNT(ID) FROM b_CatalogItem WHERE ProductId = @ProductId),
		Quantity = ISNULL((SELECT SUM(Quantity) FROM b_CatalogItem WHERE ProductId = @ProductId), 0) WHERE ID = @ProductId;		
END
GO
-- END OF SKU
--DISCOUNTS
IF OBJECT_ID (N'b_CatalogDiscount', N'U') IS NOT NULL
	DROP TABLE b_CatalogDiscount
GO	
CREATE TABLE b_CatalogDiscount(
	[Id] [int] IDENTITY(1,1) NOT NULL,
	[CurrencyId] [int] NOT NULL,
	[Value] [decimal](18, 2) NOT NULL,
	[IsPercent] [bit] NOT NULL,
	[Active] [bit] NOT NULL,
	[ActiveFromUtc] [datetime] NULL,
	[ActiveToUtc] [datetime] NULL,
	[UserRoleIds] [nvarchar](max) NULL,
	[CatalogIds] [varchar](max) NULL,
	[CatalogSectionIds] [varchar](max) NULL,
	[Filter] [xml] NOT NULL,
	[Name] [nvarchar](50) NULL,
	[SiteId] [varchar](50) NOT NULL,
	[Sort] [int] NOT NULL,
	[Description] [nvarchar](max) NULL,
	[MaxSum] [decimal](18, 2) NULL,
	[PriceTypeIds] [varchar](max) NULL,
 CONSTRAINT [PK_b_CatalogDiscount] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]

