﻿using System;
using System.Text;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.ObjectModel;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.UI;
using Bitrix.Services;
using Bitrix.Main;
using System.Collections.Generic;
using Bitrix;
using Bitrix.Security;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix.Catalog;
using Bitrix.Currency;
using Bitrix.DataTypes;
using System.Xml;
using System.IO;
using System.Globalization;

public partial class bitrix_admin_CatalogDiscountEdit : Bitrix.UI.BXAdminPage
{
	private bool currentUserCanModify = false;

	BXCatalogDiscount discount;

	List<string> PageErrors;

	protected void Page_Init(object sender, EventArgs e)
	{
		if (!this.BXUser.IsCanOperate(BXRoleOperation.Operations.ProductSettingsView))
			BXAuthentication.AuthenticationRequired();

		//currentUserCanModify = this.BXUser.IsCanOperate(BXRoleOperation.Operations.ProductSettingsManage);
		//AddButton.Visible = currentUserCanModify;

		sites = BXSite.GetList(new BXFilter(new BXFilterItem(BXSite.Fields.Active, BXSqlFilterOperators.Equal, true)),
			new BXOrderBy(new BXOrderByPair(BXSite.Fields.Sort, BXOrderByDirection.Asc)), null, null, BXTextEncoder.EmptyTextEncoder);

		currencies = BXCurrency.GetList(null, new BXOrderBy(new BXOrderByPair(BXCurrency.Fields.Iso4217AlphaCode, BXOrderByDirection.Asc)));

		priceTypes = BXCatalogPriceType.GetList(null, new BXOrderBy(new BXOrderByPair(BXCatalogPriceType.Fields.Sort, BXOrderByDirection.Asc)));

		userRoles = BXRoleManager.GetList(
			new BXFormFilter(new BXFormFilterItem("Active", true, BXSqlFilterOperators.Equal)),
			new BXOrderBy_old("RoleName", "Asc")
		);

		userRoles.Sort(delegate(BXRole a, BXRole b)
		{
			int i = RoleClass(a).CompareTo(RoleClass(b));
			return i != 0 ? i : string.Compare(a.Title, b.Title, true);
		});

		foreach (var site in sites)
			Site.Items.Add(new ListItem() { Text = site.Name, Value = site.Id });

		foreach (var currency in currencies)
			Currency.Items.Add(new ListItem() { Value = currency.Id.ToString(), Text = currency.Iso4217AlphaCode });

		foreach(var type in priceTypes)
			PriceTypes.Items.Add(new ListItem() { 
				Text = String.Format("[{0}] {1} ({2})",type.Id.ToString(), type.Code, type.GetLocalizedName()), Value = type.Id.ToString() });

		foreach (var role in userRoles)
			UserRoles.Items.Add(new ListItem()
			{
				Text = String.Format("[{0}] {1}", role.RoleId.ToString(), role.Title),
				Value = role.RoleId.ToString()
			});

		DiscountType.Items.Add(new ListItem() { Value = "percent", Text = GetMessage("DiscountType.Percent") });
		DiscountType.Items.Add(new ListItem() { Value = "sum", Text = GetMessage("DiscountType.Sum") });
	}

	public string VariantsJSON {get;set;}

	public string DefaultSelectLabel { get; set; }
	
	public string DefaultItemSelectLabel { get; set; }

	BXSiteCollection sites;

	public BXSiteCollection Sites
	{
		get { return sites ?? (sites = new BXSiteCollection()); }
	}

	BXCurrencyCollection currencies;
	BXCatalogPriceTypeCollection priceTypes;
	BXRoleCollection userRoles;


	protected void OnEditCommand(object sender, BXTabControlCommandEventArgs e)
	{
		if(e.CommandName == "apply" || e.CommandName == "save")
		{
			var tree = Request.Form[hdDiscountTree.UniqueID];
			
			var root = BXDiscountGUIHelper.ParseRuleFromClient(tree, BXLoc.CurrentLocale, true);

			var ruleString = String.Empty;
			if(root!=null)
				ruleString = BXDiscountGUIHelper.GUITreeToDiscountTree(root);

			if(SaveDiscount(ruleString) && editorError == CatalogDiscountEditorError.None)
			{
				if(e.CommandName == "save")
					GoBack();
				else
					Response.Redirect(TabControl.SelectedIndex > 0 ? string.Format("CatalogDiscountEdit.aspx?id={0}&tabindex={1}", discount.Id.ToString(), TabControl.SelectedIndex) : string.Format("CatalogDiscountEdit.aspx?id={0}", discount.Id.ToString()));
			}
		}
	}

	protected void OnToolBarButtonClick(object sender, CommandEventArgs e)
	{
		if (e.CommandName == "delete")
		{
			try
			{
                if (discount != null)
                    discount.Delete();

				GoBack();
			}
			catch (Exception ex)
			{
                this.errorText = ex.Message;
				editorError = CatalogDiscountEditorError.DeletionFailed;
			}
		}
	}

    protected override void OnPreRender(EventArgs e)
    {
        if (EditorError != CatalogDiscountEditorError.None)
        {
            Summary.AddErrorMessage(ErrorText);
            if (EditorMode == CatalogDiscountEditorMode.Modification && EditorError ==  CatalogDiscountEditorError.IsNotFound)
                TabControl.Visible = false;
        }
        base.OnPreRender(e);
    }

    private string errorText = string.Empty;
    public string ErrorText
    {
        get { return errorText; }
    }

	public BXCatalogDiscount Discount
	{
		get { return discount; }
	}

    private CatalogDiscountEditorMode editorMode = CatalogDiscountEditorMode.Creation;
    protected CatalogDiscountEditorMode EditorMode
    {
        get { return this.editorMode; }
    }

    private CatalogDiscountEditorError editorError = CatalogDiscountEditorError.None;
    protected CatalogDiscountEditorError EditorError
    {
        get { return this.editorError; }
    }

	protected void Page_Load(object sender, EventArgs e)
	{
		Page.ClientScript.RegisterClientScriptInclude("discountrules", "../js/Catalog/discountrules.js");
		var script = "BX('" + hdDiscountTree.ClientID + "').value = Bitrix.treeProcessor.SerializeTree(state, null, null, 0);";
		Page.ClientScript.RegisterOnSubmitStatement(GetType(), "submittree", script);

		VariantsJSON = BXDiscountConfiguration.ConditionVariantsToJSON(BXLoc.CurrentLocale, BXDiscountConditionVariantType.Catalog);

		DefaultSelectLabel = BXLoc.GetModuleMessage("catalog", "CatalogDiscount.AdminMessage.DefaultSelectLabel");
		DefaultItemSelectLabel = BXLoc.GetModuleMessage("catalog", "CatalogDiscount.AdminMessage.DefaultItemSelectLabel");

		LoadDiscount();

        string title = editorMode == CatalogDiscountEditorMode.Modification ? 
			string.Format(GetMessage("PageTitle.Edit"), discount != null ? 
			discount.Name : "?") : GetMessage("PageTitle.Create");

		if (EditorMode == CatalogDiscountEditorMode.Creation)
		{
			AddButton.Visible = false;
			DeleteButton.Visible = false;
		}
        MasterTitle = Page.Title = title;
	}

	public string DiscountJSON 
	{
		get
		{
			return discount != null && !String.IsNullOrEmpty(discount.Filter) ? BXDiscountGUIHelper.DiscountTreeToJSON2(discount.Filter, true) : 
				BXDiscountGUIHelper.DefaultRuleJSON(true);
		} 
	}

	static int RoleClass(BXRole role)
	{
		switch (role.RoleId)
		{
			case 2:
				return 0;
			case 3:
				return 1;
			case 1:
				return 2;
			default:
				return 3;
		}
	}

	void LoadDiscount()
	{
		int id;
		if (!int.TryParse(Request.QueryString["id"], out id))
			editorMode = CatalogDiscountEditorMode.Creation;
		else
		{
			editorMode = CatalogDiscountEditorMode.Modification;
			discount = BXCatalogDiscount.GetById(id);
			if(discount == null) 
			{
				editorError = CatalogDiscountEditorError.IsNotFound;
				return;
			}

			if (!IsPostBack)
			{
				Name.Text = discount.TextEncoder.Decode(discount.Name);
				Currency.SelectedIndex = currencies.FindIndex(x => x.Id == discount.CurrencyId);
				Site.SelectedIndex = sites.FindIndex(x => x.Id == discount.SiteId);
				Sort.Text = discount.Sort.ToString();
				if (discount.ActiveFromUtc != DateTime.MinValue)
					tiActive.StartDate = discount.ActiveFrom;
				if (discount.ActiveToUtc != DateTime.MinValue)
					tiActive.EndDate = discount.ActiveTo;

				DiscountType.SelectedIndex = discount.IsPercent ? 0 : 1;
				Description.Text = discount.TextEncoder.Decode(discount.Description) ?? "";
				Value.Text = discount.Value.ToString();
				MaxSum.Text = discount.MaxSum == 0 ? "0" : discount.MaxSum.ToString();
				Active.Checked = discount.Active;

				ListItem item;
				foreach (var type in discount.PriceTypesList)
				{
					item = PriceTypes.Items.FindByValue(type.ToString());
					if (item != null)
						item.Selected = true;
				}

				foreach (var role in discount.UserRolesList)
				{
					item = UserRoles.Items.FindByValue(role.ToString());
					if (item != null)
						item.Selected = true;
				}
			}
		}
	}

	private bool TryParseDecimal(string s, out decimal m)
	{	
		if(string.IsNullOrEmpty(s))
		{
			m = 0m;
			return false;
		}
		
		return decimal.TryParse(s, NumberStyles.AllowDecimalPoint, CultureInfo.CurrentCulture.NumberFormat, out m) 
			|| decimal.TryParse(s, NumberStyles.AllowDecimalPoint, CultureInfo.InvariantCulture.NumberFormat, out m);
	}
	
	bool SaveDiscount(string ruleString)
	{
		try
		{
			if (discount == null)
				discount = new BXCatalogDiscount();
				
			int intValue;
			decimal dValue;
			if (int.TryParse(Currency.SelectedValue, out intValue))
				discount.CurrencyId = intValue;

			discount.Active = Active.Checked;
			discount.IsPercent = DiscountType.SelectedValue == "percent";
			discount.Name = Name.Text;
			discount.Description = Description.Text;
			discount.SiteId = Site.SelectedValue;
			if (int.TryParse(Sort.Text, out intValue))
				discount.Sort = intValue;

			if (TryParseDecimal(Value.Text, out dValue))
				discount.Value = discount.IsPercent && dValue > 100m ? 100m : dValue;
			
			if (TryParseDecimal(MaxSum.Text, out dValue))
				discount.MaxSum = dValue;
				
			if (tiActive.StartDate != DateTime.MinValue)
				discount.ActiveFrom = tiActive.StartDate;

			if (tiActive.EndDate != DateTime.MaxValue)
				discount.ActiveTo = tiActive.EndDate;

			discount.PriceTypesList.Clear();
			
			foreach (ListItem item in PriceTypes.Items)
				if(item.Selected && int.TryParse(item.Value, out intValue))
					discount.PriceTypesList.Add(intValue);
			discount.UserRolesList.Clear();
			foreach (ListItem item in UserRoles.Items)
				if (item.Selected && int.TryParse(item.Value, out intValue))
					discount.UserRolesList.Add(intValue);

			discount.Filter = ruleString;
			discount.Save();

			return true;
		}
		catch(Exception exc)
		{
			errorText = exc.Message;
			editorError = editorMode == CatalogDiscountEditorMode.Creation ? CatalogDiscountEditorError.CreationFailed : CatalogDiscountEditorError.ModificationFailed;
			return false;
		}
	}

	protected string siteId = BXSite.Current.Id;
	protected string currencyCode = BXCurrency.NationalCurrency.Iso4217AlphaCode;

	protected override string BackUrl
	{
		get { return !String.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "CatalogDiscountList.aspx"; }
	}
}

public enum CatalogDiscountEditorMode
{
    Creation = 1,
    Modification = 2
}

public enum CatalogDiscountEditorError
{
    None = 0,
    IsNotFound = -1,
    CreationFailed = -2,
    ModificationFailed = -3,
    DeletionFailed = -4
}