﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using Bitrix.UI;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix;
using System.Text;
using Bitrix.Security;
using Bitrix.Catalog;
using Bitrix.Currency;

public partial class bitrix_admin_CatalogDiscountList : BXAdminPage
{
    protected override void OnPreInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXCatalog.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnPreInit(e);
    }

	protected override void OnInit(EventArgs e)
	{
		base.OnInit(e);

		var sites = BXSite.GetList(null, new BXOrderBy(new BXOrderByPair(BXSite.Fields.Name, BXOrderByDirection.Asc)), null, null, BXTextEncoder.EmptyTextEncoder);
		SiteFilter.Values.Add(new ListItem(GetMessageRaw("Kernel.All"), ""));
		foreach (var site in sites)
			SiteFilter.Values.Add(new ListItem() { Text = String.Format("[{0}] {1}",site.Id, site.Name), Value = site.Id });
	}

    protected override void OnLoad(EventArgs e)
    {
        MasterTitle = GetMessage("PageTitle");
        Page.Title = GetMessage("PageTitle");

		

        base.OnLoad(e);
    }

    protected void ItemGrid_Select(object sender, BXSelectEventArgs e)
	{
		if (!BXPrincipal.Current.IsCanOperate(BXCatalog.Operations.AdminManagement))
            return;
		List<DiscountWrapper> list = new List<DiscountWrapper>();
		BXFilter f = new BXFilter(ItemFilter.CurrentFilter, BXCatalogDiscount.Fields);

		BXCatalogDiscountCollection dList = BXCatalogDiscount.GetList(
			f,
			new BXOrderBy(BXCatalogDiscount.Fields, string.IsNullOrEmpty(e.SortExpression) ? "Id" : e.SortExpression),
			new BXSelectAdd(BXCatalogDiscount.Fields.Site,
							BXCatalogDiscount.Fields.Currency),
			new BXQueryParams(e.PagingOptions),
			BXTextEncoder.EmptyTextEncoder
		);

		foreach (var item in dList)
			list.Add(new DiscountWrapper(item, Page as BXAdminPage));
		e.Data = list;

	}


    protected void ItemGrid_SelectCount(object sender, BXSelectCountEventArgs e)
	{
		e.Count = BXCatalogDiscount.Count(new BXFilter(ItemFilter.CurrentFilter, BXCatalogDiscount.Fields));
	}

    protected void ItemGrid_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType != DataControlRowType.DataRow)
            return;

		BXGridViewRow row = (BXGridViewRow)e.Row;
		DiscountWrapper discount = (DiscountWrapper)row.DataItem;

		row.UserData.Add("ID", discount.Discount.Id);
    }


    protected void ItemGrid_Delete(object sender, BXDeleteEventArgs e)
	{
        if (!BXPrincipal.Current.IsCanOperate(BXCatalog.Operations.AdminManagement))
            return;

		BXGridView grid = (BXGridView)sender;
		BXCatalogDiscountCollection discounts;
		try
		{
			BXFilter filter = (e.Keys == null)
				? new BXFilter(ItemFilter.CurrentFilter, BXCatalogDiscount.Fields)
				: new BXFilter(new BXFilterItem(BXCatalogDiscount.Fields.Id, BXSqlFilterOperators.Equal, e.Keys["ID"]));
			discounts = BXCatalogDiscount.GetList(filter, null);
		}
		catch (Exception ex)
		{
			ErrorMessage.AddErrorMessage(ex.Message);
			return;
		}
		bool errorTextAdded = false;
		foreach (BXCatalogDiscount d in discounts)
		{
			try
			{
				d.Delete();
				e.DeletedCount++;
			}
			catch (Exception ex2)
			{
				if (!errorTextAdded)
				{
					ErrorMessage.AddErrorMessage(GetMessage("DeleteFailure"));
					errorTextAdded = true;
				}
			}
		}
	}

	public class DiscountWrapper
	{
		BXCatalogDiscount discount;
		BXAdminPage page;

		public DiscountWrapper(BXCatalogDiscount discount, BXAdminPage page)
		{
			if (discount == null)
				throw new ArgumentNullException("discount");
			if (page == null)
				throw new ArgumentNullException("page");
			this.discount = discount;
			this.page = page;
		}

		public BXCatalogDiscount Discount
		{
			get { return discount; }
		}

		public int ID
		{
			get { return discount.Id; }
		}

		public string Name
		{
			get { return discount.Name; }
		}

		public bool Active
		{
			get { return discount.Active; }
		}

		public DateTime ActiveFrom
		{
			get { return discount.ActiveFrom; }
		}

		public DateTime ActiveTo
		{
			get { return discount.ActiveTo; }
		}

		public string ActiveFromString
		{
			get { return discount.ActiveFromUtc!= DateTime.MinValue ? discount.ActiveFrom.ToString("g") : String.Empty; }
		}

		public string ActiveToString
		{
			get { return discount.ActiveToUtc != DateTime.MaxValue ? discount.ActiveTo.ToString("g") : String.Empty; }
		}

		public int Sort
		{
			get { return discount.Sort; }
		}

		public string ValueString
		{
			get { return discount.IsPercent ? String.Concat(discount.Value.ToString() + " %") : discount.Currency.MoneyToString(discount.Value); }
		}

		public bool IsPercent
		{
			get { return discount.IsPercent; }
		}

		public string SiteName
		{
			get { return discount.Site.Name; }
		}

		public string ActiveString
		{
			get { return page.GetMessage(this.Active ? "Kernel.Yes" : "Kernel.No"); }
		}
	}

}
