﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.UI;
using Bitrix.Services.Text;
using Bitrix;
using Bitrix.DataLayer;
using Bitrix.Modules;
using System.Collections.Generic;
using Bitrix.Security;
using Bitrix.Configuration;
using Bitrix.Catalog;
using System.Globalization;

public enum CatalogExtraEditorMode
{
    Creation = 1,
    Modification = 2
}

public enum CatalogExtraEditorError
{
    None = 0,
    IsNotFound = -1,
    CreationFailed = -2,
    ModificationFailed = -3,
    DeletionFailed = -4
}

public partial class bitrix_admin_CatalogExtraEdit : BXAdminPage
{
	private int? entityId = null;
	protected int EntityId
	{
        get 
        {
            if (this.entityId.HasValue)
                return this.entityId.Value;

            string s = Request.QueryString["id"];
            if (string.IsNullOrEmpty(s))
                this.entityId = 0;
            else
            {
                int i;
                this.entityId = int.TryParse(s, out i) ? i : 0;
            }
            return this.entityId.Value; 
        }
	}

    private CatalogExtraEditorMode editorMode = CatalogExtraEditorMode.Creation;
    protected CatalogExtraEditorMode EditorMode
    {
        get { return this.editorMode; }
    }

    private BXCatalogExtra entity;
    protected BXCatalogExtra Entity
    {
        get { return this.entity; }
    }

    private CatalogExtraEditorError editorError = CatalogExtraEditorError.None;
    protected CatalogExtraEditorError EditorError
    {
        get { return this.editorError; }
    }

    private string errorText = string.Empty;
    public string ErrorText
    {
        get { return this.errorText; }
    }

	protected override string BackUrl
	{
		get { return !string.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "CatalogExtraList.aspx"; }
	}

    protected override void OnInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXCatalog.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnInit(e);
    }

    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);

        TryLoadEntity();
        BXCatalogExtra entity = Entity;
        string title = EditorMode == CatalogExtraEditorMode.Modification ? string.Format(GetMessage("PageTitle.Edit"), entity != null ? HttpUtility.HtmlEncode(entity.Name) : "?") : GetMessage("PageTitle.Create");

        if (EditorMode == CatalogExtraEditorMode.Creation)
		{
			AddButton.Visible = false;
			DeleteButton.Visible = false;
		}
        MasterTitle = Page.Title = title;
    }

    private void TryLoadEntity()
	{
        int id = EntityId;
        if (id <= 0)
        {
            this.editorMode = CatalogExtraEditorMode.Creation;
            this.entity = new BXCatalogExtra();
        }
        else
        {
            this.editorMode = CatalogExtraEditorMode.Modification;
            if ((this.entity = BXCatalogExtra.GetById(id, BXTextEncoder.EmptyTextEncoder)) == null)
            {
                this.errorText = string.Format(GetMessageRaw("Error.UnableToFindEntity"), id);
                this.editorError = CatalogExtraEditorError.IsNotFound;
                return;
            }
        }

        if (!IsPostBack)
        {
            Name.Text = this.entity.Name;
            Value.Text = this.entity.Value.ToString("F4", CultureInfo.InvariantCulture.NumberFormat);
            XmlId.Text = this.entity.XmlId;
        }
        else
        {
            this.entity.Name = Name.Text;
            decimal m;

			NumberFormatInfo fi = (NumberFormatInfo)CultureInfo.InvariantCulture.NumberFormat.Clone();
			fi.NumberDecimalSeparator = ".";
			fi.NumberGroupSeparator = string.Empty;
			if(!decimal.TryParse(Value.Text, NumberStyles.Number, fi, out m))
			{
				fi.NumberDecimalSeparator = ",";
				decimal.TryParse(Value.Text, NumberStyles.Number, fi, out m);
			}

            this.entity.Value = m;
            this.entity.XmlId = XmlId.Text;
        }
	}

    private void TrySaveEntity()
    {
        if (this.entity == null)
            return;

        try
        {
            this.entity.Save();
            this.entityId = this.entity.Id;
        }
        catch (Exception exc)
        {
            this.errorText= exc.Message;
            this.editorError = EditorMode == CatalogExtraEditorMode.Creation ? CatalogExtraEditorError.CreationFailed : CatalogExtraEditorError.ModificationFailed; 
        }
    }

    protected override void OnPreRender(EventArgs e)
    {
        if (EditorError != CatalogExtraEditorError.None)
        {
            Summary.AddErrorMessage(ErrorText);
            if (EditorMode == CatalogExtraEditorMode.Modification && EditorError == CatalogExtraEditorError.IsNotFound)
                TabView.Visible = false;
        }
        base.OnPreRender(e);
    }

    protected void OnCatalogExtraEdit(object sender, BXTabControlCommandEventArgs e)
	{
		switch (e.CommandName)
		{
            case "save":
                {
                    if (IsValid)
                    {
                        TrySaveEntity();
                        if (EditorError == CatalogExtraEditorError.None)
                            GoBack(); 
                    }
                }
                break;
            case "apply":
                {
                    if (IsValid)
                    {
                        TrySaveEntity();
                        if (EditorError == CatalogExtraEditorError.None)
                            Response.Redirect(string.Format("CatalogExtraEdit.aspx?id={0}&tabindex={1}", EntityId.ToString(), TabView.SelectedIndex));
                    }
                }
                break;
			case "cancel":
				GoBack();
				break;
		}
	}

	protected void OnToolBarButtonClick(object sender, CommandEventArgs e)
	{
		if (e.CommandName == "delete")
		{
			try
			{
                BXCatalogExtra entity = Entity;
                if (entity  != null)
                    entity.Delete();

				GoBack();
			}
			catch (Exception ex)
			{
                this.errorText = ex.Message;
                this.editorError = CatalogExtraEditorError.DeletionFailed;
			}
		}
	}

    protected void MoveSelectedUp(ListItemCollection items)
    {
        var selItems = new List<ListItem>();
        for (int i = items.Count - 1; i >= 0; i--)
        {
            if (items[i].Selected)
            {
                selItems.Add(items[i]);
                items.RemoveAt(i);
            }
        }
        foreach (var i in selItems)
            items.Insert(0, i);
    }
}
