﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using Bitrix.UI;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix;
using System.Text;
using Bitrix.Security;
using Bitrix.Catalog;
using Bitrix.Main;
using System.Globalization;

public partial class bitrix_admin_CatalogExtraList : BXAdminPage
{

    protected override void OnPreInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXCatalog.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnPreInit(e);
    }

    protected override void OnLoad(EventArgs e)
    {
        ((BXAdminMasterPage)Master).PageIconId = "catalog_page_icon";
        MasterTitle = Page.Title = GetMessage("PageTitle");
        base.OnLoad(e);
    }

    protected void ItemGrid_Select(object sender, BXSelectEventArgs e)
	{
        BXCatalogExtraCollection c = BXCatalogExtra.GetList(
            new BXFilter(ItemFilter.CurrentFilter, BXCatalogExtra.Fields),
            new BXOrderBy(BXCatalogExtra.Fields, string.IsNullOrEmpty(e.SortExpression) ? "Id" : e.SortExpression),
            new BXSelect(
                BXSelectFieldPreparationMode.Normal,
                BXCatalogExtra.Fields.Id,
                BXCatalogExtra.Fields.Name,
                BXCatalogExtra.Fields.Value
                ),
            null,
            BXTextEncoder.EmptyTextEncoder
            );

        List<CatalogExtraWrapper> list = new List<CatalogExtraWrapper>();
        foreach (BXCatalogExtra item in c)
            list.Add(new CatalogExtraWrapper(item, this));

        e.Data = list;
	}


    protected void ItemGrid_SelectCount(object sender, BXSelectCountEventArgs e)
	{
        e.Count = BXCatalogExtra.Count(new BXFilter(ItemFilter.CurrentFilter, BXCatalogExtra.Fields));
	}

    protected void ItemGrid_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType != DataControlRowType.DataRow)
            return;

        BXGridViewRow row = (BXGridViewRow)e.Row;
        row.UserData.Add("ID", ((CatalogExtraWrapper)row.DataItem).ID);
    }


    protected void ItemGrid_Delete(object sender, BXDeleteEventArgs e)
	{
        if (!BXPrincipal.Current.IsCanOperate(BXCatalog.Operations.AdminManagement))
            return;

		try
		{
	        BXCatalogExtra.Delete(e.Keys["ID"]);
			e.DeletedCount++;
		}
		catch (Exception ex)
		{
			ErrorMessage.AddErrorMessage(ex.Message);
		}
	}
}
/// <summary>
/// Обёртка
/// </summary>
public class CatalogExtraWrapper
{
    BXCatalogExtra charge;
    BXAdminPage parentPage;
    public CatalogExtraWrapper(BXCatalogExtra charge, BXAdminPage parentPage)
    {
        if (charge == null)
            throw new ArgumentNullException("charge");

        if (parentPage == null)
            throw new ArgumentNullException("parentPage");

        this.charge = charge;
        this.parentPage = parentPage;
    }

    public string ID
    {
        get { return this.charge.Id.ToString(); }
    }

    public string Name
    {
        get { return HttpUtility.HtmlEncode(this.charge.Name); }
    }

    public string Value
    {
        get { return this.charge.Value.ToString("F4", CultureInfo.InvariantCulture.NumberFormat); }
    }
}
