﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.UI;
using Bitrix.Services.Text;
using Bitrix;
using Bitrix.DataLayer;
using Bitrix.Modules;
using System.Collections.Generic;
using Bitrix.Security;
using Bitrix.Configuration;
using Bitrix.Catalog;
using System.Globalization;
using Bitrix.Currency;
using Bitrix.Services;
using System.Collections.Specialized;

public enum CatalogPriceTypeEditorMode
{
    Creation = 1,
    Modification = 2
}

public enum CatalogPriceTypeEditorError
{
    None = 0,
    IsNotFound = -1,
    CreationFailed = -2,
    ModificationFailed = -3,
    DeletionFailed = -4
}

public partial class bitrix_admin_CatalogPriceTypeEditor : BXAdminPage
{
	private int? entityId = null;
	protected int EntityId
	{
        get 
        {
            if (this.entityId.HasValue)
                return this.entityId.Value;

            string s = Request.QueryString["id"];
            if (string.IsNullOrEmpty(s))
                this.entityId = 0;
            else
            {
                int i;
                this.entityId = int.TryParse(s, out i) ? i : 0;
            }
            return this.entityId.Value; 
        }
	}

    private CatalogPriceTypeEditorMode editorMode = CatalogPriceTypeEditorMode.Creation;
    protected CatalogPriceTypeEditorMode EditorMode
    {
        get { return this.editorMode; }
    }

    private BXCatalogPriceType entity;
    protected BXCatalogPriceType Entity
    {
        get { return this.entity; }
    }

    private CatalogPriceTypeEditorError editorError = CatalogPriceTypeEditorError.None;
    protected CatalogPriceTypeEditorError EditorError
    {
        get { return this.editorError; }
    }

    private string errorText = string.Empty;
    public string ErrorText
    {
        get { return this.errorText; }
    }

	protected override string BackUrl
	{
		get { return !string.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "CatalogPriceTypeList.aspx"; }
	}

    public bool IsBase
    {
        get { return this.entity != null ? this.entity.IsBase : false; }
    }


    private List<LocalizationEditor> locEdList = new List<LocalizationEditor>();

    protected override void OnInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXCatalog.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnInit(e);
        foreach(string locale in BXLoc.Locales)
        {
            HtmlGenericControl r = new HtmlGenericControl("TR");
            LocalizationContainer.Controls.Add(r);
            HtmlGenericControl c1 = new HtmlGenericControl("TD");
            r.Controls.Add(c1);
            c1.Attributes.Add("class", "field-name");
            c1.InnerHtml = string.Format(GetMessageRaw("Localization.Title"), locale);
            HtmlGenericControl c2 = new HtmlGenericControl("TD");
            r.Controls.Add(c2);
            LocalizationEditor locEd = new LocalizationEditor();
            locEd.ID = locEd.LocaleName = locale;
            c2.Controls.Add(locEd);
            locEdList.Add(locEd);
        }

        foreach (BXRole role in BXRoleManager.GetList(null, new BXOrderBy_old("RoleName", "Asc")))
        {
            RolesAllowedToView.Items.Add(new ListItem(role.Title, role.RoleId.ToString()));
            AssignedToRoles.Items.Add(new ListItem(role.Title, role.RoleId.ToString()));
        }

        BXCurrencyCollection currencies = BXCurrency.GetList(
            null,
            new BXOrderBy(
				new BXOrderByPair(BXCurrency.Fields.Sort, BXOrderByDirection.Asc),
				new BXOrderByPair(BXCurrency.Fields.Id, BXOrderByDirection.Asc)
				),
            new BXSelect(BXSelectFieldPreparationMode.Normal, BXCurrency.Fields.Id, BXCurrency.Fields.LocalizationInfos, BXCurrency.Fields.IsNational),
            null,
            BXTextEncoder.EmptyTextEncoder
            );

        foreach (BXCurrency currency in currencies)
        {
            ListItem item = new ListItem(currency.GetName(), currency.Id.ToString());
            CurrencyList.Items.Add(item);
            item.Selected = currency.IsNational;
        }

        BXCatalogExtraCollection extras = BXCatalogExtra.GetList(
            null, 
            new BXOrderBy(new BXOrderByPair(BXCatalogExtra.Fields.Name, BXOrderByDirection.Asc)), 
            new BXSelect(BXSelectFieldPreparationMode.Normal, BXCatalogExtra.Fields.Id, BXCatalogExtra.Fields.Name), 
            null, 
            BXTextEncoder.EmptyTextEncoder
            );

        ExtraList.Items.Add(new ListItem(GetMessageRaw("NotSelectedFeminine"), "0"));
        foreach (BXCatalogExtra extra in extras)
            ExtraList.Items.Add(new ListItem(extra.Name, extra.Id.ToString()));
        ExtraList.SelectedIndex = 0;

        ((BXAdminMasterPage)Master).PageIconId = "catalog_page_icon";
    }

    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);

        TryLoadEntity();
        BXCatalogPriceType entity = Entity;
        string title = EditorMode == CatalogPriceTypeEditorMode.Modification ? string.Format(GetMessage("PageTitle.Edit"), entity != null ? HttpUtility.HtmlEncode(entity.Code) : "?") : GetMessage("PageTitle.Create");

        if (EditorMode == CatalogPriceTypeEditorMode.Creation)
		{
			AddButton.Visible = false;
			DeleteButton.Visible = false;
		}
        MasterTitle = Page.Title = title;
    }

    private void TryLoadEntity()
	{
        int id = EntityId;
        if (id <= 0)
        {
            this.editorMode = CatalogPriceTypeEditorMode.Creation;
            this.entity = new BXCatalogPriceType();
        }
        else
        {
            this.editorMode = CatalogPriceTypeEditorMode.Modification;
            if ((this.entity = BXCatalogPriceType.GetById(id, BXTextEncoder.EmptyTextEncoder)) == null)
            {
                this.errorText = string.Format(GetMessageRaw("Error.UnableToFindEntity"), id);
                this.editorError = CatalogPriceTypeEditorError.IsNotFound;
                return;
            }
        }

        if (!IsPostBack)
        {
            IsBaseChk.Enabled = !(IsBaseChk.Checked = this.entity.IsBase);

            if (this.entity.ExtraId != 0)
            {
                ListItem selExtraItem = ExtraList.Items.FindByValue(this.entity.ExtraId.ToString());
                foreach (ListItem extraItem in ExtraList.Items)
                    extraItem.Selected = extraItem == selExtraItem;
            }
            if (this.entity.CurrencyId != 0)
            {
                ListItem selCurrencyItem =  CurrencyList.Items.FindByValue(this.entity.CurrencyId.ToString());
                foreach (ListItem currencyItem in CurrencyList.Items)
                    currencyItem.Selected = currencyItem == selCurrencyItem;
            }

            //if (this.entity.IsBase)
            //    ExtraList.Attributes.CssStyle.Add(HtmlTextWriterStyle.Display, "none");

            Code.Text = this.entity.Code;
            Sort.Text = this.entity.Sort.ToString();

            BXCatalogPriceType2RoleCollection roleLinks = this.entity.RoleLinks;
            foreach (BXCatalogPriceType2Role link in roleLinks)
            {
                ListItem li = (link.ViewOnly ? RolesAllowedToView : AssignedToRoles).Items.FindByValue(link.RoleId.ToString());
                if (li != null)
                    li.Selected = true;
            }

            XmlId.Text = this.entity.XmlId;

            BXCatalogPriceTypeLocalizationInfo[] locInfAry = this.entity.LocalizationInfos;
            foreach(LocalizationEditor locEd in this.locEdList)
            {
                int index = Array.FindIndex<BXCatalogPriceTypeLocalizationInfo>(
                    locInfAry, 
                    delegate(BXCatalogPriceTypeLocalizationInfo obj) { return string.Equals(obj.LocaleName, locEd.LocaleName, StringComparison.OrdinalIgnoreCase); });

                 if(index < 0)
                     continue;

                locEd.LocalizedName = locInfAry[index].Name;
            }
        }
        else
        {
            if (!this.entity.IsBase) //флаг базовой цены нельзя снять
                this.entity.IsBase = IsBaseChk.Checked;

            this.entity.Code = Code.Text;


            if (!this.entity.IsBase && !string.IsNullOrEmpty(ExtraList.SelectedValue))
            {
                int i;
                this.entity.ExtraId = int.TryParse(ExtraList.SelectedValue, out i) ? i : 0;
            }
            else
                this.entity.ExtraId = 0;

            if (!string.IsNullOrEmpty(CurrencyList.SelectedValue))
            {
                int i;
                this.entity.CurrencyId = int.TryParse(CurrencyList.SelectedValue, out i) ? i : 0;
            }

            if (string.IsNullOrEmpty(Sort.Text))
                this.entity.Sort = 0;
            else
            {
                int i;
                this.entity.Sort= int.TryParse(Sort.Text, out i) ? i : 0;
            }

            List<BXRole> roleList = new List<BXRole>();
            foreach (ListItem item in RolesAllowedToView.Items)
            {
                if (!item.Selected)
                    continue;

                BXRole role = BXRoleManager.GetById(Convert.ToInt32(item.Value));
                if (role != null)
                    roleList.Add(role);
            }
            this.entity.SetAllowedToViewRoles(roleList.ToArray());

            roleList.Clear();
            foreach (ListItem item in AssignedToRoles.Items)
            {
                if (!item.Selected)
                    continue;

                BXRole role = BXRoleManager.GetById(Convert.ToInt32(item.Value));
                if (role != null)
                    roleList.Add(role);
            }
            this.entity.SetAssignedToRoles(roleList.ToArray());

            this.entity.XmlId = XmlId.Text;

            List<BXCatalogPriceTypeLocalizationInfo> locInfLst = new List<BXCatalogPriceTypeLocalizationInfo>();
            foreach(LocalizationEditor locEd in this.locEdList)
                if(!string.IsNullOrEmpty(locEd.LocalizedName))
                    locInfLst.Add(new BXCatalogPriceTypeLocalizationInfo(locEd.LocaleName, locEd.LocalizedName));                
            this.entity.LocalizationInfos = locInfLst.ToArray();
        }
	}

    private void TrySaveEntity()
    {
        if (this.entity == null)
            return;

        try
        {
            this.entity.Save();
            this.entityId = this.entity.Id;
        }
        catch (Exception exc)
        {
            this.errorText= exc.Message;
            this.editorError = EditorMode == CatalogPriceTypeEditorMode.Creation ? CatalogPriceTypeEditorError.CreationFailed : CatalogPriceTypeEditorError.ModificationFailed; 
        }
    }

    protected override void OnPreRender(EventArgs e)
    {
        if (EditorError != CatalogPriceTypeEditorError.None)
        {
            Summary.AddErrorMessage(ErrorText);
            if (EditorMode == CatalogPriceTypeEditorMode.Modification && EditorError == CatalogPriceTypeEditorError.IsNotFound)
                TabView.Visible = false;
        }

        BXPage.Scripts.RequireUtils();
        ClientScript.RegisterStartupScript(GetType(), "Init", "Bitrix.CatalogPriceTypeEditor.instantiate();", true);
        base.OnPreRender(e);
    }

    protected void OnCatalogPriceTypeEdit(object sender, BXTabControlCommandEventArgs e)
	{
		switch (e.CommandName)
		{
            case "save":
                {
                    if (IsValid)
                    {
                        TrySaveEntity();
                        if (EditorError == CatalogPriceTypeEditorError.None)
                            GoBack(); 
                    }
                }
                break;
            case "apply":
                {
                    if (IsValid)
                    {
                        TrySaveEntity();
                        if (EditorError == CatalogPriceTypeEditorError.None)
                            Response.Redirect(string.Format("CatalogPriceTypeEdit.aspx?id={0}&tabindex={1}", EntityId.ToString(), TabView.SelectedIndex));
                    }
                }
                break;
			case "cancel":
				GoBack();
				break;
		}
	}

	protected void OnToolBarButtonClick(object sender, CommandEventArgs e)
	{
		if (e.CommandName == "delete")
		{
			try
			{
                BXCatalogPriceType entity = Entity;
                if (entity  != null)
                    entity.Delete();

				GoBack();
			}
			catch (Exception ex)
			{
                this.errorText = ex.Message;
                this.editorError = CatalogPriceTypeEditorError.DeletionFailed;
			}
		}
	}

    protected void MoveSelectedUp(ListItemCollection items)
    {
        var selItems = new List<ListItem>();
        for (int i = items.Count - 1; i >= 0; i--)
        {
            if (items[i].Selected)
            {
                selItems.Add(items[i]);
                items.RemoveAt(i);
            }
        }
        foreach (var i in selItems)
            items.Insert(0, i);
    }
}

public class LocalizationEditor: Control, IPostBackDataHandler
{
    private string localeName = string.Empty;
    public string LocaleName
    {
        get { return this.localeName; }
        set { this.localeName = value ?? string.Empty; }
    }

    private string localizedName = string.Empty;
    public string LocalizedName
    {
        get { return this.localizedName; }
        set { this.localizedName = value ?? string.Empty;  }
    }

    protected override void Render(HtmlTextWriter writer)
    {
        writer.Write("<input type=\"type\" name=\"" + UniqueID + "\" value=\"" + HttpUtility.HtmlAttributeEncode(this.localizedName) + "\"/>" );
    }

    #region IPostBackDataHandler Members
    public bool LoadPostData(string postDataKey, NameValueCollection postCollection)
    {
        if(postDataKey != UniqueID)
            return false;

        this.localizedName = postCollection[UniqueID];
        return true;

    }

    public void RaisePostDataChangedEvent()
    {
    }
    #endregion
}