﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using Bitrix.UI;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix;
using System.Text;
using Bitrix.Security;
using Bitrix.Catalog;
using Bitrix.Main;

public partial class bitrix_admin_CatalogPriceTypeList : BXAdminPage
{

    protected override void OnInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXCatalog.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnInit(e);
    }

    protected override void OnLoad(EventArgs e)
    {
        ((BXAdminMasterPage)Master).PageIconId = "catalog_page_icon";
        MasterTitle = Page.Title = GetMessage("PageTitle");
        base.OnLoad(e);
    }

    protected void ItemGrid_Select(object sender, BXSelectEventArgs e)
	{
        BXCatalogPriceTypeCollection c = BXCatalogPriceType.GetList(
            new BXFilter(ItemFilter.CurrentFilter, BXCatalogPriceType.Fields),
            new BXOrderBy(BXCatalogPriceType.Fields, string.IsNullOrEmpty(e.SortExpression) ? "Id" : e.SortExpression),
            new BXSelect(
                BXSelectFieldPreparationMode.Normal,
                BXCatalogPriceType.Fields.Id,
                BXCatalogPriceType.Fields.Code,
                BXCatalogPriceType.Fields.IsBase,
                BXCatalogPriceType.Fields.Sort,
                BXCatalogPriceType.Fields.LocalizationInfos
                ),
            null,
            BXTextEncoder.EmptyTextEncoder
            );

        List<CatalogPriceTypeWrapper> list = new List<CatalogPriceTypeWrapper>();
        foreach (BXCatalogPriceType item in c)
            list.Add(new CatalogPriceTypeWrapper(item, this));

        e.Data = list;
	}


    protected void ItemGrid_SelectCount(object sender, BXSelectCountEventArgs e)
	{
        e.Count = BXCatalogPriceType.Count(new BXFilter(ItemFilter.CurrentFilter, BXCatalogPriceType.Fields));
	}

    protected void ItemGrid_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType != DataControlRowType.DataRow)
            return;

        BXGridViewRow row = (BXGridViewRow)e.Row;
        row.UserData.Add("ID", ((CatalogPriceTypeWrapper)row.DataItem).ID);
    }


    protected void ItemGrid_Delete(object sender, BXDeleteEventArgs e)
	{
        if (!BXPrincipal.Current.IsCanOperate(BXCatalog.Operations.AdminManagement))
            return;

		try
		{
	        BXCatalogPriceType.Delete(e.Keys["ID"]);
			e.DeletedCount++;
		}
		catch (Exception ex)
		{
			ErrorMessage.AddErrorMessage(ex.Message);
		}
	}
}
/// <summary>
/// Обёртка
/// </summary>
public class CatalogPriceTypeWrapper
{
    BXCatalogPriceType charge;
    BXAdminPage parentPage;
    public CatalogPriceTypeWrapper(BXCatalogPriceType charge, BXAdminPage parentPage)
    {
        if (charge == null)
            throw new ArgumentNullException("charge");

        if (parentPage == null)
            throw new ArgumentNullException("parentPage");

        this.charge = charge;
        this.parentPage = parentPage;
    }

    public string ID
    {
        get { return this.charge.Id.ToString(); }
    }

    public string Name
    {
        get { return HttpUtility.HtmlEncode(this.charge.GetLocalizedName()); }
    }

    public string Code
    {
        get { return HttpUtility.HtmlEncode(this.charge.Code); }
    }

    public string IsBase
    {
        get { return BXMain.GetModuleMessage(this.charge.IsBase ? "Kernel.Yes" : "Kernel.No"); }
    }

    public string Sort
    {
        get { return this.charge.Sort.ToString(); }
    }
}
