﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Web.UI.WebControls;
using Bitrix.Catalog;
using Bitrix.DataLayer;
using Bitrix.IBlock;
using Bitrix.Services.Text;
using Bitrix.UI;
using System.Web;
using Bitrix.Currency;
using Bitrix.Services.Js;
using Bitrix.Modules;

namespace Bitrix.Sale.AdminPages
{

	public partial class CatalogSearch : BXAdminPage
	{
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);
		}
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);
			BXPage.Scripts.RequireUtils();
		}

		protected void GridView_Select(object sender, Bitrix.UI.BXSelectEventArgs e)
		{
			
			var filter = BuildFilter();
			var orderBy = new BXOrderBy(BXCatalog.Fields, e.SortExpression);

			var queryParams = new BXQueryParams(e.PagingOptions);

			e.Data = BXCatalog.GetList(
				filter,
				orderBy,
				null,
				queryParams,
				BXTextEncoder.EmptyTextEncoder
			)
			.ConvertAll(x => new Wrapper(x, this));

		}
		protected void GridView_SelectCount(object sender, BXSelectCountEventArgs e)
		{

			var filter = BuildFilter();
			e.Count = BXCatalog.Count(filter);
		}

		private BXFilter BuildFilter()
		{
			BXFilterOr sectionFilter = null;
			
			var mainFormFilter = new BXFormFilter(AdminFilter.CurrentFilter);
			mainFormFilter.RemoveAll(x => 
			{ 
				if (!x.filterName.StartsWith(":"))
					return false;
				
				sectionFilter = sectionFilter ?? new BXFilterOr();
				
				return true;
			});

			var filter = new BXFilter(mainFormFilter, BXCatalog.Fields);
			if (sectionFilter != null)
				filter.Add(sectionFilter);
			
			return filter;
		}

		public class Wrapper
		{
			BXCatalog item;
			CatalogSearch page;

			public Wrapper(BXCatalog item, CatalogSearch page)
			{
				this.item = item;
				this.page = page;
			}

			public string Id 
			{
				get
				{
					return item.Id.ToString();
				}
			}

			public string Active 
			{
				get
				{
					return item.IBlock.Active ? page.GetMessageRaw("Kernel.Yes") : page.GetMessageRaw("Kernel.No");
				}
			}

			public string Name
			{
				get
				{
					return HttpUtility.HtmlEncode(item.IBlock.Name ?? "");
				}
			}

			public BXCatalog Catalog
			{
				get { return item; }
			}

			public string CatalogInfo
			{
				get
				{
					var json = new Dictionary<string, object>
					{
						{ "id", item.Id},
						{ "name", item.IBlock.Name ?? ""},
			
					};
					return BXJSUtility.BuildJSON(json);
				}
			}
		}
	}
}