﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Web.UI.WebControls;
using Bitrix.Catalog;
using Bitrix.DataLayer;
using Bitrix.IBlock;
using Bitrix.Services.Text;
using Bitrix.UI;
using System.Web;
using Bitrix.Currency;
using Bitrix.Services.Js;
using Bitrix.Modules;

namespace Bitrix.Sale.AdminPages
{

	public partial class CatalogSectionSearch : BXAdminPage
	{

		List<BXCatalog> Catalogs { get; set; }


		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			Catalogs = BXCatalog.GetList(
				new BXFilter(
					new BXFilterItem(BXCatalog.Fields.IBlock.Active, BXSqlFilterOperators.Equal, "Y"),
					new BXFilterItem(BXCatalog.Fields.IBlock.Sites.SiteId, BXSqlFilterOperators.Equal, Request.QueryString["site"])
				),
				new BXOrderBy(
					new BXOrderByPair(BXCatalog.Fields.IBlock.Name, BXOrderByDirection.Asc)
				),
				new BXSelectAdd(
					BXCatalog.Fields.IBlock.ID,
					BXCatalog.Fields.IBlock.Name
				),
				null, 
				BXTextEncoder.EmptyTextEncoder
			)
			.Where(
				x => 
					x.IBlock != null 
					&& (
						BXUser.IsCanOperate(BXIBlock.Operations.IBlockPublicRead, "iblock", x.IBlockId)
						|| BXUser.IsCanOperate(BXIBlock.Operations.IBlockAdminRead, "iblock", x.IBlockId)
					)
			)
			.ToList();

			CatalogFilter.Items.Clear();
			CatalogFilter.Items.AddRange(
				Catalogs
				.ConvertAll(x => new ListItem(x.IBlock.Name, x.Id.ToString()))
				.ToArray()
			);

		}
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);
			BXPage.Scripts.RequireUtils();
		}

		protected void GridView_Select(object sender, Bitrix.UI.BXSelectEventArgs e)
		{
			
			var filter = BuildFilter();
			var orderBy = new BXOrderBy(BXIBlockSection.Fields, e.SortExpression);

			var queryParams = new BXQueryParams(e.PagingOptions);

			e.Data = BXIBlockSection.GetList(
				filter,
				orderBy,
				null,
				queryParams,
				BXTextEncoder.EmptyTextEncoder
			)
			.ConvertAll(x => new Wrapper(x, this));

		}
		protected void GridView_SelectCount(object sender, BXSelectCountEventArgs e)
		{

			var filter = BuildFilter();
			e.Count = BXIBlockSection.Count(filter);
		}

		private BXFilter BuildFilter()
		{
			BXFilterOr sectionFilter = null;
			
			var mainFormFilter = new BXFormFilter(AdminFilter.CurrentFilter);
			mainFormFilter.RemoveAll(x => 
			{ 
				if (!x.filterName.StartsWith(":"))
					return false;
				
				sectionFilter = sectionFilter ?? new BXFilterOr();
				
				return true;
			});

			var filter = new BXFilter(mainFormFilter, BXIBlockSection.Fields);
			if (sectionFilter != null)
				filter.Add(sectionFilter);
			
			return filter;
		}

		protected void Catalog_OnBuildFilter(object sender, BXCustomAdminFilter.BuildFilterEventArgs e)
		{
			int id;
			if (!string.IsNullOrEmpty(CatalogFilter.SelectedValue) && int.TryParse(CatalogFilter.SelectedValue, out id))
				e.Result.Add(new BXFormFilterItem("IBlockId", id, BXSqlFilterOperators.Equal));
		}
		protected void Catalog_OnLoadState(object sender, BXCustomAdminFilter.StateEventArgs e)
		{
			if (e.State.Contains("id"))
			{
				try
				{
					CatalogFilter.SelectedValue = (string)e.State["id"];
				}
				catch
				{
				}
			}

		}
		protected void Catalog_OnSaveState(object sender, BXCustomAdminFilter.StateEventArgs e)
		{
			e.State["id"] = CatalogFilter.SelectedValue ?? "";			
		}
		protected void Catalog_OnReset(object sender, EventArgs e)
		{
			CatalogFilter.SelectedIndex = 0;
		}

		public class Wrapper
		{
			BXIBlockSection item;
			CatalogSectionSearch page;

			public Wrapper(BXIBlockSection item, CatalogSectionSearch page)
			{
				this.item = item;
				this.page = page;
			}

			public string Id 
			{
				get
				{
					return item.Id.ToString();
				}
			}

			public string Active 
			{
				get
				{
					return item.Active ? page.GetMessageRaw("Kernel.Yes") : page.GetMessageRaw("Kernel.No");
				}
			}

			public string Name
			{
				get
				{
					return HttpUtility.HtmlEncode(item.Name ?? "");
				}
			}


			public string SectionInfo
			{
				get
				{
					var json = new Dictionary<string, object>
					{
						{ "id", item.Id},
						{ "name", item.Name ?? ""},
			
					};
					return BXJSUtility.BuildJSON(json);
				}
			}
		}
	}
}