﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.UI;
using Bitrix.Services.Text;
using Bitrix;
using Bitrix.DataLayer;
using Bitrix.Modules;
using System.Collections.Generic;
using Bitrix.Security;
using Bitrix.Configuration;
using Bitrix.Catalog;
using System.Globalization;

public enum CatalogVATRateEditorMode
{
    Creation = 1,
    Modification = 2
}

public enum CatalogVATRateEditorError
{
    None = 0,
    IsNotFound = -1,
    CreationFailed = -2,
    ModificationFailed = -3,
    DeletionFailed = -4
}

public partial class bitrix_admin_CatalogVATRateEditorEdit : BXAdminPage
{
	private int? vatRateId = null;
	protected int VATRateId
	{
        get 
        {
            if (this.vatRateId.HasValue)
                return this.vatRateId.Value;

            string s = Request.QueryString["id"];
            if (string.IsNullOrEmpty(s))
                this.vatRateId = 0;
            else
            {
                int i;
                this.vatRateId = int.TryParse(s, out i) ? i : 0;
            }
            return this.vatRateId.Value; 
        }
	}

    private CatalogVATRateEditorMode editorMode = CatalogVATRateEditorMode.Creation;
    protected CatalogVATRateEditorMode EditorMode
    {
        get { return this.editorMode; }
    }

    private BXCatalogVATRate vatRate;
    protected BXCatalogVATRate VATRate
    {
        get { return this.vatRate; }
    }

    private CatalogVATRateEditorError editorError = CatalogVATRateEditorError.None;
    protected CatalogVATRateEditorError EditorError
    {
        get { return this.editorError; }
    }

    private string errorText = string.Empty;
    public string ErrorText
    {
        get { return this.errorText; }
    }

	protected override string BackUrl
	{
		get { return !string.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "CatalogVATRateList.aspx"; }
	}

    protected override void OnInit(EventArgs e)
    {

        if (!BXPrincipal.Current.IsCanOperate(BXCatalog.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        ((BXAdminMasterPage)Master).PageIconId = "catalog_page_icon";
        base.OnInit(e);
    }

    protected override void OnLoad(EventArgs e)
    {
        TryLoadVATRate();
        BXCatalogVATRate rate = VATRate;
        string title = EditorMode == CatalogVATRateEditorMode.Modification ? string.Format(GetMessage("PageTitle.Edit"), rate != null ? HttpUtility.HtmlEncode(rate.Name) : "?") : GetMessage("PageTitle.Create");

        if (EditorMode == CatalogVATRateEditorMode.Creation)
		{
			AddButton.Visible = false;
			DeleteButton.Visible = false;
		}
        MasterTitle = Page.Title = title;
        base.OnLoad(e);
    }

    private void TryLoadVATRate()
	{
        int id = VATRateId;
        if (id <= 0)
        {
            this.editorMode = CatalogVATRateEditorMode.Creation;
            this.vatRate= new BXCatalogVATRate();
        }
        else
        {
            this.editorMode = CatalogVATRateEditorMode.Modification;
            if ((this.vatRate = BXCatalogVATRate.GetById(id, BXTextEncoder.EmptyTextEncoder)) == null)
            {
                this.errorText = string.Format(GetMessageRaw("Error.UnableToFindEntity"), id);
                this.editorError = CatalogVATRateEditorError.IsNotFound;
                return;
            }
        }

        if (!IsPostBack)
        {
            VATRateName.Text = this.vatRate.Name;
            VATRateValue.Text = this.vatRate.Value.ToString("F2", CultureInfo.InvariantCulture.NumberFormat);
            VATRateActive.Checked = this.vatRate.Active;
            VATRateSort.Text = this.vatRate.Sort.ToString();
            VATRateXmlId.Text = this.vatRate.XmlId;
        }
        else
        {
            this.vatRate.Name = VATRateName.Text;
            if (string.IsNullOrEmpty(VATRateValue.Text))
                this.vatRate.Value = 0.0m;
            else
            {
                decimal d;
                this.vatRate.Value = decimal.TryParse(VATRateValue.Text, NumberStyles.AllowDecimalPoint, CultureInfo.InvariantCulture.NumberFormat, out d) ? d : 0.0m;
            }

            this.vatRate.Active = VATRateActive.Checked;
            if (string.IsNullOrEmpty(VATRateSort.Text))
                this.vatRate.Sort = 0;
            else
            {
                int i;
                this.vatRate.Sort = int.TryParse(VATRateSort.Text, out i) ? i : 0;
            }
            this.vatRate.XmlId = VATRateXmlId.Text;
        }
	}

    private void TrySaveVATRate()
    {
        if (this.vatRate == null)
            return;

        try
        {
            this.vatRate.Save();
            this.vatRateId = this.vatRate.Id;
        }
        catch (Exception exc)
        {
            this.errorText= exc.Message;
            this.editorError = EditorMode == CatalogVATRateEditorMode.Creation ? CatalogVATRateEditorError.CreationFailed : CatalogVATRateEditorError.ModificationFailed; 
        }
    }

    protected override void OnPreRender(EventArgs e)
    {
        if (EditorError != CatalogVATRateEditorError.None)
        {
            errorMessage.AddErrorMessage(ErrorText);
            if (EditorMode == CatalogVATRateEditorMode.Modification && EditorError == CatalogVATRateEditorError.IsNotFound)
                TabControl.Visible = false;
        }

        base.OnPreRender(e);
    }

    protected void OnCatalogVATRateEdit(object sender, BXTabControlCommandEventArgs e)
	{
		switch (e.CommandName)
		{
            case "save":
                {
                    if (IsValid)
                    {
                        TrySaveVATRate();
                        if (EditorError == CatalogVATRateEditorError.None)
                            GoBack(); 
                    }
                }
                break;
            case "apply":
                {
                    if (IsValid)
                    {
                        TrySaveVATRate();
                        if (EditorError == CatalogVATRateEditorError.None)
                            Response.Redirect(string.Format("CatalogVATRateEdit.aspx?id={0}&tabindex={1}", VATRateId.ToString(), TabControl.SelectedIndex));
                    }
                }
                break;
			case "cancel":
				GoBack();
				break;
		}
	}

	protected void OnToolBarButtonClick(object sender, CommandEventArgs e)
	{
		if (e.CommandName == "delete")
		{
			try
			{
                BXCatalogVATRate rate = VATRate;
                if (rate != null)
                    rate.Delete();

				GoBack();
			}
			catch (Exception ex)
			{
                this.errorText = ex.Message;
                this.editorError = CatalogVATRateEditorError.DeletionFailed;
			}
		}
	}
}
