﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.Catalog;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix.UI;
using System.Globalization;
using System.Text;
using System.Web.Script.Serialization;
using Bitrix.IBlock;
using System.Collections.Specialized;
using Bitrix.Services;
using Bitrix.Configuration;
using Bitrix.Security;

public partial class BXCatalogIBlockElementEditor : BXIBlockElementEditor
{   
    #region BXIBlockElementEditor
    private bool? isEnabled = null;
    public override bool IsEnabled()
    {
        if (isEnabled.HasValue)
            return this.isEnabled.Value;

        string s;
        int i;
        return (this.isEnabled = !string.IsNullOrEmpty(s = Attributes["IBlockId"]) && int.TryParse(s, out i) && BXCatalog.IsIBlockExists(i)).Value;
    }

    public override void HandleSaving(BXIBlockElement element)
    {
        if (this.error != EditorError.None)
            return;

        string catalogIdStr = Attributes["IBlockId"];
        if (string.IsNullOrEmpty(catalogIdStr) || !int.TryParse(catalogIdStr, out this.catalogId) || this.catalogId <= 0)
        {
            this.error |= EditorError.CatalogIdIsNotFound;
            return;
        }

        string itemIdStr = Attributes["IBlockElementId"];
        if (string.IsNullOrEmpty(itemIdStr))
        {
            this.error |= EditorError.ItemIdIsNotFound;
            return;
        }

        if (int.TryParse(itemIdStr, out this.itemId) && this.itemId > 0)
            this.item.Id = this.itemId;

        this.item.Id = this.itemId;
        this.item.CatalogId = this.catalogId;
		
        this.item.Save();

		if(ParentCatalogItemId > 0)
			this.item.SetProductId(ParentCatalogItemId, null);

        foreach (BXCatalogPrice price in this.prices)
        {
            BXCatalogPriceType priceType = this.priceTypes.Find(delegate(BXCatalogPriceType o) { return o.Id == price.PriceTypeId; });
            if(priceType == null)
                continue;

            CatalogPriceData priceData = this.itemData.FindPriceById(price.Id);

            //удаление
            if (priceData == null || (!priceType.IsBase && priceData.Error == CatalogPriceDataError.EmptyValue))
            {
                price.Delete();
                continue;
            }

            //обновление
            price.Value = priceData.Value;
            price.ExtraId = priceData.ExtraId;
            price.QuantityFrom = priceData.QtyFrom;
            price.VATIncluded = VATIncluded.Checked;
            price.Update();
        }

        //создание
        foreach (CatalogPriceTierData tierData in this.itemData.PriceTiers)
            foreach (CatalogPriceData priceData in tierData.Prices)
            {
                if (priceData.Id > 0)
                    continue;

                BXCatalogPriceType priceType = this.priceTypes.Find(delegate(BXCatalogPriceType o) { return o.Id == priceData.PriceTypeId; });
                if (priceType == null || priceData.Error == CatalogPriceDataError.EmptyValue)
                    continue;

                BXCatalogPrice price = new BXCatalogPrice();
                price.ItemId = this.item.Id;
                price.Value = priceData.Value;
                price.PriceTypeId = priceType.Id;
                price.CurrencyId = priceType.CurrencyId;
                price.ExtraId = priceData.ExtraId;
                price.QuantityFrom = priceData.QtyFrom;
                price.VATIncluded = VATIncluded.Checked;

                price.Create();
            }        
    }

    public override bool IsValid()
    {
        if (this.error != EditorError.None)
        {
            if ((this.error & EditorError.InvalidQuantity) != 0)
                (this.errorMessageList ?? (this.errorMessageList = new List<string>())).Add(GetMessageRaw("Error.QuantityIncorrect"));

            if ((this.error & EditorError.InvalidWeight) != 0)
                (this.errorMessageList ?? (this.errorMessageList = new List<string>())).Add(GetMessageRaw("Error.WeightIncorrect"));
        }
        List<int> duplexTierQtyLst = null;
        int? minQty = null;
        for (int i = 0; i < this.itemData.PriceTiers.Length; i++)
        {
            CatalogPriceTierData tierData = this.itemData.PriceTiers[i];
            if (!minQty.HasValue || minQty > tierData.QtyFrom)
                minQty = tierData.QtyFrom;

            if (duplexTierQtyLst == null || duplexTierQtyLst.FindIndex(delegate(int qty) { return qty == tierData.QtyFrom; }) < 0)
            {
                for (int j = i + 1; j < this.itemData.PriceTiers.Length; j++)
                {
                    if (this.itemData.PriceTiers[j].QtyFrom != tierData.QtyFrom)
                        continue;
                    (duplexTierQtyLst ?? (duplexTierQtyLst = new List<int>())).Add(tierData.QtyFrom);
                    break;
                }
            }

            for (int k = 0; k < tierData.Prices.Count; k++)
            {
                CatalogPriceData priceData = tierData.Prices[k];
                if (priceData.Error == CatalogPriceDataError.None)
                    continue;
                if (priceData.IsBase && (priceData.Error & CatalogPriceDataError.EmptyValue) != 0)
                    (this.errorMessageList ?? (this.errorMessageList = new List<string>())).Add(GetMessageRaw("Error.BasePriceNotSet"));
                if ((priceData.Error & CatalogPriceDataError.InvalidValue) != 0)
                    (this.errorMessageList ?? (this.errorMessageList = new List<string>())).Add(string.Format(GetMessageRaw("Error.PriceIncorrect"), GetPriceTypeCode(priceData.PriceTypeId)));
                if ((priceData.Error & CatalogPriceDataError.NegativeOrZeroValue) != 0)
                    (this.errorMessageList ?? (this.errorMessageList = new List<string>())).Add(string.Format(GetMessageRaw("Error.PriceNegative"), GetPriceTypeCode(priceData.PriceTypeId)));
                if ((priceData.Error & CatalogPriceDataError.EmptyQtyFrom) != 0)
                    (this.errorMessageList ?? (this.errorMessageList = new List<string>())).Add(string.Format(GetMessageRaw("Error.PriceQuantityRequired"), GetPriceTypeCode(priceData.PriceTypeId)));
                if ((priceData.Error & CatalogPriceDataError.InvalidQtyFrom) != 0)
                    (this.errorMessageList ?? (this.errorMessageList = new List<string>())).Add(string.Format(GetMessageRaw("Error.PriceQuantityIncorrect"), GetPriceTypeCode(priceData.PriceTypeId)));
                if ((priceData.Error & CatalogPriceDataError.NegativeQtyFrom) != 0)
                    (this.errorMessageList ?? (this.errorMessageList = new List<string>())).Add(string.Format(GetMessageRaw("Error.PriceQuantityNegative"), GetPriceTypeCode(priceData.PriceTypeId)));
            }
        }

        if (duplexTierQtyLst != null) 
            for (int m = 0; m < duplexTierQtyLst.Count; m++)
                (this.errorMessageList ?? (this.errorMessageList = new List<string>())).Add(string.Format(GetMessageRaw("Error.TooManyPriceTiers"), duplexTierQtyLst[m].ToString()));

        if (minQty.HasValue)
        {
            if (minQty.Value <= 0)
                (this.errorMessageList ?? (this.errorMessageList = new List<string>())).Add(GetMessageRaw("Error.InvalidPriceTiers"));
            else if(minQty.Value != 1)
                (this.errorMessageList ?? (this.errorMessageList = new List<string>())).Add(GetMessageRaw("Error.ValidPriceTiersNotFound"));
        }
        return this.errorMessageList == null || this.errorMessageList.Count == 0;
    }

    private List<string> errorMessageList = null;
    public override string[] GetErrorMessages() 
    {
        return this.errorMessageList != null ? this.errorMessageList.ToArray() : new string[0];
    }
    #endregion

    internal enum EditorError
    {
        None = 0x0,
        CatalogIdIsNotFound = 0x1,
        ItemIdIsNotFound = 0x2,
        InvalidQuantity = 0x4,
        InvalidWeight = 0x8
    }

    private EditorError error = EditorError.None;
    internal new EditorError Error
    {
        get { return this.error; }
    }

    protected override void OnInit(EventArgs e)
    {
        base.OnInit(e);

        string catalogIdStr = Attributes["IBlockId"];
        if (string.IsNullOrEmpty(catalogIdStr) || !int.TryParse(catalogIdStr, out this.catalogId) || this.catalogId <= 0)
        {
            this.error |= EditorError.CatalogIdIsNotFound;
            return;
        }

		if((this.catalog = BXCatalog.GetById(this.catalogId)) != null)
		{
			this.skuCatalog = this.catalog.GetSKUCatalog();
			if(this.skuCatalog != null)
				this.skuCatalogId = skuCatalog.Id;
		}

        string itemIdStr = Attributes["IBlockElementId"];
        if (string.IsNullOrEmpty(itemIdStr))
        {
            this.error |= EditorError.ItemIdIsNotFound;
            return;
        }

        if (int.TryParse(itemIdStr, out this.itemId) && this.itemId > 0)
        {
            BXCatalogItemCollection items = BXCatalogItem.GetList(
                new BXFilter(new BXFilterItem(BXCatalogItem.Fields.Id, BXSqlFilterOperators.Equal, this.itemId)),
                null,
                null,
                new BXQueryParams(BXPagingOptions.Top(1)),
                BXTextEncoder.EmptyTextEncoder);

            if (items.Count > 0)
                this.item = items[0];
        }

        if (this.item == null)
            this.item = new BXCatalogItem(this.itemId);

		if(!this.item.IsNew 
			&& string.Equals(Request["catalogItemSkuAction"], "DELETE", StringComparison.OrdinalIgnoreCase)
			&& BXCsrfToken.CheckToken(Request[BXCsrfToken.TokenKey]))
		{
			int skuId;
			string s = Request["catalogItemSkuId"];
			if(!string.IsNullOrEmpty(s) && int.TryParse(s, out skuId))
			{
				BXCatalogItemCollection skuCol = BXCatalogItem.GetList(
					new BXFilter(
						new BXFilterItem(BXCatalogItem.Fields.Id, BXSqlFilterOperators.Equal, skuId),
						new BXFilterItem(BXCatalogItem.Fields.ProductId, BXSqlFilterOperators.Equal, this.itemId)),
					null,
					null,
					new BXQueryParams(BXPagingOptions.Top(1)));

				if(skuCol.Count > 0 && skuCol[0].IBlockElement != null)
					skuCol[0].IBlockElement.Delete();

				string backUrl = Request[BXConfigurationUtility.Constants.BackUrl];
				if(!string.IsNullOrEmpty(backUrl))
					Response.Redirect(backUrl);
			}
		}

        BXCatalogVATRateCollection col = BXCatalogVATRate.GetList(
            null,
            new BXOrderBy(
                new BXOrderByPair(BXCatalogVATRate.Fields.Sort, BXOrderByDirection.Asc),
                new BXOrderByPair(BXCatalogVATRate.Fields.Name, BXOrderByDirection.Asc)
                ),
            new BXSelect(
                BXSelectFieldPreparationMode.Normal,
                BXCatalogVATRate.Fields.Id,
                BXCatalogVATRate.Fields.Name
                ),
            null,
            BXTextEncoder.EmptyTextEncoder
            );


        VATRateList.Items.Add(new ListItem(GetMessageRaw("NotSelectedFeminine"), "0"));
        foreach (BXCatalogVATRate rate in col)
            VATRateList.Items.Add(new ListItem(rate.Name, rate.Id.ToString()));

		if(this.catalogId > 0 && this.itemId > 0)
			 this.skuItems = BXCatalogItem.GetList(
				new BXFilter(
					new BXFilterItem(BXCatalogItem.Fields.ProductId, BXSqlFilterOperators.Equal, this.itemId),
					new BXFilterItem(BXCatalogItem.Fields.Enabled, BXSqlFilterOperators.Equal, true)),
				null,
				new BXSelectAdd(BXCatalogItem.Fields.IBlockElement),
				null,
				BXTextEncoder.EmptyTextEncoder).ToArray();
		else
			this.skuItems = new BXCatalogItem[0];

        this.priceTypes = BXCatalogPriceType.GetList(
            null,
            new BXOrderBy(
                new BXOrderByPair(BXCatalogPriceType.Fields.IsBase, BXOrderByDirection.Desc),
                new BXOrderByPair(BXCatalogPriceType.Fields.Sort, BXOrderByDirection.Asc),
                new BXOrderByPair(BXCatalogPriceType.Fields.Code, BXOrderByDirection.Asc)),
                null,
                null,
                BXTextEncoder.EmptyTextEncoder);

        this.prices = BXCatalogPrice.GetList(
            new BXFilter(new BXFilterItem(BXCatalogPrice.Fields.ItemId, BXSqlFilterOperators.Equal, this.itemId)),
            new BXOrderBy(
                new BXOrderByPair(BXCatalogPrice.Fields.PriceTypeId, BXOrderByDirection.Asc),
                new BXOrderByPair(BXCatalogPrice.Fields.QuantityFrom, BXOrderByDirection.Asc)),
            null,
            null
            );

        this.extras = BXCatalogExtra.GetList(
            null,
            new BXOrderBy(new BXOrderByPair(BXCatalogExtra.Fields.Name, BXOrderByDirection.Asc)));
    }

    protected override void OnLoad(EventArgs e)
    {
        QuantityValidator.Validate();
        WeightValidator.Validate();
        base.OnLoad(e);
        if (Error != EditorError.None)
            return;

        if (!IsPostBack)
        {
            this.itemData = new CatalogItemData();
            this.itemData.Id = this.item.Id;
            this.itemData.IsNew = this.item.IsNew;

            List<CatalogPriceTierData> priceTierDataLst = new List<CatalogPriceTierData>();
            foreach (BXCatalogPriceType priceType in this.priceTypes)
            {
                foreach (BXCatalogPrice price in this.prices)
                {
                    if (price.PriceTypeId != priceType.Id)
                        continue;

                    int tierInd = priceTierDataLst.FindIndex(
                        delegate(CatalogPriceTierData o) { return o.QtyFrom == price.QuantityFrom; });

                    CatalogPriceTierData tier = null;
                    if (tierInd >= 0)
                        tier = priceTierDataLst[tierInd];
                    else
                    {
                        tier = new CatalogPriceTierData();
                        tier.QtyFrom = price.QuantityFrom;
                        priceTierDataLst.Add(tier);
                    }

                    CatalogPriceData priceData = new CatalogPriceData();
                    priceData.Id = price.Id;
                    priceData.PriceTypeId = price.PriceTypeId;
                    priceData.IsBase = price.PriceType.IsBase;
                    priceData.ExtraId = price.ExtraId;
                    priceData.Value = price.Value;
                    priceData.QtyFrom = price.QuantityFrom;
                    tier.Prices.Add(priceData);
                }
            }

            this.itemData.PriceTiers = priceTierDataLst.ToArray();

			int vatRateId = 0;
			if(item.IsNew)
			{
				if(this.catalog != null)
					vatRateId = catalog.VATRateId;
			}
            else
				vatRateId = this.item.VATRateId;

			if(vatRateId > 0)
			{
				ListItem li = VATRateList.Items.FindByValue(vatRateId.ToString());
				if (li != null)
					li.Selected = true;
			}
            //ввести округление из типа цен
            //BasePriceTbx.Text = this.basePrice.Value.ToString("F2", CultureInfo.InvariantCulture.NumberFormat);

            VATIncluded.Checked = this.prices.Count > 0 ? this.prices[0].VATIncluded : false;
            QuantityTbx.Text = this.item.Quantity.ToString();
            WeightTbx.Text = this.item.Weight.ToString("F2", CultureInfo.InvariantCulture.NumberFormat);
        }
        else
        {
            JavaScriptSerializer serializer = new JavaScriptSerializer();
            serializer.RegisterConverters(new JavaScriptConverter[] { new CatalogPriceDataJsonConv(), new CatalogPriceTierDataJsonConv(), new CatalogItemDataJsonConv() });
            this.itemData = serializer.Deserialize<CatalogItemData>(CatalogItemData.Value);
            int vatId = 0;
            this.item.VATRateId = !string.IsNullOrEmpty(VATRateList.SelectedValue) && int.TryParse(VATRateList.SelectedValue, out vatId) ? vatId : 0;

            if (!string.IsNullOrEmpty(QuantityTbx.Text))
            {
                int qty;
                if (QuantityValidator.IsValid && int.TryParse(QuantityTbx.Text, out qty))
                    this.item.Quantity = qty;
                else
                    this.error |= EditorError.InvalidQuantity;
            }
            else
                this.item.Quantity = 0;

            if (!string.IsNullOrEmpty(WeightTbx.Text))
            {
                decimal weight;
                if (WeightValidator.IsValid && decimal.TryParse(WeightTbx.Text, NumberStyles.Float, CultureInfo.InvariantCulture.NumberFormat, out weight))
                    this.item.Weight = weight;
                else
                    this.error |= EditorError.InvalidWeight;
            }
            else
                this.item.Weight = 0;
        }
    }

    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        BXPage.Scripts.RequireUtils();

        ClientScriptManager sm = Page.ClientScript;
        if (!IsPostBack)
        {
            JavaScriptSerializer serializer = new JavaScriptSerializer();
            serializer.RegisterConverters(new JavaScriptConverter[] { new CatalogPriceDataJsonConv(), new CatalogPriceTierDataJsonConv(), new CatalogItemDataJsonConv() });

            sm.RegisterStartupScript(
                GetType(),
                "Init",
                string.Format("Bitrix.CatalogIBlockElementEditor.fromObject({0});", serializer.Serialize(this.itemData)),
                true
                );
        }
        else
        {
            sm.RegisterStartupScript(
                GetType(),
                "Init",
                string.Format("Bitrix.CatalogIBlockElementEditor.fromObject({0});", CatalogItemData.Value),
                true
                );
        }

        sm.RegisterOnSubmitStatement(
            GetType(),
            "Save",
            string.Format("document.getElementById(\"{0}\").value=Bitrix.CatalogIBlockElementEditor.toString();", CatalogItemData.ClientID)
            );
    }
	protected string GetSKUCreationUrl()
	{
		if(this.itemId <= 0 || this.skuCatalogId <= 0)
			return string.Empty;

		NameValueCollection qs = HttpUtility.ParseQueryString(BXSefUrlManager.CurrentUrl.Query);
		qs["parentCatalogItemId"] = this.itemId.ToString();
		qs["iblock_id"] = this.skuCatalogId.ToString();
		qs.Remove("id");
		qs[BXConfigurationUtility.Constants.BackUrl] =  BXSefUrlManager.CurrentUrl.PathAndQuery;

		string path = BXSefUrlManager.CurrentUrl.PathAndQuery;
		int whatInd = path.IndexOf('?');
		if (whatInd >= 0)
			path = path.Substring(0, whatInd);

		return string.Concat(path, "?", qs.ToString());
	}
	protected string GetSKUModificationUrl(int id)
	{
		if(this.itemId <= 0 || this.skuCatalogId <= 0)
			return string.Empty;

		NameValueCollection qs = HttpUtility.ParseQueryString(BXSefUrlManager.CurrentUrl.Query);
		qs["parentCatalogItemId"] = this.itemId.ToString();
		qs.Remove("id");
		qs["id"] = id.ToString();
		qs["iblock_id"] = this.skuCatalogId.ToString();
		qs[BXConfigurationUtility.Constants.BackUrl] =  BXSefUrlManager.CurrentUrl.PathAndQuery;

		string path = BXSefUrlManager.CurrentUrl.PathAndQuery;
		int whatInd = path.IndexOf('?');
		if (whatInd >= 0)
			path = path.Substring(0, whatInd);

		return string.Concat(path, "?", qs.ToString());
	}
	protected string GetSKUDeletionUrl(int id)
	{
		if(this.itemId <= 0 || this.skuCatalogId <= 0)
			return string.Empty;

		NameValueCollection qs = HttpUtility.ParseQueryString(BXSefUrlManager.CurrentUrl.Query);
		qs["parentCatalogItemId"] = this.itemId.ToString();
		qs["catalogItemSkuAction"] = "delete";
		qs["catalogItemSkuId"] = id.ToString();
		qs["iblock_id"] = this.skuCatalogId.ToString();
		qs[BXCsrfToken.TokenKey] = BXCsrfToken.GenerateToken();
		qs[BXConfigurationUtility.Constants.BackUrl] =  BXSefUrlManager.CurrentUrl.PathAndQuery;

		string path = BXSefUrlManager.CurrentUrl.PathAndQuery;
		int whatInd = path.IndexOf('?');
		if (whatInd >= 0)
			path = path.Substring(0, whatInd);

		return string.Concat(path, "?", qs.ToString());
	}

	private int? parentCatalogItemId = null;
	private int ParentCatalogItemId
	{
		get 
		{ 
			if(this.parentCatalogItemId.HasValue)
				return this.parentCatalogItemId.Value;

			string s = HttpUtility.ParseQueryString(BXSefUrlManager.CurrentUrl.Query)["parentCatalogItemId"];
			int i;
			return (this.parentCatalogItemId = !string.IsNullOrEmpty(s) && int.TryParse(s, out i) ? i : 0).Value;
		}
	}

    private int catalogId = 0;
    protected int CatalogId
    {
        get { return this.catalogId; }
    }

    private BXCatalog catalog = null;
    protected BXCatalog Catalog
    {
        get { return this.catalog; }
    }

    private int itemId = 0;
    protected int ItemId
    {
        get { return this.itemId; }
    }

    private BXCatalogItem item = null;
    protected BXCatalogItem Item
    {
        get { return this.item; }
    }

    private CatalogItemData itemData = null;
    internal CatalogItemData ItemData
    {
        get { return this.itemData; }
    }

    private int skuCatalogId = 0;
    protected int SkuCatalogId
    {
        get { return this.skuCatalogId; }
    }

    private BXCatalog skuCatalog = null;
    protected BXCatalog SkuCatalog
    {
        get { return this.skuCatalog; }
    }

    private BXCatalogItem[] skuItems = null;
    protected BXCatalogItem[] SkuItems
    {
        get { return this.skuItems; }
    }

    private BXCatalogPrice basePrice = null;
    protected BXCatalogPrice BasePrice
    { 
        get { return basePrice; }
    }

    private BXCatalogPriceTypeCollection priceTypes = null;
    protected BXCatalogPriceTypeCollection PriceTypes
    {
        get { return this.priceTypes; }
    }    

    private List<BXCatalogPrice> prices = null;
    protected IList<BXCatalogPrice> Prices
    {
        get { return this.prices; }
    }

    private BXCatalogExtraCollection extras = null;
    protected IList<BXCatalogExtra> Extras
    {
        get { return this.extras; }
    }

    private string GetPriceTypeCode(int id) 
    {
        foreach (BXCatalogPriceType i in this.priceTypes)
            if (i.Id == id)
                return i.Code;
        return "<?>";
    }
}

internal class CatalogItemData
{
    private int id = 0;
    public int Id
    {
        get { return this.id; }
        set { this.id = value; }
    }

    private bool isNew = true;
    public bool IsNew
    {
        get { return this.isNew; }
        set { this.isNew = value; }
    }

    private CatalogPriceTierData[] priceTiers = null;
    public CatalogPriceTierData[] PriceTiers
    {
        get { return this.priceTiers ?? (this.priceTiers = new CatalogPriceTierData[0]); }
        set { this.priceTiers = value; }
    }

    public CatalogPriceData FindPriceById(int id)
    {
        if (this.priceTiers == null || this.priceTiers.Length == 0)
            return null;

        CatalogPriceData price = null;
        for (int i = 0; i < this.priceTiers.Length; i++)
            if ((price = this.priceTiers[i].FindPriceById(id)) != null)
                break;
        return price;
    }
}

internal class CatalogPriceTierData
{
    private List<CatalogPriceData> prices = null;
    public IList<CatalogPriceData> Prices
    {
        get { return this.prices ?? (this.prices = new List<CatalogPriceData>()); }
    }

    public CatalogPriceData FindPriceById(int id) 
    {
        if (this.prices == null || this.prices.Count == 0)
            return null;

        for(int i = 0; i < this.prices.Count; i++)
            if(this.prices[i].Id == id)
                return this.prices[i];

        return null;
    }

    private int qtyFrom = 0;
    public int QtyFrom
    {
        get { return this.qtyFrom; }
        set { this.qtyFrom = value; }
    }
}

[FlagsAttribute()]
internal enum CatalogPriceDataError
{
    None = 0,
    EmptyValue = 0x1,
    InvalidValue = 0x2,
    NegativeOrZeroValue = 0x4,
    EmptyQtyFrom = 0x8,
    InvalidQtyFrom = 0x10,
    NegativeQtyFrom = 0x20
}

internal class CatalogPriceData
{
    private int id = 0;
    public int Id
    {
        get { return this.id; }
        set { this.id = value; }
    }

    private int priceTypeId = 0;
    public int PriceTypeId
    {
        get { return this.priceTypeId; }
        set { this.priceTypeId = value; }
    }

    private bool isBase = false;
    public bool IsBase
    {
        get { return this.isBase; }
        set { this.isBase = value; }
    }

    private int extraId = 0;
    public int ExtraId
    {
        get { return this.extraId; }
        set { this.extraId = value; }
    }

    private decimal val = 0.00m;
    public decimal Value
    {
        get { return this.val; }
        set { this.val = value; }
    }

    public bool TrySetValue(object obj) 
    {
        if (obj == null || string.IsNullOrEmpty(obj.ToString()))
        {
            this.error |= CatalogPriceDataError.EmptyValue;
            return false;
        }

        try
        {
            this.val = Convert.ToDecimal(obj, CultureInfo.InvariantCulture.NumberFormat);
            if (this.val <= 0.0m)
            {
                this.error |= CatalogPriceDataError.NegativeOrZeroValue;
                return false;
            }
            return true;
        }
        catch
        {
            this.error |= CatalogPriceDataError.InvalidValue;
            return false;
        }
    }

    private int qtyFrom = 0;
    public int QtyFrom
    {
        get { return this.qtyFrom; }
        set { this.qtyFrom = value; }
    }

    public bool TrySetQtyFrom(object obj)
    {
        if (obj == null || string.IsNullOrEmpty(obj.ToString()))
        {
            this.error |= CatalogPriceDataError.EmptyQtyFrom;
            return false;
        }

        try
        {
            this.qtyFrom = Convert.ToInt32(obj);
            if (this.qtyFrom < 0)
            {
                this.error |= CatalogPriceDataError.NegativeQtyFrom;
                return false;
            }
            return true;
        }
        catch
        {
            this.error |= CatalogPriceDataError.InvalidQtyFrom;
            return false;
        }
    }

    private CatalogPriceDataError error = CatalogPriceDataError.None;
    internal CatalogPriceDataError Error
    {
        get { return this.error; }
    }
}

internal class CatalogPriceDataJsonConv : JavaScriptConverter
{
    public override IEnumerable<Type> SupportedTypes
    {
        get { return new Type[] { typeof(CatalogPriceData) }; }
    }

    public override IDictionary<string, object> Serialize(object obj, JavaScriptSerializer serializer)
    {
        Dictionary<string, object> result = new Dictionary<string, object>();
        CatalogPriceData item = obj as CatalogPriceData;
        if (item != null)
        {
            result.Add("id", item.Id);
            result.Add("typeId", item.PriceTypeId);
            result.Add("isBase", item.IsBase);
            result.Add("extraId", item.ExtraId);
            result.Add("value", item.Value.ToString("F2", CultureInfo.InvariantCulture.NumberFormat));
            result.Add("qtyFrom", item.QtyFrom.ToString());
        }
        return result;
    }

    public override object Deserialize(IDictionary<string, object> dictionary, Type type, JavaScriptSerializer serializer)
    {
        if (type != typeof(CatalogPriceData))
            return null;

        CatalogPriceData result = new CatalogPriceData();
        object obj;
        if (dictionary.TryGetValue("id", out obj))
            try
            {
                result.Id = Convert.ToInt32(obj);
            }
            catch
            {
            }

        if (dictionary.TryGetValue("typeId", out obj))
            try
            {
                result.PriceTypeId = Convert.ToInt32(obj);
            }
            catch
            {
            }


        if (dictionary.TryGetValue("isBase", out obj))
            try
            {
                result.IsBase = Convert.ToBoolean(obj);
            }
            catch
            {
            }

        if (dictionary.TryGetValue("extraId", out obj))
            try
            {
                result.ExtraId = Convert.ToInt32(obj);
            }
            catch
            {
            }

        result.TrySetValue(dictionary.TryGetValue("value", out obj) ? obj : null);
        result.TrySetQtyFrom(dictionary.TryGetValue("qtyFrom", out obj) ? obj : null);
        return result;
    }
}

internal class CatalogPriceTierDataJsonConv : JavaScriptConverter
{
    public override IEnumerable<Type> SupportedTypes
    {
        get { return new Type[] { typeof(CatalogPriceTierData) }; }
    }

    public override IDictionary<string, object> Serialize(object obj, JavaScriptSerializer serializer)
    {
        Dictionary<string, object> result = new Dictionary<string, object>();
        CatalogPriceTierData item = obj as CatalogPriceTierData;
        if (item != null)
        {
            result.Add("prices", item.Prices);
            result.Add("qtyFrom", item.QtyFrom);
        }
        return result;
    }

    public override object Deserialize(IDictionary<string, object> dictionary, Type type, JavaScriptSerializer serializer)
    {
        if (type != typeof(CatalogPriceTierData))
            return null;

        CatalogPriceTierData result = new CatalogPriceTierData();

        object obj;
        if (dictionary.TryGetValue("prices", out obj))
            foreach(CatalogPriceData price in serializer.ConvertToType<CatalogPriceData[]>(obj))
                result.Prices.Add(price);

        if (dictionary.TryGetValue("qtyFrom", out obj))
            try
            {
                result.QtyFrom = Convert.ToInt32(obj);
            }
            catch
            {
                result.QtyFrom = 0;
            }
        return result;
    }
}

internal class CatalogItemDataJsonConv : JavaScriptConverter
{
    public override IEnumerable<Type> SupportedTypes
    {
        get { return new Type[] { typeof(CatalogItemData) }; }
    }

    public override IDictionary<string, object> Serialize(object obj, JavaScriptSerializer serializer)
    {
        Dictionary<string, object> result = new Dictionary<string, object>();
        CatalogItemData item = obj as CatalogItemData;
        if (item != null)
        {
            result.Add("id", item.Id);
            result.Add("isNew", item.IsNew);
            result.Add("priceTiers", item.PriceTiers);
        }
        return result;
    }

    public override object Deserialize(IDictionary<string, object> dictionary, Type type, JavaScriptSerializer serializer)
    {
        if (type != typeof(CatalogItemData))
            return null;

        CatalogItemData result = new CatalogItemData();

        object obj;
        if (dictionary.TryGetValue("id", out obj))
            result.Id = Convert.ToInt32(obj);
        if (dictionary.TryGetValue("isNew", out obj))
            result.IsNew = Convert.ToBoolean(obj);
        if (dictionary.TryGetValue("priceTiers", out obj))
            result.PriceTiers = serializer.ConvertToType<CatalogPriceTierData[]>(obj);
        return result;
    }
}