﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using Bitrix.Components;
using Bitrix.IBlock;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix.Services;
using Bitrix.Security;
using Bitrix.Configuration;
using System.Text;
using System.Security.Cryptography;
using System.IO;
using System.Web.Hosting;
using System.Threading;
using System.Xml;
using Bitrix.Components.Editor;

namespace Bitrix.Catalog.Components
{
	public partial class CatalogImport1CComponent : BXComponent
	{
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			//Режим разработки
			if (IsComponentDesignMode || (BXConfigurationUtility.IsDesignMode && Bitrix.UI.TemplateRequisite.GetCurrentPublicPanelVisiblity(Page)))
				return;

			//Текущий режим
			string mode = Request["mode"];
			if (string.IsNullOrEmpty(mode))
				return;

			try
			{
				if (string.Equals(mode, "checkauth", StringComparison.Ordinal))
					ProcessAuthentication();
				else if (!IsAuthentificated)
					CompleteResponse(new string[] { GetMessageRaw("Error.UserIsNotAuthentificated") }, CompleteResponseStatus.Failure);
				else if(!IsAuthorized)
					CompleteResponse(new string[] { string.Format(GetMessageRaw("Error.UserIsNotAuthorized"), UserName) }, CompleteResponseStatus.Failure);
				else if (string.Equals(mode, "init", StringComparison.Ordinal))
					ProcessInitialization();
				else if (string.Equals(mode, "file", StringComparison.Ordinal))
					ProcessFileUploading();
				else if (string.Equals(mode, "import", StringComparison.Ordinal))
					ProcessImport();
			}
			catch (ThreadAbortException)
			{
			}
			catch (Exception ex)
			{
				BXLogService.LogAll(ex, BXLogMessageType.Error, "Catalog.Import.1C");
				CompleteResponse(new string[] { GetMessageRaw("Error.GeneralError") }, CompleteResponseStatus.Failure);
			}
		}

		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);
			IncludeComponentTemplate();
		}

		private StringBuilder md5Sb = null;
		private MD5 md5;
		private string Md5(string input)
		{
			if (this.md5 == null)
				this.md5 = MD5.Create();

			if (this.md5Sb == null)
				this.md5Sb = new StringBuilder();
			else if (this.md5Sb.Length != 0)
				this.md5Sb.Length = 0;

			foreach (byte b in this.md5.ComputeHash(Encoding.Unicode.GetBytes(input)))
				this.md5Sb.Append(b.ToString("x2"));

			return this.md5Sb.ToString();
		}

		#region Processing
		private void ProcessAuthentication()
		{
			string header = Page.Request.Headers["Authorization"];
			if (string.IsNullOrEmpty(header) || !header.StartsWith("Basic "))
			{
				BXLogService.LogAll(GetMessageRaw("MessageTitle.Authentification"), string.Format(GetMessageRaw("Message.AuthentificationFailedNoData"), header), BXLogMessageType.Info, "CatalogImport1C");
				CompleteResponse(new string[] { GetMessageRaw("Error.UserIsNotAuthentificated") }, CompleteResponseStatus.Failure);
			}

			string value = Encoding.GetEncoding(1251).GetString(Convert.FromBase64String(header.Substring("Basic".Length).Trim()));
			int index = value.IndexOf(':');
			string login = value.Remove(index),
				password = value.Substring(index + 1),
				provider;
			if (!BXAuthentication.Authenticate(login, password, out provider))
			{
				BXLogService.LogAll(GetMessageRaw("MessageTitle.Authentification"), string.Format(GetMessageRaw("Message.AuthentificationFailed"), provider, login), BXLogMessageType.Info, "CatalogImport1C");
				CompleteResponse(new string[] { GetMessageRaw("Error.UserIsNotAuthentificated") }, CompleteResponseStatus.Failure);
			}

			BXLogService.LogAll(GetMessageRaw("MessageTitle.Authentification"), string.Format(GetMessageRaw("Message.AuthentificationSuccessfull"), login), BXLogMessageType.Info, "CatalogImport1C");
			BXOptionManager.SetOption(BXCatalogModuleConfiguration.Id, "ctlgImp1c:lgn:" + Md5(Parent.TemplateControl.AppRelativeVirtualPath), login);
			BXOptionManager.SetOption(BXCatalogModuleConfiguration.Id, "ctlgImp1c:prvdr:" + Md5(Parent.TemplateControl.AppRelativeVirtualPath), provider);

			HttpCookie cookie = BXAuthentication.GetAuthCookie(login, provider, false);
			CompleteResponse(new string[] { cookie.Name, cookie.Value }, CompleteResponseStatus.Success);
		}
		private void ProcessInitialization()
		{
			try
			{
				BXOptionManager.DeleteOption(optionEntity, ContextOptionName, BXSite.Current.Id);

				DirectoryInfo di = new DirectoryInfo(HostingEnvironment.MapPath(UploadPath));
				if (!di.Exists)
					di.Create();
				else
				{
					foreach (FileInfo f in di.GetFiles())
						f.Delete();

					foreach (DirectoryInfo d in di.GetDirectories())
						d.Delete(true);
				}
			}
			catch (Exception ex)
			{
				BXLogService.LogAll(ex, BXLogMessageType.Error, "Catalog.Import.1C");
				CompleteResponse(new string[] { GetMessageRaw("Error.TempDirectory") }, CompleteResponseStatus.Failure);
				return;
			}
			CompleteResponse(new string[] { "zip=no", string.Concat("file_limit=", FileChunkMaxSize.ToString()) }, CompleteResponseStatus.None);
		}
		private void ProcessFileUploading()
		{
			string fileName = Request["filename"];
			if (string.IsNullOrEmpty(fileName))
			{
				CompleteResponse(new string[] { GetMessageRaw("Error.FileNameIsNotFound") }, CompleteResponseStatus.Failure);
				return;
			}
			try
			{
				FileInfo fi = new FileInfo(HostingEnvironment.MapPath(VirtualPathUtility.Combine(UploadPath, fileName)));
				DirectoryInfo di = fi.Directory;
				if (!di.Exists)
					di.Create();

				using (FileStream fs = fi.OpenWrite())
				{
					fs.Position = fs.Length;
					byte[] byteAry = Request.BinaryRead(Request.TotalBytes);
					fs.Write(byteAry, 0, byteAry.Length);
				}
			}
			catch (ThreadAbortException)
			{
			}
			catch (Exception ex)
			{
				BXLogService.LogAll(ex, BXLogMessageType.Error, "Catalog.Import.1C");
				CompleteResponse(new string[] { string.Format(GetMessageRaw("Error.FileUploadingFailed"), fileName) }, CompleteResponseStatus.Failure);
				return;
			}
			CompleteResponse(null, CompleteResponseStatus.Success);
		}

		private string contextOptionName = null;
		private string ContextOptionName
		{
			get { return this.contextOptionName ?? (this.contextOptionName = string.Concat("context:user:", BXIdentity.Current.Id)); }
		}

		private static readonly string optionEntity = "BX_COMPONENT_CATALOG_IMPORT_1C";

		private void ProcessImport()
		{
			string fileName = Request["filename"];
			if (string.IsNullOrEmpty(fileName))
			{
				CompleteResponse(new string[] { GetMessageRaw("Error.FileNameIsNotFound") }, CompleteResponseStatus.Failure);
				return;
			}

			try
			{
				BXCmlImportSettings settings = new BXCmlImportSettings();
				settings.BlockTypeId = IBlockTypeId;
				foreach (string siteId in SiteIds)
					if (!string.IsNullOrEmpty(siteId))
						settings.SiteList.Add(siteId);

				settings.FileNames = new string[] { fileName };
				settings.EnableIterativeProcessing = EnableIterativeProcessing;
				settings.IterationLength = IterationLength;
				settings.ElementOmissionProcessing = ElementAction;
				settings.SectionOmissionProcessing = SectionAction;

				BXCmlImportContext context = null;
				string contextXml = BXOptionManager.GetOptionString(optionEntity, ContextOptionName, string.Empty, BXSite.Current.Id);
				if(!string.IsNullOrEmpty(contextXml))
				{
					using(TextReader r = new StringReader(contextXml))
					{
						XmlReaderSettings rs = new XmlReaderSettings();
						rs.CloseInput = false;
						rs.ConformanceLevel = ConformanceLevel.Fragment;
						using(XmlReader xr = XmlReader.Create(r, rs))
						{
							xr.MoveToContent();
							context = new BXCmlImportContext();
							context.ReadXml(xr);
						}
					}
				}

				if(context == null)
					context = new BXCmlImportContext();

				(new BXCmlImport()).Process(settings, context);

				StringBuilder sb = new StringBuilder();
				XmlWriterSettings ws = new XmlWriterSettings();
				ws.ConformanceLevel = ConformanceLevel.Fragment;
				ws.OmitXmlDeclaration = true;
				ws.Indent = false;
				using(XmlWriter xw = XmlWriter.Create(sb, ws))
					context.WriteXml(xw);

				BXOptionManager.SetOptionString(optionEntity, ContextOptionName, sb.ToString(), BXSite.Current.Id);
				CompleteResponseStatus status = CompleteResponseStatus.None;
				if(!context.Progress.IsSuccessful)
					status = CompleteResponseStatus.Failure;
				else if(context.Progress.IsStageCompleted)
					status = CompleteResponseStatus.Success;
				else
					status = CompleteResponseStatus.Progress;

				CompleteResponse(new string[] { context.Progress.Status }, status);
			}
			catch(ThreadAbortException)
			{
			}
			catch(Exception ex)
			{
				BXLogService.LogAll(ex, BXLogMessageType.Error, "Catalog.Import.1C");
				CompleteResponse(new string[] { string.Format(GetMessageRaw("Error.FileImportFailed"), fileName) }, CompleteResponseStatus.Failure);
			}
		}
		#endregion

		private enum CompleteResponseStatus
		{
			None = 0,
			Progress,
			Success,
			Failure
		}

		private void CompleteResponse(string[] messageAry, CompleteResponseStatus status)
		{
			HttpResponse r = Response;
			r.Buffer = true;
			r.Clear();
			r.ContentType = "text/plain";
			r.ContentEncoding = Encoding.GetEncoding(1251);

			if (status == CompleteResponseStatus.Progress)
				r.Write("progress\n");
			else if (status == CompleteResponseStatus.Success)
				r.Write("success\n");
			else if (status == CompleteResponseStatus.Failure)
				r.Write("failure\n");

			if (messageAry != null && messageAry.Length > 0)
				for (int i = 0; i < messageAry.Length; i++)
				{
					r.Write(messageAry[i]);
					r.Write("\n");
				}
			r.End();
		}

		public bool IsAuthentificated
		{
			get { return BXPrincipal.Current != null && BXPrincipal.Current.Identity != null && BXPrincipal.Current.Identity.IsAuthenticated; }
		}

		public bool IsAuthorized
		{
			get { return BXPrincipal.Current != null && BXPrincipal.Current.IsCanOperate(BXCatalog.Operations.DataExchange); }
		}

		public string UserName
		{
			get { return BXPrincipal.Current != null && BXPrincipal.Current.Identity != null ? BXPrincipal.Current.Identity.Name : string.Empty; }
		}

		private static string uploadPath = null;
		public static string UploadPath
		{
			get { return uploadPath ?? (uploadPath = VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine(BXConfigurationUtility.Options.UploadFolderPath, "1c_catalog"))); }
		}

		protected override void PreLoadComponentDefinition()
		{
			Title = GetMessageRaw("Title");
			Description = GetMessageRaw("Description");
			Icon = "images/1c-import.gif";

			Group = new BXComponentGroup("catalogue", GetMessageRaw("Group"), 10, BXComponentGroup.Content);

			BXCategory mainCategory = BXCategory.Main,
				addSettingsCategory = BXCategory.AdditionalSettings,
				imageSettingsCategory = new BXCategory(GetMessageRaw("Category.ImageSettings"), "ImageSettings", 300);

			#region Main
			ParamsDefinition.Add(
				"IBlockTypeId",
				new BXParamSingleSelection(
					GetMessageRaw("Param.InfoBlockType"), string.Empty, mainCategory));

			ParamsDefinition.Add(
				"SiteIds",
				new BXParamMultiSelection(
					GetMessageRaw("Param.Sites"), string.Empty, mainCategory));

			ParamsDefinition.Add(
				"EnableIterativeProcessing",
				new BXParamYesNo(
					GetMessageRaw("Param.EnableIterativeProcessing"), false, mainCategory,
					new ParamClientSideActionGroupViewSwitch(ClientID,"EnableIterativeProcessing", "Enable","Disable")));

			ParamsDefinition.Add(
				"IterationLength",
				new BXParamText(
					GetMessageRaw("Param.IterationLength"), "30", mainCategory,
					new ParamClientSideActionGroupViewMember(ClientID, "IterationLength", new string[]{ "Enable" })));
			#endregion

			#region ImageSettings
			ParamsDefinition.Add(
				"EnableCreationOfPreviewImage",
				new BXParamYesNo(
					GetMessageRaw("Param.EnableCreationOfPreviewImage"), true, imageSettingsCategory));

			ParamsDefinition.Add(
				"PreviewMaxImageWidth",
				new BXParamText(
					GetMessageRaw("Param.PreviewImageMaxWidth"), "100", imageSettingsCategory));

			ParamsDefinition.Add(
				"PreviewMaxImageHeight",
				new BXParamText(
					GetMessageRaw("Param.PreviewImageMaxHeight"), "100", imageSettingsCategory));

			ParamsDefinition.Add(
				"EnableDetailImageResize",
				new BXParamYesNo(
					GetMessageRaw("Param.EnableDetailImageResize"), true, imageSettingsCategory));

			ParamsDefinition.Add(
				"DetailImageMaxWidth",
				new BXParamText(
					GetMessageRaw("Param.DetailImageMaxWidth"), "300", imageSettingsCategory));

			ParamsDefinition.Add(
				"DetailImageMaxHeight",
				new BXParamText(
					GetMessageRaw("Param.DetailImageMaxHeight"), "300", imageSettingsCategory));
			#endregion

			#region AddSettings
			ParamsDefinition.Add(
				"ElementAction",
				new BXParamSingleSelection(
					GetMessageRaw("Param.ElementAction"), BXCmlOmissionProcessing.Delete.ToString("G"), mainCategory));

			ParamsDefinition.Add(
				"SectionAction",
				new BXParamSingleSelection(
					GetMessageRaw("Param.SectionAction"), BXCmlOmissionProcessing.Delete.ToString("G"), mainCategory));

			ParamsDefinition.Add(
				"FileChunkMaxSize",
				new BXParamText(
					GetMessageRaw("Param.FileChunkMaxSize"), "204800", mainCategory));
			#endregion
		}

		protected override void LoadComponentDefinition()
		{
			#region IBlockType
			List<BXParamValue> typeValues = ParamsDefinition["IBlockTypeId"].Values;
			typeValues.Clear();
			typeValues.Add(new BXParamValue(GetMessageRaw("SelectIBlockType"), string.Empty));
			BXIBlockTypeCollection blockTypes = BXIBlockType.GetList(
				null,
				new BXOrderBy(new BXOrderByPair(BXIBlockType.Fields.Name, BXOrderByDirection.Asc)),
				null,
				null,
				BXTextEncoder.EmptyTextEncoder);

			foreach (BXIBlockType blockType in blockTypes)
				typeValues.Add(new BXParamValue(blockType.Translations[BXLoc.CurrentLocale].Name, blockType.Id.ToString()));

			ParamsDefinition["IBlockTypeId"].RefreshOnDirty = true;
			#endregion

			#region Sites
			List<BXParamValue> siteValues = ParamsDefinition["SiteIds"].Values;
			siteValues.Clear();
			BXSiteCollection sites = BXSite.GetList(null,
				new BXOrderBy(new BXOrderByPair(BXSite.Fields.Sort, BXOrderByDirection.Asc)),
				new BXSelect(BXSite.Fields.ID, BXSite.Fields.Name),
				null,
				BXTextEncoder.EmptyTextEncoder);

			foreach (BXSite site in sites)
				siteValues.Add(new BXParamValue(site.Name, site.Id));
			#endregion

			#region ElementAction, SectionAction
			List<BXParamValue> elementActionValues = ParamsDefinition["ElementAction"].Values,
				sectionActionValues = ParamsDefinition["SectionAction"].Values;

			elementActionValues.Clear();
			sectionActionValues.Clear();

			BXParamValue[] actionAry = new BXParamValue[]
			{
				new BXParamValue(GetMessageRaw("Option.NoAction"), BXCmlOmissionProcessing.None.ToString("F")),
				new BXParamValue(GetMessageRaw("Option.Deactivate"), BXCmlOmissionProcessing.Deactivate.ToString("F")),
				new BXParamValue(GetMessageRaw("Option.Delete"), BXCmlOmissionProcessing.Delete.ToString("F"))
			};

			foreach (BXParamValue action in actionAry)
			{
				elementActionValues.Add(action);
				sectionActionValues.Add(action);
			}
			#endregion
		}

		public int IBlockTypeId
		{
			get { return Parameters.GetInt("IBlockTypeId", 0); }
			set { Parameters["IBlockTypeId"] = (value > 0 ? value : 0).ToString(); }
		}

		private string[] siteIds = null;
		public string[] SiteIds
		{
			get
			{
				if (this.siteIds != null)
					return this.siteIds;

				List<string> lst = Parameters.GetListString("SiteIds", null);
				return this.siteIds = lst != null ? lst.ToArray() : new string[0];
			}
			set { Parameters["SiteIds"] = BXStringUtility.ListToCsv(this.siteIds = value ?? new string[0]); }
		}

		private BXCmlOmissionProcessing? elementAction = null;
		public BXCmlOmissionProcessing ElementAction
		{
			get { return (this.elementAction ?? (this.elementAction = Parameters.GetEnum<BXCmlOmissionProcessing>("ElementAction", BXCmlOmissionProcessing.Delete))).Value; }
			set { Parameters["ElementAction"] = (this.elementAction = value).Value.ToString("G"); }
		}

		private BXCmlOmissionProcessing? sectionAction = null;
		public BXCmlOmissionProcessing SectionAction
		{
			get { return (this.sectionAction ?? (this.sectionAction = Parameters.GetEnum<BXCmlOmissionProcessing>("SectionAction", BXCmlOmissionProcessing.Delete))).Value; }
			set { Parameters["SectionAction"] = (this.sectionAction = value).Value.ToString("G"); }
		}

		public int FileChunkMaxSize
		{
			get { return Parameters.GetInt("FileChunkMaxSize", 204800); }
			set { Parameters["FileChunkMaxSize"] = (value > 0 ? value : 204800).ToString(); }
		}

		public bool EnableIterativeProcessing
		{
			get { return Parameters.GetBool("EnableIterativeProcessing", false); }
			set { Parameters["EnableIterativeProcessing"] = value.ToString(); }
		}

		public int IterationLength
		{
			get { return Parameters.GetInt("IterationLength", 30); }
			set { Parameters["IterationLength"] = (value > 0 ? value : 30).ToString(); }
		}
	}

	public class CatalogImport1CTemplate : BXComponentTemplate<CatalogImport1CComponent>
	{
	}
}
