﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.UI;
using Bitrix.Services.Text;
using Bitrix;
using Bitrix.DataLayer;
using Bitrix.Modules;
using System.Collections.Generic;
using Bitrix.Security;
using Bitrix.Configuration;
using System.Globalization;
using Bitrix.Currency;
using Bitrix.Services;

public partial class bitrix_admin_CurrencyEdit : BXAdminPage
{
	private int? entityId = null;
	protected int EntityId
	{
		get
		{
			if (this.entityId.HasValue)
				return this.entityId.Value;

			string s = Request.QueryString["id"];
			if (string.IsNullOrEmpty(s))
				this.entityId = 0;
			else
			{
				int i;
				this.entityId = int.TryParse(s, out i) ? i : 0;
			}
			return this.entityId.Value;
		}
	}

	private CurrencyEditorMode editorMode = CurrencyEditorMode.Creation;
	protected CurrencyEditorMode EditorMode
	{
		get { return this.editorMode; }
	}

	private BXCurrency entity;
	protected BXCurrency Entity
	{
		get { return this.entity; }
	}

	private List<CurrencyLocalizationInfoWrapper> localizationInfos = null;
	public IList<CurrencyLocalizationInfoWrapper> LocalizationInfos
	{
		get { return this.localizationInfos ?? (this.localizationInfos = new List<CurrencyLocalizationInfoWrapper>()); }
	}

	private CurrencyLocalizationInfoWrapper currentLocalizationInfo = null;
	public CurrencyLocalizationInfoWrapper CurrentLocalizationInfo
	{
		get { return this.currentLocalizationInfo; }
	}

	private CurrencyEditorError editorError = CurrencyEditorError.None;
	protected CurrencyEditorError EditorError
	{
		get { return this.editorError; }
	}

	private string errorText = string.Empty;
	public string ErrorText
	{
		get { return this.errorText; }
	}

	protected override string BackUrl
	{
		get { return !string.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "CurrencyList.aspx"; }
	}

	protected override void OnInit(EventArgs e)
	{
		if (!BXPrincipal.Current.IsCanOperate(BXCurrency.Operations.AdminManagement))
		    BXAuthentication.AuthenticationRequired();

		base.OnInit(e);
	}

	protected override void OnLoad(EventArgs e)
	{
		TryLoadEntity();
		BXCurrency entity = Entity;
		string title = EditorMode == CurrencyEditorMode.Modification ? string.Format(GetMessage("PageTitle.Edit"), entity != null ? HttpUtility.HtmlEncode(entity.GetName()) : "?") : GetMessage("PageTitle.Create");

		if (EditorMode == CurrencyEditorMode.Creation)
		{
			AddButton.Visible = false;
			DeleteButton.Visible = false;
		}
		MasterTitle = Page.Title = title;
		base.OnLoad(e);
	}

	private void TryLoadEntity()
	{
		int id = EntityId;
		if (id <= 0)
		{
			this.editorMode = CurrencyEditorMode.Creation;
			this.entity = new BXCurrency();
		}
		else
		{
			this.editorMode = CurrencyEditorMode.Modification;
			if ((this.entity = BXCurrency.GetById(id, BXTextEncoder.EmptyTextEncoder)) == null)
			{
				this.errorText = string.Format(GetMessageRaw("Error.UnableToFindEntity"), id);
				this.editorError = CurrencyEditorError.IsNotFound;
				return;
			}
		}

		if (!IsPostBack)
		{
			IsNational.Enabled = !(IsNational.Checked = this.entity.IsNational);

			IsoCode.Text = this.entity.Iso4217AlphaCode;
			Sort.Text = this.entity.Sort.ToString();
			XmlId.Text = this.entity.XmlId;
		}
		else
		{
            if (!this.entity.IsNational) //флаг национальной валюты нельзя снять
                this.entity.IsNational = IsNational.Checked;

			this.entity.Iso4217AlphaCode = IsoCode.Text;

			if (string.IsNullOrEmpty(Sort.Text))
				this.entity.Sort = 0;
			else
			{
				int i;
				this.entity.Sort = int.TryParse(Sort.Text, out i) ? i : 0;
			}
			this.entity.XmlId = XmlId.Text;

			List<BXCurrencyLocalizationInfo> infoLst = new List<BXCurrencyLocalizationInfo>();
			foreach (string locale in BXLoc.Locales)
			{
				BXCurrencyLocalizationInfo info = new BXCurrencyLocalizationInfo(locale);
				info.Name = Request.Form[UniqueID + "$LocalizationInfoName$" + locale];
				info.Symbol = Request.Form[UniqueID + "$LocalizationInfoSymbol$" + locale];
				info.Format = Request.Form[UniqueID + "$LocalizationInfoFormat$" + locale];

				if(info.IsEmpty())
					continue;

				infoLst.Add(info);
			}

			this.entity.LocalizationInfos = infoLst.ToArray();
		}

		this.localizationInfos = new List<CurrencyLocalizationInfoWrapper>();
		foreach (string locale in BXLoc.Locales)
		{
			CurrencyLocalizationInfoWrapper locInfo = new CurrencyLocalizationInfoWrapper(Array.Find<BXCurrencyLocalizationInfo>(this.entity.LocalizationInfos, item => string.Equals(item.LocaleName, locale, StringComparison.OrdinalIgnoreCase)) ?? new BXCurrencyLocalizationInfo(locale), this);
			if (this.currentLocalizationInfo == null && string.Equals(locale, BXLoc.CurrentLocale, StringComparison.Ordinal))
				this.currentLocalizationInfo = locInfo;
			this.localizationInfos.Add(locInfo);
		}
	}

	private void TrySaveEntity()
	{
		if (this.entity == null)
			return;

		try
		{
			this.entity.Save();
			this.entityId = this.entity.Id;
		}
		catch (Exception exc)
		{
			this.errorText = exc.Message;
			this.editorError = EditorMode == CurrencyEditorMode.Creation ? CurrencyEditorError.CreationFailed : CurrencyEditorError.ModificationFailed;
		}
	}

	protected override void OnPreRender(EventArgs e)
	{
		if (EditorError != CurrencyEditorError.None)
		{
			errorMessage.AddErrorMessage(ErrorText);
			if (EditorMode == CurrencyEditorMode.Modification && EditorError == CurrencyEditorError.IsNotFound)
				TabControl.Visible = false;
		}

		base.OnPreRender(e);
	}

	protected void OnCurrencyEditCommand(object sender, BXTabControlCommandEventArgs e)
	{
		switch (e.CommandName)
		{
			case "save":
				{
					if (IsValid)
					{
						TrySaveEntity();
						if (EditorError == CurrencyEditorError.None)
							GoBack();
					}
				}
				break;
			case "apply":
				{
					if (IsValid)
					{
						TrySaveEntity();
						if (EditorError == CurrencyEditorError.None)
							Response.Redirect(string.Format("CurrencyEdit.aspx?id={0}&tabindex={1}", EntityId.ToString(), TabControl.SelectedIndex));
					}
				}
				break;
			case "cancel":
				GoBack();
				break;
		}
	}

	protected void OnToolBarButtonClick(object sender, CommandEventArgs e)
	{
		if (!BXPrincipal.Current.IsCanOperate(BXCurrency.Operations.AdminManagement))
		    BXAuthentication.AuthenticationRequired();

		if (e.CommandName == "delete")
		{
			try
			{
				if (this.entity != null)
					this.entity.Delete();

				GoBack();
			}
			catch (Exception ex)
			{
				this.errorText = ex.Message;
				this.editorError = CurrencyEditorError.DeletionFailed;
			}
		}
	}
}

public class CurrencyLocalizationInfoWrapper
{
	private BXCurrencyLocalizationInfo charge = null;
	private BXAdminPage page = null;

	public CurrencyLocalizationInfoWrapper(BXCurrencyLocalizationInfo charge, BXAdminPage page)
	{
		if (charge == null)
			throw new ArgumentNullException("charge");
		this.charge = charge;

		if (page == null)
			throw new ArgumentNullException("page");
		this.page = page;
	}

	public string LocaleName
	{
		get { return this.charge.LocaleName; }
	}

	public string Name
	{
		get { return this.charge.Name; }
	}

	public string Symbol
	{
		get { return this.charge.Symbol; }
	}

	public string Format
	{
		get { return this.charge.Format; }
	}
}

public enum CurrencyEditorMode
{
	Creation = 1,
	Modification = 2
}

public enum CurrencyEditorError
{
	None = 0,
	IsNotFound = -1,
	CreationFailed = -2,
	ModificationFailed = -3,
	DeletionFailed = -4
}