﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.UI;
using Bitrix.Services.Text;
using Bitrix;
using Bitrix.DataLayer;
using Bitrix.Modules;
using System.Collections.Generic;
using Bitrix.Security;
using Bitrix.Configuration;
using System.Globalization;
using Bitrix.Currency;


public partial class bitrix_admin_CurrencyExchangeRateEditorEdit : BXAdminPage
{
	private int? entityId = null;
	protected int EntityId
	{
        get 
        {
            if (this.entityId.HasValue)
                return this.entityId.Value;

            string s = Request.QueryString["id"];
            if (string.IsNullOrEmpty(s))
                this.entityId = 0;
            else
            {
                int i;
                this.entityId = int.TryParse(s, out i) ? i : 0;
            }
            return this.entityId.Value; 
        }
	}

    private BXExchangeRate entity;
    protected BXExchangeRate Entity
    {
        get { return this.entity; }
    }

    private CurrencyExchRateEditorMode editorMode = CurrencyExchRateEditorMode.Creation;
    protected CurrencyExchRateEditorMode EditorMode
    {
        get { return this.editorMode; }
    }

    private CurrencyExchRateEditorError editorError = CurrencyExchRateEditorError.None;
    protected CurrencyExchRateEditorError EditorError
    {
        get { return this.editorError; }
    }

    private string errorText = string.Empty;
    public string ErrorText
    {
        get { return this.errorText; }
    }

	protected override string BackUrl
	{
		get { return !string.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "CurrencyExchangeRateList.aspx"; }
	}

    protected override void OnInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXCurrency.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        BXCurrencyCollection currencies = BXCurrency.GetList(
            null,
            new BXOrderBy(
				new BXOrderByPair(BXCurrency.Fields.Sort, BXOrderByDirection.Asc),
				new BXOrderByPair(BXCurrency.Fields.Id, BXOrderByDirection.Asc)
				),
            new BXSelect(
				BXSelectFieldPreparationMode.Normal, 
				BXCurrency.Fields.Id, 
				BXCurrency.Fields.Iso4217AlphaCode,
				BXCurrency.Fields.LocalizationInfos, 
				BXCurrency.Fields.IsNational),
            null,
            BXTextEncoder.EmptyTextEncoder
            );

        foreach (BXCurrency currency in currencies)
        {
			if(currency.IsNational)
				continue;

			string code = currency.Iso4217AlphaCode, name = currency.GetName();
            CurrencyList.Items.Add(new ListItem(string.Equals(code, name, StringComparison.InvariantCulture) ? code : string.Concat(code, " (", name, ")"), currency.Id.ToString()));
        }

        base.OnInit(e);
    }

    protected override void OnLoad(EventArgs e)
    {
        TryLoadEntity();
		if(EditorMode == CurrencyExchRateEditorMode.Modification)
		{
			string currencyCode = "-", issued = "-";
			if(this.entity != null)
			{
				if(this.entity.Currency != null)
					currencyCode = this.entity.Currency.Iso4217AlphaCode;

				issued = this.entity.Issued.ToString("d");
			}
			MasterTitle = Page.Title = string.Format(GetMessage("PageTitle.Edit"), currencyCode, BXCurrency.NationalCurrency != null ? BXCurrency.NationalCurrency.Iso4217AlphaCode : "-", issued);
		}
        else
		{
			MasterTitle = Page.Title = GetMessage("PageTitle.Create");

			AddButton.Visible = false;
			DeleteButton.Visible = false;
		}
        base.OnLoad(e);
    }

    private void TryLoadEntity()
	{
        if (EntityId <= 0)
        {
            this.editorMode = CurrencyExchRateEditorMode.Creation;
            this.entity= new BXExchangeRate();
        }
        else
        {
            this.editorMode = CurrencyExchRateEditorMode.Modification;
            if ((this.entity = BXExchangeRate.GetById(EntityId, BXTextEncoder.EmptyTextEncoder)) == null)
            {
                this.errorText = string.Format(GetMessageRaw("Error.UnableToFindEntity"), EntityId);
                this.editorError = CurrencyExchRateEditorError.IsNotFound;
                return;
            }
        }

        if (!IsPostBack)
        {
			IssuedDate.Date = this.entity.Issued.Date;

			ListItem currencyItem = CurrencyList.Items.FindByValue(this.entity.CurrencyId.ToString());
			if(currencyItem != null)
				currencyItem.Selected = true;
			else
				CurrencyList.SelectedIndex = -1;

            RateValue.Text = this.entity.Value.ToString("F4", CultureInfo.InvariantCulture.NumberFormat);
            XmlId.Text = this.entity.XmlId;
        }
        else
        {
			this.entity.Issued = IssuedDate.Date.Date;
			int currencyId;
			this.entity.CurrencyId = int.TryParse(CurrencyList.SelectedValue, out currencyId) && currencyId >= 0 ? currencyId : 0;

            if (string.IsNullOrEmpty(RateValue.Text))
                this.entity.Value = 0.0m;
            else
            {
                decimal d;
                this.entity.Value = decimal.TryParse(RateValue.Text, NumberStyles.AllowDecimalPoint, CultureInfo.InvariantCulture.NumberFormat, out d) && d >= 0m ? d : 0m;
            }
            this.entity.XmlId = XmlId.Text;
        }
	}

    private void TrySaveEntity()
    {
        if (this.entity == null)
            return;

        try
        {
            this.entity.Save();
            this.entityId = this.entity.Id;
        }
        catch (Exception exc)
        {
            this.errorText= exc.Message;
            this.editorError = EditorMode == CurrencyExchRateEditorMode.Creation ? CurrencyExchRateEditorError.CreationFailed : CurrencyExchRateEditorError.ModificationFailed; 
        }
    }

    protected override void OnPreRender(EventArgs e)
    {
        if (EditorError != CurrencyExchRateEditorError.None)
        {
            errorMessage.AddErrorMessage(ErrorText);
            if (EditorMode == CurrencyExchRateEditorMode.Modification && EditorError == CurrencyExchRateEditorError.IsNotFound)
                TabControl.Visible = false;
        }

        base.OnPreRender(e);
    }

    protected void OnCurrencyExchRateEdit(object sender, BXTabControlCommandEventArgs e)
	{
		switch (e.CommandName)
		{
            case "save":
                {
                    if (IsValid)
                    {
                        TrySaveEntity();
                        if (EditorError == CurrencyExchRateEditorError.None)
                            GoBack(); 
                    }
                }
                break;
            case "apply":
                {
                    if (IsValid)
                    {
                        TrySaveEntity();
                        if (EditorError == CurrencyExchRateEditorError.None)
                            Response.Redirect(string.Format("CurrencyExchangeRateEdit.aspx?id={0}&tabindex={1}", EntityId.ToString(), TabControl.SelectedIndex));
                    }
                }
                break;
			case "cancel":
				GoBack();
				break;
		}
	}

	protected void OnToolBarButtonClick(object sender, CommandEventArgs e)
	{
        if (!BXPrincipal.Current.IsCanOperate(BXCurrency.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

		if (e.CommandName == "delete")
		{
			try
			{
                if (this.entity != null)
                    this.entity.Delete();

				GoBack();
			}
			catch (Exception ex)
			{
                this.errorText = ex.Message;
                this.editorError = CurrencyExchRateEditorError.DeletionFailed;
			}
		}
	}
}

public enum CurrencyExchRateEditorMode
{
    Creation = 1,
    Modification = 2
}

public enum CurrencyExchRateEditorError
{
    None = 0,
    IsNotFound = -1,
    CreationFailed = -2,
    ModificationFailed = -3,
    DeletionFailed = -4
}