﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using Bitrix.UI;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix;
using System.Text;
using Bitrix.Security;
using Bitrix.Main;
using System.Globalization;
using Bitrix.Currency;

public partial class bitrix_admin_CurrencyExchangeRateList : BXAdminPage
{

    protected override void OnPreInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXCurrency.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnPreInit(e);
    }

	protected override void OnInit(EventArgs e)
	{
		BXCurrencyCollection currencies = BXCurrency.GetList(
			null,
			new BXOrderBy(
				new BXOrderByPair(BXCurrency.Fields.Sort, BXOrderByDirection.Asc),
				new BXOrderByPair(BXCurrency.Fields.Id, BXOrderByDirection.Asc)),
			new BXSelect(
				BXSelectFieldPreparationMode.Normal, 
				BXCurrency.Fields.Id, 
				BXCurrency.Fields.Iso4217AlphaCode,
 				BXCurrency.Fields.LocalizationInfos,
				BXCurrency.Fields.IsNational),
			null,
			BXTextEncoder.EmptyTextEncoder
			);

		CurrencyFilter.Values.Add(new ListItem(GetMessageRaw("CurrencyFilter.NotSelected"), string.Empty));
		foreach (BXCurrency currency in currencies)
		{
			if (currency.IsNational)
				continue;
			CurrencyFilter.Values.Add(new ListItem(currency.GetName(), currency.Id.ToString()));
		}
		base.OnInit(e);
	}

    protected override void OnLoad(EventArgs e)
    {
        MasterTitle = Page.Title = GetMessage("PageTitle");
        base.OnLoad(e);
    }

    protected void ItemGrid_Select(object sender, BXSelectEventArgs e)
	{
		BXExchangeRateCollection c = BXExchangeRate.GetList(
			new BXFilter(ItemFilter.CurrentFilter, BXExchangeRate.Fields),
			new BXOrderBy(BXExchangeRate.Fields, string.IsNullOrEmpty(e.SortExpression) ? "Id" : e.SortExpression),
			new BXSelect(
				BXSelectFieldPreparationMode.Normal,
				BXExchangeRate.Fields.Id,
				BXExchangeRate.Fields.Currency.Id,
				BXExchangeRate.Fields.Currency.Iso4217AlphaCode,
				BXExchangeRate.Fields.Issued,
				BXExchangeRate.Fields.Value),
			null,
			BXTextEncoder.EmptyTextEncoder);

		List<CurrencyExchRateWrapper> list = new List<CurrencyExchRateWrapper>();
        foreach (BXExchangeRate rate in c)
			list.Add(new CurrencyExchRateWrapper(rate, this));

        e.Data = list;
	}

    protected void ItemGrid_SelectCount(object sender, BXSelectCountEventArgs e)
	{
        e.Count = BXExchangeRate.Count(new BXFilter(ItemFilter.CurrentFilter, BXExchangeRate.Fields));
	}

	protected void ItemGrid_RowDataBound(object sender, GridViewRowEventArgs e)
	{
		if (e.Row.RowType != DataControlRowType.DataRow)
			return;

		BXGridViewRow row = (BXGridViewRow)e.Row;
		row.UserData.Add("ID", ((CurrencyExchRateWrapper)row.DataItem).ID);
	}

    protected void ItemGrid_Delete(object sender, BXDeleteEventArgs e)
	{
        if (!BXPrincipal.Current.IsCanOperate(BXCurrency.Operations.AdminManagement))
            return;

		try
		{
	        BXExchangeRate.Delete(e.Keys["ID"]);
			e.DeletedCount++;
		}
		catch (Exception ex)
		{
			ErrorMessage.AddErrorMessage(ex.Message);
		}
	}
}
/// <summary>
/// Обёртка
/// </summary>
public class CurrencyExchRateWrapper
{
    BXExchangeRate charge;
    BXAdminPage page;
	public CurrencyExchRateWrapper(BXExchangeRate charge, BXAdminPage page)
    {
        if (charge == null)
            throw new ArgumentNullException("charge");

        if (page == null)
            throw new ArgumentNullException("page");

        this.charge = charge;
        this.page = page;
    }

    public string ID
    {
        get { return this.charge.Id.ToString(); }
    }

    public string CurrencyCode
    {
        get { return this.charge.Currency != null ? this.charge.Currency.Iso4217AlphaCode : string.Empty; }
    }

	public string Issued
	{
		get { return this.charge.Issued.ToString("d"); }
	}

    public string Value
    {
        get { return this.charge.Value.ToString("F4", CultureInfo.InvariantCulture.NumberFormat); }
    }
}
