﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using Bitrix.UI;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix;
using System.Text;
using Bitrix.Security;
using Bitrix.Main;
using System.Globalization;
using Bitrix.Currency;

public partial class bitrix_admin_CurrencyList : BXAdminPage
{
    protected override void OnPreInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXCurrency.Operations.AdminManagement))
	            BXAuthentication.AuthenticationRequired();

        base.OnPreInit(e);
    }

    protected override void OnLoad(EventArgs e)
    {
        //((BXAdminMasterPage)Master).PageIconId = "catalog_page_icon";
        MasterTitle = Page.Title = GetMessage("PageTitle");
        base.OnLoad(e);
    }

    protected void ItemGrid_Select(object sender, BXSelectEventArgs e)
	{
        List<CurrencyWrapper> list = new List<CurrencyWrapper>();
        foreach (BXCurrency currency in InnerCurrencies)
            list.Add(new CurrencyWrapper(currency, this));

        e.Data = list;
	}

    protected void ItemGrid_SelectCount(object sender, BXSelectCountEventArgs e)
	{
        e.Count = InnerCurrencies.Count;
	}

    protected void ItemGrid_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType != DataControlRowType.DataRow)
            return;

        BXGridViewRow row = (BXGridViewRow)e.Row;
        row.UserData.Add("ID", ((CurrencyWrapper)row.DataItem).ID);
    }


    protected void ItemGrid_Delete(object sender, BXDeleteEventArgs e)
	{
        if (!BXPrincipal.Current.IsCanOperate(BXCurrency.Operations.AdminManagement))
            return;

		try
		{
	        BXCurrency.Delete(e.Keys["ID"]);
			e.DeletedCount++;
			if(this.currencies != null)
				this.currencies = null;
		}
		catch (Exception ex)
		{
			ErrorMessage.AddErrorMessage(ex.Message);
		}
	}

	private BXCurrencyCollection currencies = null;
	protected BXCurrencyCollection InnerCurrencies
	{
		get 
		{
			if(this.currencies != null)
				return this.currencies;

			return this.currencies = BXCurrency.GetList(
				null,
				new BXOrderBy(
					new BXOrderByPair(BXCurrency.Fields.Sort, BXOrderByDirection.Asc),
					new BXOrderByPair(BXCurrency.Fields.Id, BXOrderByDirection.Asc)),
				null,
				null,
				BXTextEncoder.EmptyTextEncoder);
		}
	}
}
/// <summary>
/// Обёртка
/// </summary>
public class CurrencyWrapper
{
    BXCurrency charge;
    BXAdminPage page;
    public CurrencyWrapper(BXCurrency charge, BXAdminPage page)
    {
        if (charge == null)
            throw new ArgumentNullException("charge");
        this.charge = charge;

        if (page == null)
            throw new ArgumentNullException("page");
        this.page = page;
    }

    public string ID
    {
        get { return this.charge.Id.ToString(); }
    }

	public string Iso4217AlphaCode
	{
		get { return this.charge.Iso4217AlphaCode; }
	}

	public string National
	{
		get { return this.page.GetMessageRaw(this.charge.IsNational ? "Kernel.Yes" : "Kernel.No"); }
	}

    public string Name
    {
        get { return HttpUtility.HtmlEncode(this.charge.GetName()); }
    }


    public string Sort
    {
        get { return this.charge.Sort.ToString(); }
    }
}
