﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.Security;
using Bitrix.Components;
using System.Collections.Generic;
using Bitrix;
using Bitrix.UI;

using Bitrix.IBlock;
using Bitrix.Services;
using Bitrix.DataLayer;
using Bitrix.DataTypes;
using Bitrix.IO;
using Bitrix.Services.Text;
using Bitrix.Configuration;
using System.Text;
using Bitrix.UI.Popup;

public partial class CatalogElement : BXComponent
{
	private BXIBlockElement element;

	public BXIBlockElement Element
	{
		get { return this.element; }
		set { this.element = value; }
	}

	public int ElementId
	{
		get { return Parameters.Get<int>("ElementId"); }
	}

	public string ElementCode
	{
		get { return Parameters.Get<string>("ElementCode", String.Empty); }
	}

	public string ElementName
	{
		set { ComponentCache["ElementName"] = value; }
		get
		{
			if (!ComponentCache.ContainsKey("ElementName"))
				return null;
			return ComponentCache.Get<string>("ElementName");
		}
	}

	public BXFile PreviewImage
	{
		get { return ComponentCache.Get("PreviewImage") as BXFile; }
		set { ComponentCache["PreviewImage"] = value; }
	}

	public BXFile DetailImage
	{
		get { return ComponentCache.Get("DetailImage") as BXFile; }
		set { ComponentCache["DetailImage"] = value; }
	}

	public int IBlockId
	{
		get
		{
			return Parameters.Get("IBlockId", 0);
		}
	}

	public string PropertyKeywordsCode
	{
		get
		{
			return Parameters.Get<string>("PropertyKeywords", "-");
		}
	}

	public string PropertyKeywordsValue
	{
		set { ComponentCache["PropertyKeywords"] = value; }
		get
		{
			if (!ComponentCache.ContainsKey("PropertyKeywords"))
				return null;
			return ComponentCache.Get<string>("PropertyKeywords");
		}
	}

	public string PropertyDescriptionCode
	{
		get
		{
			return Parameters.Get<string>("PropertyDescription", "-");
		}
	}

	public string PropertyDescriptionValue
	{
		set { ComponentCache["PropertyDescription"] = value; }
		get
		{
			if (!ComponentCache.ContainsKey("PropertyDescription"))
				return null;
			return ComponentCache.Get<string>("PropertyDescription");
		}
	}

	public Dictionary<string, Dictionary<string, string>> Properties
	{
		get { return ComponentCache.Get("Properties") as Dictionary<string, Dictionary<string, string>>; }
		set { ComponentCache["Properties"] = value; }
	}

	public string IBlockName
	{
		set { ComponentCache["IBlockName"] = value; }
		get
		{
			if (!ComponentCache.ContainsKey("IBlockName"))
				return null;
			return ComponentCache.Get<string>("IBlockName");
		}
	}

	public int IBlockTypeId
	{
		set { ComponentCache["IBlockTypeID"] = value; }
		get { return ComponentCache.Get<int>("IBlockTypeID"); }
	}

	public string IBlockElementName
	{
		set { ComponentCache["IBlockElementName"] = value; }
		get { return ComponentCache.Get<string>("IBlockElementName"); }
	}

	public string iblockElementSectionId
	{
		get { return "IBlockElementMenuSection";}
	}

	public string iblockElementButtonMenuId
	{
		get { return String.Format("IBlockElementButtonMenu_{0}", IBlockId); }
	}

	public string errorMessage = String.Empty;

	protected void Page_Load(object sender, EventArgs e)
	{
		if (!IsCached(((BXPrincipal)Page.User).GetAllRoles(true)))
		{
			BXIBlock iblock = null;
			bool badParams = false;

			if (IBlockId <= 0)
			{
				badParams = true;
				errorMessage = GetMessage("EmptyIBlockCode");
			}
			else if (ElementId <= 0)
			{
				badParams = true;
				errorMessage = GetMessage("ElementNotFound");
			}
			else
			{
				iblock = BXIBlock.GetById(IBlockId);
				if (iblock == null)
				{
					badParams = true;
					errorMessage = GetMessage("WrongIBlockCode");
				}
			}

			BXIBlockElementCollection elementCollection = null;
			if (!badParams)
			{
				IBlockName = iblock.Name;
				IBlockTypeId = iblock.TypeId;
				IBlockElementName = iblock.ElementName;

				BXFilter elementFilter = new BXFilter(
					new BXFilterItem(BXIBlockElement.Fields.Active, BXSqlFilterOperators.Equal, "Y"),
					new BXFilterItem(BXIBlockElement.Fields.CheckPermissions, BXSqlFilterOperators.Equal, "Y"),
					new BXFilterItem(BXIBlockElement.Fields.IBlock.ID, BXSqlFilterOperators.Equal, IBlockId)
				);

				if (!String.IsNullOrEmpty(ElementCode))
				{
					elementFilter.Add(new BXFilterItem(BXIBlockElement.Fields.Code, BXSqlFilterOperators.Equal, ElementCode));

					if (ElementId > 0)
						elementFilter.Add(new BXFilterItem(BXIBlockElement.Fields.ID, BXSqlFilterOperators.Equal, ElementId));
				}
				else
					elementFilter.Add(new BXFilterItem(BXIBlockElement.Fields.ID, BXSqlFilterOperators.Equal, ElementId));

				elementCollection = BXIBlockElement.GetList(elementFilter, null);
				if (elementCollection == null || elementCollection.Count == 0)
				{
					badParams = true;
					errorMessage = GetMessage("ElementNotFound");
				}
			}
	
			if (badParams)
			{
				AbortCache();

				ScriptManager ajax = ScriptManager.GetCurrent(Page);
				if (!IsComponentDesignMode && BXConfigurationUtility.ShowMode == BXShowMode.View && (ajax == null || !ajax.IsInAsyncPostBack))
				{
					Response.StatusCode = 404;
					Response.TrySkipIisCustomErrors = true;
				}

				IncludeComponentTemplate();
				return;
			}

			element = elementCollection[0];

			ElementName = element.Name;
			PreviewImage = element.PreviewImageId > 0 ? element.PreviewImage : null;
			DetailImage = element.DetailImageId > 0 ? element.DetailImage : null;

			//Properties
			if (element.Properties != null && element.Properties.Count > 0)
			{
				if (element.Properties.ContainsKey(PropertyKeywordsCode))
					PropertyKeywordsValue = GetCustomProperty(element.Properties[PropertyKeywordsCode], ", ");

				if (element.Properties.ContainsKey(PropertyDescriptionCode))
					PropertyDescriptionValue = GetCustomProperty(element.Properties[PropertyDescriptionCode], ", ");

				List<string> propertiesCode = Parameters.GetListString("Properties");
				Dictionary<string, Dictionary<string, string>> elementProperties = new BXParamsBag<Dictionary<string, string>>();

				foreach (BXCustomField field in iblock.CustomFields)
				{
					if (propertiesCode.Contains(field.Name))
					{
						Dictionary<string, string> propertyValues = new Dictionary<string, string>(2);
						propertyValues.Add("Name", field.EditFormLabel);
						propertyValues.Add("Value", GetCustomProperty(element.Properties[field.Name], "&nbsp;/&nbsp;"));
						elementProperties[field.Name] = propertyValues;
					}
				}

				Properties = elementProperties.Count > 0 ? elementProperties : null;
			}

			IncludeComponentTemplate();
		}

		BXPublicPage bitrixPage = Page as BXPublicPage;
		if (bitrixPage != null && !IsComponentDesignMode)
		{
			if (Parameters.Get<bool>("SetPageTitle", true) && ElementName != null)
				bitrixPage.BXTitle = BXTextEncoder.DefaultEncoder.Decode(ElementName);

			if (PropertyDescriptionValue != null)			
				bitrixPage.Keywords["description"] = PropertyDescriptionValue;

			if (PropertyKeywordsValue != null)
				bitrixPage.Keywords["keywords"] = PropertyKeywordsValue;
		}	
	}

	private string GetCustomProperty(BXCustomProperty property, string separator)
	{
		if (property.IsMultiple && property.Values.Count > 0)
		{
			StringBuilder returnString = new StringBuilder();
			returnString.Append(BXTextEncoder.DefaultEncoder.Encode(property.Values[0].ToString()));

			for (int j = 1; j < property.Values.Count; j++)
			{
				returnString.Append(separator);
				returnString.Append(BXTextEncoder.DefaultEncoder.Encode(property.Values[j].ToString()));
			}

			if (returnString.Length > 0)
				return returnString.ToString();
		}
		else if (property.Value != null)
		{
			return BXTextEncoder.DefaultEncoder.Encode(property.Value.ToString());
		}

		return null;
	}

	//Создаем секцию и кнопку для выпадающего меню в панели администратора
	public override void CreatePublicPanelMenu(BXPublicPanelMenuSectionList sectionList)
	{
		if (!Parameters.Get<bool>("AddAdminPanelButtons") || Element == null)
			return;

		if (sectionList == null)
			throw new ArgumentNullException("sectionList");

		if (!BXIBlock.IsUserCanOperate(IBlockId, BXIBlock.Operations.IBlockAdminRead) ||
			!BXIBlock.IsUserCanOperate(IBlockId, BXIBlock.Operations.IBlockModifyElements))
			return;

		//Создаем секцию
		BXPublicPanelMenuSection iblockElementSection = sectionList.FindByID(iblockElementSectionId);
		if (iblockElementSection == null)
		{
			BXPublicPanelMenuSectionInfo sectionInfo = new BXPublicPanelMenuSectionInfo(iblockElementSectionId, 100);
			iblockElementSection = new BXPublicPanelMenuSection(sectionInfo);
			sectionList.Add(iblockElementSection);
		}

		//Создаем кнопку
		BXPublicPanelMenu elementEditMenu = iblockElementSection.FindByID(iblockElementButtonMenuId);
		if (elementEditMenu == null)
		{
			elementEditMenu = new BXPublicPanelMenu();
			elementEditMenu.ID = iblockElementButtonMenuId;
			elementEditMenu.BackgroundImagePath = BXPath.Combine(BXComponentManager.GetFolder(Name), "/images/iblock.gif");
			elementEditMenu.Caption = IBlockName;
			elementEditMenu.Hint = IBlockName;
			elementEditMenu.ShowMode = BXShowMode.NonView;
			iblockElementSection.Add(elementEditMenu);
		}
	}

	//Заполняем выпадающее меню пунктами "редактировать элемент" и "добавить элемент"
	public override void PopulatePublicPanelMenu(BXPublicPanelMenuSectionList sectionList)
	{
		if (!Parameters.Get<bool>("AddAdminPanelButtons") || Element == null)
			return;

		if (sectionList == null)
			throw new ArgumentNullException("sectionList");

		if (!BXIBlock.IsUserCanOperate(IBlockId, BXIBlock.Operations.IBlockAdminRead) ||
			!BXIBlock.IsUserCanOperate(IBlockId, BXIBlock.Operations.IBlockModifyElements))
			return;

		BXPublicPanelMenuSection iblockElementSection = sectionList.FindByID(iblockElementSectionId);
		if (iblockElementSection == null)
			return;

		BXPublicPanelMenu elementEditMenu = iblockElementSection.FindByID(iblockElementButtonMenuId);
		if (elementEditMenu == null)
			return;

		if (elementEditMenu.PopupMenu == null)
			elementEditMenu.PopupMenu = new BXPopupMenuV2();

		BXPopupMenuItem[] menuItems = GetMenuItems();
		for (int menuIndex = 0; menuIndex < menuItems.Length; menuIndex++)
		{
			bool itemExists = false;
			foreach (BXPopupMenuBaseItem item in elementEditMenu.PopupMenu.Items)
			{
				if (item.ID == menuItems[menuIndex].ID)
				{
					itemExists = true;
					break;
				}
			}

			if (!itemExists)
				elementEditMenu.PopupMenu.AddItem(menuItems[menuIndex]);
		}
	}

	public BXPopupMenuItem[] GetMenuItems()
	{
		BXPopupMenuItem[] menuItems = new BXPopupMenuItem[2];

		menuItems[0] = new BXPopupMenuItem();
		menuItems[0].BackgroundImagePath = BXPath.Combine(BXComponentManager.GetFolder(Name), "/images/edit_element.gif");
		menuItems[0].ID = String.Format("element_edit_{0}", ElementId);
		menuItems[0].Text = String.Format(GetMessageRaw("EditElementMenuItem"), String.IsNullOrEmpty(IBlockElementName) ? GetMessageRaw("IBlockElementName") : IBlockElementName.ToLower());
		menuItems[0].ClientClickScript =
			String.Format(
				"jsUtils.Redirect(arguments, '{0}?id={1}&iblock_id={2}&type_id={3}')",
				VirtualPathUtility.ToAbsolute("~/bitrix/admin/IBlockElementEdit.aspx"),
				ElementId,
				IBlockId,
				IBlockTypeId
			);

		menuItems[1] = new BXPopupMenuItem();
		menuItems[1].Text = String.Format(GetMessageRaw("CreateElementMenuItem"), String.IsNullOrEmpty(IBlockElementName) ? GetMessageRaw("IBlockElementName") : IBlockElementName.ToLower());
		menuItems[1].ID = String.Format("element_new_{0}", IBlockId);
		menuItems[1].BackgroundImagePath = BXPath.Combine(BXComponentManager.GetFolder(Name), "/images/new_element.gif");
		menuItems[1].ClientClickScript =
			String.Format(
				"jsUtils.Redirect(arguments, '{0}?iblock_id={1}&type_id={2}')",
				VirtualPathUtility.ToAbsolute("~/bitrix/admin/IBlockElementEdit.aspx"),
				IBlockId,
				IBlockTypeId
			);

		return menuItems;
	}

	//Создаем контекстное меню
	public override BXComponentPopupMenuInfo PopupMenuInfo
	{
		get
		{
			BXComponentPopupMenuInfo info = base.PopupMenuInfo;

			if (BXIBlock.IsUserCanOperate(IBlockId, BXIBlock.Operations.IBlockAdminRead) &&
				BXIBlock.IsUserCanOperate(IBlockId, BXIBlock.Operations.IBlockModifyElements))
			{
				info.CreateCustomPopupMenuItems = delegate(BXShowMode showMode)
				{
					if (showMode != BXShowMode.View)
						return GetMenuItems();
					else
						return null;
				};
			}
			return info;
		}
	}

	protected override void PreLoadComponentDefinition()
	{
		Title = GetMessage("Title");
		Description = GetMessage("Description");
		Icon = "images/cat_detail.gif";

		//Group = new BXComponentGroup("catalog", GetMessage("Group"), 100, BXComponentGroup.Content);

		ParamsDefinition.Add(BXParametersDefinition.Cache);

		ParamsDefinition.Add(
			"IBlockTypeId",
			new BXParamSingleSelection(
				GetMessageRaw("InfoBlockType"),
				String.Empty,
				BXCategory.Main
			)
		);

		ParamsDefinition.Add(
			"IBlockId",
			new BXParamSingleSelection(
				GetMessageRaw("InfoBlockCode"),
				String.Empty,
				BXCategory.Main
			)
		);

		ParamsDefinition.Add(
			"ElementId",
			new BXParamText(
				GetMessageRaw("ElementID_1"),
				"0",
				BXCategory.Main
			));

		ParamsDefinition.Add(
			"ElementCode",
			new BXParamText(
			GetMessageRaw("ElementCode"),
			String.Empty,
			BXCategory.Main
		));

		ParamsDefinition.Add(
			"Properties",
			new BXParamMultiSelection(
				GetMessageRaw("Properties"),
				"-",
				BXCategory.AdditionalSettings
			));

		ParamsDefinition.Add(
			"PropertyKeywords",
			new BXParamSingleSelection(
				GetMessageRaw("SetPageKeywordsFromProperty"),
				"-",
				BXCategory.AdditionalSettings
			));
		ParamsDefinition.Add(
			"PropertyDescription",
			new BXParamSingleSelection(
				GetMessageRaw("SetPageDescriptionFromProperty"),
				"-",
				BXCategory.AdditionalSettings
			));

		ParamsDefinition.Add(
			"AddAdminPanelButtons",
			new BXParamYesNo(
				GetMessageRaw("AddButtonsForThisComponentToAdminPanel"),
				false,
				BXCategory.AdditionalSettings
			));
		ParamsDefinition.Add(
			"SetPageTitle",
			new BXParamYesNo(
				GetMessageRaw("SetPageTitle"),
				true,
				BXCategory.AdditionalSettings
			));
	}

	protected override void LoadComponentDefinition()
	{
		#region MainGroup

		List<BXParamValue> types = new List<BXParamValue>();
		types.Add(new BXParamValue("-", ""));
		BXIBlockTypeCollection typeCollection = BXIBlockType.GetList(null, new BXOrderBy(new BXOrderByPair(BXIBlockType.Fields.Sort, BXOrderByDirection.Asc)));
		foreach (BXIBlockType t in typeCollection)
			types.Add(new BXParamValue(t.Translations[BXLoc.CurrentLocale].Name, t.Id.ToString()));

		ParamsDefinition["IBlockTypeId"].Values = types;
		ParamsDefinition["IBlockTypeId"].RefreshOnDirty = true;

		BXFilter iblockFilter = new BXFilter();
		if (Parameters.ContainsKey("IBlockTypeId"))
		{
			int typeId;
			int.TryParse(Parameters["IBlockTypeId"], out typeId);
			if (typeId > 0)
				iblockFilter.Add(new BXFilterItem(BXIBlock.Fields.Type.ID, BXSqlFilterOperators.Equal, typeId));
		}
		if (!string.IsNullOrEmpty(DesignerSite))
			iblockFilter.Add(new BXFilterItem(BXIBlock.Fields.Sites.SiteId, BXSqlFilterOperators.Equal, DesignerSite));

		List<BXParamValue> iblocks = new List<BXParamValue>();
		BXIBlockCollection iblockCollection = BXIBlock.GetList(iblockFilter, new BXOrderBy(new BXOrderByPair(BXIBlock.Fields.Sort, BXOrderByDirection.Asc)));
		foreach (BXIBlock b in iblockCollection)
			iblocks.Add(new BXParamValue(b.Name, b.Id.ToString()));

		ParamsDefinition["IBlockId"].Values = iblocks;
		ParamsDefinition["IBlockId"].RefreshOnDirty = true;

		#endregion

		#region View
		List<BXParamValue> properties = new List<BXParamValue>();
		if (Parameters.ContainsKey("IBlockId"))
		{
			int iblockId;
			int.TryParse(Parameters["IBlockId"], out iblockId);
			if (iblockId > 0)
			{
				BXCustomFieldCollection cfCollection = BXCustomEntityManager.GetFields("IBlock_" + iblockId.ToString());
				foreach (BXCustomField f in cfCollection)
					properties.Add(new BXParamValue(f.EditFormLabel, f.Name));
			}
		}

		ParamsDefinition["Properties"].Values = properties;
		#endregion

		#region Additional
		ParamsDefinition["PropertyKeywords"].Values = properties;
		ParamsDefinition["PropertyDescription"].Values = properties;
		#endregion
	}
}

public partial class CatalogElementTemplate : BXComponentTemplate<CatalogElement>
{
	protected override void Render(HtmlTextWriter writer)
	{
		if (IsComponentDesignMode && Component.Element == null)
			writer.Write(HttpUtility.HtmlEncode(BXLoc.GetMessage(Component, "YouHaveToAdjustTheComponent")));
		else
			base.Render(writer);
	}
}
