﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.Security;
using Bitrix.Components;
using System.Collections.Generic;
using Bitrix;
using Bitrix.UI;

using Bitrix.IBlock;
using Bitrix.Services;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix.DataTypes;
using Bitrix.IO;

public partial class bitrix_components_bitrix_catalog_section_list_component : BXComponent
{
    int SectionId = 0, ParentSectionId = 0;
    string SectionCode = String.Empty;

    #region UrlTemplates

    public string SectionURL
    {
        get
        {
            if (EnableSef)
            {
                return Parameters.Get<string>("SEF_URL_TEMPLATES_section", "#SECTION_ID#/");
            }
            else
            {
                return Parameters.Get<string>("SectionURL", String.Format("?{0}=#SECTION_ID#", AliasSectionId));
            }
        }
    }

    public string SectionPageURL
    {
        get
        {
            if (EnableSef)
            {
                return Parameters.Get<string>("SEF_URL_TEMPLATES_section_page", "#SECTION_ID#/Page/#PAGE_ID#/");
            }
            else
            {
                return Parameters.Get<string>("SectionPageURL",
                                       String.Format("?{0}=#SECTION_ID#&{1}=#PAGE_ID#", AliasSectionId, AliasPageId));
            }
        }
    }

    #endregion

    #region UrlParams

    public string ParamSectionId
    {
        get
        {
            return Parameters.Get<string>("ParamSectionId", "section_id");
        }
    }

    public string ParamSectionCode
    {
        get
        {
            return Parameters.Get<string>("ParamSectionCode", "section_code");
        }
    }

    public string ParamPageId
    {
        get
        {
            return Parameters.Get<string>("ParamPageId", "page_id");
        }
    }

    public string AliasSectionId
    {
        get
        {
            return Parameters.Get<string>("VARIABLE_ALIASES_SECTION_ID", "SECTION_ID");
        }
    }

    public string AliasSectionCode
    {
        get
        {
            return Parameters.Get<string>("VARIABLE_ALIASES_SECTION_CODE", "SECTION_CODE");
        }
    }

    public string AliasPageId
    {
        get
        {
            return Parameters.Get<string>("VARIABLE_ALIASES_PAGE_ID", "PAGE_ID");
        }
    }
    #endregion

    public string SEFFolder
    {
        get
        {
			return Parameters.Get<string>("SEFFolder", "catalog").Trim('/', '\\');
        }
    }

    public int IBlockTypeId
    {
        get
        {
            return Parameters.Get("IBlockTypeId", 0);
        }
    }

    public int IBlockId
    {
        get
        {
            return Parameters.Get("IBlockId", 0);
        }
    }

    public int Depth
    {
        get
        {
            return Parameters.Get<int>("Depth", 2);
        }
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        #region InitializeStartup
        SectionId = Parameters.Get("SectionId", 0);

        bool COUNT_ELEMENTS = Parameters.Get<bool>("COUNT_ELEMENTS", false);
        string SortByWhat = Parameters.Get<string>("SortBy", "sort");
        string OrderByWhat = Parameters.Get<string>("OrderBy", "asc");
        #endregion

        BXParamsBag<object> request = new BXParamsBag<object>();

        if (EnableSef)
        {
            BXParamsBag<string> urlToPage = new BXParamsBag<string>();
            urlToPage.Add("Catalog.List0", SectionURL);
            urlToPage.Add("Catalog.List1", SectionPageURL);

            MapVariable(SEFFolder, urlToPage, request);

            Results["ParamSectionId"] = ParamSectionId;
            Results["ParamSectionCode"] = ParamSectionCode;
        }
        else
        {
            BXParamsBag<string> variableAlias = new BXParamsBag<string>();
            variableAlias[ParamSectionId] = AliasSectionId;
            variableAlias[ParamSectionCode] = AliasSectionCode;

            MapVariable(variableAlias, request);

            Results["ParamSectionId"] = AliasSectionId;
            Results["ParamSectionCode"] = AliasSectionCode;
        }

        //IBlockId = request.Get("iblock_id", IBlockId);
        SectionId = request.Get(ParamSectionId, SectionId);
        SectionCode = request.Get<string>(ParamSectionCode, SectionCode);

        // Rewrite default variables
        request.Add(ParamSectionId, SectionId);
        request.Add(ParamSectionCode, SectionCode);        

        Results["SectionURL"] = SectionURL;

        Results["SectionId"] = SectionId;
        Results["SectionCode"] = SectionCode;

        if (EnableSef)
            Results["CatalogRoot"] = BXPath.ToVirtualAbsolutePath(SEFFolder);
        else
            Results["CatalogRoot"] = Parameters.Get<string>("CatalogRoot", BXSefUrlManager.CurrentUrl.AbsolutePath);

        if (!IsCached(IBlockId, SectionId, SectionCode))
        {
            if (IBlockId > 0)
            {
                List<Dictionary<string, string>> sList = new List<Dictionary<string, string>>();
                                        
                int i = 0, baseDepth = 0;
                BXIBlockSectionCollection sections = null;


                if (SectionId <= 0 && String.IsNullOrEmpty(SectionCode))
                {
                    sections = BXIBlockSection.GetTree(IBlockId, Depth + 1);
                }
                else
                {
                    #region SectionList
                    BXOrderBy sOrder = new BXOrderBy();
					sOrder.Add(BXIBlockSection.Fields, String.Format("{0} {1}", SortByWhat, OrderByWhat));
					//sOrder.Add(BXIBlockSection.Fields, String.Format("{0} {1}", "id", "asc"));
                    BXIBlockSection parentSection = null;

                    if (String.IsNullOrEmpty(SectionCode))
                    {
                        if (SectionId > 0)
                            parentSection = BXIBlockSection.GetById(SectionId);
                    }
                    else
                    {
                        BXFilter sParentFilter = new BXFilter(
							new BXFilterItem(BXIBlockSection.Fields.Code, BXSqlFilterOperators.Equal, SectionCode),
							new BXFilterItem(BXIBlockSection.Fields.Active, BXSqlFilterOperators.Equal, "Y")
                            );

                        BXIBlockSectionCollection sCol = BXIBlockSection.GetList(sParentFilter, null);
                        if (sCol != null && sCol.Count > 0)
                            parentSection = sCol[0];
                    }

                    BXFilter sFilter = new BXFilter(
							new BXFilterItem(BXIBlockSection.Fields.IBlock.ID, BXSqlFilterOperators.Equal, IBlockId),
							new BXFilterItem(BXIBlockSection.Fields.Active, BXSqlFilterOperators.Equal, "Y")
                        );

                    if (parentSection != null)
                    {
                        ParentSectionId = parentSection.Id;                        
                        baseDepth = parentSection.DepthLevel;

						sFilter.Add(new BXFilterItem(BXIBlockSection.Fields.LeftMargin, BXSqlFilterOperators.GreaterOrEqual, parentSection.LeftMargin));
						sFilter.Add(new BXFilterItem(BXIBlockSection.Fields.RightMargin, BXSqlFilterOperators.Less, parentSection.RightMargin));
						sFilter.Add(new BXFilterItem(BXIBlockSection.Fields.DepthLevel, BXSqlFilterOperators.LessOrEqual, parentSection.DepthLevel + Depth));

                        sections = BXIBlockSection.GetList(sFilter, sOrder);
                    }
                    #endregion
                    #region NavChain
                    BXIBlockSectionCollection navChain = null;

                    if (SectionId > 0)
                    {
                        navChain = BXIBlockSection.GetNavChain(SectionId);
                    }
                    else if (ParentSectionId > 0)
                    {
                        navChain = BXIBlockSection.GetNavChain(ParentSectionId);                        
                    }

                    Results["NavChain"] = navChain;
                    SetupNavchain(navChain, request);  
                    #endregion
                }
                Results["DataVolume"] = (sections != null) ? sections.Count : 0;
				if (sections != null)
				{
					string str = String.Empty;
					if (EnableSef)
						str = String.Concat(SEFFolder, "/", SectionURL);
					else
						str = SectionURL;

					foreach (BXIBlockSection s in sections)
					{
						Dictionary<string, string> v = new Dictionary<string, string>();
						if (s.DepthLevel == 1)
						{
							i++;
							v.Add("I", (i < 10) ? "0" + i : i.ToString());
						}
						v.Add("SectionId", s.Id.ToString());
						v.Add("Depth", (s.DepthLevel - baseDepth).ToString());
						v.Add("Name", s.Name);
						if (COUNT_ELEMENTS)
							v.Add("ElementsCount", s.ElementsCount.ToString());
						if (!String.IsNullOrEmpty(SectionURL))
						{
							request["iblock_id"] = s.IBlockId;
							request["section_id"] = s.Id;
							request["section_code"] = String.IsNullOrEmpty(s.Code) ? String.Empty : s.Code;
							v.Add("Path", BXUri.ToRelativeUri(MakeLink(str, request)));
						}
						sList.Add(v);
					}
				}

                Results["Section"] = sList;

                Results["EnableAjaxA"] = Parameters.Get<bool>("EnableAjax", false)?
                    "EnableAjax=\"True\"": "";
            }

            IncludeComponentTemplate();
        }
        else
        {
             // 1. NavChain
            if (Results.ContainsKey("NavChain")) SetupNavchain((BXIBlockSectionCollection)Results["NavChain"], request);
        }
	}

    public virtual void SetupNavchain(BXIBlockSectionCollection navChain, BXParamsBag<object> request)
    {
        string SEFFolder = Parameters.Get<string>("SEFFolder", "/catalog");

        if (navChain != null)
        {           
            string parentSectionUrl, parentSectionCode = String.Empty;
            for (int j = 0; j < navChain.Count; j++)
            {
                if (navChain[j] == null) continue;
                request["section_id"] = navChain[j].Id;
                request["section_code"] = navChain[j].Code;
                string sectionUrl = MakeLink(SectionURL, request);
                if (EnableSef) 
					sectionUrl = BXUri.Combine(SEFFolder, sectionUrl);

                if (j != 0)
                {
                    request["section_id"] = navChain[j].SectionId;
                    request["section_code"] = parentSectionCode;
                    parentSectionUrl = MakeLink(SectionURL, request);
                    if (EnableSef)
						parentSectionUrl = BXUri.Combine(SEFFolder, parentSectionUrl);
                }
                else
                {
                    parentSectionCode = navChain[0].Code;
                    parentSectionUrl = (string)Results["CatalogRoot"];
                }

                BXSiteMapManager.AddNode(
                    parentSectionUrl,
                    sectionUrl,
                    navChain[j].Name);

                parentSectionCode = navChain[j].Code;
            }
        }
    }

    protected override void PreLoadComponentDefinition()
    {
        Title = GetMessage("Title");
        Description = GetMessage("Description");
        //Group = new BXComponentGroup("catalog", GetMessage("Group"), 100, BXComponentGroup.Content);
        Icon = "images/sections_top_count.gif";

        ParamsDefinition.Add(BXParametersDefinition.Cache);

        ParamsDefinition.Add(
            "IBlockTypeId",
            new BXParamSingleSelection(
                GetMessageRaw("InfoBlockType"),
                String.Empty,
                BXCategory.Main
            )
        );

        ParamsDefinition.Add(
            "IBlockId",
            new BXParamSingleSelection(
                GetMessageRaw("InfoBlockCode"),
                String.Empty,
                BXCategory.Main
            )
        );

        ParamsDefinition.Add(
            "Depth",
            new BXParamText(
                GetMessageRaw("SubsectionsDisplayDepth"),
                "2",
                BXCategory.Main
            ));


        ParamsDefinition.Add(
            "SectionId",
            new BXParamText(
                GetMessageRaw("SectionNumber"),
                "0",
                BXCategory.Main
            ));

        ParamsDefinition.Add(
            "COUNT_ELEMENTS",
            new BXParamYesNo(
                GetMessageRaw("DisplayQuantityOfElementsInSection"),
                true,
                BXCategory.Main
            ));

        ParamsDefinition.Add(BXParametersDefinition.Sef);

        ParamsDefinition.Add(
            "SEF_URL_TEMPLATES_section",
            new BXParamText(
                GetMessageRaw("PageAddress"),
                "?SECTION_ID=#SECTION_ID#",
                BXCategory.Sef
            ));

        ParamsDefinition.Add(
            "VARIABLE_ALIASES_SECTION_ID",
            new BXParamText(
                GetMessageRaw("SectionID"),
                "SECTION_ID",
                BXCategory.Sef
            ));

        ParamsDefinition.Add(
            "VARIABLE_ALIASES_SECTION_CODE",
            new BXParamText(
                GetMessageRaw("SectionMnemonicCodeID"),
                "SECTION_CODE",
                BXCategory.Sef
            ));

        ParamsDefinition.Add(
            "DISPLAY_PANEL",
            new BXParamYesNo(
                GetMessageRaw("AddButtonsForThisComponentToAdminPanel"),
                false,
                BXCategory.AdditionalSettings
            ));
    }

    protected override void LoadComponentDefinition()
    {
        #region MainGroup

        List<BXParamValue> types = new List<BXParamValue>();
        types.Add(new BXParamValue("-", ""));
		BXIBlockTypeCollection typeCollection = BXIBlockType.GetList(null, new BXOrderBy(new BXOrderByPair(BXIBlockType.Fields.Sort, BXOrderByDirection.Asc)));
		foreach (BXIBlockType t in typeCollection)
			types.Add(new BXParamValue(t.Translations[BXLoc.CurrentLocale].Name, t.Id.ToString()));

        ParamsDefinition["IBlockTypeId"].Values = types;
        ParamsDefinition["IBlockTypeId"].RefreshOnDirty = true;

        BXFilter iblockFilter = new BXFilter();
        if (Parameters.ContainsKey("IBlockTypeId"))
        {
            int typeId;
            int.TryParse(Parameters["IBlockTypeId"], out typeId);
            if (typeId > 0)
				iblockFilter.Add(new BXFilterItem(BXIBlock.Fields.Type.ID, BXSqlFilterOperators.Equal, typeId));
        }
        if (!string.IsNullOrEmpty(DesignerSite))
			iblockFilter.Add(new BXFilterItem(BXIBlock.Fields.Sites.SiteId, BXSqlFilterOperators.Equal, DesignerSite));

        List<BXParamValue> iblocks = new List<BXParamValue>();
		BXIBlockCollection iblockCollection = BXIBlock.GetList(iblockFilter, new BXOrderBy(new BXOrderByPair(BXIBlock.Fields.Sort, BXOrderByDirection.Asc)));
		foreach (BXIBlock b in iblockCollection)
			iblocks.Add(new BXParamValue(b.Name, b.Id.ToString()));

        ParamsDefinition["IBlockId"].Values = iblocks;
        ParamsDefinition["IBlockId"].RefreshOnDirty = true;

        #endregion
    }
}
