﻿using System;
using System.Collections.Generic;
using System.Web;
using Bitrix;

using Bitrix.Components;
using Bitrix.DataLayer;
using Bitrix.IBlock;
using Bitrix.Modules;
using Bitrix.Services;
using Bitrix.UI;
using Bitrix.DataTypes;
using Bitrix.Components.Editor;

public partial class bitrix_components_bitrix_catalog_component : BXComponent
{    
    private string ElementCode = String.Empty;
    private int ElementId = 0;
    //private int IBlockId = 0;
    //private int IBlockTypeId = 0;
    private int PageId = 0;

    private string SectionCode = String.Empty;
    private int SectionId = 0;

    #region UrlTemplates

    public string SectionURL
    {
        get
        {
            if (EnableSef)
            {
				return Parameters.Get<string>("SEF_URL_TEMPLATES_section", "#SECTION_ID#/");
            }
            else
            {
                return Parameters.Get<string>("SectionURL", String.Format("?{0}=#SECTION_ID#", AliasSectionId));
            }
        }
    }

    public string SectionPageURL
    {
        get
        {
            if (EnableSef)
            {
				return Parameters.Get<string>("SEF_URL_TEMPLATES_section_page", "#SECTION_ID#/Page/#PAGE_ID#/");
            }
            else
            {
                return Parameters.Get<string>("SectionPageURL",
                                       String.Format("?{0}=#SECTION_ID#&{1}=#PAGE_ID#", AliasSectionId, AliasPageId));
            }
        }
    }

    public string ElementURL
    {
        get
        {
            if (EnableSef)
            {
				return Parameters.Get<string>("SEF_URL_TEMPLATES_element", "#SECTION_ID#/#ELEMENT_ID#/");
            }
            else
            {
                return Parameters.Get<string>("ElementURL",
                                       String.Format("?{0}=#SECTION_ID#&{1}=#ELEMENT_ID#", AliasSectionId,
                                                     AliasElementId));
            }
        }
    }

    public string TOPElementURL
    {
        get
        {
            if (EnableSef)
            {
                return Parameters.Get<string>("SEF_URL_TEMPLATES_TOPElement", "TOP/#ELEMENT_ID#/");
            }
            else
            {
                return Parameters.Get<string>("TOPElementURL", String.Format("?{0}=#ELEMENT_ID#", AliasElementId));
            }
        }
    }

    #endregion

    #region UrlParams

    public string ParamSectionId
    {
        get
        {
            return Parameters.Get<string>("ParamSectionId", "section_id");
        }
    }

    public string ParamElementId
    {
        get
        {
            return Parameters.Get<string>("ParamElementId", "element_id");
        }
    }

    public string ParamPageId
    {
        get
        {
            return Parameters.Get<string>("ParamPageId", "page_id");
        }
    }

    public string ParamSectionCode
    {
        get
        {
            return Parameters.Get<string>("ParamSectionCode", "section_code");
        }
    }

    public string ParamElementCode
    {
        get
        {
            return Parameters.Get<string>("ParamElementCode", "element_code");
        }
    }

    public string AliasSectionId
    {
        get
        {
            return Parameters.Get<string>("VARIABLE_ALIASES_SECTION_ID", "SECTION_ID");
        }
    }

    public string AliasElementId
    {
        get
        {
            return Parameters.Get<string>("VARIABLE_ALIASES_ELEMENT_ID", "ELEMENT_ID");
        }
    }

    public string AliasPageId
    {
        get
        {
            return Parameters.Get<string>("VARIABLE_ALIASES_PAGE_ID", "PAGE_ID");
        }
    }

    public string AliasSectionCode
    {
        get
        {
            return Parameters.Get<string>("VARIABLE_ALIASES_SECTION_CODE", "SECTION_CODE");
        }
    }

    public string AliasElementCode
    {
        get
        {
            return Parameters.Get<string>("VARIABLE_ALIASES_ELEMENT_CODE", "ELEMENT_CODE");
        }
    }

    #endregion

    public string SEFFolder
    {
        get
        {
            return Parameters.Get<string>("SEFFolder", "/catalog");
        }
    }

    public int IBlockTypeId
    {
        get
        {
            return Parameters.Get("IBlockTypeId", 0);
        }
    }

    public int IBlockId
    {
        get
        {
            return Parameters.Get("IBlockId", 0);
        }
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        #region HackUnSef

        BXParamsBag<string> urlToPage = new BXParamsBag<string>();
        BXParamsBag<object> request = new BXParamsBag<object>();
        string PageKey = String.Empty;        

        urlToPage.Add("ElementTOP", TOPElementURL);
        urlToPage.Add("SectionPage", SectionPageURL);
        urlToPage.Add("Element", ElementURL);
        urlToPage.Add("Section", SectionURL);

        if (EnableSef)
        {
            PageKey = MapVariable(SEFFolder, urlToPage, request, String.Empty);

            Results["ParamSectionId"] = ParamSectionId;
            Results["ParamElementId"] = ParamElementId;
            Results["ParamPageId"] = ParamPageId;
            Results["ParamElementCode"] = ParamElementCode;
            Results["ParamSectionCode"] = ParamSectionCode;
        }
        else
        {
            BXParamsBag<string> variableAliases = new BXParamsBag<string>();
            variableAliases.Add(ParamSectionId, AliasSectionId);
            variableAliases.Add(ParamElementId, AliasElementId);
            variableAliases.Add(ParamPageId, AliasPageId);
            variableAliases.Add(ParamSectionCode, AliasSectionCode);
            variableAliases.Add(ParamElementCode, AliasElementCode);
            MapVariable(variableAliases, request);

            Results["ParamSectionId"] = AliasSectionId;
            Results["ParamElementId"] = AliasElementId;
            Results["ParamPageId"] = AliasPageId;
            Results["ParamElementCode"] = AliasElementCode;
            Results["ParamSectionCode"] = AliasSectionCode;
        }

        SectionId = request.Get(ParamSectionId, SectionId);
        ElementId = request.Get(ParamElementId, ElementId);
        PageId = request.Get(ParamPageId, PageId);
        ElementCode = request.Get(ParamElementCode, ElementCode);
        SectionCode = request.Get(ParamSectionCode, SectionCode);

        if (!EnableSef)
        {
            if (PageId > 0) PageKey = "SectionPage";
            else if (ElementId > 0 || !String.IsNullOrEmpty(ElementCode)) PageKey = "Element";
            else if (SectionId > 0 || !String.IsNullOrEmpty(SectionCode)) PageKey = "Section";
        }

        #endregion

        if (String.IsNullOrEmpty(SectionCode) && SectionId <= 0 && IBlockId > 0)
        {
            Results["ShowTopElements"] = true;
        }
        
        Results["SectionURL"] = SectionURL;
        Results["SectionPageURL"] = SectionPageURL;
        Results["ElementURL"] = ElementURL;
        Results["TOPElementURL"] = TOPElementURL;

        Results["SectionId"] = SectionId;
        Results["ElementId"] = ElementId;
        Results["PageId"] = PageId;
        Results["ElementCode"] = ElementCode;
        Results["SectionCode"] = SectionCode;

        Results["PageKey"] = PageKey;

        if (String.IsNullOrEmpty(PageKey))
            IncludeComponentTemplate();
        else if (PageKey == "Element" || PageKey == "ElementTOP")
            IncludeComponentTemplate("element");
        else
            IncludeComponentTemplate();
    }
        
    protected override void PreLoadComponentDefinition()
    {
        Title = GetMessage("Title");
        Description = GetMessage("Description");
        Icon = "images/catalog.gif";

        //Group = new BXComponentGroup("catalog", GetMessage("Group"), 100, BXComponentGroup.Content);

        #region MainGroup
        ParamsDefinition.Add(
            "IBlockTypeId",
            new BXParamSingleSelection(
                GetMessageRaw("InfoBlockType"),
                String.Empty,
                BXCategory.Main
                )
            );
        ParamsDefinition["IBlockTypeId"].RefreshOnDirty = true;

        ParamsDefinition.Add(
            "IBlockId",
            new BXParamSingleSelection(
                GetMessageRaw("InfoBlockCode"),
                String.Empty,
                BXCategory.Main
                )
            );
        ParamsDefinition["IBlockId"].RefreshOnDirty = true;
        #endregion

        #region UrlTemplates

        //ParamsDefinition.Add(
        //    "BasketURL",
        //    new BXParam(
        //        "URL, ведущий на страницу с корзиной покупателя",
        //        BXParamType.Text,
        //        "basket.aspx",
        //        "UrlTemplates",
        //        "Шаблоны ссылок",
        //        null
        //    ));
        //ParamsDefinition.Add(
        //    "ParamAction",
        //    new BXParam(
        //        "Название переменной, в которой передается действие",
        //        BXParamType.Text,
        //        "action",
        //        "UrlTemplates",
        //        "Шаблоны ссылок",
        //        null
        //    ));
        //ParamsDefinition.Add(
        //    "ParamItemId",
        //    new BXParam(
        //        "Название переменной, в которой передается код товара для покупки",
        //        BXParamType.Text,
        //        "id",
        //        "UrlTemplates",
        //        "Шаблоны ссылок",
        //        null
        //    ));
        //ParamsDefinition.Add(
        //    "ParamSectionId",
        //    new BXParam(
        //        "Название переменной, в которой передается код группы",
        //        BXParamType.Text,
        //        "SECTION_ID",
        //        "UrlTemplates",
        //        "Шаблоны ссылок",
        //        null
        //    ));

        #endregion

        #region SefConfig

        BXParamsBag<BXParam> sefParBag = BXParametersDefinition.Sef;
        //sefParBag["EnableSEF"].OnChangeClientScript = "if(typeof(ShowSefUrls) == 'function'){ShowSefUrls(this);}";
        sefParBag["EnableSEF"].ClientSideAction = new ParamClientSideActionShowSefRelated(true);
        sefParBag["SEFFolder"].ClientSideAction = new ParamClientSideActionShowSefRelated(ParamClientSideActionShowSefRelatedParticipantType.Sef);
        ParamsDefinition.Add(sefParBag);



        ParamsDefinition.Add(
            "VARIABLE_ALIASES_SECTION_ID",
            new BXParam(
                GetMessageRaw("SectionID"),
                BXParamType.Text,
                "SECTION_ID",
                BXCategory.Sef,
                null, 
                new ParamClientSideActionShowSefRelated(ParamClientSideActionShowSefRelatedParticipantType.NonSef)
                ));

        ParamsDefinition.Add(
            "VARIABLE_ALIASES_ELEMENT_ID",
            new BXParam(
                GetMessageRaw("ElementID"),
                BXParamType.Text,
                "ELEMENT_ID",
                BXCategory.Sef,
                null,
                new ParamClientSideActionShowSefRelated(ParamClientSideActionShowSefRelatedParticipantType.NonSef)
                ));
        ParamsDefinition.Add(
            "VARIABLE_ALIASES_PAGE_ID",
            new BXParam(
                GetMessageRaw("PageID"),
                BXParamType.Text,
                "PAGE_ID",
                BXCategory.Sef,
                null,
                new ParamClientSideActionShowSefRelated(ParamClientSideActionShowSefRelatedParticipantType.NonSef)
                ));
        ParamsDefinition.Add(
            "VARIABLE_ALIASES_ELEMENT_CODE",
            new BXParam(
                GetMessageRaw("ElementMnemonicCodeID"),
                BXParamType.Text,
                "ELEMENT_CODE",
                BXCategory.Sef,
                null,
                new ParamClientSideActionShowSefRelated(ParamClientSideActionShowSefRelatedParticipantType.NonSef)
                ));
        ParamsDefinition.Add(
            "VARIABLE_ALIASES_SECTION_CODE",
            new BXParam(
                GetMessageRaw("SectionMnemonicCodeID"),
                BXParamType.Text,
                "SECTION_CODE",
                BXCategory.Sef,
                null,
                new ParamClientSideActionShowSefRelated(ParamClientSideActionShowSefRelatedParticipantType.NonSef)
                ));

        ParamsDefinition.Add(
            "SEF_URL_TEMPLATES_section",
            new BXParam(
                GetMessageRaw("Section"),
                BXParamType.Text,
                "#SECTION_ID#/",
                BXCategory.Sef,
                null,
                new ParamClientSideActionShowSefRelated(ParamClientSideActionShowSefRelatedParticipantType.Sef)
                ));
        ParamsDefinition.Add(
            "SEF_URL_TEMPLATES_section_page",
            new BXParam(
                GetMessageRaw("SectionPage"),
                BXParamType.Text,
                "#SECTION_ID#/Page/#PAGE_ID#/",
                BXCategory.Sef,
                null,
                new ParamClientSideActionShowSefRelated(ParamClientSideActionShowSefRelatedParticipantType.Sef)
                ));
        ParamsDefinition.Add(
            "SEF_URL_TEMPLATES_element",
            new BXParam(
                GetMessageRaw("ElementDetailedInfo"),
                BXParamType.Text,
                "#SECTION_ID#/#ELEMENT_ID#",
                BXCategory.Sef,
                null,
                new ParamClientSideActionShowSefRelated(ParamClientSideActionShowSefRelatedParticipantType.Sef)
                ));
        ParamsDefinition.Add(
            "SEF_URL_TEMPLATES_TOPElement",
            new BXParam(
                GetMessageRaw("TopElementDetailedInfo"),
                BXParamType.Text,
                "TOP/#ELEMENT_ID#",
                BXCategory.Sef,
                null,
                new ParamClientSideActionShowSefRelated(ParamClientSideActionShowSefRelatedParticipantType.Sef)
                ));

        #endregion

        #region CacheConfig

        ParamsDefinition.Add(
            "CacheOnFilter",
            new BXParamYesNo(
                GetMessageRaw("CacheIfFilterEnabled"),
                true,
                BXCategory.CacheSettings
                ));

        #endregion

        #region TOPElements

        List<BXParamValue> sortFields = new List<BXParamValue>();
        sortFields.Add(new BXParamValue("ID", "ID"));
        sortFields.Add(new BXParamValue(GetMessageRaw("Title"), "name"));
        sortFields.Add(new BXParamValue(GetMessageRaw("DateOfModification"), "updatedate"));
        sortFields.Add(new BXParamValue(GetMessageRaw("SortIndex"), "sort"));
        sortFields.Add(new BXParamValue(GetMessageRaw("QuantityOfViewsAvg"), "sort")); /// ?
        sortFields.Add(new BXParamValue(GetMessageRaw("ActiveFromDate"), "activefromdate"));
        sortFields.Add(new BXParamValue(GetMessageRaw("ActiveUntilDate"), "activetodate"));

        ParamsDefinition.Add(
            "TOPSecSortBy",
            new BXParam(
                GetMessageRaw("TopSectionSortBy.Title"),
                BXParamType.SingleSelection,
                "sort",
                new BXCategory(GetMessageRaw("TopSectionSortBy.Category"), "TopSettings", 110),
                sortFields
                )
            );

        ParamsDefinition.Add(
            "TOPSecSortOrder",
            new BXParamSort(
                GetMessageRaw("TopSectionSortOrder.Title"),
                BXCategory.Categories["TopSettings"]
                )
            );

        ParamsDefinition.Add(
            "COUNT_ELEMENTS",
            new BXParamYesNo(
                GetMessageRaw("DisplayQuantityOfElementsInSection"),
                true,
                BXCategory.Main
                ));

        ParamsDefinition.Add(
            "ShowTopElements",
            new BXParamYesNo(
                GetMessageRaw("DisplayTopOfElements"),
                true,
                BXCategory.Categories["TopSettings"]
                )
            );

        ParamsDefinition.Add(
            "TOPPerPageCount",
            new BXParamText(
                GetMessageRaw("ElementsPerPage"),
                "30",
                BXCategory.Categories["TopSettings"]
                ));

        ParamsDefinition.Add(
            "TOPPerLineCount",
            new BXParamText(
                GetMessageRaw("ElementsPerTableRow"),
                "3",
                BXCategory.Categories["TopSettings"]
                ));

        ParamsDefinition.Add(
            "TOPElementsSortBy",
            new BXParamSingleSelection(
                GetMessageRaw("TopElementsSortBy.Title"),
                "sort",
                BXCategory.Categories["TopSettings"],
                sortFields
                )
            );

        ParamsDefinition.Add(
            "TOPElementsSortOrder",
            new BXParamSort(
                GetMessageRaw("TopElementsSortOrder.Title"),
                BXCategory.Categories["TopSettings"]
                )
            );

        ParamsDefinition.Add(
            "TOPProperties",
            new BXParamMultiSelectionWithText(
                GetMessageRaw("Properties"),                
                "-",
                BXCategory.Categories["TopSettings"]
                ));

        #endregion

        #region ElementsList
        ParamsDefinition.Add(
            "ELPerPageCount",
            new BXParamText(
                GetMessageRaw("ElementsPerPage"),
                "30",
                BXCategory.ListSettings
                ));

        ParamsDefinition.Add(
            "ELPerLineCount",
            new BXParamText(
                GetMessageRaw("ElementsPerTableRow"),
                "3",
                BXCategory.ListSettings
                ));

        ParamsDefinition.Add(
            "ELSortBy",
            new BXParamSingleSelection(
                GetMessageRaw("TopSectionSortBy.Title"),
                "sort",
                BXCategory.ListSettings,
                sortFields
                )
            );

        ParamsDefinition.Add(
            "ELSortOrder",
            new BXParamSort(
                GetMessageRaw("TopSectionSortOrder.Title"),
                BXCategory.ListSettings
                )
            );

        ParamsDefinition.Add(
            "ELProperties",
            new BXParamMultiSelectionWithText(
                GetMessageRaw("Properties"),                
                "-",
                BXCategory.ListSettings
                ));

        ParamsDefinition.Add(
            "ELShowSubelements",
            new BXParamYesNo(
                GetMessageRaw("DisplaySubsectionElements"),
                true,
                BXCategory.ListSettings
                ));

        #endregion

        #region DetailView
        ParamsDefinition.Add(
            "DTProperties",
            new BXParamMultiSelectionWithText(
                GetMessageRaw("Properties"),
                "-",
                BXCategory.DetailSettings
                ));
        #endregion

        #region Additional

        ParamsDefinition.Add(
            "AddAdminPanelButtons",
            new BXParamYesNo(
                GetMessageRaw("AddButtonsForThisComponentToAdminPanel"),
                false,
                BXCategory.AdditionalSettings
                ));
        ParamsDefinition.Add(
            "SetPageTitle",
            new BXParamYesNo(
                GetMessageRaw("SetPageTitle"),
                true,
                BXCategory.AdditionalSettings
                ));

        #endregion

        ParamsDefinition.Add(BXParametersDefinition.Cache);
        ParamsDefinition.Add(BXParametersDefinition.Paging);
		ParamsDefinition.Add(BXParametersDefinition.Ajax);

		ParamsDefinition.Add(BXParametersDefinition.Menu(DesignerSite));

        ParamsDefinition.Remove("PagingAllowAll");
        Parameters["PagingAllowAll"] = false.ToString();
    }

    protected override void LoadComponentDefinition()
    {
        #region MainGroup
        List<BXParamValue> types = new List<BXParamValue>();
        types.Add(new BXParamValue("-", ""));
        BXIBlockTypeCollection typeCollection = BXIBlockType.GetList(null, new BXOrderBy(new BXOrderByPair(BXIBlockType.Fields.Sort, BXOrderByDirection.Asc)));
        foreach (BXIBlockType t in typeCollection)
            types.Add(new BXParamValue(t.Translations[BXLoc.CurrentLocale].Name, t.Id.ToString()));

        ParamsDefinition["IBlockTypeId"].Values = types;

		BXFilter iblockFilter = new BXFilter();
		if (Parameters.ContainsKey("IBlockTypeId"))
		{
			int typeId;
			int.TryParse(Parameters["IBlockTypeId"], out typeId);
			if (typeId > 0)
				iblockFilter.Add(new BXFilterItem(BXIBlock.Fields.Type.ID, BXSqlFilterOperators.Equal, typeId));
		}
		if (!string.IsNullOrEmpty(DesignerSite))
			iblockFilter.Add(new BXFilterItem(BXIBlock.Fields.Sites.SiteId, BXSqlFilterOperators.Equal, DesignerSite));

        List<BXParamValue> iblocks = new List<BXParamValue>();
        BXIBlockCollection iblockCollection = BXIBlock.GetList(iblockFilter, new BXOrderBy(new BXOrderByPair(BXIBlock.Fields.Sort, BXOrderByDirection.Asc)));
        foreach (BXIBlock b in iblockCollection)
            iblocks.Add(new BXParamValue(b.Name, b.Id.ToString()));

        ParamsDefinition["IBlockId"].Values = iblocks;
        #endregion

        #region TOPElements
        List<BXParamValue> properties = new List<BXParamValue>();
        if (Parameters.ContainsKey("IBlockId"))
        {
            int iblockId;
            int.TryParse(Parameters["IBlockId"], out iblockId);
            if (iblockId > 0)
            {
                BXCustomFieldCollection cfCollection = BXCustomEntityManager.GetFields("IBlock_" + iblockId);
                foreach (BXCustomField f in cfCollection)
                    properties.Add(new BXParamValue(f.EditFormLabel, f.Name));
            }
        }

        ParamsDefinition["TOPProperties"].Values = properties;
        #endregion

        #region ELProperties
        ParamsDefinition["ELProperties"].Values = properties;
        #endregion

        #region DetailView
        ParamsDefinition["DTProperties"].Values = properties;
        #endregion

        ParamsDefinition.Add(BXParametersDefinition.Paging);
        ParamsDefinition.Add(BXParametersDefinition.Search);

        foreach (string parDefKey in ParamsDefinition.Keys)
            ParamsDefinition[parDefKey].AdjustPresentation(this);
	}

    public override void ProcessMessage(BXCommand cmd, bool executionContext, string executionVirtualPath)
    {
        if (cmd.Action == "Bitrix.Search.ProvideUrl")
        {
            if (cmd.Parameters.Get<string>("moduleId") != "iblock")
                return;
            int iblockId = Parameters.Get("IBlockId", -1);
            if (iblockId < 0 || cmd.Parameters.Get<int>("itemGroup") != iblockId)
                return;
            int elementId = cmd.Parameters.Get("itemId", -1);
            if (elementId < 0)
                return;

            BXIBlockElement ibe = BXIBlockElement.GetById(elementId);
            BXIBlockSection ibs = null;
            if (ibe != null)
                foreach (int sid in ibe.Sections.ToList())
                {
                    ibs = BXIBlockSection.GetById(sid);
                    if (ibs != null)
                        break;
                }
            if (ibe == null || ibs == null)
                return;

            Results["SECTION_ID"] = ibs.Id;
            Results["ELEMENT_ID"] = ibe.Id;
            Results["SECTION_CODE"] = ibs.Code ?? string.Empty;
            Results["ELEMENT_CODE"] = ibe.Code ?? string.Empty;

            string url;
            if (Parameters.Get<bool>("EnableSEF"))
                url =
                    MakeLink(Parameters.Get<string>("SEFFolder"),
                             Parameters.Get<string>("SEF_URL_TEMPLATES_element", "#SECTION_ID#/#ELEMENT_ID#/"));
            else
                url = MakeLink(string.Format(
                                   "{0}?{1}=#SECTION_ID#&{2}=#ELEMENT_ID#",
                                   VirtualPathUtility.ToAbsolute(executionVirtualPath),
                                   Parameters.Get<string>("VARIABLE_ALIASES_SECTION_ID", "SECTION_ID"),
                                   Parameters.Get<string>("VARIABLE_ALIASES_ELEMENT_ID", "ELEMENT_ID")
                                   ));
            cmd.AddCommandResult("bitrix:catalog@" + executionVirtualPath,
                                 new BXCommandResult(BXCommandResultType.Ok, url));
        }

		if (cmd.Action == "Bitrix.Main.GeneratePublicMenu")
		{
			//Bitrix.Diagnostics.BXDump.ToFile(
			//    String.Format(
			//        "executionVirtualPath={0}, uriPath={1}, uri={2}, SefFolder={3}, GenerateMenuType={4}, menuType={5}", 
			//        executionVirtualPath, 
			//        cmd.Parameters.Get<string>("uriPath"), 
			//        cmd.Parameters.Get<string>("uri"), 
			//        Bitrix.IO.BXPath.ToVirtualRelativePath(SEFFolder.TrimStart('\\', '/')),
			//        Parameters.Get("GenerateMenuType", "left"),
			//        cmd.Parameters.Get<string>("menuType")
			//    )
			//);

			//if (!cmd.Parameters.Get<string>("uriPath").StartsWith(Bitrix.IO.BXPath.ToVirtualRelativePath(SEFFolder.TrimStart('\\', '/')) + "/", StringComparison.InvariantCultureIgnoreCase))
				//return;

			//Генерируем меню только для тех адресов, которые выводит сам компонент.
			if (!Parameters.Get<bool>("EnableSEF") && !executionVirtualPath.Equals(cmd.Parameters.Get<string>("uriPath"), StringComparison.InvariantCultureIgnoreCase))
				return;
			else if (Parameters.Get<bool>("EnableSEF") && !cmd.Parameters.Get<string>("uriPath").StartsWith(Bitrix.IO.BXPath.ToVirtualRelativePath(SEFFolder.TrimStart('\\', '/')) + "/", StringComparison.InvariantCultureIgnoreCase))
				return;

			if (!Parameters.Get("GenerateMenuType", "left").Equals(cmd.Parameters.Get<string>("menuType"), StringComparison.InvariantCultureIgnoreCase))
				return;

			int iblockId = Parameters.Get("IBlockId", -1);
			if (iblockId < 0)
			    return;

			BXParamsBag<string> urlToPage = new BXParamsBag<string>();
			BXParamsBag<object> request = new BXParamsBag<object>();
			string PageKey = String.Empty;

			urlToPage.Add("ElementTOP", TOPElementURL.TrimEnd('/', '\\'));
			urlToPage.Add("SectionPage", SectionPageURL.TrimEnd('/', '\\'));
			urlToPage.Add("Element", ElementURL.TrimEnd('/', '\\'));
			urlToPage.Add("Section", SectionURL.TrimEnd('/', '\\'));

			if (EnableSef)
			{
				PageKey = MapVariable(SEFFolder, urlToPage, request, String.Empty, BXUri.ToRelativeUri(cmd.Parameters.Get<string>("uri")));
			}
			else
			{
				BXParamsBag<string> variableAliases = new BXParamsBag<string>();
				variableAliases.Add(ParamSectionId, AliasSectionId);
				variableAliases.Add(ParamElementId, AliasElementId);
				variableAliases.Add(ParamPageId, AliasPageId);
				variableAliases.Add(ParamSectionCode, AliasSectionCode);
				variableAliases.Add(ParamElementCode, AliasElementCode);

				MapVariable(variableAliases, request, BXUri.ToRelativeUri(cmd.Parameters.Get<string>("uri")));
			}

			SectionId = request.Get(ParamSectionId, SectionId);
			ElementId = request.Get(ParamElementId, ElementId);
			PageId = request.Get(ParamPageId, PageId);
			ElementCode = request.Get(ParamElementCode, ElementCode);
			SectionCode = request.Get(ParamSectionCode, SectionCode);

			List<BXPublicMenuItem> resultList = null;
			string parentLevelUri = null;

			if (ElementId > 0 || !String.IsNullOrEmpty(ElementCode))
			{
				if (SectionId > 0 || !String.IsNullOrEmpty(SectionCode))
				{
					BXParamsBag<object> p = new BXParamsBag<object>();
					p["SECTION_ID"] = SectionId;
					p["SECTION_CODE"] = SectionCode;
					parentLevelUri = MakeMenuUri(executionVirtualPath, p);
				}
				else
				{
					BXFilter elFilter = new BXFilter(
						new BXFilterItem(BXIBlockElement.Fields.Active, BXSqlFilterOperators.Equal, "Y"),
						new BXFilterItem(BXIBlockElement.Fields.ActiveDate, BXSqlFilterOperators.Equal, "Y")
					);
					if (ElementId > 0)
						elFilter.Add(new BXFilterItem(BXIBlockElement.Fields.ID, BXSqlFilterOperators.Equal, ElementId));
					else
						elFilter.Add(new BXFilterItem(BXIBlockElement.Fields.Code, BXSqlFilterOperators.Equal, ElementCode));

					BXIBlockElementCollection collElement = BXIBlockElement.GetList(elFilter, null);
					if (collElement != null)
					{
						if (collElement.Count > 0)
						{
							BXIBlockElement.BXInfoBlockElementSectionCollection es = collElement[0].Sections;
							if (es != null)
							{
								if (es.Count > 0)
								{
									SectionId = es[0].SectionId;

									BXParamsBag<object> p = new BXParamsBag<object>();
									p["SECTION_ID"] = SectionId;

									if (Parameters.Get<bool>("EnableSEF") && SectionURL.Contains("#SECTION_CODE#"))
									{
										BXIBlockSectionCollection collSection = BXIBlockSection.GetList(
											new BXFilter(
												new BXFilterItem(BXIBlockSection.Fields.ActiveGlobal, BXSqlFilterOperators.Equal, "Y"),
												new BXFilterItem(BXIBlockSection.Fields.Active, BXSqlFilterOperators.Equal, "Y"),
												new BXFilterItem(BXIBlockSection.Fields.ID, BXSqlFilterOperators.Equal, SectionId),
												new BXFilterItem(BXIBlockSection.Fields.IBlock.ID, BXSqlFilterOperators.Equal, iblockId)
											),
											null
										);
										if (collSection != null)
										{
											if (collSection.Count > 0)
											{
												SectionCode = collSection[0].Code;
												p["SECTION_CODE"] = SectionCode;
											}
										}
									}

									parentLevelUri = MakeMenuUri(executionVirtualPath, p);
								}
							}
						}
					}
				}
			}

			if (parentLevelUri == null)
			{
				if (SectionId > 0 || !String.IsNullOrEmpty(SectionCode))
				{
					int parentSectionId = 0;
					bool parentSectionFound = false;

					if (SectionId <= 0)
					{
						BXIBlockSectionCollection coll = BXIBlockSection.GetList(
							new BXFilter(
								new BXFilterItem(BXIBlockSection.Fields.ActiveGlobal, BXSqlFilterOperators.Equal, "Y"),
								new BXFilterItem(BXIBlockSection.Fields.Active, BXSqlFilterOperators.Equal, "Y"),
								new BXFilterItem(BXIBlockSection.Fields.Code, BXSqlFilterOperators.Equal, SectionCode),
								new BXFilterItem(BXIBlockSection.Fields.IBlock.ID, BXSqlFilterOperators.Equal, iblockId)
							),
							null
						);
						if (coll != null)
						{
							if (coll.Count > 0)
							{
								SectionId = coll[0].Id;
								parentSectionId = coll[0].SectionId;
								parentSectionFound = true;
							}
						}
					}

					if (SectionId > 0)
					{
						BXIBlockSectionCollection coll = BXIBlockSection.GetList(
							new BXFilter(
								new BXFilterItem(BXIBlockSection.Fields.SectionId, BXSqlFilterOperators.Equal, SectionId),
								new BXFilterItem(BXIBlockSection.Fields.ActiveGlobal, BXSqlFilterOperators.Equal, "Y"),
								new BXFilterItem(BXIBlockSection.Fields.Active, BXSqlFilterOperators.Equal, "Y"),
								new BXFilterItem(BXIBlockSection.Fields.IBlock.ID, BXSqlFilterOperators.Equal, iblockId)
							),
							new BXOrderBy(
								new BXOrderByPair(BXIBlockSection.Fields.Sort, BXOrderByDirection.Asc),
								new BXOrderByPair(BXIBlockSection.Fields.Name, BXOrderByDirection.Asc)
							)
						);

						resultList = ExtractMenuItemsFromCollection(executionVirtualPath, resultList, coll);

						if (resultList == null)
						{
							if (!parentSectionFound)
							{
								coll = BXIBlockSection.GetList(
									new BXFilter(
										new BXFilterItem(BXIBlockSection.Fields.ActiveGlobal, BXSqlFilterOperators.Equal, "Y"),
										new BXFilterItem(BXIBlockSection.Fields.Active, BXSqlFilterOperators.Equal, "Y"),
										new BXFilterItem(BXIBlockSection.Fields.ID, BXSqlFilterOperators.Equal, SectionId),
										new BXFilterItem(BXIBlockSection.Fields.IBlock.ID, BXSqlFilterOperators.Equal, iblockId)
									),
									null
								);
								if (coll != null)
								{
									if (coll.Count > 0)
									{
										parentSectionFound = true;
										parentSectionId = coll[0].SectionId;
									}
								}
							}

							if (parentSectionFound && parentSectionId > 0)
							{
								BXParamsBag<object> p = new BXParamsBag<object>();
								p["SECTION_ID"] = parentSectionId;

								if (Parameters.Get<bool>("EnableSEF") && SectionURL.Contains("#SECTION_CODE#"))
								{
									coll = BXIBlockSection.GetList(
										new BXFilter(
											new BXFilterItem(BXIBlockSection.Fields.ActiveGlobal, BXSqlFilterOperators.Equal, "Y"),
											new BXFilterItem(BXIBlockSection.Fields.Active, BXSqlFilterOperators.Equal, "Y"),
											new BXFilterItem(BXIBlockSection.Fields.ID, BXSqlFilterOperators.Equal, parentSectionId),
											new BXFilterItem(BXIBlockSection.Fields.IBlock.ID, BXSqlFilterOperators.Equal, iblockId)
										),
										null
									);
									if (coll != null)
									{
										if (coll.Count > 0)
											p["SECTION_CODE"] = coll[0].Code;
									}
								}

								parentLevelUri = MakeMenuUri(executionVirtualPath, p);
							}
							else
							{
								if (Parameters.Get<bool>("EnableSEF"))
									parentLevelUri = MakeLink(Parameters.Get<string>("SEFFolder"), "");
								else
									parentLevelUri = BXUri.ToAppRelativeUri(executionVirtualPath);
							}
						}
					}
				}
			}

			if (parentLevelUri == null)
			{
				if (resultList == null)
				{
					BXIBlockSectionCollection coll = BXIBlockSection.GetList(
						new BXFilter(
							new BXFilterItem(BXIBlockSection.Fields.SectionId, BXSqlFilterOperators.Equal, null),
							new BXFilterItem(BXIBlockSection.Fields.ActiveGlobal, BXSqlFilterOperators.Equal, "Y"),
							new BXFilterItem(BXIBlockSection.Fields.Active, BXSqlFilterOperators.Equal, "Y"),
							new BXFilterItem(BXIBlockSection.Fields.IBlock.ID, BXSqlFilterOperators.Equal, iblockId)
						),
						new BXOrderBy(
							new BXOrderByPair(BXIBlockSection.Fields.Sort, BXOrderByDirection.Asc),
							new BXOrderByPair(BXIBlockSection.Fields.Name, BXOrderByDirection.Asc)
						)
					);

					resultList = ExtractMenuItemsFromCollection(executionVirtualPath, resultList, coll);
				}
			}

			BXPublicMenu.BXLoadMenuCommandParameter r = new BXPublicMenu.BXLoadMenuCommandParameter(resultList, true, parentLevelUri);
			cmd.AddCommandResult("bitrix:catalog@" + executionVirtualPath, new BXCommandResult(BXCommandResultType.Ok, r));
		}
	}

	private List<BXPublicMenuItem> ExtractMenuItemsFromCollection(string executionVirtualPath, List<BXPublicMenuItem> resultList, BXIBlockSectionCollection coll)
	{
		resultList = null;
		if (coll != null && coll.Count > 0)
		{
			resultList = new List<BXPublicMenuItem>();
			foreach (BXIBlockSection s in coll)
			{
				BXPublicMenuItem i1 = new BXPublicMenuItem();
				i1.Title = s.Name;

				BXParamsBag<object> p = new BXParamsBag<object>();
				p["SECTION_ID"] = s.Id;
				p["SECTION_CODE"] = s.Code ?? string.Empty;
				i1.Links.Add(MakeMenuUri(executionVirtualPath, p));
				i1.Sort = s.Sort;
				resultList.Add(i1);
			}
		}
		return resultList;
	}

	private string MakeMenuUri(string executionVirtualPath, BXParamsBag<object> p)
	{
		string url;
		if (Parameters.Get<bool>("EnableSEF"))
			url = MakeLink(
					Parameters.Get<string>("SEFFolder"),
					SectionURL,
					p);
		else
			url = MakeLink(
					string.Format(
						"{0}?{1}=#SECTION_ID#",
						BXUri.ToAppRelativeUri(executionVirtualPath),
						Parameters.Get<string>("VARIABLE_ALIASES_SECTION_ID", "SECTION_ID")
					),
					p);
		return url;
	}
}
