﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using Bitrix.Security;
using Bitrix.Configuration;
using Bitrix.Services;
using System.Web.Script.Serialization;
using Bitrix.IBlock;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using System.Text;
using Bitrix.Services.Js;
using Bitrix.Sale;
using Bitrix.Services.Location;
using Bitrix;
using System.Collections.Specialized;
using Bitrix.Sale.UI;


public partial class bitrix_modules_Sale_Options : BXControl
{
	private Dictionary<int, BXSaleExportMapping> Exports = new Dictionary<int, BXSaleExportMapping>();
	private Dictionary<int, Mapping> Mappings = new Dictionary<int, Mapping>();
	protected BXSaleCustomerTypeCollection CustomerTypes;
	private bool isPostBack;

	private bool? isAuthorized = null;
	public bool IsAuthorized
	{
		get { return (this.isAuthorized ?? (this.isAuthorized = BXPrincipal.Current.IsCanOperate(BXRoleOperation.Operations.ProductSettingsManage))).Value; }
	}

	private string backUrl = null;
	public string BackUrl
	{
		get { return this.backUrl ?? (this.backUrl = Request.QueryString[BXConfigurationUtility.Constants.BackUrl] ?? BXSefUrlManager.CurrentUrl.ToString()); }
	}

	private SaleModuleOptionsEditorError editorError = SaleModuleOptionsEditorError.None;
	public SaleModuleOptionsEditorError EditorError
	{
		get { return this.editorError; }
		private set { this.editorError = value; }
	}

	private string siteId = null;
	public string SiteId
	{
		get { return this.siteId ?? (this.siteId = Request["siteId"] ?? string.Empty); }
	}

	private BXSale config = null;
	public BXSale Config
	{
		get { return this.config; }
	}

	private string siteConfigUrlTemplate = null;
	protected string SiteConfigUrlTemplate
	{
		get
		{
			if (this.siteConfigUrlTemplate != null)
				return this.siteConfigUrlTemplate;

			NameValueCollection qsParams = HttpUtility.ParseQueryString(BXSefUrlManager.CurrentUrl.Query);
			qsParams.Remove("siteId");
			qsParams.Add("siteId", "__BX_SITE_ID__");
			UriBuilder ub = new UriBuilder(BXSefUrlManager.CurrentUrl);
			ub.Query = qsParams.ToString();

			return (this.siteConfigUrlTemplate = ub.Uri.PathAndQuery);
		}
	}

	protected override void OnInit(EventArgs e)
	{
		base.OnInit(e);
		isPostBack = (Request.Form["sale_options_postback"] != null);

		CustomerTypes = BXSaleCustomerType.GetList(
			null,
			new BXOrderBy(
				new BXOrderByPair(BXSaleCustomerType.Fields.Site.ID, BXOrderByDirection.Asc),
				new BXOrderByPair(BXSaleCustomerType.Fields.Id, BXOrderByDirection.Asc)
			),
			null,
			null,
			BXTextEncoder.EmptyTextEncoder
		);
		var exportMappings = BXSaleExportMapping.GetList(
			null,
			null,
			null,
			null,
			BXTextEncoder.EmptyTextEncoder
		);

		foreach (var t in CustomerTypes)
			Exports[t.Id] = 
				exportMappings.Find(x => x.CustomerTypeId == t.Id) 
				?? new BXSaleExportMapping(BXTextEncoder.EmptyTextEncoder) { CustomerTypeId = t.Id };
		
		if (isPostBack)
		{
			foreach (var p in Exports)
			{
				p.Value.ExportType = 
					Request.Form["saleOrderMappingType$" + p.Key.ToString()] == "JUR" 
					? BXSaleExportMappingExportType.Juristic
					: BXSaleExportMappingExportType.Physical;
			}			
		}


		BindingControls.DataSource = CustomerTypes;
		BindingControls.DataBind();

		foreach (var t in CustomerTypes)
		{
			var tab = new BXTabControlTab();
			tab.Text = string.Format("[{0}] {1}", t.SiteId, t.Name);
			tab.Title = t.Name;
			Bindings.Tabs.Add(tab);

			var renderer = new Control();
			renderer.SetRenderMethodDelegate(GetRenderMethod(Mappings[t.Id].RenderSource));
			tab.Controls.Add(renderer);
		}
		if (Bindings.Tabs.Count > 0)
			Bindings.SelectedIndex = 0;



		tabContainer.ShowApplyButton = tabContainer.ShowSaveButton = IsAuthorized;
		tabContainer.ShowCancelButton = BackUrl.Length > 0;

		ListItemCollection countryItems = CountryList.Items;
		countryItems.Clear();

		string curLangCode = BXLoc.CurrentLocale.ToUpperInvariant();
		if (curLangCode == "EN")
			curLangCode = "US";

		foreach (BXCountry country in BXCountry.GetAll())
		{
			if (string.Equals(country.Iso3166AlphaCode, curLangCode, StringComparison.Ordinal))
				countryItems.Insert(0, new ListItem(country.NativeName, country.Iso3166NumericCode.ToString()));
			else
				countryItems.Add(new ListItem(string.Concat(country.GetLocalizedName(curLangCode), " / " + country.NativeName), country.Iso3166NumericCode.ToString()));
		}

		BXSiteCollection siteColl = BXSite.GetList(null,
			new BXOrderBy(new BXOrderByPair(BXSite.Fields.Sort, BXOrderByDirection.Asc)),
			new BXSelect(BXSite.Fields.ID, BXSite.Fields.Name),
			null,
			BXTextEncoder.EmptyTextEncoder
			);

		ListItemCollection siteItems = SiteList.Items;
		siteItems.Clear();

		siteItems.Add(new ListItem(GetMessageRaw("SelectItem.AllSites"), string.Empty));
		foreach (BXSite site in siteColl)
			siteItems.Add(new ListItem(string.Format("[{0}] {1}", site.Id, site.Name), site.Id));
	}

	RenderMethod GetRenderMethod(Control control)
	{
		return (a, b) => control.RenderControl(a);
	}

	protected override void OnLoad(EventArgs e)
	{
		base.OnLoad(e);
		LoadData();
		if (!IsPostBack)
		{
			ListItem li = null;

			if (!string.IsNullOrEmpty(SiteId) && (li = SiteList.Items.FindByValue(SiteId)) != null)
				li.Selected = true;

			if (this.config.CountryNumericCode > 0 && (li = CountryList.Items.FindByValue(this.config.CountryNumericCode.ToString())) != null)
				li.Selected = true;

			PostalCodeTbx.Text = this.config.PostalCode;
			LocationNameTbx.Text = this.config.LocationName;
		}
	}

	protected override void OnPreRender(EventArgs e)
	{
		base.OnPreRender(e);
		BXPage.Scripts.RequireUtils();
	}

	protected void OnTabCommand(object sender, BXTabControlCommandEventArgs e)
	{
		SaleModuleOptionsEditorCommand? cmd = null;

		if (string.Equals(e.CommandName, "save", StringComparison.Ordinal))
			cmd = SaleModuleOptionsEditorCommand.Save;
		else if (string.Equals(e.CommandName, "apply", StringComparison.Ordinal))
			cmd = SaleModuleOptionsEditorCommand.Apply;

		if (!cmd.HasValue)
			Response.Redirect(BackUrl);

		if (Page.IsValid && (cmd.Value == SaleModuleOptionsEditorCommand.Apply || cmd.Value == SaleModuleOptionsEditorCommand.Save))
			SaveData();

		if (EditorError == SaleModuleOptionsEditorError.None)
		{
			if (cmd.Value == SaleModuleOptionsEditorCommand.Save && BackUrl.Length > 0)
				Page.Response.Redirect(BackUrl);
			else
			{
				LoadData();
				successMessage.Visible = true;
			}
		}
	}

	protected void BindingControls_ItemDataBound(object sender, RepeaterItemEventArgs e)
	{
		if (e.Item.ItemType != ListItemType.Item && e.Item.ItemType != ListItemType.AlternatingItem)
			return;

		var t = (BXSaleCustomerType)e.Item.DataItem;
		var m = new Mapping
		{
			PhysicalMapping = (OrderMappingEditor)e.Item.FindControl("PhysicalMapping"),
			JuristicMapping = (OrderMappingEditor)e.Item.FindControl("JuristicMapping"),
			RenderSource = e.Item
		};
		Mappings[t.Id] = m;

		m.PhysicalMapping.CustomerTypeId = t.Id;
		m.PhysicalMapping.Fields.AddRange(
			new Dictionary<string, string>
		    {
		        { "AGENT_NAME", GetMessageRaw("AgentName") },
				{ "FULL_NAME", GetMessageRaw("FullName") },
				{ "SURNAME", GetMessageRaw("LastName") },
				{ "NAME", GetMessageRaw("FirstName") },
				{ "SECOND_NAME", GetMessageRaw("SecondName") },
				{ "BIRTHDAY", GetMessageRaw("Birthday") },
				{ "MALE", GetMessageRaw("Sex") },
				{ "INN", GetMessageRaw("INN") },
				{ "KPP", GetMessageRaw("KPP") },
				{ "ADDRESS_FULL", GetMessageRaw("FullAddress") },
				{ "INDEX", GetMessageRaw("PostalCode") },
				{ "COUNTRY", GetMessageRaw("Country") },
				{ "REGION", GetMessageRaw("Region") },
				{ "STATE", GetMessageRaw("State") },
				{ "TOWN", GetMessageRaw("Town") },
				{ "CITY", GetMessageRaw("City") },
				{ "STREET", GetMessageRaw("Street") },
				{ "BUILDING", GetMessageRaw("Building") },
				{ "HOUSE", GetMessageRaw("House") },
				{ "FLAT", GetMessageRaw("Flat") },
				{ "PHONE", GetMessageRaw("Phone") },
				{ "EMAIL", GetMessageRaw("Email") },
				{ "CONTACT_PERSON", GetMessageRaw("ContactPerson") },
				{ "F_ADDRESS_FULL", GetMessageRaw("RealFullAddress") },
				{ "F_INDEX", GetMessageRaw("RealPostalCode") },
				{ "F_COUNTRY", GetMessageRaw("RealCountry") },
				{ "F_REGION", GetMessageRaw("RealRegion") },
				{ "F_STATE", GetMessageRaw("RealState") },
				{ "F_TOWN", GetMessageRaw("RealTown") },
				{ "F_CITY", GetMessageRaw("RealCity") },
				{ "F_STREET", GetMessageRaw("RealStreet") },
				{ "F_BUILDING", GetMessageRaw("RealBuilding") },
				{ "F_HOUSE", GetMessageRaw("RealHouse") },
				{ "F_FLAT", GetMessageRaw("RealFlat") }
		    }
		);

		m.JuristicMapping.CustomerTypeId = t.Id;
		m.JuristicMapping.Fields.AddRange(
			new Dictionary<string, string>
		    {
		        { "AGENT_NAME", GetMessageRaw("AgentName") },
				{ "FULL_NAME", GetMessageRaw("FullName") },
				{ "ADDRESS_FULL", GetMessageRaw("FullAddress") },
				{ "INDEX", GetMessageRaw("PostalCode") },
				{ "COUNTRY", GetMessageRaw("Country") },
				{ "REGION", GetMessageRaw("Region") },
				{ "STATE", GetMessageRaw("State") },
				{ "TOWN", GetMessageRaw("Town") },
				{ "CITY", GetMessageRaw("City") },
				{ "STREET", GetMessageRaw("Street") },
				{ "BUILDING", GetMessageRaw("Building") },
				{ "HOUSE", GetMessageRaw("House") },
				{ "FLAT", GetMessageRaw("Flat") },
				{ "INN", GetMessageRaw("INN") },
				{ "KPP", GetMessageRaw("KPP") },
				{ "EGRPO", GetMessageRaw("EGRPO") },
				{ "OKVED", GetMessageRaw("OKVED") },
				{ "OKDP", GetMessageRaw("OKDP") },
				{ "OKOPF", GetMessageRaw("OKOPF") },
				{ "OKFC", GetMessageRaw("OKFC") },
				{ "OKPO", GetMessageRaw("OKPO") },
				{ "ACCOUNT_NUMBER", GetMessageRaw("AccountNumber") },
				{ "B_NAME", GetMessageRaw("BankName") },
				{ "B_BIK", GetMessageRaw("BankBIK") },
				{ "B_ADDRESS_FULL", GetMessageRaw("BankFullAddress") },
				{ "B_INDEX", GetMessageRaw("BankPostalCode") },
				{ "B_COUNTRY", GetMessageRaw("BankCountry") },
				{ "B_REGION", GetMessageRaw("BankRegion") },
				{ "B_STATE", GetMessageRaw("BankState") },
				{ "B_TOWN", GetMessageRaw("BankTown") },
				{ "B_CITY", GetMessageRaw("BankCity") },
				{ "B_STREET", GetMessageRaw("BankStreet") },
				{ "B_BUILDING", GetMessageRaw("BankBuilding") },
				{ "B_HOUSE", GetMessageRaw("BankHouse") },
				{ "PHONE", GetMessageRaw("Phone") },
				{ "EMAIL", GetMessageRaw("Email") },
				{ "CONTACT_PERSON", GetMessageRaw("ContactPerson") },
				{ "F_ADDRESS_FULL", GetMessageRaw("RealFullAddress") },
				{ "F_INDEX", GetMessageRaw("RealPostalCode") },
				{ "F_COUNTRY", GetMessageRaw("RealCountry") },
				{ "F_REGION", GetMessageRaw("RealRegion") },
				{ "F_STATE", GetMessageRaw("RealState") },
				{ "F_TOWN", GetMessageRaw("RealTown") },
				{ "F_CITY", GetMessageRaw("RealCity") },
				{ "F_STREET", GetMessageRaw("RealStreet") },
				{ "F_BUILDING", GetMessageRaw("RealBuilding") },
				{ "F_HOUSE", GetMessageRaw("RealHouse") },
				{ "F_FLAT", GetMessageRaw("RealFlat") }
		    }
		);

		if (!isPostBack)
		{
			var ex = Exports[t.Id];
			(ex.ExportType == BXSaleExportMappingExportType.Physical ? m.PhysicalMapping : m.JuristicMapping)
			.LoadMappings(ex.Mapping);
		}
	}

	private void SaveData()
	{
		if (this.config == null || EditorError != SaleModuleOptionsEditorError.None)
			return;

		try
		{
			//this.config.SiteId = SiteId;
			if (!string.IsNullOrEmpty(CountryList.SelectedValue))
			{
				int i;
				this.config.CountryNumericCode = int.TryParse(CountryList.SelectedValue, out i) ? i : 0;
			}
			else
				this.config.CountryNumericCode = 0;

			this.config.PostalCode = this.PostalCodeTbx.Text;
			this.config.LocationName = this.LocationNameTbx.Text;

			this.config.Save();

			foreach (var p in Exports)
			{
				var m = Mappings[p.Key];
				var mappingControl = p.Value.ExportType == BXSaleExportMappingExportType.Physical ? m.PhysicalMapping : m.JuristicMapping;
				
				p.Value.Mapping.Clear();
				mappingControl.SaveMappings(p.Value.Mapping);

				p.Value.Save();
			}
		}
		catch (Exception exc)
		{
			EditorError = SaleModuleOptionsEditorError.DataSavingFailed;
			errorMessage.AddErrorMessage(Encode(exc.Message));
		}
	}

	private void LoadData()
	{
		try
		{
			this.config = BXSale.GetBySiteId(SiteId);
		}
		catch (Exception exc)
		{
			EditorError = SaleModuleOptionsEditorError.DataLoadingFailed;
			errorMessage.AddErrorMessage(Encode(exc.Message));
		}
	}

	protected string GetMappingStyle(int id, bool physical)
	{
		return 
			(Exports[id].ExportType == BXSaleExportMappingExportType.Physical) == physical
			? ""
			: @"style=""display:none;""";
	}

	protected string GetMappingSelected(int id, bool physical)
	{
		return 
			(Exports[id].ExportType == BXSaleExportMappingExportType.Physical) == physical
			? @"selected=""selected"""
			: "";
	}

	class Mapping
	{
		public Control RenderSource;
		public OrderMappingEditor PhysicalMapping;
		public OrderMappingEditor JuristicMapping;
	}
}

public enum SaleModuleOptionsEditorError
{
	None = 0,
	DataLoadingFailed = -1,
	DataSavingFailed = -2
}

public enum SaleModuleOptionsEditorCommand
{
	Save = 1,
	Apply
}