﻿IF OBJECT_ID(N'b_SaleCart', N'U') IS NULL
CREATE TABLE b_SaleCart
(
ID INT IDENTITY(1,1) CONSTRAINT PK_b_SaleCart PRIMARY KEY,
UserId INT NULL,
UniqueId CHAR(38) NOT NULL,
ResolverId NVARCHAR(50) NOT NULL
)

GO
IF OBJECT_ID(N'b_SaleCartItem', N'U') IS NULL
CREATE TABLE b_SaleCartItem
(
	ID INT IDENTITY(1,1) CONSTRAINT PK_b_SaleCartItem PRIMARY KEY,
	CartId INT NOT NULL,
	SiteId varchar(50) NOT NULL,
	ElementId INT NULL,
	ElementCode NVARCHAR(50) NULL,
	Quantity INT NOT NULL,
	IsPutAside BIT NOT NULL,
	NotifyWhenInStock BIT NOT NULL,
	DetailUrl NVARCHAR(MAX),
	CreatedUtc DATETIME NOT NULL,
	LastUpdatedUtc DATETIME NOT NULL
	)
	
GO
IF OBJECT_ID(N'b_SaleOrderStatus', N'U') IS NULL
CREATE TABLE b_SaleOrderStatus(
	Id INT IDENTITY(1,1) CONSTRAINT PK_b_SaleOrderStatus NOT NULL PRIMARY KEY CLUSTERED,
	IsFinal bit NOT NULL,
	XmlId NVARCHAR(30) NULL,
	Localization XML,	
	Sort INT NOT NULL
)

GO

IF OBJECT_ID(N'SaleCartUpdateQuantity', N'P') IS NOT NULL
DROP PROCEDURE SaleCartUpdateQuantity

GO

CREATE PROCEDURE SaleCartUpdateQuantity
@cartId INT,
@itemData XML,
@userId INT
AS
	SET NOCOUNT ON;
	
	DECLARE @items TABLE ( Id int, Quantity int )
	
	IF (@cartId is NULL)
		RETURN
	
	IF (NOT EXISTS ( SELECT * FROM b_SaleCart WHERE ID = @cartId AND UserId = @userId))
		RETURN
	
	
	INSERT @items ( Id, Quantity )
	SELECT t.c.value('./id[1]','int') as idValue, t.c.value('./q[1]','int') as qValue  from
	@itemData.nodes('items/item') t(c)
	
	update sci set sci.Quantity = i.Quantity
	from 
	@items i 
	join b_SaleCartItem sci on i.Id = sci.ID
	where  
	CartId = @cartId

	DELETE FROM b_SaleCartItem 
	WHERE CartId = @cartId AND Quantity <= 0
GO

IF OBJECT_ID(N'b_SaleOrder', N'U') IS NULL
CREATE TABLE b_SaleOrder(
	[Id] int IDENTITY(1,1) NOT NULL CONSTRAINT PK_b_SaleOrder PRIMARY KEY,
	[SiteId] varchar(50) NOT NULL,
	[UserId] int NULL,
	[CustomerTypeId] int NOT NULL,
	[CurrencyId] int NOT NULL,
	[StatusId] int NOT NULL,
	[Sum] decimal(18, 2) NOT NULL,
	[SumPaid] decimal(18, 2) NOT NULL,
	[StatusSetDateUtc] datetime NOT NULL,
	[StatusSetEmployeeId] int NULL,
	[Cancelled] [bit] NOT NULL,
	[ReasonCancelled] [nvarchar](30) NULL,
	[CancelledDateUtc] [datetime] NULL,
	[CancelledEmployeeId] [int] NULL,
	[Paid] [bit] NOT NULL,
	[PaidDateUtc] [datetime] NULL,
	[PaidEmployeeId] [int] NULL,
	[PaymentDocument] [nvarchar](256) NULL,
	[PaymentDocumentDateUtc] [datetime] NULL,
	[PaymentSystemId] [int] NULL,
	[DeliveryCost] [decimal](18, 2) NULL,
	[DeliveryVariant] [nvarchar](128) NULL,
	[CreatedUtc] [datetime] NOT NULL,
	[LastUpdatedUtc] [datetime] NOT NULL,
	[AllowShipping] [bit] NOT NULL,
	[AllowShippingDateUtc] [datetime] NULL,
	[AllowShippingEmployeeId] [int] NULL,
	[ShippingDocument] [nvarchar](256) NULL,
	[ShippingDocumentDateUtc] [datetime] NULL,
	[PaymentSystemResponseXml] XML NULL,
	[CustomerComments] NVARCHAR(MAX) NULL,
	[EmployeeComments] NVARCHAR(MAX) NULL,
	[AdditionalInfo] NVARCHAR(MAX) NULL,
	[AutoRecalculate] BIT NOT NULL CONSTRAINT DF_b_SaleOrder_AutoRecalculate DEFAULT (1),
	[DiscountName] [nvarchar](50) NULL,
	[DiscountValue] [decimal](18, 4) NULL,
	[CouponId] [int] NULL
)
	
GO

IF OBJECT_ID(N'b_SaleCustomerType', N'U') IS NULL
CREATE TABLE b_SaleCustomerType
(
	Id INT IDENTITY(1,1) CONSTRAINT PK_b_SaleCustomerType PRIMARY KEY CLUSTERED,
	SiteId VARCHAR(50) NOT NULL,
	[Name] NVARCHAR(256) NOT NULL,
	Code VARCHAR(30) NOT NULL,
	Sort INT NOT NULL CONSTRAINT [DF_b_SaleCustomerType_Sort] DEFAULT (100),
	XmlId VARCHAR(256) NULL
)
	
GO

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'b_SaleCustomerType') AND name = N'IX_BX_b_SaleCustomerType_SiteId')
CREATE NONCLUSTERED INDEX IX_BX_b_SaleCustomerType_SiteId ON b_SaleCustomerType
(
	SiteId ASC
)

GO

IF OBJECT_ID(N'b_SaleOrderField', N'U') IS NULL
CREATE TABLE b_SaleOrderField
(
	Id int IDENTITY(1,1) CONSTRAINT PK_b_SaleOrderField PRIMARY KEY CLUSTERED,
	Code varchar(255) NOT NULL,
	[Name] nvarchar(255) NOT NULL,
	CustomerTypeId int NOT NULL,
	CustomTypeId nvarchar(255) NOT NULL,
	Required bit NOT NULL CONSTRAINT DF_b_SaleOrderField_Required DEFAULT (0),
	Multiple bit NOT NULL CONSTRAINT DF_b_SaleOrderField_Multiple DEFAULT (0),
	Settings nvarchar(max) NULL,
	Sort int NOT NULL CONSTRAINT DF_b_SaleOrderField_Sort DEFAULT (100),
	ShowInFilter bit NOT NULL CONSTRAINT DF_b_SaleOrderField_ShowInFilter DEFAULT (0),
	UseAsPostalCode bit NOT NULL CONSTRAINT DF_b_SaleOrderField_UseAsPostalCode DEFAULT (0),
	UseAsProfileName bit NOT NULL CONSTRAINT DF_b_SaleOrderField_UseAsProfileName DEFAULT (0),
	UseAsEmail BIT NOT NULL CONSTRAINT DF_b_SaleOrderField_UseAsEmail DEFAULT (0),
	UseAsLocation BIT NOT NULL CONSTRAINT DF_b_SaleOrderField_UseAsLocation DEFAULT (0)
)

GO

IF OBJECT_ID(N'b_SaleOrderValues', N'U') IS NULL
CREATE TABLE dbo.b_SaleOrderValues
(
	OrderId int NOT NULL,
	OrderFieldId int NOT NULL,
	Code varchar(255) NOT NULL,
	Name nvarchar(255) NOT NULL,
	Value nvarchar(255) NOT NULL
)
GO

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'b_SaleOrderValues') AND name = N'IX_BX_b_SaleOrderValues_OrderId')
CREATE CLUSTERED INDEX IX_BX_b_SaleOrderValues_OrderId ON dbo.b_SaleOrderValues
(
	OrderId
)

GO
IF OBJECT_ID(N'b_SaleProduct', N'U') IS NULL
CREATE TABLE [dbo].[b_SaleProduct](
	[Id] [int] IDENTITY(1,1) NOT NULL PRIMARY KEY,
	[OrderId] [int] NOT NULL,
	[Name] [nvarchar](255) NOT NULL,
	[DetailUrl] [nvarchar](max) NOT NULL,
	[Price] [decimal](18, 4) NOT NULL,
	[Quantity] [int] NOT NULL,
	[VatIncluded] [bit] NOT NULL,
	[VatRate] [decimal](18, 3) NULL,
	[Weight] [decimal](18, 2) NOT NULL,
	[DiscountName] [nvarchar](255) NULL,
	[DiscountValue] [decimal](18, 4) NULL,
	CartResolverId NVARCHAR(50) NOT NULL,
	CartElementId INT NULL,
	CartElementCode NVARCHAR(255) NULL,
	CartCatalogCode NVARCHAR(255) NULL,
	[CouponId] [int] NULL
)
GO


IF OBJECT_ID(N'SaleCart_RemoveItems', N'P') IS NOT NULL
DROP PROCEDURE SaleCart_RemoveItems
GO

CREATE PROCEDURE SaleCart_RemoveItems
@cartId INT,
@ids VARCHAR(MAX)
AS
	SET NOCOUNT ON
	
	DECLARE @Num		int
	DECLARE @Pos		int
	DECLARE @NextPos	int
	DECLARE @strId		varchar(256)
	DECLARE @intId		int
	
	DECLARE @tbIds TABLE ( id INT )

	SET @Num = 0
	SET @Pos = 1
	WHILE(@Pos <= LEN(@ids))
	BEGIN
		SELECT @NextPos = CHARINDEX(',', @ids,  @Pos)
		IF (@NextPos = 0 OR @NextPos IS NULL)
			SELECT @NextPos = LEN(@ids) + 1
		SELECT @strId = RTRIM(LTRIM(SUBSTRING(@ids, @Pos, @NextPos - @Pos)))
		SELECT @Pos = @NextPos+1
		
		BEGIN TRY
			SET @intId = CONVERT(INT,@strId)
			INSERT INTO @tbIds VALUES (@intId)
		END TRY
		BEGIN CATCH
		END CATCH
		
		SET @Num = @Num + 1
	END
	
	DELETE FROM b_SaleCartItem WHERE ID in ( SELECT id FROM @tbIds ) AND CartId = @cartId
GO

IF OBJECT_ID(N'SaleCart_Merge', N'P') IS NOT NULL
DROP PROCEDURE SaleCart_Merge
GO
CREATE PROCEDURE SaleCart_Merge
@sourceCartId INT,
@destCartId INT,
@userId INT
AS
	SET NOCOUNT ON
	
	DECLARE @sExists bit, @destExists bit, @sourceUserId int, @destUserId int
	
	SELECT @sourceUserId = UserId FROM b_SaleCart WHERE ID = @sourceCartId

	SELECT @destUserId = UserId FROM b_SaleCart WHERE ID = @destCartId
	
	IF(@sourceUserId NOT IN(@userId, 0) AND @destUserId NOT IN(@userId, 0)) BEGIN
		SELECT -1
		RETURN
	END
		
	IF(@sourceUserId IS NULL AND @destUserId NOT IN (@userId, 0)) BEGIN
		SELECT -2
		RETURN
	END
	
	IF(@destUserId IS NULL AND @sourceUserId NOT IN (@userId, 0)) BEGIN
		SELECT -3
		RETURN
	END
	
	IF(@sourceUserId IS NULL AND @destUserId = 0) BEGIN
		UPDATE b_SaleCart SET UserId = @userId WHERE ID = @destCartId
		
		SELECT @destCartId
		RETURN
	END
	
	IF(@destUserId IS NULL AND @sourceUserId = 0) BEGIN
		UPDATE b_SaleCart SET UserId = @userId WHERE ID = @sourceCartId
		SELECT @sourceCartId
		RETURN
	END
	
	--merge
	
	DECLARE @items TABLE(ElementId INT, ElementCode NVARCHAR(50), Quantity INT)
	
	DECLARE @ElementId INT, @ElementCode NVARCHAR(50), @Quantity INT
	--ElementId
	UPDATE s SET Quantity = items.Quantity
	
	FROM b_SaleCartItem s JOIN 
	(
		SELECT ElementId, SUM(Quantity) as Quantity, SiteId
		FROM b_SaleCartItem 
		WHERE CartId IN (@sourceCartId, @destCartId) AND ElementId IS NOT NULL
		GROUP BY ElementId, SiteId 	) items on items.ElementId = s.ElementId AND items.SiteId = s.SiteId
	WHERE 
		s.CartId = @destCartId

	UPDATE src
		 SET CartId = @destCartId
	FROM
		b_SaleCartItem src 
	WHERE 
		CartId = @sourceCartId AND ElementId IS NOT NULL
		AND ElementId NOT IN ( SELECT ElementId FROM b_SaleCartItem WHERE CartId = @destCartId AND SiteId = src.SiteId )
		
	--ElementCode
	UPDATE s SET Quantity = items.Quantity
	
	FROM b_SaleCartItem s JOIN 
	(
		SELECT ElementCode, SUM(Quantity) as Quantity, SiteId
		FROM b_SaleCartItem 
		WHERE CartId IN (@sourceCartId, @destCartId) AND ElementCode IS NOT NULL AND ElementId IS NULL
		GROUP BY ElementCode, SiteId ) items on items.ElementCode = s.ElementCode AND items.SiteId = s.SiteId
	WHERE 
		s.CartId = @sourceCartId

	UPDATE src
		 SET CartId = @destCartId
	FROM
		b_SaleCartItem src 
	WHERE 
		CartId = @sourceCartId 
		AND ElementCode NOT IN ( SELECT ElementCode FROM b_SaleCartItem WHERE CartId = @destCartId AND SiteId = src.SiteId)
		
	DELETE FROM b_SaleCartItem WHERE CartId = @sourceCartId
	
	DELETE FROM b_SaleCart WHERE ID = @sourceCartId
	
	SELECT @destCartId
GO

IF OBJECT_ID(N'SaleOrder_SetValues', N'P') IS NOT NULL
DROP PROCEDURE SaleOrder_SetValues

GO
CREATE PROCEDURE SaleOrder_SetValues
	@orderId int, 
	@values xml
AS
BEGIN
	SET NOCOUNT ON;
	
	DELETE FROM b_SaleOrderValues WHERE OrderId = @orderId;
	
	INSERT INTO b_SaleOrderValues (OrderId, OrderFieldId,  Code, Name, Value)
	SELECT
		@orderId, 
		Properties.p.value('@id', 'INT'),
		Properties.p.value('@code', 'VARCHAR(255)'),
		Properties.p.value('@name', 'NVARCHAR(255)'),
		Properties.p.value('.', 'NVARCHAR(255)')
	FROM 
		@values.nodes('/properties/p') as Properties(p);
END
GO

IF OBJECT_ID(N'SaleOrder_GetValues', N'P') IS NOT NULL
DROP PROCEDURE SaleOrder_GetValues

GO
CREATE PROCEDURE SaleOrder_GetValues
	@orderId int
AS
BEGIN
	SET NOCOUNT ON;
	
	SELECT
		OrderFieldId,
		Code,
		Name,
		Value
	FROM 
		b_SaleOrderValues 
	WHERE 
		OrderId = @orderId;
END
GO

IF OBJECT_ID(N'SaleOrder_DeleteValues', N'P') IS NOT NULL
DROP PROCEDURE SaleOrder_DeleteValues

GO
CREATE PROCEDURE SaleOrder_DeleteValues
	@orderId int
AS
BEGIN
	SET NOCOUNT ON;
	
	DELETE FROM
		b_SaleOrderValues 
	WHERE 
		OrderId = @orderId;
END
GO

IF OBJECT_ID(N'b_SaleDeliverySrvCfg', N'U') IS NULL
CREATE TABLE b_SaleDeliverySrvCfg
(
	ID INT IDENTITY(1,1) NOT NULL,
	ServiceName NVARCHAR(1024) NOT NULL,
	ServiceNameHash INT NOT NULL,
	SiteId VARCHAR(50) NOT NULL,
	Active BIT NOT NULL,
	Name NVARCHAR(1024) NULL,
	[Description] NVARCHAR(MAX) NULL,
	Extra DECIMAL(18,2) NOT NULL,
	Sort INT NOT NULL,
	SettingsXml XML NULL,
	CreatedUtc DATETIME NOT NULL,
	LastUpdatedUtc DATETIME NOT NULL,
	CONSTRAINT [PK_b_SaleDeliverySrvCfg_Id] PRIMARY KEY CLUSTERED (ID ASC)
);
GO

IF OBJECT_ID(N'b_SaleCustomDeliverySrv', N'U') IS NULL
CREATE TABLE b_SaleCustomDeliverySrv
(
	ID INT IDENTITY(1,1) NOT NULL,
	Code NVARCHAR(32) NOT NULL,
	Active BIT NOT NULL,
	SiteId VARCHAR(50) NOT NULL,
	DisplayName NVARCHAR(1024) NULL,
	[Description] NVARCHAR(MAX) NULL,
	DeliveryTimeFrom INT NOT NULL,
	DeliveryTimeTo INT NOT NULL,
	WeightFrom INT NOT NULL,
	WeightTo INT NOT NULL,
	CurrencyId INT NOT NULL,
	CostFrom DECIMAL(18, 2) NOT NULL,
	CostTo DECIMAL(18, 2) NOT NULL,
	DeliveryCost DECIMAL(18, 2) NOT NULL,
	Sort INT NOT NULL,
	CreatedUtc DATETIME NOT NULL,
	LastUpdatedUtc DATETIME NOT NULL,
	XmlId VARCHAR(256) NULL,
	CONSTRAINT [PK_b_SaleCustomDeliverySrv_Id] PRIMARY KEY CLUSTERED (ID ASC)
);
GO

IF OBJECT_ID(N'b_SaleUserValues', N'U') IS NULL
CREATE TABLE b_SaleUserValues(
	[Id] [int] IDENTITY(1,1) NOT NULL CONSTRAINT [PK_b_SaleUserValues] PRIMARY KEY NONCLUSTERED,
	[UserId] [int] NOT NULL,
	[CustomerTypeId] [int] NOT NULL,
	[Name] [nvarchar](255) NOT NULL,
	[Values] [xml] NOT NULL,
	[LastUpdatedUtc] [datetime] NOT NULL
);
GO

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'b_SaleUserValues') AND name = N'IX_BX_b_SaleUserValues_UserId_CustomerTypeId')
CREATE CLUSTERED INDEX IX_BX_b_SaleUserValues_UserId_CustomerTypeId ON b_SaleUserValues
(
	[UserId] ASC,
	[CustomerTypeId] ASC
);
GO

IF OBJECT_ID(N'b_SalePaymentSystem', N'U') IS NULL
CREATE TABLE b_SalePaymentSystem
(
	ID INT IDENTITY(1,1) NOT NULL,
	SiteId VARCHAR(50) NOT NULL,
	Active BIT NOT NULL,
	Name NVARCHAR(1024) NOT NULL,
	[Description] NVARCHAR(MAX) NULL,
	Sort INT NOT NULL,
	CreatedUtc DATETIME NOT NULL,
	LastUpdatedUtc DATETIME NOT NULL,
	XmlId VARCHAR(256) NULL,
	CONSTRAINT [PK_b_SalePaymentSystem_Id] PRIMARY KEY CLUSTERED (ID ASC)
);
GO


IF OBJECT_ID(N'b_SalePaymentSystemConfig', N'U') IS NULL
CREATE TABLE b_SalePaymentSystemConfig
(
	ID INT IDENTITY(1,1) NOT NULL,
	Name NVARCHAR(1024) NOT NULL,
	PaymentSystemId  INT NOT NULL,
	CustomerTypeId INT NOT NULL,
	ControllerTypeId NVARCHAR(256) NOT NULL,
	ControllerSettingsXml XML NULL,
	CreatedUtc DATETIME NOT NULL,
	LastUpdatedUtc DATETIME NOT NULL,
	CONSTRAINT [PK_b_SalePaymentSystemConfig_Id] PRIMARY KEY CLUSTERED (ID ASC)
);
GO

IF OBJECT_ID(N'SalePaymentSystemConfig_SetValues', N'P') IS NOT NULL
	DROP PROCEDURE SalePaymentSystemConfig_SetValues;
GO

CREATE PROCEDURE SalePaymentSystemConfig_SetValues
	@PaymentSysmemId INT
AS
BEGIN
	SET NOCOUNT ON;
	DELETE FROM b_SalePaymentSystemConfig WHERE  PaymentSystemId = @PaymentSysmemId;
END
GO

IF OBJECT_ID(N'b_SaleExportMapping', N'U') IS NULL
CREATE TABLE b_SaleExportMapping
(
	CustomerTypeId INT NOT NULL CONSTRAINT PK_b_SaleExportMapping_CustomerTypeId PRIMARY KEY NONCLUSTERED,
	ExportType INT NOT NULL CONSTRAINT DF_b_SaleExportMapping_ExportType DEFAULT(0),
	Mapping NVARCHAR(MAX) NOT NULL
);
GO


IF OBJECT_ID(N'b_SaleDiscount', N'U') IS NULL
CREATE TABLE b_SaleDiscount(
	[Id] [int] IDENTITY(1,1) NOT NULL,
	[CurrencyId] [int] NOT NULL,
	[SiteId] [varchar](50) NOT NULL,
	[Value] [decimal](18, 2) NOT NULL,
	[Type] [tinyint] NOT NULL,
	[Active] [bit] NOT NULL,
	[MaxElementsCount] [int] NOT NULL CONSTRAINT [DF_MaxElementsCount] DEFAULT(0),
	[UseCoupons] [bit] NOT NULL  CONSTRAINT [DF_UserCoupons] DEFAULT(0),
	[Name] [nvarchar](50) NULL,
	[MaxSum] [decimal](18, 2) NULL,
	[ActiveFromUtc] [datetime] NULL,
	[ActiveToUtc] [datetime] NULL,
	[UserRoleIds] [nvarchar](max) NULL,
	[Filter] [xml] NOT NULL,
	[FilterApply] [xml] NOT NULL,
	[Sort] [int] NOT NULL,
	[Description] [nvarchar](max) NULL,

 CONSTRAINT [PK_b_SaleDiscount] PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO
IF OBJECT_ID(N'b_SaleDiscountCoupon', N'U') IS NULL
CREATE TABLE b_SaleDiscountCoupon(
	[Id] [int] IDENTITY(1,1) NOT NULL,
	[DiscountId] [int] NOT NULL,
	[Active] [bit] NOT NULL,
	[IsOneTime] [bit] NOT NULL,
	[AppliedDateUtc] [datetime] NULL,
	[CouponString] [nvarchar](50) NOT NULL CONSTRAINT [C_CouponString_Unique] UNIQUE,
PRIMARY KEY CLUSTERED 
(
	[Id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_b_SaleDiscountCoupon_CouponString')
	CREATE INDEX IX_b_SaleDiscountCoupon_CouponString ON b_SaleDiscountCoupon(CouponString ASC);
GO


IF OBJECT_ID(N'b_SaleDiscountCouponInCart', N'U') IS NULL
CREATE TABLE b_SaleDiscountCouponInCart(
	[CartId] [int] NOT NULL,
	[CouponString] [nvarchar](50) NOT NULL,
	[Active] [bit] NOT NULL,
	[CouponId] [int] NOT NULL,
	[DiscountId] [int] NULL,
	PRIMARY KEY CLUSTERED 
(
	[CartId] ASC, [CouponId] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

IF OBJECT_ID(N'Sale_DeactivateCoupons', N'P') IS NOT NULL
DROP PROCEDURE Sale_DeactivateCoupons
GO

CREATE PROCEDURE Sale_DeactivateCoupons
@userId INT,
@coupons XML
AS
	BEGIN
		DECLARE @items TABLE ( Id int )
		
		INSERT @items ( Id)
		SELECT t.c.value('.','int') as idValue  from
		@coupons.nodes('root/d') t(c)
		
		UPDATE sdi
		SET Active = 0
		FROM
			b_SaleDiscountCouponInCart sdi join b_SaleCart sc on sdi.CartId = sc.ID
			join b_SaleDiscountCoupon sdc on sdc.Id = sdi.CouponId
			join @items i on i.Id = sdc.Id
		WHERE
			sc.UserId = @userId
			AND sdc.IsOneTime = 1
			AND sdc.Active = 1
			
		UPDATE sdc
		SET Active = 0, AppliedDateUtc = GETDATE()
		FROM
			b_SaleDiscountCouponInCart sdi join b_SaleCart sc on sdi.CartId = sc.ID
			join b_SaleDiscountCoupon sdc on sdc.Id = sdi.CouponId
			join @items i on i.Id = sdc.Id
		WHERE
			sc.UserId = @userId
			AND sdc.IsOneTime = 1
			AND sdc.Active = 1
	END
GO
IF OBJECT_ID(N'Sale_FindCoupon', N'P') IS NOT NULL
DROP PROCEDURE Sale_FindCoupon
GO
CREATE PROCEDURE Sale_FindCoupon
@coupon NVARCHAR(50)
AS
BEGIN
	SET NOCOUNT ON
	DECLARE @foundStr NVARCHAR(50),
	@oneTime BIT,
	@count INT,
	@couponId INT
	
	select @foundStr = CouponString, @oneTime = IsOneTime, @couponId = Id
	FROM
		b_SaleDiscountCoupon sdc 
	WHERE
		CouponString = @coupon	
	
	IF(@foundStr IS NULL)
		RETURN;

	
	IF(@oneTime = 1 AND EXISTS(SELECT * FROM b_SaleDiscountCouponInCart WHERE CouponId = @couponId)) 
		RETURN;
	
	SELECT * from b_SaleDiscountCoupon WHERE Id = @couponId	
END

GO

IF OBJECT_ID(N'Sale_GetDiscountsForCoupons', N'P') IS NOT NULL
DROP PROCEDURE Sale_GetDiscountsForCoupons
GO
CREATE PROCEDURE Sale_GetDiscountsForCoupons 
@coupons XML
AS 
	BEGIN
	SET NOCOUNT ON
	CREATE TABLE #coupons ( CouponString NVARCHAR(50) COLLATE DATABASE_DEFAULT)
	
	insert #coupons
	select t.c.value('.','NVARCHAR(50)') 
	from @coupons.nodes('/root/coupon') as t(c) 
	
	SELECT d.*
	FROM 
		b_SaleDiscount d 
		JOIN b_SaleDiscountCoupon cdc ON cdc.DiscountId = d.Id
		JOIN #coupons c on c.CouponString = cdc.CouponString
	WHERE
		cdc.Active = 1 AND ( cdc.IsOneTime = 0 OR cdc.AppliedDateUtc IS NULL )
		AND d.Active = 1 AND GETDATE() > ISNULL(d.ActiveFromUtc, '19000101')
		AND GETDATE() < ISNULL (d.ActiveToUtc, '99990101')
		AND d.UseCoupons = 1
	
	DROP TABLE #coupons
	END
GO
IF OBJECT_ID(N'Sale_ActivateCoupons', N'P') IS NOT NULL
DROP PROCEDURE Sale_ActivateCoupons
GO
CREATE PROCEDURE Sale_ActivateCoupons
@orderId INT
AS 
	BEGIN
	SET NOCOUNT ON
	
	DECLARE @userId INT
	
	SELECT @userId = UserId FROM b_SaleOrder WHERE Id = @orderId
	
	IF ( @userId IS NULL )
		RETURN
	
	UPDATE sdc SET Active = 1, AppliedDateUtc = NULL
	FROM 
		b_SaleProduct sp 
		JOIN b_SaleOrder so on so.Id = sp.OrderId
		JOIN b_SaleDiscountCoupon sdc on (sdc.Id = sp.CouponId OR sdc.Id = so.CouponId)
	WHERE
		so.UserId = @userId
		AND so.Id = @orderId

	--order coupons
	UPDATE sdci SET Active = 1
	FROM 

		b_SaleProduct sp 
		JOIN b_SaleOrder so on so.Id = sp.OrderId
		JOIN b_SaleDiscountCoupon sdc on (sdc.Id = sp.CouponId OR sdc.Id = so.CouponId)
		JOIN b_SaleDiscountCouponInCart sdci on sdci.CouponId = sdc.Id
		JOIN b_SaleCart sc on sc.ID = sdci.CartId
	WHERE
		so.UserId = @userId
		AND so.Id = @orderId
		AND sc.UserId = @userId
	END