﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.UI;
using Bitrix.Services.Text;
using Bitrix;
using Bitrix.DataLayer;
using Bitrix.Modules;
using System.Collections.Generic;
using Bitrix.Security;
using Bitrix.Configuration;
using Bitrix.Sale;
using Bitrix.Currency;
using System.Globalization;

public partial class bitrix_admin_saleCustomDeliveryEdit : BXAdminPage
{
    private int? serviceId = null;
    protected int ServiceId
    {
        get
        {
            if (this.serviceId.HasValue)
                return this.serviceId.Value;

            string s = Request.QueryString["id"];
            if (string.IsNullOrEmpty(s))
                return (this.serviceId = 0).Value;

            int i;
            return (this.serviceId = int.TryParse(s, out i) ? i : 0).Value;
        }
    }

    private CustomDeliveryEditorMode editorMode = CustomDeliveryEditorMode.Creation;
    protected CustomDeliveryEditorMode EditorMode
    {
        get { return this.editorMode; }
    }

    private BXCustomizableDeliveryService service = null;
    protected BXCustomizableDeliveryService Service
    {
        get { return this.service; }
    }

    private string errorMessageStr = string.Empty;
    protected string ErrorMessage
    {
        get { return this.errorMessageStr; }
    }

    private CustomDeliveryEditorError editorError = CustomDeliveryEditorError.None;
    protected CustomDeliveryEditorError EditorError
    {
        get { return this.editorError; }
    }

    protected override string BackUrl
    {
        get { return !string.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "SaleCustomDeliveryList.aspx"; }
    }

    protected override void OnInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        BXCurrencyCollection currencies = BXCurrency.GetList(
            null,
            new BXOrderBy(
				new BXOrderByPair(BXCurrency.Fields.Sort, BXOrderByDirection.Asc),
				new BXOrderByPair(BXCurrency.Fields.Id, BXOrderByDirection.Asc)
				),
            new BXSelect(BXSelectFieldPreparationMode.Normal, BXCurrency.Fields.Id, BXCurrency.Fields.LocalizationInfos, BXCurrency.Fields.IsNational),
            null,
            BXTextEncoder.EmptyTextEncoder
            );

        CurrencyList.Items.Clear();
        foreach (BXCurrency currency in currencies)
        {
            ListItem item = new ListItem(currency.GetName(), currency.Id.ToString());
            CurrencyList.Items.Add(item);
        }


        BXSiteCollection sites = BXSite.GetList(null,
            new BXOrderBy(new BXOrderByPair(BXSite.Fields.Sort, BXOrderByDirection.Asc)),
            new BXSelect(BXSite.Fields.ID, BXSite.Fields.Name),
            null,
            BXTextEncoder.EmptyTextEncoder
            );

        SiteList.Items.Clear();
        SiteList.Items.Add(new ListItem(GetMessageRaw("SelectItem.AllSites"), string.Empty));
        foreach (BXSite site in sites)
        {
            ListItem item = new ListItem(string.Format("[{0}] {1}", site.Id, site.Name), site.Id);
            SiteList.Items.Add(item);
        }

        TryLoadService();
        base.OnInit(e);
    }

    protected override void OnLoad(EventArgs e)
    {
        MasterTitle = Page.Title = EditorMode == CustomDeliveryEditorMode.Modification ? string.Format(GetMessage("PageTitle.Modify"), Service != null ? HttpUtility.HtmlEncode(Service.DisplayName) : ServiceId.ToString()) : GetMessage("PageTitle.Create");
        base.OnLoad(e);
    }

    private void TryLoadService()
    {
        if (ServiceId <= 0)
        {
            this.editorMode = CustomDeliveryEditorMode.Creation;
            this.service = new BXCustomizableDeliveryService();
            this.service.CurrencyId = BXCurrency.NationalCurrencyId;
        }
        else
        {
            this.editorMode = CustomDeliveryEditorMode.Modification;
            BXCustomizableDeliveryServiceCollection col = BXCustomizableDeliveryService.GetList(
                new BXFilter(new BXFilterItem(BXCustomizableDeliveryService.Fields.Id, BXSqlFilterOperators.Equal, ServiceId)),
                null,
                null,
                new BXQueryParams(new BXPagingOptions(0, 1)),
				BXTextEncoder.EmptyTextEncoder);

            if ((this.service = col.Count > 0 ? col[0] : null) == null)
            {
                this.errorMessageStr = string.Format(GetMessageRaw("Error.UnableToFindService"), ServiceId);
                this.editorError = CustomDeliveryEditorError.IsNotFound;
                return;
            }
        }

        ActiveChkBx.Checked = this.service.Active;
        CodeTbx.Text = this.service.Code;
        ListItem siteItem = SiteList.Items.FindByValue(this.service.SiteId);
        if (siteItem != null)
            siteItem.Selected = true;

        DisplayNameTbx.Text = this.service.DisplayName;
        if (this.service.DeliveryTimeFrom != 0)
            DeliveryTimeFromTbx.Text = this.service.DeliveryTimeFrom.ToString();

        if (this.service.DeliveryTimeTo != 0)
            DeliveryTimeToTbx.Text = this.service.DeliveryTimeTo.ToString();

        if (this.service.WeightFrom != 0)
            WeightFromTbx.Text = this.service.WeightFrom.ToString();

        if (this.service.WeightTo != 0)
            WeightToTbx.Text = this.service.WeightTo.ToString();

        ListItem currencyItem = CurrencyList.Items.FindByValue(this.service.CurrencyId.ToString());
        if (currencyItem != null)
            currencyItem.Selected = true;

        BXCurrency currency = this.service.Currency;
        if (this.service.CostFrom != 0m)
            CostFromTbx.Text = currency != null ? currency.MoneyToString(this.service.CostFrom, CultureInfo.InvariantCulture, false) : this.service.CostFrom.ToString("F2");

        if (this.service.CostTo != 0m)
            CostToTbx.Text = currency != null ? currency.MoneyToString(this.service.CostTo, CultureInfo.InvariantCulture, false) : this.service.CostTo.ToString("F2");

        if(this.service.DeliveryCost != 0)
            DeliveryCostTbx.Text = currency != null ? currency.MoneyToString(this.service.DeliveryCost, CultureInfo.InvariantCulture, false) : this.service.DeliveryCost.ToString("F2");

        DescriptionTbx.Text = this.service.Description;

        if(this.service.Sort != 0)
            SortTbx.Text = this.service.Sort.ToString();

        XmlIdTbx.Text = this.service.XmlId;
    }

    private void TrySaveService()
    {
        if (this.service == null)
            return;

        try
        {
            this.service.Active = ActiveChkBx.Checked;
            this.service.Code = CodeTbx.Text;
            this.service.SiteId = SiteList.SelectedValue;
            this.service.DisplayName = DisplayNameTbx.Text;

            int i;
            this.service.DeliveryTimeFrom = int.TryParse(DeliveryTimeFromTbx.Text, out i) ? i : 0;
            this.service.DeliveryTimeTo = int.TryParse(DeliveryTimeToTbx.Text, out i) ? i : 0;
            this.service.WeightFrom = int.TryParse(WeightFromTbx.Text, out i) ? i : 0;
            this.service.WeightTo = int.TryParse(WeightToTbx.Text, out i) ? i : 0;
            this.service.CurrencyId = int.TryParse(CurrencyList.SelectedValue, out i) ? i : BXCurrency.NationalCurrencyId;

            decimal m;
            this.service.CostFrom = decimal.TryParse(CostFromTbx.Text, NumberStyles.Number, CultureInfo.InvariantCulture.NumberFormat, out m) ? m : 0m;
            this.service.CostTo = decimal.TryParse(CostToTbx.Text, NumberStyles.Number, CultureInfo.InvariantCulture.NumberFormat, out m) ? m : 0m;
            this.service.DeliveryCost = decimal.TryParse(DeliveryCostTbx.Text, NumberStyles.Number, CultureInfo.InvariantCulture.NumberFormat, out m) ? m : 0m;
            this.service.Description = DescriptionTbx.Text;
            this.service.Sort = int.TryParse(SortTbx.Text, out i) ? i : 100;
            this.service.XmlId = XmlIdTbx.Text;

            this.service.Save();
            this.serviceId = this.service.Id;
        }
        catch (Exception exc)
        {
            this.errorMessageStr = exc.Message;
            this.editorError = EditorMode == CustomDeliveryEditorMode.Creation ? CustomDeliveryEditorError.Creation : CustomDeliveryEditorError.Modification;
        }
    }

    protected override void OnPreRender(EventArgs e)
    {
        if (EditorMode != CustomDeliveryEditorMode.Modification)
            AddButton.Visible = DeleteButton.Visible = false;

        if (EditorError != CustomDeliveryEditorError.None)
        {
            errorMessage.AddErrorMessage(ErrorMessage);
            if (EditorMode == CustomDeliveryEditorMode.Modification && EditorError == CustomDeliveryEditorError.IsNotFound)
                TabControl.Visible = false;

            AddButton.Visible = DeleteButton.Visible = false;
        }
        base.OnPreRender(e);
    }


    protected void OnCommand(object sender, BXTabControlCommandEventArgs e)
    {
        switch (e.CommandName)
        {
            case "save":
                {
                    if (IsValid && EditorError == CustomDeliveryEditorError.None)
                    {
                        TrySaveService();
                        if (EditorError == CustomDeliveryEditorError.None)
                            GoBack();
                    }
                }
                break;
            case "apply":
                {
                    if (IsValid && EditorError == CustomDeliveryEditorError.None)
                    {
                        TrySaveService();
                        if (EditorError == CustomDeliveryEditorError.None)
                            Response.Redirect(string.Format("SaleCustomDeliveryEdit.aspx?id={0}&tabindex={1}", ServiceId.ToString(), TabControl.SelectedIndex));
                    }
                }
                break;
            case "cancel":
                GoBack();
                break;
        }
    }

    protected void OnToolBarButtonClick(object sender, CommandEventArgs e)
    {
        if (e.CommandName == "delete")
        {
            try
            {
                if (Service != null)
                    Service.Delete();

                GoBack();
            }
            catch (Exception ex)
            {
                this.errorMessageStr = ex.Message;
                this.editorError = CustomDeliveryEditorError.Deletion;
            }
        }
    }
}

public enum CustomDeliveryEditorMode
{
    Creation = 1,
    Modification = 2
}

public enum CustomDeliveryEditorError
{
    None = 0,
    IsNotFound = 1,
    Creation = 3,
    Modification = 4,
    Deletion = 5
}