﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using Bitrix.UI;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix;
using System.Text;
using Bitrix.Security;
using Bitrix.Sale;
using Bitrix.Currency;
using System.Globalization;

public partial class bitrix_admin_SaleCustomDeliveryList : BXAdminPage
{
    protected override void OnPreInit(EventArgs e)
    {
        if (!BXUser.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnPreInit(e);
    }

    protected override void OnInit(EventArgs e)
    {
        MasterTitle = Page.Title = GetMessage("PageTitle");
        base.OnInit(e);
    }

    protected void ItemGrid_Select(object sender, BXSelectEventArgs e)
    {
        if (!BXUser.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

		BXCustomizableDeliveryServiceCollection col  = BXCustomizableDeliveryService.GetList(
			null,
			new BXOrderBy(BXCustomizableDeliveryService.Fields, string.IsNullOrEmpty(e.SortExpression) ? "Id" : e.SortExpression)
			);

        List<ServiceWrapper> list = new List<ServiceWrapper>();
        foreach (BXCustomizableDeliveryService item in col)
            list.Add(new ServiceWrapper(item, this));

		e.Data = list;
    }

    protected void ItemGrid_SelectCount(object sender, BXSelectCountEventArgs e)
    {
        e.Count = BXCustomizableDeliveryService.Count(null);
    }

    protected void ItemGrid_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType != DataControlRowType.DataRow)
            return;

        BXGridViewRow row = (BXGridViewRow)e.Row;
        ServiceWrapper wrapper = (ServiceWrapper)row.DataItem;

        row.UserData.Add("ID", wrapper.Id);
    }

    protected void ItemGrid_Delete(object sender, BXDeleteEventArgs e)
	{
        if (!BXUser.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            return;

        BXCustomizableDeliveryServiceCollection col;

        try
        {
            BXFilter f = null;
            if(e.Keys != null)
                f = new BXFilter(new BXFilterItem(BXCustomizableDeliveryService.Fields.Id, BXSqlFilterOperators.Equal, e.Keys["ID"]));
            col = BXCustomizableDeliveryService.GetList(f, null);

        }
        catch (Exception ex)
        {
            ErrorMessage.AddErrorMessage(ex.Message);
            return;
        }
        bool aboutErrorAdd = false;
        foreach (BXCustomizableDeliveryService srv in col)
        {
            try
            {
                srv.Delete();
                e.DeletedCount++;
            }
            catch(Exception exc)
            {
                if (!aboutErrorAdd)
                {
                    ErrorMessage.AddErrorMessage(exc.Message);
                    aboutErrorAdd = true;
                }
            }
        }

        if(aboutErrorAdd)
            ErrorMessage.AddErrorMessage(GetMessage("CustomizableDeliveryService.DeleteFailure"));
	}
}
/// <summary>
/// Обёртка для Настраиваемой Службы Доставки
/// </summary>
public class ServiceWrapper
{
    BXCustomizableDeliveryService charge;
    BXAdminPage page;
    public ServiceWrapper(BXCustomizableDeliveryService charge, BXAdminPage page)
    {
        if (charge == null)
            throw new ArgumentNullException("charge");

        if (page == null)
            throw new ArgumentNullException("page");

        this.charge = charge;
        this.page = page;
    }

    public string Id
    {
        get { return this.charge.Id.ToString(); }
    }

    public string Code
    {
        get { return this.charge.Code; }
    }

    public string DisplayName
    {
        get { return this.charge.DisplayName; }
    }

    public string Sort
    {
        get { return this.charge.Sort.ToString(); }
    }

    public string SiteId
    {
        get { return this.charge.SiteId; }
    }

    public bool IsActive
    {
        get { return this.charge.Active; }
    }

    public string Active
    {
        get { return this.page.GetMessageRaw(this.charge.Active ? "Kernel.Yes" : "Kernel.No"); }
    }

    public string Weight
    {
        get 
        { 
            if(this.charge.WeightFrom == 0 && this.charge.WeightTo == 0)
                return this.page.GetMessageRaw("WeightNotDefined");

            if(this.charge.WeightFrom != 0 && this.charge.WeightTo == 0)
                return string.Format(this.page.GetMessageRaw("WeightRangeLeftBoundTemplate"), this.charge.WeightFrom.ToString());

            if(this.charge.WeightFrom == 0 && this.charge.WeightTo != 0)
                return string.Format(this.page.GetMessageRaw("WeightRangeRightBoundTemplate"), this.charge.WeightTo.ToString());

            return string.Format(this.page.GetMessageRaw("WeightRangeTemplate"), this.charge.WeightFrom.ToString(), this.charge.WeightTo.ToString()); 
        }
    }

    public string Cost
    {
        get 
        {
            if(this.charge.CostFrom == 0 && this.charge.CostTo == 0)
                return this.page.GetMessageRaw("CostNotDefined");

            BXCurrency currency = this.charge.Currency;

            if(this.charge.CostFrom != 0 && this.charge.CostTo == 0)
            {
                if(currency != null)
                    return string.Format(this.page.GetMessageRaw("CostRangeLeftBoundTemplate"), currency.MoneyToString(this.charge.CostFrom, CultureInfo.InvariantCulture, false), currency.GetSymbol());
                return string.Format(this.page.GetMessageRaw("CostRangeLeftBoundTemplate"), this.charge.CostFrom.ToString("F2"), string.Empty);
            }
            if(this.charge.CostFrom == 0 && this.charge.CostTo != 0)
            {
                if(currency != null)
                    return string.Format(this.page.GetMessageRaw("CostRangeRightBoundTemplate"), currency.MoneyToString(this.charge.CostTo, CultureInfo.InvariantCulture, false), currency.GetSymbol());
                return string.Format(this.page.GetMessageRaw("CostRangeRightBoundTemplate"), this.charge.CostTo.ToString("F2"), string.Empty);
            }

            if(currency != null)
                return string.Format(this.page.GetMessageRaw("CostRangeTemplate"), currency.MoneyToString(this.charge.CostFrom, CultureInfo.InvariantCulture, false), currency.MoneyToString(this.charge.CostTo, CultureInfo.InvariantCulture, false), currency.GetSymbol()); 

            return string.Format(this.page.GetMessageRaw("CostRangeTemplate"), this.charge.CostFrom.ToString("F2"), this.charge.CostTo.ToString("F2"), string.Empty); 
        }
    }

    public string DeliveryCost
    {
        get 
        {
            BXCurrency currency = this.charge.Currency;
            if(currency != null)
                return string.Format(this.page.GetMessageRaw("DeliveryCostTemplate"), currency.MoneyToString(this.charge.DeliveryCost, CultureInfo.InvariantCulture, false), currency.GetSymbol()); 

            return this.charge.DeliveryCost.ToString("F2"); 
        }
    }

	public string SiteInfo
	{
		get { return !string.IsNullOrEmpty(this.charge.SiteId) ? this.charge.SiteId : this.page.GetMessageRaw("AllSites");  }
	}
}
