﻿using Bitrix.UI;
using Bitrix.Security;
using System.Security;
using Bitrix.Services.Text;
using System;
using System.Linq;
using Bitrix.Modules;
using System.Web.UI.WebControls;
using Bitrix.DataLayer;

namespace Bitrix.Sale.AdminPages
{
	public partial class CustomerTypeEdit : BXAdminPage
	{
		#region Messages
		int showMessage;

		bool HasErrors { get { return showMessage == -1; } }

		void PrepareResultMessage()
		{
			SuccessMessage.Visible = showMessage > 0;
		}
		void ShowErrorHtml(string html)
		{
			showMessage = -1;
			ErrorMessage.AddErrorMessage(html);
		}
		void ShowErrorText(string text)
		{
			ShowErrorHtml(Encode(text));
		}
		void ShowError(Exception ex)
		{
			showMessage = -1;
			ErrorMessage.AddErrorMessage(Encode(ex.Message));
		}
		void ShowOk()
		{
			if (showMessage == 0)
				showMessage = 1;
		}
		#endregion
		public bool CanManage { get; set; }
		public BXSaleCustomerType Item { get; set; }
		protected override string BackUrl
		{
			get
			{
				return base.BackUrl ?? "SaleCustomerTypeList.aspx";
			}
		}
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			if (!BXUser.IsCanOperate(BXSaleModule.Operations.AdminManagement))
				BXAuthentication.AuthenticationRequired();

			Site.Items.Clear();
			Site.Items.AddRange(
				(
					from s in BXSite.GetAllSitesReadOnly()
					where (s.Active)
					select new ListItem(string.Format("[{0}] {1}", s.TextEncoder.Decode(s.Id), s.TextEncoder.Decode(s.Name)), s.TextEncoder.Decode(s.Id))
				)
				.ToArray()
			);

			var idString = Request.QueryString["id"];
			if (idString != null)
			{
				int id;
				if (!int.TryParse(idString, out id) || id <= 0 || (Item = BXSaleCustomerType.GetById(id, BXTextEncoder.EmptyTextEncoder)) == null)
					Response.Redirect("SaleCustomerTypeList.aspx"); // не удалось найти такой объект

				DoLoad();
				//Code.Enabled = false;
			}
			else
			{
				var siteId = Request["siteId"];
				if(!string.IsNullOrEmpty(siteId))
				{
					ListItem siteItem = Site.Items.FindByValue(siteId);
					if(siteItem != null)
						siteItem.Selected = true;
				}
				Sort.Text = "100";
			}

			CanManage = true;
		}
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);

			Title = GetMessageRaw("PageTitle");
			MasterTitle = Item != null ? GetMessageRaw("MasterTitle.Edit") : GetMessageRaw("MasterTitle.New");

			PrepareResultMessage();

			TabControl.ShowSaveButton = CanManage;
			TabControl.ShowApplyButton = CanManage;

			AddButton.Visible = CanManage && Item != null;
			DeleteButton.Visible = CanManage && Item != null;
		}
		void DoLoad()
		{
			Name.Text = Item.Name;
			Sort.Text = Item.Sort.ToString();
			Site.SelectedValue = Item.SiteId;
			Code.Text = Item.Code;
			XmlId.Text = Item.XmlId ?? "";
		}
		void DoSave()
		{
			if (!Page.IsValid)
			{
				showMessage = -1;
				return;
			}
			try
			{
				if (!CanManage)
					throw new SecurityException(GetMessageRaw("Error.InsufficientRightsToEdit"));

				var item = Item ?? new BXSaleCustomerType(BXTextEncoder.EmptyTextEncoder);

				var sort = Sort.Text.Trim();
				var site =
				(
					from s in BXSite.GetAllSitesReadOnly()
					where (s.Active && s.TextEncoder.Decode(s.Id) == Site.SelectedValue)
					select s
				)
				.FirstOrDefault();

				if (site == null)
					throw new Exception(GetMessageRaw("Error.SiteNotFound"));

				var newCode = Code.Text.Trim().ToUpperInvariant();
				var items = BXSaleCustomerType.GetList(new BXFilter(
					new BXFilterItem(BXSaleCustomerType.Fields.Code, BXSqlFilterOperators.Equal, newCode),
					new BXFilterItem(BXSaleCustomerType.Fields.Site.ID, BXSqlFilterOperators.Equal, site.Id))
					, null);
				if (items.Count > 0)
					throw new Exception(GetMessageRaw("Error.CodeNotUnique"));
				item.Code = newCode;

				item.Name = Name.Text.Trim();
				item.Sort = string.IsNullOrEmpty(sort) ? 100 : int.Parse(sort);
				item.SiteId = site.TextEncoder.Decode(site.Id);

				if (!String.IsNullOrEmpty(XmlId.Text.Trim()))
					item.XmlId = XmlId.Text.Trim();

				item.Save();

				Item = item;
				ShowOk();
			}
			catch (Exception ex)
			{
				ShowError(ex);
			}
		}
		protected void TabControl_Command(object sender, BXTabControlCommandEventArgs e)
		{
			if (e.CommandName == "cancel")
				GoBack();

			DoSave();
			if (HasErrors)
				return;

			if (e.CommandName == "save")
				GoBack();
			else
				Response.Redirect("SaleCustomerTypeEdit.aspx?id=" + Item.Id.ToString());
		}
		protected void ActionBar_CommandClick(object sender, CommandEventArgs e)
		{
			if (e.CommandName != "delete")
				return;

			if (Item == null)
				return;

			if (!CanManage)
			{
				ShowErrorText(GetMessageRaw("Error.InsufficientRightsToDelete"));
				return;
			}

			bool success = false;
			try
			{
				Item.Delete();
				success = true;
			}
			catch (BXEventException ex)
			{
				foreach (string s in ex.Messages)
					ShowErrorText(s);
			}
			catch
			{
				ErrorMessage.AddErrorMessage(GetMessageRaw("Error.Delete"));
			}

			if (success)
				GoBack();
		}
	}
}
