﻿using System;
using System.Linq;
using System.Web.UI.WebControls;
using Bitrix.DataLayer;
using Bitrix.Sale.SaleCart;
using Bitrix.Security;
using Bitrix.Services;
using Bitrix.Services.Text;
using Bitrix.UI;

namespace Bitrix.Sale.AdminPages
{
	public partial class CustomerTypeList : BXAdminPage
	{
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
				BXAuthentication.AuthenticationRequired();

			SiteFilter.Values.AddRange(
				(
					from s in BXSite.GetAllSitesReadOnly()
					where (s.Active)
					select new ListItem(s.TextEncoder.Decode(s.Name), s.TextEncoder.Decode(s.Id))
				)
				.ToArray()
			);
		}
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);

			MasterTitle = GetMessageRaw("MasterTitle");
		}

		protected void GridView_Select(object sender, BXSelectEventArgs e)
		{
			var filter = new BXFilter(AdminFilter.CurrentFilter, BXSaleCustomerType.Fields);
			var orderBy = new BXOrderBy(BXSaleCustomerType.Fields, e.SortExpression);
			var select = new BXSelectAdd(
				BXSaleCustomerType.Fields.Site.Name
			);

			e.Data = BXSaleCustomerType.GetList(filter, orderBy, select, new BXQueryParams(e.PagingOptions), BXTextEncoder.EmptyTextEncoder);
		}
		protected void GridView_SelectCount(object sender, BXSelectCountEventArgs e)
		{
			var filter = new BXFilter(AdminFilter.CurrentFilter, BXSaleCustomerType.Fields);
			e.Count = BXSaleCustomerType.Count(filter);
		}
		protected void GridView_Delete(object sender, BXDeleteEventArgs e)
		{
			try
			{
				var filter =
					(e.Keys == null)
					? new BXFilter(AdminFilter.CurrentFilter, BXSaleCustomerType.Fields)
					: new BXFilter(new BXFilterItem(BXSaleCustomerType.Fields.Id, BXSqlFilterOperators.Equal, e.Keys["Id"]));
				
				var items = BXSaleCustomerType.GetList(filter, null);
				foreach (var item in items)
				{
					try
					{
						item.Delete();
						e.DeletedCount++;
					}
					catch (Exception ex)
					{
						ErrorMessage.AddErrorMessage(string.Format(GetMessageRaw("Error.UnableToDelete"), item.Id));
						BXLogService.LogAll(ex, BXLogMessageType.Error, "SaleCustomerTypeList.aspx, Delete");
					}
				}
			}
			catch (Exception ex)
			{
				ErrorMessage.AddErrorMessage(ex.Message);
				return;
			}
		}

		protected void GridView_RowDataBound(object sender, GridViewRowEventArgs e)
		{
			if (e.Row.RowType != DataControlRowType.DataRow)
				return;

			var row = (BXGridViewRow)e.Row;
			var item = (BXSaleCustomerType)row.DataItem;
			row.UserData.Add("Id", item.Id);			
		}
	}
}