﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using Bitrix.CommunicationUtility;
using System.Text;
using Bitrix.Services.Js;
using System.Web.Script.Serialization;
using Bitrix.Security;
using Bitrix.Services;
using Bitrix.Services.Text;
using Bitrix.Sale;
using System.Globalization;
using Bitrix;
using Bitrix.DataLayer;
using System.Collections.Specialized;


public partial class bitrix_admin_SaleDeliveryHandlerEdit : BXAdminPage
{
    private JavaScriptSerializer PrepareJsonSerializer()
    {
        JavaScriptSerializer serializer = new JavaScriptSerializer();
        if (this.serviceConfigEditor != null)
            serializer.RegisterConverters(this.serviceConfigEditor.CreateJsonConverters());

        return serializer;
    }

    protected override void OnInit(EventArgs e)
    {
        base.OnInit(e);

        if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        BXSiteCollection siteColl = BXSite.GetList(null,
            new BXOrderBy(new BXOrderByPair(BXSite.Fields.Sort, BXOrderByDirection.Asc)),
            new BXSelect(BXSite.Fields.ID, BXSite.Fields.Name),
            null,
            BXTextEncoder.EmptyTextEncoder
            );

        ListItemCollection siteItems = TargetSite.Items;
        siteItems.Clear();

        siteItems.Add(new ListItem(GetMessageRaw("SelectItem.AllSites"), string.Empty));
        foreach (BXSite site in siteColl)
            siteItems.Add(new ListItem(string.Format("[{0}] {1}", site.Id, site.Name), site.Id));

        if (!string.IsNullOrEmpty(SiteId))
        {
            ListItem selSiteItem = siteItems.FindByValue(SiteId);
            if (selSiteItem != null)
                selSiteItem.Selected = true;
        }
    }

    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);

        if (!string.IsNullOrEmpty(ServiceName))
            this.serviceConfig = BXSaleDeliveryManager.GetServiceConfig(ServiceName, SiteId);

        if (this.serviceConfig == null)
        {
            this.editorError = SaleDeliveryHandlerEditorError.IsNotFound;
            this.errorMessageText = GetMessageRaw("Message.ServiceConfigIsNotFound");
            return;
        }

		if(IsCopyMode && this.serviceConfig.IsPersistent)
			Reload(TabControl.SelectedIndex, new string[]{ "copy" }, null);

        this.serviceConfigEditor = this.serviceConfig.CreateConfigEditor();

		TargetSite.Enabled = IsCopyMode;

        if (!IsPostBack)
        {
            Active.Checked = this.serviceConfig.Active;
            Name.Text = this.serviceConfig.Name;
            Description.Text = this.serviceConfig.Description;
            Extra.Text = this.serviceConfig.Extra.ToString("F3", CultureInfo.InvariantCulture.NumberFormat);
            Sort.Text = this.serviceConfig.Sort.ToString();
        }

        MasterTitle = Page.Title = string.Format(GetMessage( IsCopyMode ? "PageTitle.Copy" : "PageTitle.Edit"), this.serviceConfig.ServiceDisplayName, this.serviceConfig.ServiceName);
    }

    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        if (EditorError != SaleDeliveryHandlerEditorError.None)
        {
            errorMessage.AddErrorMessage(ErrorMessageText);
            if (EditorError == SaleDeliveryHandlerEditorError.IsNotFound)
                TabControl.Visible = false;
        }

        if (this.serviceConfigEditor != null)
        {
            ConfigData.Value = PrepareJsonSerializer().Serialize(this.serviceConfigEditor);
            RegisterScriptIncludeBatch(this.serviceConfigEditor.GetClientSideFilePaths());

            ClientScript.RegisterOnSubmitStatement(GetType(), "SaveData", "Bitrix.SaleDeliveryHandlerEditor.getInstance().save();");
        }

        BXPage.Scripts.RequireUtils();
    }

    private string serviceName = null;
    protected string ServiceName
    {
        get { return this.serviceName ?? (this.serviceName = Request.QueryString["serviceName"] ?? string.Empty); }
    }

    private string siteId = null;
    protected string SiteId
    {
        get { return this.siteId ?? (this.siteId = Request.QueryString["siteId"] ?? string.Empty); }
    }

    protected string DescriptionHtml
    {
        get { return this.serviceConfig != null ? this.serviceConfig.ServiceDescriptionHtml : string.Empty; }
    }

    private BXSaleDeliveryServiceConfig serviceConfig = null;
    protected BXSaleDeliveryServiceConfig ServiceConfig
    {
        get { return this.serviceConfig; }
    }


    private BXSaleDeliveryServiceConfigEditor serviceConfigEditor = null;
    protected BXSaleDeliveryServiceConfigEditor ServiceConfigEditor
    {
        get { return this.serviceConfigEditor; }
    }

	private bool? isCopyMode = null;
	public bool IsCopyMode
	{
		get
		{
			if(this.isCopyMode.HasValue)
				return this.isCopyMode.Value;

			return (this.isCopyMode = Request["copy"] != null).Value;
		}
	}

    protected override string BackUrl
    {
        get { return !string.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "SaleDeliveryHandlerList.aspx"; }
    }

    private void TrySave()
    {
        if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            return;

        if (this.editorError != SaleDeliveryHandlerEditorError.None)
            return;

        try
        {
            string configStr = ConfigData.Value;
            if (!string.IsNullOrEmpty(configStr))
            {
                this.serviceConfigEditor = PrepareJsonSerializer().Deserialize<BXSaleDeliveryServiceConfigEditor>(configStr);
                this.serviceConfig = this.serviceConfigEditor != null ? this.serviceConfigEditor.Config : null;
            }

            if (this.serviceConfig == null)
            {
                this.editorError = SaleDeliveryHandlerEditorError.IsNotFound;
                this.errorMessageText = GetMessageRaw("Message.ServiceConfigIsNotFound");
                return;
            }

			this.serviceConfig.SiteId = TargetSite.SelectedValue;
            this.serviceConfig.Active = Active.Checked;
            this.serviceConfig.Name = Name.Text;
            this.serviceConfig.Description = Description.Text;

            decimal d;
            this.serviceConfig.Extra = decimal.TryParse(Extra.Text, NumberStyles.Number, CultureInfo.InvariantCulture.NumberFormat, out d) ? d : 0m;

            int i;
            this.serviceConfig.Sort = int.TryParse(Sort.Text, out i) ? i : 100;

            this.serviceConfig.Save();
        }
        catch (Exception exc)
        {
            this.errorMessageText = exc.Message;
            this.editorError = SaleDeliveryHandlerEditorError.Modification;
        }
    }

    private string errorMessageText = string.Empty;
    protected string ErrorMessageText
    {
        get { return this.errorMessageText; }
    }

    private SaleDeliveryHandlerEditorError editorError = SaleDeliveryHandlerEditorError.None;
    protected SaleDeliveryHandlerEditorError EditorError
    {
        get { return this.editorError; }
    }

    protected void OnTabControlCommand(object sender, BXTabControlCommandEventArgs e)
    {
        switch (e.CommandName)
        {
            case "save":
                {
                    if (IsValid && EditorError == SaleDeliveryHandlerEditorError.None)
                    {
                        TrySave();
                        if (EditorError == SaleDeliveryHandlerEditorError.None)
                            GoBack("~/bitrix/admin/SaleDeliveryHandlerList.aspx");
                    }
                }
                break;
            case "apply":
                {
                    if (IsValid && EditorError == SaleDeliveryHandlerEditorError.None)
                    {
                        TrySave();
                        if (EditorError == SaleDeliveryHandlerEditorError.None && ServiceConfig != null)
                            Response.Redirect(string.Format("SaleDeliveryHandlerEdit.aspx?serviceName={0}&siteId={1}&tabindex={2}", HttpUtility.UrlEncode(ServiceConfig.ServiceName), HttpUtility.UrlEncode(ServiceConfig.SiteId), TabControl.SelectedIndex));
                    }
                }
                break;
            case "cancel":
                GoBack();
                break;
        }
    }
}

public enum SaleDeliveryHandlerEditorError
{
    None = 0,
    IsNotFound = -1,
    Modification = -2
}