﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using Bitrix.UI;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix;
using System.Text;
using Bitrix.Security;
using Bitrix.Sale;

public partial class bitrix_admin_SaleDeliveryHandlerList : BXAdminPage
{
    protected override void OnPreInit(EventArgs e)
    {
	    if (!BXUser.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnPreInit(e);
    }

	protected override void OnInit(EventArgs e)
	{
        MasterTitle = Page.Title = GetMessage("PageTitle");
		base.OnInit(e);
	}

    private BXSaleDeliveryServiceConfig[] serviveCfgAry = null;
    protected BXSaleDeliveryServiceConfig[] GetServiveCfgAry()
    {
        return this.serviveCfgAry ?? (this.serviveCfgAry = BXSaleDeliveryManager.GetAllServiceConfigs());
    }

    protected void ItemGrid_Select(object sender, BXSelectEventArgs e)
	{
	    if (!BXUser.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        List<ServiceConfigWrapper> list = new List<ServiceConfigWrapper>();
        foreach (BXSaleDeliveryServiceConfig cfg in GetServiveCfgAry())
            list.Add(new ServiceConfigWrapper(cfg, this));

        e.Data = list;
	}

    protected void ItemGrid_SelectCount(object sender, BXSelectCountEventArgs e)
	{
        e.Count = GetServiveCfgAry().Length;
	}

	private string[] systemCommands = new string[] { "copyToCustom" };
	private string[] customCommands = new string[] { "edit", "delete" };

    protected void ItemGrid_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType != DataControlRowType.DataRow)
            return;

        BXGridViewRow row = (BXGridViewRow)e.Row;
        ServiceConfigWrapper wrapper = (ServiceConfigWrapper)row.DataItem;

        row.UserData.Add("ServiceName", wrapper.ServiceName);
        row.UserData.Add("SiteId", wrapper.SiteId);

		row.AllowedCommandsList = wrapper.IsDefult ? systemCommands : customCommands;
    }

    protected void ItemGrid_Delete(object sender, BXDeleteEventArgs e)
	{
        if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            return;

		try
		{
			if(this.serviveCfgAry == null)
				this.serviveCfgAry = BXSaleDeliveryManager.GetAllServiceConfigs();

			BXSaleDeliveryServiceConfig cfg = Array.Find<BXSaleDeliveryServiceConfig>(
				this.serviveCfgAry, 
				x => string.Equals(x.ServiceName, (string)e.Keys["ServiceName"], StringComparison.InvariantCultureIgnoreCase) 
					&& string.Equals(x.SiteId, (string)e.Keys["SiteId"], StringComparison.InvariantCultureIgnoreCase)
					&& !x.IsDefault);

			if(cfg != null)
			{
				cfg.Delete();
				e.DeletedCount++;
				this.serviveCfgAry = null;
			}
			else
			{
				cfg = Array.Find<BXSaleDeliveryServiceConfig>(
					this.serviveCfgAry, 
					x => string.Equals(x.ServiceName, (string)e.Keys["ServiceName"], StringComparison.InvariantCultureIgnoreCase) 
						&& string.Equals(x.SiteId, (string)e.Keys["SiteId"], StringComparison.InvariantCultureIgnoreCase)
						&& x.IsDefault);
				if(cfg != null)
					ErrorMessage.AddErrorMessage(GetMessageRaw("Error.CantDeleteDefaultConfiguration"));
			}

		}
		catch (Exception ex)
		{
			ErrorMessage.AddErrorMessage(ex.Message);
		}
	}
}

/// <summary>
/// Обёртка для Информера Службы Доставки
/// </summary>
public class ServiceConfigWrapper
{
    BXSaleDeliveryServiceConfig _charge;
    BXAdminPage _parentPage;
    public ServiceConfigWrapper(BXSaleDeliveryServiceConfig charge, BXAdminPage parentPage)
    {
        if (charge == null)
            throw new ArgumentNullException("charge");

        if (parentPage == null)
            throw new ArgumentNullException("parentPage");

        _charge = charge;
        _parentPage = parentPage;
    }

    public string ServiceName
    {
        get { return _charge.ServiceName; }
    }

    public string Name
    {
        get { return _charge.Name; }
    }

    public string Sort
    {
        get { return _charge.Sort.ToString(); }
    }

    public string SiteId
    {
        get { return _charge.SiteId; }
    }

    public bool IsActive
    {
        get { return _charge.IsDefault ? !_charge.IsPersistent : _charge.Active; }
    }

    public bool IsDefult
    {
        get { return _charge.IsDefault; }
    }

    public string Active
    {
        get { return _parentPage.GetMessageRaw(IsActive ? "Kernel.Yes" : "Kernel.No"); }
    }

	public string Configuration
	{
        get { return _parentPage.GetMessageRaw(_charge.IsDefault ? _charge.IsPersistent ? "Configuration.SystemOverriden" : "Configuration.SystemDefault" : "Configuration.Custom" ); }
	}
}
