﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.UI;
using Bitrix.Services.Text;
using Bitrix;
using Bitrix.DataLayer;
using Bitrix.Modules;
using System.Collections.Generic;
using Bitrix.Security;
using Bitrix.Configuration;
using Bitrix.Sale;
using System.Globalization;

public enum SaleDiscountCouponEditorMode
{
    Creation = 1,
    Modification = 2
}

public enum SaleDiscountCouponEditorError
{
    None = 0,
    IsNotFound = -1,
    CreationFailed = -2,
    ModificationFailed = -3,
    DeletionFailed = -4
}

public partial class bitrix_admin_SaleDiscountCouponEdit : BXAdminPage
{
	private int? entityId = null;
	protected int EntityId
	{
        get 
        {
            if (this.entityId.HasValue)
                return this.entityId.Value;

            string s = Request.QueryString["id"];
            if (string.IsNullOrEmpty(s))
                this.entityId = 0;
            else
            {
                int i;
                this.entityId = int.TryParse(s, out i) ? i : 0;
            }
            return this.entityId.Value; 
        }
	}

    private SaleDiscountCouponEditorMode editorMode = SaleDiscountCouponEditorMode.Creation;
    protected SaleDiscountCouponEditorMode EditorMode
    {
        get { return this.editorMode; }
    }

    private BXSaleDiscountCoupon entity;
    protected BXSaleDiscountCoupon Entity
    {
        get { return this.entity; }
    }

    private SaleDiscountCouponEditorError editorError = SaleDiscountCouponEditorError.None;
    protected SaleDiscountCouponEditorError EditorError
    {
        get { return this.editorError; }
    }

    private string errorText = string.Empty;
    public string ErrorText
    {
        get { return this.errorText; }
    }

	protected override string BackUrl
	{
		get { return !string.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "SaleDiscountCouponList.aspx"; }
	}

    protected override void OnInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnInit(e);
    }

	protected override void OnInitComplete(EventArgs e)
	{
		base.OnInitComplete(e);

		var discounts = BXSaleDiscount.GetList(new BXFilter(new BXFilterItem(BXSaleDiscount.Fields.Active, BXSqlFilterOperators.Equal, true)),
			new BXOrderBy(new BXOrderByPair(BXSaleDiscount.Fields.Name, BXOrderByDirection.Asc)));
		foreach (var discount in discounts)
			Discount.Items.Add(new ListItem { Text = discount.Name, Value = discount.Id.ToString() });
	}

    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);

        TryLoadEntity();
        BXSaleDiscountCoupon entity = Entity;
        string title = EditorMode == SaleDiscountCouponEditorMode.Modification ? 
			string.Format(GetMessage("PageTitle.Edit"), entity != null ? 
			" #"+ entity.Id.ToString() : "?") : GetMessage("PageTitle.Create");

		if (EditorMode == SaleDiscountCouponEditorMode.Creation)
		{
			AddButton.Visible = false;
			DeleteButton.Visible = false;
			Discount.Visible = true;
			DiscountName.Visible = false;
			Active.Checked = true;

		}
		else
		{
			Discount.Visible = false;
			DiscountName.Visible = true;
			DiscountName.Text = Entity.Discount.Name;
		}
        MasterTitle = Page.Title = title;
    }

    private void TryLoadEntity()
	{
        int id = EntityId;
        if (id <= 0)
        {
            this.editorMode = SaleDiscountCouponEditorMode.Creation;
            this.entity = new BXSaleDiscountCoupon();
        }
        else
        {
            this.editorMode = SaleDiscountCouponEditorMode.Modification;
            if ((this.entity = BXSaleDiscountCoupon.GetById(id, BXTextEncoder.EmptyTextEncoder)) == null)
            {
                this.errorText = string.Format(GetMessageRaw("Error.UnableToFindEntity"), id);
                this.editorError = SaleDiscountCouponEditorError.IsNotFound;
                return;
            }
        }

        if (!IsPostBack)
        {
			Code.Text = this.entity.CouponString;
			IsOneTime.Checked = this.entity.IsOneTime;
			Active.Checked = this.entity.Active;
        }
        else
        {
			this.entity.IsOneTime = IsOneTime.Checked;
			this.entity.Active = Active.Checked;
			if(Code.Text!=String.Empty)
				this.entity.CouponString = Code.Text;
			int discountId = 0;

			if (editorMode == SaleDiscountCouponEditorMode.Creation && int.TryParse(Discount.SelectedValue, out discountId))
			{
				this.entity.DiscountId = discountId;

			}
        }
	}

    private void TrySaveEntity()
    {
        if (this.entity == null)
            return;

        try
        {
            this.entity.Save();
            this.entityId = this.entity.Id;
        }
        catch (Exception exc)
        {
			var msg = GetMessage("Error."+exc.Message);
            this.errorText= string.IsNullOrEmpty(msg) ? exc.Message : msg;
            this.editorError = EditorMode == SaleDiscountCouponEditorMode.Creation ? SaleDiscountCouponEditorError.CreationFailed : SaleDiscountCouponEditorError.ModificationFailed; 
        }
    }

    protected override void OnPreRender(EventArgs e)
    {
        if (EditorError != SaleDiscountCouponEditorError.None)
        {
            Summary.AddErrorMessage(ErrorText);
            if (EditorMode == SaleDiscountCouponEditorMode.Modification && EditorError == SaleDiscountCouponEditorError.IsNotFound)
                TabView.Visible = false;
        }
        base.OnPreRender(e);
    }

    protected void OnSaleDiscountCouponEdit(object sender, BXTabControlCommandEventArgs e)
	{
		switch (e.CommandName)
		{
            case "save":
                {
                    if (IsValid)
                    {
                        TrySaveEntity();
                        if (EditorError == SaleDiscountCouponEditorError.None)
                            GoBack(); 
                    }
                }
                break;
            case "apply":
                {
                    if (IsValid)
                    {
                        TrySaveEntity();
                        if (EditorError == SaleDiscountCouponEditorError.None)
                            Response.Redirect(string.Format("SaleDiscountCouponEdit.aspx?id={0}&tabindex={1}", EntityId.ToString(), TabView.SelectedIndex));
                    }
                }
                break;
			case "cancel":
				GoBack();
				break;
		}
	}

	protected void OnToolBarButtonClick(object sender, CommandEventArgs e)
	{
		if (e.CommandName == "delete")
		{
			try
			{
                BXSaleDiscountCoupon entity = Entity;
                if (entity  != null)
                    entity.Delete();

				GoBack();
			}
			catch (Exception ex)
			{
                this.errorText = ex.Message;
                this.editorError = SaleDiscountCouponEditorError.DeletionFailed;
			}
		}
	}

    protected void MoveSelectedUp(ListItemCollection items)
    {
        var selItems = new List<ListItem>();
        for (int i = items.Count - 1; i >= 0; i--)
        {
            if (items[i].Selected)
            {
                selItems.Add(items[i]);
                items.RemoveAt(i);
            }
        }
        foreach (var i in selItems)
            items.Insert(0, i);
    }
}
