﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using Bitrix.UI;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix;
using System.Text;
using Bitrix.Security;
using Bitrix.Sale;
using Bitrix.Main;
using System.Globalization;

public partial class bitrix_admin_SaleDiscountCouponList : BXAdminPage
{

    protected override void OnPreInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnPreInit(e);
    }

	protected override void OnInit(EventArgs e)
	{
		var discounts = BXSaleDiscount.GetList(null, new BXOrderBy(new BXOrderByPair(BXSaleDiscount.Fields.Name, BXOrderByDirection.Asc)));
		DiscountFilter.Values.Add(new ListItem(GetMessageRaw("Kernel.All"), ""));
		foreach (var d in discounts)
			DiscountFilter.Values.Add(new ListItem() { Text = String.Format("[{0}] {1} [{2}]", d.Id.ToString(), d.Name, d.SiteId), Value = d.Id.ToString() });

		base.OnInit(e);
	}

    protected override void OnLoad(EventArgs e)
    {
        ((BXAdminMasterPage)Master).PageIconId = "sale_page_icon";
        MasterTitle = Page.Title = GetMessage("PageTitle");


        base.OnLoad(e);
    }

    protected void ItemGrid_Select(object sender, BXSelectEventArgs e)
	{
        BXSaleDiscountCouponCollection c = BXSaleDiscountCoupon.GetList(
            new BXFilter(ItemFilter.CurrentFilter, BXSaleDiscountCoupon.Fields),
            new BXOrderBy(BXSaleDiscountCoupon.Fields, string.IsNullOrEmpty(e.SortExpression) ? "Id" : e.SortExpression),
            new BXSelectAdd(
                BXSaleDiscountCoupon.Fields.Discount
                ),
            new BXQueryParams(e.PagingOptions),
            BXTextEncoder.EmptyTextEncoder
            );

        List<SaleDiscountCouponWrapper> list = new List<SaleDiscountCouponWrapper>();
        foreach (BXSaleDiscountCoupon item in c)
            list.Add(new SaleDiscountCouponWrapper(item, this));

        e.Data = list;
	}


    protected void ItemGrid_SelectCount(object sender, BXSelectCountEventArgs e)
	{
        e.Count = BXSaleDiscountCoupon.Count(new BXFilter(ItemFilter.CurrentFilter, BXSaleDiscountCoupon.Fields));
	}

    protected void ItemGrid_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType != DataControlRowType.DataRow)
            return;

        BXGridViewRow row = (BXGridViewRow)e.Row;
        row.UserData.Add("ID", ((SaleDiscountCouponWrapper)row.DataItem).ID);
    }


    protected void ItemGrid_Delete(object sender, BXDeleteEventArgs e)
	{
        if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            return;

		BXGridView grid = (BXGridView)sender;
		BXSaleDiscountCouponCollection coupons;
		try
		{
			BXFilter filter = (e.Keys == null)
				? new BXFilter(ItemFilter.CurrentFilter, BXSaleDiscount.Fields)
				: new BXFilter(new BXFilterItem(BXSaleDiscountCoupon.Fields.Id, BXSqlFilterOperators.Equal, e.Keys["ID"]));
			coupons = BXSaleDiscountCoupon.GetList(filter, null);
		}
		catch (Exception ex)
		{
			ErrorMessage.AddErrorMessage(ex.Message);
			return;
		}
		bool errorTextAdded = false;
		foreach (BXSaleDiscountCoupon c in coupons)
		{
			try
			{
				c.Delete();
				e.DeletedCount++;
			}
			catch (Exception ex2)
			{
				if (!errorTextAdded)
				{
					ErrorMessage.AddErrorMessage(ex2.Message);
					errorTextAdded = true;
				}
			}
		}
	}
}
/// <summary>
/// Обёртка
/// </summary>
public class SaleDiscountCouponWrapper
{
    BXSaleDiscountCoupon coupon;
    BXAdminPage parentPage;
    public SaleDiscountCouponWrapper(BXSaleDiscountCoupon coupon, BXAdminPage parentPage)
    {
		if (coupon == null)
			throw new ArgumentNullException("coupon");

        if (parentPage == null)
            throw new ArgumentNullException("parentPage");

		this.coupon = coupon;
        this.parentPage = parentPage;
    }

    public string ID
    {
		get { return this.coupon.Id.ToString(); }
    }

    public string CouponString
    {
		get { return HttpUtility.HtmlEncode(this.coupon.CouponString); }
    }

	public bool Active
	{
		get { return this.coupon.Active; }
	}

	public string ActiveString
	{
		get { return parentPage.GetMessage(this.coupon.Active ? "Kernel.Yes" : "Kernel.No"); ; }
	}

	public bool IsOneTime
	{
		get { return this.coupon.IsOneTime; }
	}

	public string DiscountName
	{
		get { return this.coupon.Discount.Name; }
	}

	public string AppliedDateString
	{
		get { return this.coupon.AppliedDateUtc != DateTime.MinValue ? this.coupon.AppliedDate.ToString("g") : String.Empty; }
	}

	public DateTime AppliedDate
	{
		get { return this.coupon.AppliedDate; }
	}

	public DateTime AppliedDateUtc
	{
		get { return this.coupon.AppliedDateUtc; }
	}

	public string IsOneTimeString
	{
		get { return parentPage.GetMessage(this.coupon.IsOneTime ? "Kernel.Yes" : "Kernel.No"); ; }
	}
}
