﻿using System;
using System.Text;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.ObjectModel;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.UI;
using Bitrix.Services;
using Bitrix.Main;
using System.Collections.Generic;
using Bitrix;
using Bitrix.Security;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using Bitrix.Sale;
using Bitrix.Currency;
using Bitrix.DataTypes;
using Bitrix.Catalog;
using System.Xml;
using System.IO;
using System.Globalization;

public partial class bitrix_admin_SaleDiscountEdit : Bitrix.UI.BXAdminPage
{

	BXSaleDiscount discount;

	List<string> pageErrors;

	public string VariantsJSON {get;set;}

	public string ApplyVariantsJSON { get; set; }

	public string DefaultSelectLabel { get; set; }
	
	public string DefaultItemSelectLabel { get; set; }

	BXSiteCollection sites;

	public BXSiteCollection Sites
	{
		get { return sites ?? (sites = new BXSiteCollection()); }
	}

	BXCurrencyCollection currencies;

	BXRoleCollection userRoles;


	protected void OnEditCommand(object sender, BXTabControlCommandEventArgs e)
	{
		if(e.CommandName == "apply" || e.CommandName == "save")
		{
			var root = BXDiscountGUIHelper.ParseRuleFromClient(hdDiscountTree.Value, BXLoc.CurrentLocale, false);

			var ruleString = String.Empty;
			if(root!=null)
				ruleString = BXDiscountGUIHelper.GUITreeToDiscountTree(root);

			var applyString = String.Empty;
			root = BXDiscountGUIHelper.ParseRuleFromClient(hdApply.Value, BXLoc.CurrentLocale, false);
			if(root!=null)
				applyString = BXDiscountGUIHelper.GUITreeToDiscountTree(root);

			if(SaveDiscount(ruleString, applyString) && editorError == SaleDiscountEditorError.None)
			{
				if(e.CommandName == "save")
					GoBack();
				else
					Response.Redirect(TabControl.SelectedIndex > 0 ? string.Format("SaleDiscountEdit.aspx?id={0}&tabindex={1}", discount.Id.ToString(), TabControl.SelectedIndex) : string.Format("SaleDiscountEdit.aspx?id={0}", discount.Id.ToString()));
			}
		}
	}

    protected override void OnPreRender(EventArgs e)
    {
        if (EditorError != SaleDiscountEditorError.None)
        {
		    foreach (var error in pageErrors)
		        Summary.AddErrorMessage(error);
            if (EditorMode == SaleDiscountEditorMode.Modification && EditorError ==  SaleDiscountEditorError.IsNotFound)
                TabControl.Visible = false;
        }
        base.OnPreRender(e);
    }

	protected void OnToolBarButtonClick(object sender, CommandEventArgs e)
	{
		if (e.CommandName == "delete")
		{
			try
			{
                if (discount != null)
                    discount.Delete();

				GoBack();
			}
			catch (Exception ex)
			{
                pageErrors.Add(ex.Message);
				editorError = SaleDiscountEditorError.DeletionFailed;
			}
		}
	}

	public BXSaleDiscount Discount
	{
		get { return discount; }
	}

	public int CurrencyId
	{
		get { return BXCurrency.NationalCurrencyId; }
	}

	protected void Page_Init(object sender, EventArgs e)
	{
		if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
			BXAuthentication.AuthenticationRequired();

		sites = BXSite.GetList(new BXFilter(new BXFilterItem(BXSite.Fields.Active, BXSqlFilterOperators.Equal, true)),
			new BXOrderBy(new BXOrderByPair(BXSite.Fields.Sort, BXOrderByDirection.Asc)), null, null, BXTextEncoder.EmptyTextEncoder);

		currencies = BXCurrency.GetList(null, new BXOrderBy(new BXOrderByPair(BXCurrency.Fields.Iso4217AlphaCode, BXOrderByDirection.Asc)));


		userRoles = BXRoleManager.GetList(
			new BXFormFilter(new BXFormFilterItem("Active", true, BXSqlFilterOperators.Equal)),
			new BXOrderBy_old("RoleName", "Asc")
		);

		userRoles.Sort(delegate(BXRole a, BXRole b)
		{
			int i = RoleClass(a).CompareTo(RoleClass(b));
			return i != 0 ? i : string.Compare(a.Title, b.Title, true);
		});

		foreach (var site in sites)
			Site.Items.Add(new ListItem() { Text = site.Name, Value = site.Id });
		foreach (var currency in currencies)
			Currency.Items.Add(new ListItem() { Value = currency.Id.ToString(), Text = currency.Iso4217AlphaCode });

		foreach (var role in userRoles)
			UserRoles.Items.Add(new ListItem()
			{
				Text = String.Format("[{0}] {1}", role.RoleId.ToString(), role.Title),
				Value = role.RoleId.ToString()
			});

		foreach (var val in Enum.GetValues(typeof(BXSaleDiscountType)))
		{
			var str = (BXSaleDiscountType)val;
			DiscountType.Items.Add(new ListItem(GetMessage("DiscountType." + str.ToString()), ((byte)val).ToString()));
		}

		//DiscountType.Items.Add(new ListItem() { Value = "percent", Text = GetMessage("DiscountType.Percent") });
		//DiscountType.Items.Add(new ListItem() { Value = "sum", Text = GetMessage("DiscountType.Sum") });
	}

	protected void Page_Load(object sender, EventArgs e)
	{
		Title = GetMessage("PageTitle");
		pageErrors = new List<string>();
		((BXAdminMasterPage)Master).Title = GetMessage("MasterTitle");
		Page.ClientScript.RegisterClientScriptInclude("discountrules", "../js/Catalog/discountrules.js");
		var script = "BX('" + hdDiscountTree.ClientID + "').value = Bitrix.treeProcessor.SerializeTree(discountState, null, null, 0);" + "BX('" + hdApply.ClientID + "').value = Bitrix.treeProcessor.SerializeTree(applyState, null, null, 0);";
		Page.ClientScript.RegisterOnSubmitStatement(GetType(), "submittree", script);


		ApplyVariantsJSON = BXDiscountConfiguration.ConditionVariantsToJSON(BXLoc.CurrentLocale, BXDiscountConditionVariantType.SaleApply);
		VariantsJSON = BXDiscountConfiguration.ConditionVariantsToJSON(BXLoc.CurrentLocale, BXDiscountConditionVariantType.Sale);
		DefaultSelectLabel = BXLoc.GetModuleMessage("Sale", "SaleDiscount.AdminMessage.DefaultSelectLabel");

		DefaultItemSelectLabel = BXLoc.GetModuleMessage("Sale", "SaleDiscount.AdminMessage.DefaultItemSelectLabel");

		LoadDiscount();

		var msg = GetMessage("Label.AutoGenerate");
		var index = msg.IndexOf("#input#");
		if (index != -1)
		{
			InfoBeforeGen.Text = msg.Substring(0, index);
			InfoAfterGen.Text = msg.Substring(index + 7);
		}
		else
			InfoBeforeGen.Text = msg;

        string title = editorMode == SaleDiscountEditorMode.Modification ? 
			string.Format(GetMessage("PageTitle.Edit"), discount != null ? 
			discount.Name : "?") : GetMessage("PageTitle.Create");

		if (EditorMode == SaleDiscountEditorMode.Creation)
		{
			AddButton.Visible = false;
			DeleteButton.Visible = false;
		}
        MasterTitle = Page.Title = title;
	}

	public string DiscountJSON 
	{
		get
		{
			return discount != null && !String.IsNullOrEmpty(discount.Filter) ? BXDiscountGUIHelper.DiscountTreeToJSON2(discount.Filter, false) : 
				BXDiscountGUIHelper.DefaultRuleJSON(false);
		} 
	}

	public string ApplyDiscountJSON
	{
		get
		{
			return discount != null && !String.IsNullOrEmpty(discount.FilterApply) ? BXDiscountGUIHelper.DiscountTreeToJSON2(discount.FilterApply, false) :
				BXDiscountGUIHelper.DefaultRuleJSON(false, true);
		}
	}

	static int RoleClass(BXRole role)
	{
		switch (role.RoleId)
		{
			case 2:
				return 0;
			case 3:
				return 1;
			case 1:
				return 2;
			default:
				return 3;
		}
	}

	void LoadDiscount()
	{
		int id;
		if (!int.TryParse(Request.QueryString["id"], out id))
			editorMode = SaleDiscountEditorMode.Creation;
		else
		{
			editorMode = SaleDiscountEditorMode.Modification;
			discount = BXSaleDiscount.GetById(id);
			if(discount == null) 
			{
				editorError = SaleDiscountEditorError.IsNotFound;
				return;
			}

			if (!IsPostBack)
			{
				Name.Text = discount.Name;
				Currency.SelectedIndex = currencies.FindIndex(x => x.Id == discount.CurrencyId);
				Site.SelectedIndex = sites.FindIndex(x => x.Id == discount.SiteId);
				Sort.Text = discount.Sort.ToString();

				if (discount.ActiveFromUtc != DateTime.MinValue)
					tiActive.StartDate = discount.ActiveFrom;
				if (discount.ActiveToUtc != DateTime.MinValue)
					tiActive.EndDate = discount.ActiveTo;

				Description.Text = discount.Description ?? "";
				Value.Text = discount.Value.ToString();
				MaxSum.Text = discount.MaxSum == 0? "0": discount.MaxSum.ToString();
				MaxElementsCount.Text = discount.MaxElementsCount.ToString();
				Active.Checked = discount.Active;
				UseCoupons.Checked = discount.UseCoupons;
				DiscountType.SelectedIndex = (int)discount.Type;
				ListItem item;

				foreach (var role in discount.UserRolesList)
				{
					item = UserRoles.Items.FindByValue(role.ToString());
					if (item != null)
						item.Selected = true;
				}



				var coupons = BXSaleDiscountCoupon.GetList(
					new BXFilter(new BXFilterItem(BXSaleDiscountCoupon.Fields.IsOneTime, BXSqlFilterOperators.Equal, false),
								new BXFilterItem(BXSaleDiscountCoupon.Fields.DiscountId, BXSqlFilterOperators.Equal, discount.Id)),
								null);

				if (coupons.Count > 0)
				{
					MultiCouponCode.Text = coupons[0].CouponString;
					MultiCoupon.Checked = true;
					CouponsToGenerate.Text = "0";
					IsMultiCoupon = true;
				}
				else
				{
					IsMultiCoupon = false;
				}
			}
		}
	}

	public bool IsMultiCoupon
	{
		get;
		set;
	}

	private bool TryParseDecimal(string s, out decimal m)
	{	
		if(string.IsNullOrEmpty(s))
		{
			m = 0m;
			return false;
		}
		
		return decimal.TryParse(s, NumberStyles.AllowDecimalPoint, CultureInfo.CurrentCulture.NumberFormat, out m) 
			|| decimal.TryParse(s, NumberStyles.AllowDecimalPoint, CultureInfo.InvariantCulture.NumberFormat, out m);
	}

	bool SaveDiscount(string ruleString, string applyString)
	{
		try
		{
			if (discount == null)
				discount = new BXSaleDiscount();

			int intValue;
			byte byteValue;
			decimal dValue;
			if (int.TryParse(Currency.SelectedValue, out intValue))
				discount.CurrencyId = intValue;

			discount.Active = Active.Checked;
			discount.Name = Name.Text;
			discount.Description = Description.Text;
			discount.SiteId = Site.SelectedValue;
			discount.UseCoupons = UseCoupons.Checked;


			if (byte.TryParse(DiscountType.SelectedValue, out byteValue) && Enum.IsDefined(typeof(BXSaleDiscountType), byteValue))
				discount.Type = (BXSaleDiscountType)byteValue;
			else
			{
				pageErrors.Add(GetMessage("Error.WrongDiscountType"));
				return false;
			}

			if (int.TryParse(Sort.Text, out intValue))
				discount.Sort = intValue;

			if (int.TryParse(MaxElementsCount.Text, out intValue))
				discount.MaxElementsCount = intValue;

			if (TryParseDecimal(Value.Text, out dValue))
			{
				if (discount.Type == BXSaleDiscountType.PercentFromProductPrice && dValue > 100)
				{
					pageErrors.Add(GetMessage("Error.ValueMoreThanHundredPercent"));
					editorError = editorMode == SaleDiscountEditorMode.Creation ? SaleDiscountEditorError.CreationFailed : SaleDiscountEditorError.ModificationFailed;
				}
				discount.Value = dValue;
			}
			else
			{
				editorError = editorMode == SaleDiscountEditorMode.Creation ? SaleDiscountEditorError.CreationFailed : SaleDiscountEditorError.ModificationFailed;
				pageErrors.Add(GetMessage("Error.WrongDiscountValue"));
			}

			if (editorError != SaleDiscountEditorError.None)
				return false;

			if (TryParseDecimal(MaxSum.Text, out dValue))
				discount.MaxSum = dValue;


			if (tiActive.StartDate != DateTime.MinValue)
				discount.ActiveFrom = tiActive.StartDate;

			if (tiActive.EndDate != DateTime.MaxValue)
				discount.ActiveTo = tiActive.EndDate;

			discount.UserRolesList.Clear();
			foreach (ListItem item in UserRoles.Items)
				if (item.Selected && int.TryParse(item.Value, out intValue))
					discount.UserRolesList.Add(intValue);

			discount.Filter = ruleString;
			discount.FilterApply = applyString;
			discount.Save();

			//coupons
			if (MultiCoupon.Checked)
			{
				var couponString = MultiCouponCode.Text.Trim();
				if (couponString == String.Empty)
					return true;

				var coupons = BXSaleDiscountCoupon.GetList(
					new BXFilter(new BXFilterItem(BXSaleDiscountCoupon.Fields.Discount.Id, BXSqlFilterOperators.Equal, discount.Id)
						), null);
				var couponExists = coupons.Exists(x => x.CouponString.Equals(couponString, StringComparison.OrdinalIgnoreCase));
				foreach (var coupon in coupons)
					if (!coupon.CouponString.Equals(couponString, StringComparison.OrdinalIgnoreCase))
						coupon.Delete();
				if (!couponExists)
				{
					var c = new BXSaleDiscountCoupon();
					c.DiscountId = discount.Id;
					c.IsOneTime = false;
					c.CouponString = couponString.ToUpperInvariant();
					c.Active = true;
					c.Save();
				}
			}
			else //many coupons
			{
				int quantity = 0;
				if (int.TryParse(CouponsToGenerate.Text, out quantity))
				{
					string lastCode = "asda";
					for (var i = 0; i < quantity; i++)
					{
						var c = new BXSaleDiscountCoupon();
						c.DiscountId = discount.Id;
						c.IsOneTime = true;

						c.CouponString = BXSaleDiscountCoupon.GenerateCouponCode(lastCode);
						int j = 0;
						while (j < 100)
						{
							var items = BXSaleDiscountCoupon.GetList(new BXFilter(new BXFilterItem(BXSaleDiscountCoupon.Fields.CouponString, BXSqlFilterOperators.Equal, c.CouponString)), null);
							if (items.Count == 0)
								break;
							lastCode = c.CouponString;
							c.CouponString = BXSaleDiscountCoupon.GenerateCouponCode(lastCode);
							j++;
						}

						c.Active = true;
						c.Save();
						lastCode = c.CouponString;
					}
				}
			}

			MultiCoupon.Checked = true;
			CouponsToGenerate.Text = "0";

			return true;
		}
		catch(Exception exc)
		{
			pageErrors.Add(exc.Message);
			editorError = editorMode == SaleDiscountEditorMode.Creation ? SaleDiscountEditorError.CreationFailed : SaleDiscountEditorError.ModificationFailed;
			return false;
		}
	}

	protected override string BackUrl
	{
		get { return !String.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "SaleDiscountList.aspx"; }
	}

	protected string siteId = BXSite.Current.Id;
	protected string currencyCode = BXCurrency.NationalCurrency.Iso4217AlphaCode;

    private SaleDiscountEditorMode editorMode = SaleDiscountEditorMode.Creation;
    protected SaleDiscountEditorMode EditorMode
    {
        get { return this.editorMode; }
    }

    private SaleDiscountEditorError editorError = SaleDiscountEditorError.None;
    protected SaleDiscountEditorError EditorError
    {
        get { return this.editorError; }
    }
}

public enum SaleDiscountEditorMode
{
    Creation = 1,
    Modification = 2
}

public enum SaleDiscountEditorError
{
    None = 0,
    IsNotFound = -1,
    CreationFailed = -2,
    ModificationFailed = -3,
    DeletionFailed = -4
}