﻿using Bitrix.UI;
using Bitrix.Security;
using System.Security;
using Bitrix.Services.Text;
using System;
using System.Linq;
using Bitrix.Modules;
using System.Web.UI.WebControls;
using System.Web.UI;
using Bitrix.DataLayer;
using System.Collections.Generic;
using Bitrix.Services;
using Bitrix.Services.Js;
using Bitrix.Currency;
using System.Web;
using Bitrix.Sale.SaleCart;
using System.Globalization;

namespace Bitrix.Sale.AdminPages
{
	public partial class OrderEdit : BXAdminPage, IPostBackEventHandler
	{
		#region Messages
		int showMessage;

		bool HasErrors { get { return showMessage == -1; } }

		void PrepareResultMessage()
		{
			SuccessMessage.Visible = showMessage > 0;
		}
		void ShowErrorHtml(string html)
		{
			showMessage = -1;
			ErrorMessage.AddErrorMessage(html);
		}
		void ShowErrorText(string text)
		{
			ShowErrorHtml(Encode(text));
		}
		void ShowError(Exception ex)
		{
			showMessage = -1;
			ErrorMessage.AddErrorMessage(Encode(ex.Message));
		}
		void ShowOk()
		{
			if (showMessage == 0)
				showMessage = 1;
		}
		#endregion

		protected BXSaleOrder Item { get; set; }
		protected BXSaleProductCollection Products { get; set; }
		protected List<ProductInfo> ProductInfos { get; set; }
		protected BXSaleOrderStatusCollection Statuses { get; set; }
		protected BXSalePaymentSystemCollection PaymentSystems { get; set; }
		protected List<BXSaleDeliveryServiceInfo> Deliveries { get; set; }
		protected override string BackUrl
		{
			get
			{
				return base.BackUrl ?? "SaleOrderList.aspx";
			}
		}

		private bool deliveriesInitialized;
		protected BXCustomizableDeliveryService customizableDelivery;
		protected BXSalePaymentSystem paymentSystem;
		protected string siteId;
		protected BXSaleCustomerType currentCustomerType;
		protected BXSaleCustomerTypeCollection customerTypes;
		protected BXSaleOrderFieldCollection fieldCollection;
		protected List<FieldInfo> fields;
		protected bool fieldsLoaded;
		protected int destinationCountryCode;
		protected string postCode;
		protected string deliveryVariant;

		private void InitReasonsCancelled()
		{
			ReasonCancelled.Items.Clear();
			ReasonCancelled.Items.Add(new ListItem(GetMessageRaw("Option.NotSet"), ""));
			ReasonCancelled.Items.AddRange
			(
				(
					from BXSaleOrderCancelReason r in Enum.GetValues(typeof(BXSaleOrderCancelReason))
					select new ListItem(BXSaleOrderCancelReasonLocalization.GetLocalizedName(r), r.ToString())
				)
				.ToArray()
			);
		}
		private void InitStatuses()
		{
			Status.Items.Clear();
			if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderExitStatus, BXSaleModuleConfiguration.Id, Item.StatusId))
			{
				var i = Statuses.FindIndex(x => x.Id == Item.StatusId);
				if (i >= 0)
				{
					var s = Statuses[i];
					Status.Items.Add(new ListItem(string.Format("[{0}] {1}", s.XmlId, s.LocalizedName), s.Id.ToString()));
				}
			}
			else
			{
				Status.Items.AddRange
				(
					(
						from s in Statuses
						where s.Id == Item.StatusId || BXUser.IsCanOperate(BXSaleModule.Operations.OrderEnterStatus, BXSaleModuleConfiguration.Id, s.Id)
						select new ListItem(
							string.Format("[{0}] {1}", s.XmlId, s.LocalizedName),
							s.Id.ToString()
						)
					).ToArray()
				);
			}
		}
		private void InitSites()
		{
			siteId = Request.Form["site"] ?? Item.SiteId;
		}
		private void InitItem()
		{
			var idString = Request.QueryString["id"];
			int id = 0;
			if (idString != null)
			{
				if (!int.TryParse(idString, out id) || id <= 0)
					GoBack(); // не удалось найти такой объект

				Item = BXSaleOrder.GetList(
					new BXFilter(
						new BXFilterItem(BXSaleOrder.Fields.Id, BXSqlFilterOperators.Equal, id)
					),
					null,
					new BXSelectAdd(
						BXSaleOrder.Fields.User,
						BXSaleOrder.Fields.Site,
						BXSaleOrder.Fields.Currency,
						BXSaleOrder.Fields.CustomerType,
						BXSaleOrder.Fields.Status,

						BXSaleOrder.Fields.StatusSetEmployee.UserId,
						BXSaleOrder.Fields.StatusSetEmployee.UserName,

						BXSaleOrder.Fields.CancelledEmployee.UserId,
						BXSaleOrder.Fields.CancelledEmployee.UserName,

						BXSaleOrder.Fields.PaidEmployee.UserId,
						BXSaleOrder.Fields.PaidEmployee.UserName,

						BXSaleOrder.Fields.AllowShippingEmployee.UserId,
						BXSaleOrder.Fields.AllowShippingEmployee.UserName,

						BXSaleOrder.Fields.PaymentSystem.Name,
						BXSaleOrder.Fields.PaymentSystem.SiteId
					),
					new BXQueryParams(new BXPagingOptions(0, 1)),
					BXTextEncoder.EmptyTextEncoder
				)
				.FirstOrDefault();

				if (Item == null)
					GoBack();

				Products = BXSaleProduct.GetList(
					new BXFilter(
						new BXFilterItem(BXSaleProduct.Fields.OrderId, BXSqlFilterOperators.Equal, Item.Id)
					),
					new BXOrderBy(
						new BXOrderByPair(BXSaleProduct.Fields.Id, BXOrderByDirection.Asc)
					),
					null,
					null,
					BXTextEncoder.EmptyTextEncoder
				);
			}
			else
				GoBack();
		}
		private void LoadItem()
		{
			Status.SelectedValue = Item.StatusId.ToString();
			if (Item.User != null)
			{
				User.HiddenValue = Item.UserId.ToString();
				User.TextBoxValue = string.Format(
					"{0} {1} ({2})",
					Item.User.FirstName,
					Item.User.LastName,
					Item.User.UserName
				);
			}
			else
			{
				User.HiddenValue = Item.UserId <= 0 ? "" : Item.UserId.ToString();
				User.TextBoxValue = "";
			}
			SumPaid.Text = Item.SumPaid.ToString();

			Cancelled.Checked = Item.Cancelled;
			ReasonCancelled.SelectedValue = Item.ReasonCancelled != null ? Item.ReasonCancelled.ToString() : "";

			Paid.Checked = Item.Paid;
			PaymentDocument.Text = Item.PaymentDocument;
			PaymentDocumentDate.Date = Item.PaymentDocumentDateUtc != DateTime.MinValue ? Item.PaymentDocumentDate : DateTime.MinValue;

			AllowShipping.Checked = Item.AllowShipping;
			ShippingDocument.Text = Item.ShippingDocument;
			ShippingDocumentDate.Date = Item.ShippingDocumentDateUtc != DateTime.MinValue ? Item.ShippingDocumentDate : DateTime.MinValue;

			CustomerComments.Text = Item.CustomerComments;
			EmployeeComments.Text = Item.EmployeeComments;
			AdditionalInfo.Text = Item.AdditionalInfo;
			Recalculate.Checked = Item.AutoRecalculate;
		}
		private void ReadProducts()
		{
			ProductInfos = new List<ProductInfo>();
			var idsString = Request.Form["productsIds"];
			if (string.IsNullOrEmpty(idsString))
				return;
			foreach (var id in idsString.Split('|'))
			{
				var prefix = "product_" + id + "_";
				ProductInfos.Add(new ProductInfo
				{
					Id = Request.Form[prefix + "id"],
					CartElementId = Request.Form[prefix + "cartelementid"] ?? "",
					CartElementCode = Request.Form[prefix + "cartelementcode"] ?? "",
					CartCatalogCode = Request.Form[prefix + "cartcatalogcode"] ?? "",
					Name = Request.Form[prefix + "name"] ?? "",
					Url = Request.Form[prefix + "url"] ?? "",
					Price = Request.Form[prefix + "price"] ?? "",
					Weight = Request.Form[prefix + "weight"] ?? "",
					Quantity = Request.Form[prefix + "quantity"] ?? "",
					Vat = Request.Form[prefix + "vat"] ?? "",
					VatIncluded = Request.Form[prefix + "vatincluded"] ?? ""
				});
			}
		}
		private void LoadProducts()
		{
			ProductInfos = new List<ProductInfo>();
			foreach (var p in Products)
			{
				ProductInfos.Add(new ProductInfo
				{
					Id = p.Id != 0 ? p.Id.ToString() : null,
					CartElementId = p.CartElementId.ToString(),
					CartElementCode = p.CartElementCode ?? "",
					CartCatalogCode = p.CartCatalogCode ?? "",
					Name = p.Name ?? "",
					Url = p.DetailUrl ?? "",
					Price = p.Price.ToString("F2"),
					Weight = p.Weight.ToString("N"),
					Quantity = p.Quantity.ToString(),
					Vat = p.VatRate.ToString("N"),
					VatIncluded = p.VatIncluded ? "Y" : ""
				});
			}
		}
		private bool SaveItem()
		{
			bool hasErrors = !IsValid;

			int id;
			if (BXStringUtility.IsNullOrTrimEmpty(User.TextBoxValue) || !int.TryParse(User.HiddenValue, out id))
				Item.UserId = 0;
			else
			{
				var user = Bitrix.Security.BXUser.GetById(id);
				if (user == null)
				{
					ErrorMessage.AddErrorMessage(GetMessageRaw("Error.UserNotFound"));
					hasErrors = true;
				}
				else
					Item.UserId = id;
			}

			if (!hasErrors)
				Item.SumPaid = BXStringUtility.IsNullOrTrimEmpty(SumPaid.Text) ? 0m : decimal.Parse(SumPaid.Text);

			if (!BXSite.GetAllSitesReadOnly().Any(x => x.TextEncoder.Decode(x.Id) == siteId))
			{
				ErrorMessage.AddErrorMessage(GetMessageRaw("Error.SiteNotFound"));
				hasErrors = true;
			}
			else
				Item.SiteId = siteId;

			if (currentCustomerType == null)
			{
				ErrorMessage.AddErrorMessage(GetMessageRaw("Error.CustomerTypeNotFound"));
				hasErrors = true;
			}
			else
				Item.CustomerTypeId = currentCustomerType.Id;

			Item.PaymentSystemId = paymentSystem != null ? paymentSystem.Id : 0;

			var props = new BXCustomPropertyCollection();
			foreach (var f in fields)
			{
				f.Errors = f.Errors ?? new List<string>();
				f.Editor.DoSave(f.FormFieldName, props, f.Errors);
				if (f.Errors.Count > 0)
				{
					hasErrors = true;
					foreach (var e in f.Errors)
						ErrorMessage.AddErrorMessage(e);
				}
				if (f.Field.UseAsPostalCode)
				{
					BXCustomProperty p;
					if (props.TryGetValue(f.Field.Code, out p) && p != null)
						postCode = p.Value != null ? p.Value.ToString() : null;
				}
			}
			fieldsLoaded = true;


			// deliveries should be calculated after properties
			InitDeliveries();
			if (Item.DeliveryVariant != deliveryVariant)
			{
				if (string.IsNullOrEmpty(deliveryVariant))
				{
					Item.DeliveryVariant = null;
					if (Recalculate.Checked)
						Item.DeliveryCost = 0m;
				}
				else
				{
					bool found = false;
					foreach (var d in Deliveries)
					{
						d.Context.Response.ClearVariants();
						d.Process();
						foreach (var v in d.Context.Response.Variants)
						{
							if (v.Id != deliveryVariant)
								continue;

							found = true;
							Item.DeliveryVariant = v.Id;
							if (Recalculate.Checked)
								Item.DeliveryCost = BXCurrency.ConvertMoney(v.Cost, v.Currency, Item.Currency);							
							break;
						}
						if (found)
							break;
					}
				}
			}

			if (!Recalculate.Checked)
			{
				var costString = Request.Form["deliveryCost"];
				decimal cost;
				if (BXStringUtility.IsNullOrTrimEmpty(costString))
					Item.DeliveryCost = 0m;
				else if (decimal.TryParse(costString.Trim(), out cost))
					Item.DeliveryCost = cost;
				else 
				{
					hasErrors = true;
					ErrorMessage.AddErrorText(GetMessageRaw("Error.DeliveryCostIncorrect"));
				}
			}


			if (BXUser.IsCanOperate(BXSaleModule.Operations.OrderCancel, BXSaleModuleConfiguration.Id, Item.StatusId))
			{
				Item.Cancelled = Cancelled.Checked;
				if (Item.Cancelled)
				{
					try
					{
						Item.ReasonCancelled = (BXSaleOrderCancelReason)Enum.Parse(typeof(BXSaleOrderCancelReason), ReasonCancelled.SelectedValue);
					}
					catch
					{
						Item.ReasonCancelled = null;
					}
				}
			}

			if (BXUser.IsCanOperate(BXSaleModule.Operations.OrderPay, BXSaleModuleConfiguration.Id, Item.StatusId))
			{
				Item.Paid = Paid.Checked;
				Item.PaymentDocument = PaymentDocument.Text;
				Item.PaymentDocumentDate = PaymentDocumentDate.Date;
			}

			if (BXUser.IsCanOperate(BXSaleModule.Operations.OrderAllowShipping, BXSaleModuleConfiguration.Id, Item.StatusId))
			{
				Item.AllowShipping = AllowShipping.Checked;
				Item.ShippingDocument = ShippingDocument.Text;
				Item.ShippingDocumentDate = ShippingDocumentDate.Date;
			}


			// we should change the status last of all (because of security checks depending on status)
			if (!string.IsNullOrEmpty(Status.SelectedValue)
				&& int.TryParse(Status.SelectedValue, out id)
				&& id != Item.StatusId
				&& BXUser.IsCanOperate(BXSaleModule.Operations.OrderExitStatus, BXSaleModuleConfiguration.Id, Item.StatusId)
				&& BXUser.IsCanOperate(BXSaleModule.Operations.OrderEnterStatus, BXSaleModuleConfiguration.Id, id)
				&& Statuses.Exists(x => x.Id == id)
			)
				Item.StatusId = id;

			Item.CustomerComments = CustomerComments.Text;
			Item.EmployeeComments = EmployeeComments.Text;
			Item.AdditionalInfo = AdditionalInfo.Text;
			Item.AutoRecalculate = Recalculate.Checked;

			var data = new List<ProductData>();
			foreach (var info in ProductInfos)
			{
				var d = new ProductData();
				int t1;

				if (info.Id != null)
				{
					if (!int.TryParse(info.Id.Trim(), out t1))
					{
						ErrorMessage.AddErrorText(string.Format(GetMessageRaw("Error.ItemIdIncorrect")));
						hasErrors = true;
					}
					else 
						d.Id = t1;
				}

				if (BXStringUtility.IsNullOrTrimEmpty(info.CartElementId) || !int.TryParse(info.CartElementId.Trim(), out t1))
				{
					ErrorMessage.AddErrorText(string.Format(GetMessageRaw("Error.ItemElementIdIncorrect")));
					hasErrors = true;
				}
				else 
					d.CartElementId = t1;

				decimal t2;
				if (BXStringUtility.IsNullOrTrimEmpty(info.Price) || !decimal.TryParse(info.Price.Trim(), out t2))
				{
					ErrorMessage.AddErrorText(string.Format(GetMessageRaw("Error.ItemPriceIncorrect")));
					hasErrors = true;
				}
				else 
					d.Price = t2;
				
				if (!BXStringUtility.IsNullOrTrimEmpty(info.Weight))
				{
					if (!decimal.TryParse(info.Weight.Trim(), out t2))
					{
						ErrorMessage.AddErrorText(string.Format(GetMessageRaw("Error.ItemPriceNegative")));
						hasErrors = true;
					}
					else if (t2 < 0m)
					{
						ErrorMessage.AddErrorText(string.Format(GetMessageRaw("Error.ItemWeightNegative")));
						hasErrors = true;
					}
					else 
						d.Weight = t2;
				}

				if (!BXStringUtility.IsNullOrTrimEmpty(info.Vat))
				{
					if (!decimal.TryParse(info.Vat.Trim(), out t2))
					{
						ErrorMessage.AddErrorText(string.Format(GetMessageRaw("Error.ItemVatIncorrect")));
						hasErrors = true;
					}
					else if (t2 < 0m)
					{
						ErrorMessage.AddErrorText(string.Format(GetMessageRaw("Error.ItemVatNegative")));
						hasErrors = true;
					}
					else 
						d.Vat = t2;
				}

				if (BXStringUtility.IsNullOrTrimEmpty(info.Quantity))
				{
					ErrorMessage.AddErrorText(string.Format(GetMessageRaw("Error.ItemQuantityRequired")));
					hasErrors = true;
				}
				if (!int.TryParse(info.Quantity.Trim(), out t1))
				{
					ErrorMessage.AddErrorText(string.Format(GetMessageRaw("Error.ItemQuantityIncorrect")));
					hasErrors = true;
				}
				else if (t1 < 1)
				{
					ErrorMessage.AddErrorText(string.Format(GetMessageRaw("Error.ItemQuantityNotPositive")));
					hasErrors = true;
				}
				else 
					d.Quantity = t1;

				d.CartElementCode = info.CartElementCode;
				d.CartCatalogCode = info.CartCatalogCode;
				d.Name = info.Name;
				d.Url = info.Url;
				d.VatIncluded = info.VatIncluded == "Y";	
			
				data.Add(d);
			}
			
			if (hasErrors)
				return false;


			List<BXSaleProduct> products = new List<BXSaleProduct>(Products);
			List<BXSaleProduct> productsToSave = new List<BXSaleProduct>();
			foreach (var d in data)
			{
				int i;
				BXSaleProduct p;
				if (d.Id != null && (i = products.FindIndex(x => x.Id == d.Id.Value)) >= 0)
				{
					p = products[i];
					products.RemoveAt(i);
				}
				else 
				{
					p = new BXSaleProduct(BXTextEncoder.EmptyTextEncoder)
					{
						CartResolverId = BXCatalogSaleCartItemResolver.ResolverId,
						OrderId = Item.Id,
						DiscountName = "",
						DiscountValue = 0m
					};
				}

				p.CartElementId = d.CartElementId;
				p.CartElementCode = d.CartElementCode;
				p.CartCatalogCode = d.CartCatalogCode;
				p.Name = d.Name;
				p.DetailUrl = d.Url;
				p.Price = d.Price;
				p.Weight = d.Weight;
				p.Quantity = d.Quantity;
				p.VatRate = d.Vat;
				p.VatIncluded = d.VatIncluded;
				productsToSave.Add(p);
			}

			try
			{
				var sum = 0m;
				foreach (var p in productsToSave)
					sum += p.GetTotalCost();
				sum += Item.DeliveryCost;
				Item.Sum = sum;
				
				using (var tran = new BXSqlTransaction())
				{
					Item.Save(tran.Connection, tran);
					BXSaleOrderValueManager.SetValues(Item.Id, fieldCollection, props, tran);
					foreach(var p in products)
						p.Delete(tran.Connection, tran);
					foreach(var p in productsToSave)
						p.Save(tran.Connection, tran);
					tran.Commit();
				}
			}
			catch (Exception ex)
			{
				ErrorMessage.AddErrorText(ex.Message);
				return false;
			}

			return true;
		}

		private void InitDeliveries()
		{
			if (deliveriesInitialized)
				return;

			if (string.IsNullOrEmpty(Request.Form["destinationCountryCode"]) || !int.TryParse(Request.Form["destinationCountryCode"], out destinationCountryCode))
				destinationCountryCode = BXSale.Current.CountryNumericCode;

			var context = new BXSaleDeliveryContext();
			context.SiteId = siteId;
			context.Request.Cost = Item.Sum - Item.DeliveryCost;
			context.Request.Currency = Item.Currency;
			context.Request.Source.CountryCode = Bitrix.Services.Location.BXCountry.GetByCode(BXSale.Current.CountryNumericCode).Iso3166AlphaCode;
			context.Request.Source.PostalCode = BXSale.Current.PostalCode;
			context.Request.Destination.CountryCode = Bitrix.Services.Location.BXCountry.GetByCode(destinationCountryCode).Iso3166AlphaCode;
			context.Request.Destination.PostalCode = postCode;
			context.IgnoreRestrictions = true;

			decimal weight = 0m;
			foreach(var p in ProductInfos)
			{
				decimal unitWeight;
				int quantity;
			  

				if (BXStringUtility.IsNullOrTrimEmpty(p.Weight) || !decimal.TryParse(p.Weight.Trim(), out unitWeight) || unitWeight <= 0m)
					continue;
					
				if (BXStringUtility.IsNullOrTrimEmpty(p.Quantity) || !int.TryParse(p.Quantity.Trim(), out quantity) || quantity < 1)
					continue;

				  weight += unitWeight * quantity;
			}
			context.Request.WeightInGrams = (int)weight;

			Deliveries = new List<BXSaleDeliveryServiceInfo>(BXSaleDeliveryManager.GetDeliveryServiceInfos(context));
			deliveriesInitialized = true;

			if (IsPostBack)
				deliveryVariant = Request.Form["deliveries"];
			else
				deliveryVariant = Item.DeliveryVariant;
		}
		private void InitPaymentSystems()
		{
			PaymentSystems = BXSalePaymentSystem.GetList(
				new BXFilter(
					new BXFilterItem(BXSalePaymentSystem.Fields.SiteId, BXSqlFilterOperators.Equal, siteId),
					new BXFilterOr(
						new BXFilterItem(BXSalePaymentSystem.Fields.Active, BXSqlFilterOperators.Equal, true),
						new BXFilterItem(BXSalePaymentSystem.Fields.Id, BXSqlFilterOperators.Equal, Item.PaymentSystemId)
					)
				),
				new BXOrderBy(
					new BXOrderByPair(BXSalePaymentSystem.Fields.Sort, BXOrderByDirection.Asc),
					new BXOrderByPair(BXSalePaymentSystem.Fields.Id, BXOrderByDirection.Asc)
				),
				null,
				null,
				BXTextEncoder.EmptyTextEncoder
			);

			if (IsPostBack)
			{
				int id;
				var val = Request.Form["paymentSystem"];
				if (!string.IsNullOrEmpty(val) && int.TryParse(val, out id))
					paymentSystem = PaymentSystems.Find(x => x.Id == id);
			}
			else
			{
				paymentSystem = PaymentSystems.Find(x => x.Id == Item.PaymentSystemId);
			}
		}
		private void InitProperties()
		{
			customerTypes = BXSaleCustomerType.GetList(
				new BXFilter(new BXFilterItem(BXSaleCustomerType.Fields.Site.ID, BXSqlFilterOperators.Equal, siteId)),
				new BXOrderBy(new BXOrderByPair(BXSaleCustomerType.Fields.Sort, BXOrderByDirection.Asc)),
				null,
				null,
				BXTextEncoder.EmptyTextEncoder
			);

			if (IsPostBack)
			{
				int id;
				var val = Request.Form["customerType"];
				if (!string.IsNullOrEmpty(val) && int.TryParse(val, out id))
					currentCustomerType = customerTypes.Find(x => x.Id == id);
			}
			else
			{
				currentCustomerType = customerTypes.Find(x => x.Id == Item.CustomerTypeId);
			}
			if (currentCustomerType == null && customerTypes.Count > 0)
				currentCustomerType = customerTypes[0];
		}
		private void BuildFields()
		{
			fieldCollection = currentCustomerType != null ? BXSaleOrderField.GetCustomerFields(currentCustomerType.Id, null) : new BXSaleOrderFieldCollection();
			fields = fieldCollection.ConvertAll(x => new FieldInfo { Field = x });

			foreach (var f in fields)
			{
				f.Adapter = f.Field.GetCustomFieldAdapter();
				f.Editor = f.Field.CustomType.CreatePublicEditor();
				f.Editor.Init(f.Adapter);
				f.FormFieldName = "property$" + f.Field.Code;
				f.ClientId = "property_" + f.Field.Code.ToLowerInvariant();
			}
		}
		private bool CheckAjax()
		{
			if (Request.QueryString["action"] != "delivery")
				return false;

			var data = new List<Dictionary<string, string>>();

			var context = new BXSaleDeliveryContext();
			context.SiteId = Request.QueryString["site"];
			context.Request.Cost = Item.Sum - Item.DeliveryCost;
			context.Request.Currency = Item.Currency;
			context.Request.Source.CountryCode = Bitrix.Services.Location.BXCountry.GetByCode(BXSale.Current.CountryNumericCode).Iso3166AlphaCode;
			context.Request.Source.PostalCode = BXSale.Current.PostalCode;
			context.Request.Destination.CountryCode = Bitrix.Services.Location.BXCountry.GetByCode(int.Parse(Request.QueryString["country"])).Iso3166AlphaCode;
			context.Request.Destination.PostalCode = Request.QueryString["postalcode"];
			context.IgnoreRestrictions = true;

			double weight;
			if (!string.IsNullOrEmpty(Request.QueryString["weight"]) && double.TryParse(Request.QueryString["weight"], NumberStyles.Float, CultureInfo.InvariantCulture, out weight))
				context.Request.WeightInGrams = (int)weight;
			
			foreach (var d in BXSaleDeliveryManager.GetDeliveryServiceInfos(context))
			{
				context.Response.ClearVariants();
				d.Process();
				foreach (var v in context.Response.Variants)
				{
					var item = new Dictionary<string, string>
						{
							{ "value", v.Id },
							{ "text", v.Title },
							{ "cost", BXCurrency.ConvertMoney(v.Cost, v.Currency, Item.Currency).ToString("F2") }
						};
					data.Add(item);
				}
			}

			Response.Write(BXJSUtility.BuildJSArray(data));
			Response.End();
			return true;

		}

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			InitItem();

			if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderView, BXSaleModuleConfiguration.Id, Item.StatusId))
				BXAuthentication.AuthenticationRequired();
			if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderManage, BXSaleModuleConfiguration.Id, Item.StatusId))
				BXAuthentication.AuthenticationRequired();

			if (CheckAjax())
				return;

			Statuses = BXSaleOrderStatus.GetList(
				null,
				new BXOrderBy(new BXOrderByPair(BXSaleOrderStatus.Fields.Sort, BXOrderByDirection.Asc)),
				null,
				null,
				BXTextEncoder.EmptyTextEncoder
			);

			InitSites();
			InitStatuses();
			InitReasonsCancelled();
			InitProperties();
			InitPaymentSystems();
			//InitDeliveries();
			BuildFields();
			if (!IsPostBack)
			{
				LoadItem();
				LoadProducts();
			}
			else 
				ReadProducts();
		}
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);

			Title = string.Format(GetMessageRaw("PageTitle"), Item.Id);
			MasterTitle = string.Format(GetMessageRaw("Heading.Order"), Item.Id, Item.Created.ToString("g"));

			PrepareResultMessage();

			User.AttachScript();

			ViewButton.Href = string.Concat("SaleOrderView.aspx?id=", Item.Id);
					

			if (!fieldsLoaded && !IsPostBack)
			{
				var values = BXSaleOrderValueManager.GetValues(Item.Id, fieldCollection, null);
				foreach (var f in fields)
				{
					BXCustomProperty p;
					if (!values.TryGetValue(f.Field.Code, out p))
						p = new BXCustomProperty(f.Field.Code, f.Field.Id, f.Field.CustomType.DbType, f.Field.Multiple, null);

					if (f.Field.UseAsPostalCode)
						postCode = p.Value != null ? p.Value.ToString() : null;

					f.Editor.Load(p);
				}
				fieldsLoaded = true;
			}

			if (!fieldsLoaded && IsPostBack)
			{
				foreach (var f in fields)
				{
					f.Errors = new List<string>();
					f.Editor.DoValidate(f.FormFieldName, f.Errors);
					if (f.Field.UseAsPostalCode)
					{
						var props = new BXCustomPropertyCollection();
						f.Editor.DoSave(f.FormFieldName, props, f.Errors);
						BXCustomProperty p;
						if (props.TryGetValue(f.Field.Code, out p) && p != null)
							postCode = p.Value != null ? p.Value.ToString() : null;
					}
				}

				fieldsLoaded = true;
			}
			InitDeliveries();


			BXPage.RegisterScriptInclude("~/bitrix/js/Main/Core/core.js");
			BXPage.RegisterScriptInclude("~/bitrix/js/Main/Core/core_ajax.js");
		}
	
		protected void ActionBar_CommandClick(object sender, CommandEventArgs e)
		{
			if (e.CommandName != "delete")
				return;

			if (Item == null)
				return;

			bool success = false;
			try
			{
				Item.Delete();
				success = true;
			}
			catch (BXEventException ex)
			{
				foreach (string s in ex.Messages)
					ShowErrorText(s);
			}
			catch
			{
				ErrorMessage.AddErrorMessage(GetMessageRaw("Error.Delete"));
			}

			if (success)
				GoBack();
		}
		protected void Order_Save(object sender, EventArgs e)
		{
			if (!SaveItem())
				return;
			GoBack();
		}
		protected void Order_Apply(object sender, EventArgs e)
		{
			if (!SaveItem())
				return;
			Reload(TabControl.SelectedIndex);
		}
		protected void Order_Cancel(object sender, EventArgs e)
		{
			GoBack();
		}

		protected List<CustomProperty> GetCustomProperties()
		{
			var result = new List<CustomProperty>();
			var fields = BXSaleOrderField.GetCustomerFields(Item.CustomerTypeId, null);
			var values = BXSaleOrderValueManager.GetValues(Item.Id, fields, null);

			foreach (var v in values.Values)
			{
				var f = fields[v.Name];
				if (f != null)
				{
					var view = f.CustomType.CreatePublicView();
					view.Init(v, f.GetCustomFieldAdapter());
					result.Add(new CustomProperty
					{
						TitleHtml = f.Name,
						ValueHtml = view.GetHtml("<br/>"),
						Sort = f.Sort
					});
				}
				else
				{
					result.Add(new CustomProperty
					{
						TitleHtml = Encode(v.UserLikeName),
						ValueHtml = string.Join("<br/>", v.Values.ConvertAll(x => Encode(x.ToString())).ToArray()),
						Sort = int.MaxValue
					});
				}
			}
			result.Sort((x, y) => x.Sort - y.Sort);
			return result;
		}
		protected string Hidden(string name, string value)
		{
			return string.Concat(
				@"<input type=""hidden"" name=""", 
				HttpUtility.HtmlAttributeEncode(name ?? ""),
				@""" id=""", 
				HttpUtility.HtmlAttributeEncode(name ?? ""),
				@""" value=""", 
				HttpUtility.HtmlAttributeEncode(value ?? ""),
				@""" />"
			);
		}

		protected class CustomProperty
		{
			public string TitleHtml { get; set; }
			public string ValueHtml { get; set; }
			public int Sort { get; set; }
		}
		protected class FieldInfo
		{
			public BXSaleOrderField Field;
			public BXCustomField Adapter;
			public BXCustomTypePublicEdit Editor;
			public string FormFieldName;
			public string ClientId;
			public List<string> Errors;
			public bool Valid = true;
		}
		protected class ProductInfo
		{
			public string Id;
			public string CartElementId;
			public string CartElementCode;
			public string CartCatalogCode;
			public string Name;
			public string Url;
			public string Price;
			public string Weight;
			public string Quantity;
			public string Vat;
			public string VatIncluded;
		}
		protected class ProductData
		{
			public int? Id;
			public int CartElementId;
			public string CartElementCode;
			public string CartCatalogCode;
			public string Name;
			public string Url;
			public decimal Price;
			public decimal Weight;
			public int Quantity;
			public decimal Vat;
			public bool VatIncluded;
		}



		#region IPostBackEventHandler Members

		public void RaisePostBackEvent(string eventArgument)
		{
			if (eventArgument == "change_customer_type" || eventArgument == "change_site")
			{
				var values = BXSaleOrderValueManager.GetValues(Item.Id, fieldCollection, null);
				foreach (var f in fields)
				{
					BXCustomProperty p;
					if (!values.TryGetValue(f.Field.Code, out p))
						p = new BXCustomProperty(f.Field.Code, f.Field.Id, f.Field.CustomType.DbType, f.Field.Multiple, null);

					if (f.Field.UseAsPostalCode)
						postCode = p.Value != null ? p.Value.ToString() : null;

					f.Editor.Load(p);
				}
				fieldsLoaded = true;
			}
		}

		#endregion
	}
}