﻿using Bitrix.UI;
using Bitrix.Security;
using System.Security;
using Bitrix.Services.Text;
using System;
using System.Linq;
using Bitrix.Modules;
using System.Web.UI.WebControls;
using System.Web.UI;

namespace Bitrix.Sale.AdminPages
{
	public partial class OrderFieldEdit : BXAdminPage
	{
		#region Messages
		int showMessage;

		bool HasErrors { get { return showMessage == -1; } }

		void PrepareResultMessage()
		{
			SuccessMessage.Visible = showMessage > 0;
		}
		void ShowErrorHtml(string html)
		{
			showMessage = -1;
			ErrorMessage.AddErrorMessage(html);
		}
		void ShowErrorText(string text)
		{
			ShowErrorHtml(Encode(text));
		}
		void ShowError(Exception ex)
		{
			showMessage = -1;
			ErrorMessage.AddErrorMessage(Encode(ex.Message));
		}
		void ShowOk()
		{
			if (showMessage == 0)
				showMessage = 1;
		}
		#endregion

		protected BXCustomField fieldAdapter;
		protected BXCustomType customType;
		protected IBXCustomTypeSetting settings;
		protected IBXCustomTypeAdvancedSetting advancedSettings;

		public bool CanManage { get; set; }
		public BXSaleOrderField Item { get; set; }
		protected override string BackUrl
		{
			get
			{
				return base.BackUrl ?? "SaleOrderFieldList.aspx";
			}
		}

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			if (!BXUser.IsCanOperate(BXSaleModule.Operations.AdminManagement))
				BXAuthentication.AuthenticationRequired();

			var idString = Request.QueryString["id"];
			if (idString != null)
			{
				int id;
				if (!int.TryParse(idString, out id) || id <= 0 || (Item = BXSaleOrderField.GetById(id, BXTextEncoder.EmptyTextEncoder)) == null)
					Response.Redirect("SaleOrderFieldList.aspx"); // не удалось найти такой объект
				fieldAdapter = Item.GetCustomFieldAdapter();
				customType = Item.CustomType;
				
				Control control = null;
				try
				{
					control = customType.Settings;
				}
				catch
				{
				}
				if (control != null && (settings = control as IBXCustomTypeSetting) != null)
				{
					settings.ValidationGroup = TabControl.ValidationGroup;
					settings.SetSettings(fieldAdapter.Settings);
					SettingsPlaceholder.Controls.Add(control);
				}

				control = null;
				try
				{
					control = customType.AdvancedSettings;
				}
				catch
				{
				}
				if (control != null && (advancedSettings = control as IBXCustomTypeAdvancedSetting) != null)
				{
					advancedSettings.ValidationGroup = TabControl.ValidationGroup;
					advancedSettings.Initialize(fieldAdapter);
					//advancedSettings.SetSettings(Item.Settings);
					AdvancedSettingsPlaceholder.Controls.Add(control);
				}

				DoLoad();
			}
			else
			{
				CustomerType.Items.AddRange(
					(
						from ct in BXSaleCustomerType.GetList(null, null, null, null, BXTextEncoder.EmptyTextEncoder)
						select new ListItem(ct.Name, ct.Id.ToString())
					)
					.ToArray()
				);
				CustomType.Items.AddRange(
					(
						from ct in BXCustomTypeManager.CustomTypes.Values
						where !IsObsolete(ct)
						orderby ct.Title
						select new ListItem(ct.Title, ct.TypeName) { Selected = string.Equals(ct.TypeName, "Bitrix.System.Text", StringComparison.OrdinalIgnoreCase) }
					)
					.ToArray()
				);
				Sort.Text = "100";
			}

			CanManage = true;
		}
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);

			Title = GetMessageRaw("PageTitle");
			MasterTitle = Item != null ? GetMessageRaw("MasterTitle.Edit") : GetMessageRaw("MasterTitle.New");

			PrepareResultMessage();

			TabControl.ShowSaveButton = CanManage;
			TabControl.ShowApplyButton = CanManage;

			AddButton.Visible = CanManage && Item != null;
			DeleteButton.Visible = CanManage && Item != null;
		}

		void DoLoad()
		{
			Code.Text = Item.Code;
			Name.Text = Item.Name;
			Required.Checked = Item.Required;
			Multiple.Checked = Item.Multiple;
			Sort.Text = Item.Sort.ToString();
			UseAsPostalCode.Checked = Item.UseAsPostalCode;
			UseAsLocation.Checked = Item.UseAsLocation;
			UseAsProfileName.Checked = Item.UseAsProfileName;
			UseAsEmail.Checked = Item.UseAsEmail;
			ShowInFilter.Checked = Item.ShowInFilter;
		}
		void DoSave()
		{
			if (!Page.IsValid)
			{
				showMessage = -1;
				return;
			}
			try
			{
				if (!CanManage)
					throw new SecurityException(GetMessageRaw("Message.InsufficientRightsEdit"));

				var item = Item ?? new BXSaleOrderField(BXTextEncoder.EmptyTextEncoder);
				if (item.IsNew)
				{
					int customerTypeId;
					if (!int.TryParse(CustomerType.SelectedValue, out customerTypeId) || BXSaleCustomerType.GetById(customerTypeId) == null)
						throw new Exception(GetMessageRaw("Message.CustomerTypeNotFound"));

					BXCustomType type;
					if (!BXCustomTypeManager.CustomTypes.TryGetValue(CustomType.SelectedValue, out type))
						throw new Exception(GetMessageRaw("Message.FieldTypeNotFound"));
					
					item.CustomerTypeId = customerTypeId;
					item.CustomTypeId = type.TypeName;
				}

				int sort;
				if (!int.TryParse(Sort.Text.Trim(), out sort))
					sort = 100;

				item.Code = Code.Text.Trim().ToUpperInvariant();
				item.Name = Name.Text.Trim();
				item.Required = Required.Checked;
				item.Multiple = Multiple.Checked;
				item.Sort = sort;
				if (settings != null)
					item.Settings.Assign(settings.GetSettings());
				
				item.UseAsPostalCode = UseAsPostalCode.Checked;
				item.UseAsLocation = UseAsLocation.Checked;
				item.UseAsProfileName = UseAsProfileName.Checked;
				item.UseAsEmail = UseAsEmail.Checked;
				item.ShowInFilter = ShowInFilter.Checked;

				item.Save();

				if (advancedSettings != null)
					advancedSettings.Save();

				Item = item;
				ShowOk();
			}
			catch (Exception ex)
			{
				ShowError(ex);
			}
		}

		protected void TabControl_Command(object sender, BXTabControlCommandEventArgs e)
		{
			if (e.CommandName == "cancel")
				GoBack();

			DoSave();
			if (HasErrors)
				return;

			if (e.CommandName == "save")
				GoBack();
			else
				Response.Redirect("SaleOrderFieldEdit.aspx?id=" + Item.Id.ToString());
		}
		protected void ActionBar_CommandClick(object sender, CommandEventArgs e)
		{
			if (e.CommandName != "delete")
				return;

			if (Item == null)
				return;

			if (!CanManage)
			{
				ShowErrorText(GetMessageRaw("Message.InsufficientRightsDelete"));
				return;
			}

			bool success = false;
			try
			{
				Item.Delete();
				success = true;
			}
			catch (BXEventException ex)
			{
				foreach (string s in ex.Messages)
					ShowErrorText(s);
			}
			catch
			{
				ErrorMessage.AddErrorMessage(GetMessageRaw("Message.ErrorDelete"));
			}

			if (success)
				GoBack();
		}

		bool IsObsolete(BXCustomType type)
		{
			for (Type t = type.GetType(); t != typeof(BXCustomType) && t != null; t = t.BaseType)
			{
				object[] attributes = t.GetCustomAttributes(typeof(ObsoleteAttribute), false);
				if (attributes != null && attributes.Length != 0)
					return true;
			}
			return false;
		}
	}
}