﻿using System;
using System.Linq;
using System.Web.UI.WebControls;
using Bitrix.DataLayer;
using Bitrix.Sale.SaleCart;
using Bitrix.Security;
using Bitrix.Services;
using Bitrix.Services.Text;
using Bitrix.UI;

namespace Bitrix.Sale.AdminPages
{
	public partial class OrderFieldList : BXAdminPage
	{
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
				BXAuthentication.AuthenticationRequired();

			CustomerTypeFilter.Values.AddRange(
				(
					from ct in BXSaleCustomerType.GetList(null, null, null, null, BXTextEncoder.EmptyTextEncoder)
					select new ListItem(string.Format("[{0}] {1} ({2})", ct.Id.ToString(), ct.Name, ct.SiteId), ct.Id.ToString())
				)
				.ToArray()
			);
			CustomTypeFilter.Values.AddRange(
				(
					from ct in BXCustomTypeManager.CustomTypes.Values
					where !IsObsolete(ct)
					select new ListItem(ct.Title, ct.TypeName)
				)
				.ToArray()
			);
		}
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);

			Title = GetMessageRaw("PageTitle");
			MasterTitle = GetMessageRaw("MasterTitle");
		}

		protected void GridView_Select(object sender, BXSelectEventArgs e)
		{
			var filter = new BXFilter(AdminFilter.CurrentFilter, BXSaleOrderField.Fields);
			var orderBy = new BXOrderBy(BXSaleOrderField.Fields, e.SortExpression);
			var select = new BXSelectAdd(
				BXSaleOrderField.Fields.CustomerType.Name,
				BXSaleOrderField.Fields.CustomerType.Site.ID
			);

			e.Data = BXSaleOrderField.GetList(filter, orderBy, select, new BXQueryParams(e.PagingOptions), BXTextEncoder.EmptyTextEncoder);
		}
		protected void GridView_SelectCount(object sender, BXSelectCountEventArgs e)
		{
			var filter = new BXFilter(AdminFilter.CurrentFilter, BXSaleOrderField.Fields);
			e.Count = BXSaleOrderField.Count(filter);
		}
		protected void GridView_Delete(object sender, BXDeleteEventArgs e)
		{
			try
			{
				var filter =
					(e.Keys == null)
					? new BXFilter(AdminFilter.CurrentFilter, BXSaleOrderField.Fields)
					: new BXFilter(new BXFilterItem(BXSaleOrderField.Fields.Id, BXSqlFilterOperators.Equal, e.Keys["Id"]));

				var items = BXSaleOrderField.GetList(filter, null);
				foreach (var item in items)
				{
					try
					{
						item.Delete();
						e.DeletedCount++;
					}
					catch (Exception ex)
					{
						ErrorMessage.AddErrorMessage(string.Format(GetMessageRaw("Error.Delete"), item.Id));
						BXLogService.LogAll(ex, BXLogMessageType.Error, "SaleOrderFieldList.aspx, Delete");
					}
				}
			}
			catch (Exception ex)
			{
				ErrorMessage.AddErrorMessage(ex.Message);
				return;
			}
		}

		protected void GridView_RowDataBound(object sender, GridViewRowEventArgs e)
		{
			if (e.Row.RowType != DataControlRowType.DataRow)
				return;

			var row = (BXGridViewRow)e.Row;
			var item = (BXSaleOrderField)row.DataItem;
			row.UserData.Add("Id", item.Id);
		}

		bool IsObsolete(BXCustomType type)
		{
			for (Type t = type.GetType(); t != typeof(BXCustomType) && t != null; t = t.BaseType)
			{
				object[] attributes = t.GetCustomAttributes(typeof(ObsoleteAttribute), false);
				if (attributes != null && attributes.Length != 0)
					return true;
			}
			return false;
		}
	}
}