﻿using System;
using System.Linq;
using System.Web.UI.WebControls;
using Bitrix.DataLayer;
using Bitrix.Sale.SaleCart;
using Bitrix.Security;
using Bitrix.Services;
using Bitrix.Services.Text;
using Bitrix.UI;
using System.Web;
using System.Text;
using System.Collections.Generic;
using Bitrix.Currency;
using System.Data;
using System.Web.UI;

namespace Bitrix.Sale.AdminPages
{
	public partial class OrderList : BXAdminPage
	{
		private Dictionary<int, BXSaleProductCollection> products = new Dictionary<int, BXSaleProductCollection>();
		string[] viewOperations = new[] { "view" };
		string[] editOperations = new[] { "view", "edit", "delete" };
		Dictionary<int, BXSaleOrderFieldCollection> orderFields;

		private void LoadFields()
		{
			orderFields = new Dictionary<int, BXSaleOrderFieldCollection>();
			foreach (var site in BXSite.GetAllSitesReadOnly())
				foreach (var ct in BXSaleCustomerType.GetAllForSite(site.TextEncoder.Decode(site.Id)))
					orderFields[ct.Id] = BXSaleOrderField.GetCustomerFields(ct.Id, null);
		}
		private void AddCustomColumns()
		{
			var used = new List<string>();

			foreach (var fields in orderFields.Values)
			{
				foreach (var f in fields)
				{
					if (used.Contains(f.Code))
						continue;
					used.Add(f.Code);

					var column = new TemplateField();
					column.Visible = false;
					column.HeaderText = f.TextEncoder.Decode(f.Name);
					column.ItemTemplate = new ValueTemplate(f.Code);
					GridView.Columns.Add(column);
				}
			}
		}
		private void AddCustomFilters()
		{
			var filtered = new List<BXSaleOrderField>();

			foreach (var c in orderFields.Values)
				foreach (var f in c)
				{
					if (f.ShowInFilter && !filtered.Exists(x => x.Code == f.Code))
						filtered.Add(f);
				}

			foreach (var field in filtered)
			{
				var type = field.CustomType;
				if (type == null)
					continue;

				var f = field.GetCustomFieldAdapter();
				BXAdminFilterBaseItem control;
				try
				{
					control = type.GetFilter(f) as BXAdminFilterBaseItem;
				}
				catch
				{
					continue;
				}
				if (control == null)
					continue;
				control.Text = field.TextEncoder.Decode(field.Name);
				AdminFilter.Items.Add(control);
			}
		}
		private BXFilter MakeFilter()
		{
			var filter = new BXFormFilter(AdminFilter.CurrentFilter);
			var specials = new List<BXFormFilterItem>();
			for (int i = filter.Count - 1; i >= 0; i--)
			{
				if (filter[i].filterName.StartsWith("@"))
				{
					specials.Add(filter[i]);
					filter.RemoveAt(i);
				}
			}

			var result = new BXFilter(filter, BXSaleOrder.Fields);
			foreach (var s in specials)
			{
				int i = s.filterName.LastIndexOf(':');
				int start = i < 0 ? 1 : i + 1;
				if (start >= s.filterName.Length - 1)
					continue;
				var code = s.filterName.Substring(start);
				result.Add(new BXFilterItem(BXSaleOrder.Fields.Values[code], s.filterOperator, s.filterValue));
			}

			if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderView))
			{
				var idStrings = BXUser.GetOperatableExternalIds(new[] { BXSaleModule.Operations.OrderView });

				var ids = new List<int>(idStrings.Length);
				int id;
				foreach (var s in idStrings)
				{
					if (int.TryParse(s, out id))
						ids.Add(id);
				}
				result.Add(new BXFilterItem(BXSaleOrder.Fields.StatusId, BXSqlFilterOperators.In, ids));
			}

			return result;
		}

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			
			AddButton.Visible = false;//CanManage;
			//MultiActionMenuToolbar.Visible = CanManage;

			SiteFilter.Items.Clear();
			SiteFilter.Items.Add(new ListItem(GetMessageRaw("Kernel.All"), ""));
			SiteFilter.Items.AddRange(
				BXSite.GetAllSitesReadOnly()
				.Where(x => x.Active)
				.Select(x => new ListItem(x.TextEncoder.Decode(x.Name), x.TextEncoder.Decode(x.Id)))
				.ToArray()
			);

			CurrencyFilter.Items.Clear();
			CurrencyFilter.Items.Add(new ListItem(GetMessageRaw("Kernel.All"), ""));
			CurrencyFilter.Items.AddRange(
				BXCurrency.GetList(
					null,
				new BXOrderBy(
					new BXOrderByPair(BXCurrency.Fields.Sort, BXOrderByDirection.Asc),
					new BXOrderByPair(BXCurrency.Fields.Id, BXOrderByDirection.Asc)
					),					
					new BXSelect(BXCurrency.Fields.Id, BXCurrency.Fields.LocalizationInfos),
					null,
					BXTextEncoder.EmptyTextEncoder
				)
				.Select(x => new ListItem(x.GetName(), x.Id.ToString()))
				.ToArray()
			);

			StatusFilter.Values.Clear();
			StatusFilter.Values.AddRange(
				BXSaleOrderStatus.GetList(
					null,
					new BXOrderBy(new BXOrderByPair(BXSaleOrderStatus.Fields.Sort, BXOrderByDirection.Asc)),
					new BXSelect(BXSaleOrderStatus.Fields.Id, BXSaleOrderStatus.Fields.XmlId, BXSaleOrderStatus.Fields.Localization),
					null,
					BXTextEncoder.EmptyTextEncoder
				)
				.Select(x => new ListItem(string.Format("[{0}] {1}", x.XmlId, x.LocalizedName), x.Id.ToString()))
				.ToArray()
			);

			CustomerTypeFilter.Values.Clear();
			CustomerTypeFilter.Values.AddRange(
				BXSaleCustomerType.GetList(
					null,
					new BXOrderBy(
						new BXOrderByPair(BXSaleCustomerType.Fields.Site.ID, BXOrderByDirection.Asc),
						new BXOrderByPair(BXSaleCustomerType.Fields.Sort, BXOrderByDirection.Asc)
					),
					null,
					null,
					BXTextEncoder.EmptyTextEncoder
				)
				.Select(x => new ListItem(string.Format("[{0}] {1} ({2})", x.Id, x.Name, x.SiteId), x.Id.ToString()))
				.ToArray()
			);

			PaymentSystemFilter.Values.Clear();
			PaymentSystemFilter.Values.AddRange(
				BXSalePaymentSystem.GetList(
					null,
					new BXOrderBy(
						new BXOrderByPair(BXSalePaymentSystem.Fields.Sort, BXOrderByDirection.Asc)
					),
					null,
					null,
					BXTextEncoder.EmptyTextEncoder
				)
				.Select(x => new ListItem(string.Format("[{0}] {1} ({2})", x.Id, x.Name, x.SiteId), x.Id.ToString()))
				.ToArray()
			);

			LoadFields();
			AddCustomFilters();
			AddCustomColumns();
		}
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);

			Title = GetMessageRaw("PageTitle");
			MasterTitle = GetMessageRaw("MasterTitle");

			UserFilter.AttachScript();
		}

		protected void GridView_Select(object sender, BXSelectEventArgs e)
		{
			var filter = MakeFilter();
			var orderBy = new BXOrderBy(BXSaleOrder.Fields, e.SortExpression);

			var columns = GridView.GetVisibleColumnsKeys();
			var select = new BXSelectAdd
			{
				BXSaleOrder.Fields.Currency,
			
				BXSaleOrder.Fields.Site.ID,
				BXSaleOrder.Fields.Site.Name,

				BXSaleOrder.Fields.CustomerType.Id,
				BXSaleOrder.Fields.CustomerType.Name,
			};

			if (Array.IndexOf(columns, "Status") != -1)
			{
				select.Add(BXSaleOrder.Fields.Status.XmlId);
				select.Add(BXSaleOrder.Fields.Status.Localization);
			}

			if (Array.IndexOf(columns, "Paid") != -1)
				select.Add(BXSaleOrder.Fields.PaidEmployee.UserName);

			if (Array.IndexOf(columns, "Cancelled") != -1)
				select.Add(BXSaleOrder.Fields.CancelledEmployee.UserName);

			if (Array.IndexOf(columns, "PaymentSystem") != -1)
				select.Add(BXSaleOrder.Fields.PaymentSystem.Name);

			if (Array.IndexOf(columns, "User") != -1)
			{
				select.Add(BXSaleOrder.Fields.User.UserName);
				select.Add(BXSaleOrder.Fields.User.FirstName);
				select.Add(BXSaleOrder.Fields.User.LastName);				
			}

			var data = BXSaleOrder.GetList(filter, orderBy, select, new BXQueryParams(e.PagingOptions), BXTextEncoder.EmptyTextEncoder)
				.ConvertAll(x => new HtmlWrapper(x, this));



			products.Clear();
			var productColumns = new[] {
				"Products",
				"ProductNames",
				"ProductIds",
				"ProductPrices",
				"ProductQuantities",
				"ProductWeights",
				"ProductCodes",
				"ProductVatRates" 
			};
			if (Array.Exists(productColumns, x => Array.IndexOf(columns, x) >= 0))
			{
				var productsList = BXSaleProduct.GetList(
					new BXFilter(
						new BXFilterItem(BXSaleProduct.Fields.OrderId, BXSqlFilterOperators.In, data.ConvertAll(x => x.Item.Id))
					),
					new BXOrderBy(
						new BXOrderByPair(BXSaleProduct.Fields.OrderId, BXOrderByDirection.Asc),
						new BXOrderByPair(BXSaleProduct.Fields.Id, BXOrderByDirection.Asc)
					),
					null,
					null,
					BXTextEncoder.EmptyTextEncoder
				);

				int currentOrderId = -1;
				BXSaleProductCollection currentCollection = null;
				foreach (var p in productsList)
				{
					if (currentOrderId != p.OrderId)
					{
						currentCollection = new BXSaleProductCollection();
						currentOrderId = p.OrderId;
						products[currentOrderId] = currentCollection;
					}
					currentCollection.Add(p);
				}
			}

			e.Data = data;
		}
		protected void GridView_SelectCount(object sender, BXSelectCountEventArgs e)
		{
			var filter = MakeFilter();
			e.Count = BXSaleOrder.Count(filter);
		}
		protected void GridView_Delete(object sender, BXDeleteEventArgs e)
		{
			try
			{
				var filter =
					(e.Keys == null)
					? new BXFilter(AdminFilter.CurrentFilter, BXSaleOrder.Fields)
					: new BXFilter(new BXFilterItem(BXSaleOrder.Fields.Id, BXSqlFilterOperators.Equal, e.Keys["Id"]));

				var items = BXSaleOrder.GetList(filter, null);
				foreach (var item in items)
				{
					try
					{
						if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderManage, BXSaleModuleConfiguration.Id, item.StatusId))
						{
							ErrorMessage.AddErrorMessage(string.Format(GetMessageRaw("Error.InsufficientRightsDelete"), item.Id));
							continue;
						}
						item.Delete();
						e.DeletedCount++;
					}
					catch (Exception ex)
					{
						ErrorMessage.AddErrorMessage(string.Format(GetMessageRaw("Error.Delete"), item.Id));
						BXLogService.LogAll(ex, BXLogMessageType.Error, "SaleOrderList.aspx, Delete");
					}
				}
			}
			catch (Exception ex)
			{
				ErrorMessage.AddErrorMessage(ex.Message);
				return;
			}
		}
		protected void GridView_RowDataBound(object sender, GridViewRowEventArgs e)
		{
			if (e.Row.RowType != DataControlRowType.DataRow)
				return;

			var row = (BXGridViewRow)e.Row;
			var item = (HtmlWrapper)row.DataItem;
			row.UserData.Add("Id", item.Item.Id);
			row.AllowedCommandsList =
				BXUser.IsCanOperate(BXSaleModule.Operations.OrderManage, BXSaleModuleConfiguration.Id, item.Item.StatusId)
				? editOperations 
				: viewOperations;
		}

		protected void SiteCurrency_OnBuildFilter(object sender, BXCustomAdminFilter.BuildFilterEventArgs e)
		{
			if (!string.IsNullOrEmpty(SiteFilter.SelectedValue))
				e.Result.Add(new BXFormFilterItem("SiteId", SiteFilter.SelectedValue, BXSqlFilterOperators.Equal));
			if (!string.IsNullOrEmpty(CurrencyFilter.SelectedValue))
				e.Result.Add(new BXFormFilterItem("CurrencyId", int.Parse(CurrencyFilter.SelectedValue), BXSqlFilterOperators.Equal));
		}
		protected void SiteCurrency_OnLoadState(object sender, BXCustomAdminFilter.StateEventArgs e)
		{
			if (e.State.Contains("siteId"))
				SiteFilter.SelectedValue = (string)e.State["siteId"];
			if (e.State.Contains("currencyId"))
				CurrencyFilter.SelectedValue = (string)e.State["currencyId"];
		}
		protected void SiteCurrency_OnSaveState(object sender, BXCustomAdminFilter.StateEventArgs e)
		{
			e.State["siteId"] = SiteFilter.SelectedValue ?? "";
			e.State["currencyId"] = CurrencyFilter.SelectedValue ?? "";
		}
		protected void SiteCurrency_OnReset(object sender, EventArgs e)
		{
			SiteFilter.SelectedIndex = 0;
			CurrencyFilter.SelectedIndex = 0;
		}

		protected class HtmlWrapper
		{
			private BXSaleOrder item;
			private OrderList page;
			private BXCustomPropertyCollection values;

			public HtmlWrapper(BXSaleOrder item, OrderList page)
			{
				this.item = item;
				this.page = page;
			}

			public int Id
			{
				get
				{
					return item.Id;
				}
			}
			public BXSaleOrder Item
			{
				get
				{
					return item;
				}
			}

			string BuildProductList(Action<BXSaleProduct, StringBuilder> builder)
			{
				var s = new StringBuilder();

				BXSaleProductCollection list;
				if (page.products.TryGetValue(Item.Id, out list))
				{
					foreach (var p in list)
					{
						if (s.Length != 0)
							s.Append(@"<hr width=""90%"" size=""1"" />");
						builder(p, s);
					}
				}

				return s.Length > 0 ? s.ToString() : "&nbsp;";
			}

			public string IdInfo
			{
				get
				{
					return string.Format(
						page.GetMessageRaw("OrderIdTemplate"),
						Item.Id,
						HttpUtility.HtmlEncode(Item.Created.ToString("g"))
					);
				}
			}
			public string Paid
			{
				get
				{
					if (!Item.Paid)
						return page.GetMessageRaw("Kernel.No");

					return string.Concat(
						page.GetMessageRaw("Kernel.Yes"),

						Item.PaidDateUtc != DateTime.MinValue
						? string.Format(
							@"<br/>{0}",
							HttpUtility.HtmlEncode(Item.PaidDate.ToString("g"))
						)
						: null,

						Item.PaidEmployee != null
						? string.Format(
							@"<br/>[<a href=""AuthUserEdit.aspx?id={0}"">{0}</a>] ({1})",
							Item.PaidEmployee.UserId,
							HttpUtility.HtmlEncode(Item.PaidEmployee.UserName)
						)
						: null
					);
				}
			}
			public string Cancelled
			{
				get
				{
					if (!Item.Cancelled)
						return page.GetMessageRaw("Kernel.No");

					return string.Concat(
						page.GetMessageRaw("Kernel.Yes"),

						Item.CancelledDateUtc != DateTime.MinValue
						? string.Format(
							@"<br/>{0}",
							HttpUtility.HtmlEncode(Item.CancelledDate.ToString("g"))
						)
						: null,

						Item.CancelledEmployee != null
						? string.Format(
							@"<br/>[<a href=""AuthUserEdit.aspx?id={0}"">{0}</a>] ({1})",
							Item.CancelledEmployee.UserId,
							HttpUtility.HtmlEncode(Item.CancelledEmployee.UserName)
						)
						: null
					);
				}
			}
			public string Status
			{
				get
				{
					return string.Format(
						@"[<a href=""SaleOrderStatusEdit.aspx?id={0}"">{1}</a>] {2}<br/>{3}",

						Item.StatusId,
						HttpUtility.HtmlEncode(Item.Status.XmlId),
						HttpUtility.HtmlEncode(Item.Status.LocalizedName),
						HttpUtility.HtmlEncode(Item.StatusSetDate.ToString("g"))
					);
				}
			}
			public string Sum
			{
				get
				{
					return HttpUtility.HtmlEncode(Item.Currency.MoneyToString(Item.Sum));
				}
			}
			public string User
			{
				get
				{
					if (Item.User == null)
						return "-";

					return string.Format(
						@"[<a href=""AuthUsersEdit.aspx?Id={0}"">{0}</a>] {1} {2} ({3})",
						Item.UserId,
						HttpUtility.HtmlEncode(Item.User.LastName),
						HttpUtility.HtmlEncode(Item.User.FirstName),
						HttpUtility.HtmlEncode(Item.User.UserName)
					);
				}
			}
			public string PaymentSystem
			{
				get
				{
					if (Item.PaymentSystem == null)
						return "-";

					return string.Format(
						@"[<a href=""SalePaymentSystemEdit.aspx?Id={0}"">{0}</a>] {1}",
						Item.PaymentSystemId,
						HttpUtility.HtmlEncode(Item.PaymentSystem.Name)
					);
				}
			}
			public string Products
			{
				get
				{
					return BuildProductList((p, s) =>
					{
						s.AppendFormat(
							page.GetMessageRaw("ProductTemplate"),
							p.CartElementId,
							HttpUtility.HtmlAttributeEncode(p.DetailUrl),
							HttpUtility.HtmlEncode(p.Name),
							p.Quantity
						);
					});
				}
			}
			public string Site
			{
				get
				{
					return string.Format(
						@"[{0}] {1}",
						HttpUtility.HtmlEncode(Item.Site.Id),
						HttpUtility.HtmlEncode(Item.Site.Name)
					);
				}
			}
			public string CustomerType
			{
				get
				{
					return string.Format(
						@"[<a href=""SaleCustomerTypeEdit.aspx?id={0}"">{0}</a>] {1}",
						Item.CustomerType.Id,
						HttpUtility.HtmlEncode(Item.CustomerType.Name)
					);
				}
			}
			public string DeliveryCost
			{
				get
				{
					return HttpUtility.HtmlEncode(Item.Currency.MoneyToString(Item.DeliveryCost));
				}
			}
			public string DeliveryVariant
			{
				get
				{
					string s;
					if (!BXSaleDeliveryManager.TryGetVariantDescription(item.DeliveryVariant, out s))
						return "&nbsp;";

					return HttpUtility.HtmlEncode(s);
				}
			}
			public string LastUpdated
			{
				get
				{
					return HttpUtility.HtmlEncode(Item.LastUpdated.ToString("g"));
				}
			}
			public string Comments
			{
				get
				{
					var s = new StringBuilder();
					if (!BXStringUtility.IsNullOrTrimEmpty(Item.CustomerComments))
					{
						if (s.Length > 0)
							s.Append(@"<hr width=""90%"" size=""1"" />");
						s.Append(BXStringUtility.HtmlEncodeEx(Item.CustomerComments));
					}
					if (!BXStringUtility.IsNullOrTrimEmpty(Item.AdditionalInfo))
					{
						if (s.Length > 0)
							s.Append(@"<hr width=""90%"" size=""1"" />");
						s.Append(BXStringUtility.HtmlEncodeEx(Item.AdditionalInfo));
					}
					if (!BXStringUtility.IsNullOrTrimEmpty(Item.EmployeeComments))
					{
						if (s.Length > 0)
							s.Append(@"<hr width=""90%"" size=""1"" />");
						s.Append(BXStringUtility.HtmlEncodeEx(Item.EmployeeComments));
					}

					return s.Length > 0 ? s.ToString() : "&nbsp;";
				}
			}
			public string ProductNames
			{
				get
				{
					return BuildProductList((p, s) =>
					{
						s.AppendFormat(
							@"<a href=""{0}"">{1}</a>",
							HttpUtility.HtmlAttributeEncode(p.DetailUrl),
							HttpUtility.HtmlEncode(p.Name)
						);
					});
				}
			}
			public string ProductIds
			{
				get
				{
					return BuildProductList((p, s) =>
					{
						s.Append(p.CartElementId);
					});
				}
			}
			public string ProductPrices
			{
				get
				{
					return BuildProductList((p, s) =>
					{
						s.Append(HttpUtility.HtmlEncode(Item.Currency.MoneyToString(p.GetUnitCost())));
					});
				}
			}
			public string ProductQuantities
			{
				get
				{
					return BuildProductList((p, s) =>
					{
						s.AppendFormat(page.GetMessageRaw("Quantity"), p.Quantity);
					});
				}
			}
			public string ProductWeights
			{
				get
				{
					return BuildProductList((p, s) =>
					{
						s.Append(p.Weight);
					});
				}
			}
			public string ProductCodes
			{
				get
				{
					return BuildProductList((p, s) =>
					{
						s.Append(!string.IsNullOrEmpty(p.CartElementCode) ? HttpUtility.HtmlEncode(p.CartElementCode) : "&nbsp;");
					});
				}
			}
			public string ProductVatRates
			{
				get
				{
					return BuildProductList((p, s) =>
					{
						s.Append(p.VatRate.ToString("F2"));
					});
				}
			}

			public BXCustomPropertyCollection Values
			{
				get
				{
					if (values == null)
						BuildValues();
					return values;
				}
			}

			private void BuildValues()
			{
				var fields = page.orderFields[Item.CustomerTypeId];
				values = BXSaleOrderValueManager.GetValues(Item.Id, fields, null);
			}
		}
		protected class ValueTemplate : ITemplate
		{
			string code;

			public ValueTemplate(string code)
			{
				this.code = code;
			}

			public void InstantiateIn(Control container)
			{
				container.Controls.Add(new ValueRenderer(code));
			}
		}
		protected class ValueRenderer : Control
		{
			HtmlWrapper w;
			string code;

			public new OrderList Page
			{
				get
				{
					return (OrderList)base.Page;
				}
			}

			public ValueRenderer(string code)
			{
				this.code = code;
			}

			public override void DataBind()
			{
				var row = (GridViewRow)BindingContainer;
				w = (HtmlWrapper)row.DataItem;
			}

			protected override void Render(HtmlTextWriter writer)
			{
				BXCustomProperty p;
				var field = Page.orderFields[w.Item.CustomerTypeId][code];
				if (field == null || !w.Values.TryGetValue(code, out p))
				{
					writer.Write("&nbsp;");
					return;
				}

				var v = field.CustomType.CreatePublicView();
				v.Init(p, field.GetCustomFieldAdapter());
				if (v.IsEmpty)
				{
					writer.Write("&nbsp;");
					return;
				}

				v.Render("<br/>", writer);
			}
		}
	}
}
