﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.UI;
using Bitrix.Services;
using Bitrix.Modules;
using Bitrix;
using Bitrix.Security;
using Bitrix.Services.Text;
using Bitrix.Sale;
using Bitrix.DataLayer;
using System.Collections.Generic;

public partial class bitrix_admin_SaleOrderStatusEdit : Bitrix.UI.BXAdminPage
{
	private bool currentUserCanModify = false;
	protected BXLanguageCollection Languages;
	protected BXSaleOrderStatus status;
	List<BXRole> allRoles;
	List<BXRoleOperation> allOperations;
	
	List<BXRole> Roles
	{
		get
		{
			if (allRoles == null)
			{
				allRoles = BXRoleManager.GetList(
					new BXFormFilter(new BXFormFilterItem("Active", true, BXSqlFilterOperators.Equal)),
					new BXOrderBy_old("RoleName", "Asc")
				);
			}
			return allRoles;
		}
	}
	List<BXRoleOperation> Operations
	{
		get
		{
			if (allOperations == null)
			{
				var names = BXSaleModule.Operations.GetStatusOperationNames();
				allOperations = BXRoleOperationManager.GetList(
					new BXFormFilter(new BXFormFilterItem("ModuleId", BXSaleModuleConfiguration.Id, BXSqlFilterOperators.Equal)),
					new BXOrderBy_old("OperationName", "Asc")
				);
				allOperations = allOperations.FindAll(x => Array.IndexOf(names, x.OperationName) >= 0);
			}

			return allOperations;
		}
	}
	public int StatusId
	{
		get
		{
			int id = -1;
			if (int.TryParse(Request.QueryString["id"], out id))
				return id;
			return 0;
		}
	}

	private void BindStatus(int statusId)
	{
		status = BXSaleOrderStatus.GetById(statusId);

		if (status != null && status.Id == BXSaleModuleConfiguration.InitialOrderStatusId)
			txtCode.Enabled = false;

		if (status == null)
			status = new BXSaleOrderStatus();

	}
	protected void SetTabTitle()
	{
		BXTabControl1.Tabs[0].Title = GetMessage("TabTitle");
	}
	private void SaveStatus(bool isApply)
	{
		if (!currentUserCanModify)
		{
			errorMessage.AddErrorMessage(GetMessage("Exception.YouDontHaveRightsToPerformThisOperation"));
			return;
		}

		var status = BXSaleOrderStatus.GetById(StatusId, BXTextEncoder.EmptyTextEncoder);

		if (status == null)
			status = new BXSaleOrderStatus();

		bool isNew = status.IsNew;

		int sort = 0;
		if (int.TryParse(txtSort.Text, out sort))
		{
			status.Sort = sort;
		}

		status.IsFinal = IsFinal.Checked;

		if (isNew || status.Id != BXSaleModuleConfiguration.InitialOrderStatusId)
			status.XmlId = txtCode.Text;

		foreach (var lang in Languages)
		{
			var formName = Request.Form["locname_" + lang.Id];
			var formDesc = Request.Form["locdesc_" + lang.Id];
			if (!String.IsNullOrEmpty(formName))
			{
				status.LocalizationInfo[lang.Id] = new BXSaleOrderStatusLocalizationInfo(formName, formDesc ?? String.Empty);
			}
		}

		try
		{
			status.Save();
			SaveAccessState(status, isNew);
		}
		catch (Exception ex)
		{
			errorMessage.AddErrorMessage(ex.Message);
			return;
		}

		if (isApply)
			Response.Redirect(string.Format("SaleOrderStatusEdit.aspx?id={0}", status.Id));
		else
			Response.Redirect("SaleOrderStatusList.aspx");
	}
	bool ValidateFields()
	{
		var hasErrors = false;

		foreach (var lang in Languages)
		{
			var formName = Request.Form["locname_"+lang.Id];
			if (String.IsNullOrEmpty(formName))
			{
				hasErrors = true;
				errorMessage.AddErrorMessage(String.Format(GetMessage("Error.NameNotSpecified"), lang.Name));
			}
		}

		return !hasErrors;
	}
	private void PrepareAccessState()
	{
		AccessEdit.FillStandardRoles(Roles, false);
		
		foreach (BXRoleOperation op in Operations)
			AccessEdit.Operations.Add(op.OperationId.ToString(), op.OperationTitle);
	}
	private void LoadAccessState()
	{
		//SKIP THIS THING IF AJAX
		ScriptManager sm = ScriptManager.GetCurrent(this);
		if (sm != null && sm.IsInAsyncPostBack)
		{ 
			bool skip = true;
			for (Control parent = AccessEdit.Parent; parent != null; parent = parent.Parent)
			{
				UpdatePanel panel = parent as UpdatePanel;
				if (panel != null && panel.IsInPartialRendering)
				{ 
					skip = false; 
					break;
				}
			}
			if (skip)
				return;
		}

		AccessEdit.State.Clear();
		
		
		var statusIdString = (status != null && !status.IsNew) ? status.Id.ToString() : null;
		var links = BXRoleOperationManager.GetLinks(BXSaleModuleConfiguration.Id);
		links = links.FindAll(x => Roles.Exists(y => x.RoleId == y.RoleId));
		links = links.FindAll(x => Operations.Exists(y => x.OperationId == y.OperationId));
		links = links.FindAll(x => string.IsNullOrEmpty(x.OperationExternalId) || statusIdString != null && x.OperationExternalId == statusIdString);
		
		var processedRoles = new List<int>();
		foreach (var link in links)
		{
			var roleInfo = AccessEdit.Roles[link.RoleId.ToString()];
			var stringOperationId = link.OperationId.ToString();
			BXOperationsEditOperationInfo opInfo;
			if (!roleInfo.Operations.TryGetValue(stringOperationId, out opInfo))
				roleInfo.Operations.Add(stringOperationId, opInfo = new BXOperationsEditOperationInfo());
			if (string.IsNullOrEmpty(link.OperationExternalId))
				opInfo.InheritedState = BXOperationsEditInheritedOperationState.Allowed;
			else 
				opInfo.State = BXOperationsEditOperationState.Allowed;

			int i = processedRoles.BinarySearch(link.RoleId);
			if (i < 0)
				processedRoles.Insert(~i, link.RoleId);
		}
		foreach (var roleId in processedRoles)
			AccessEdit.State.SetRoleFromData(roleId.ToString());
	}
	private void SaveAccessState(BXSaleOrderStatus status, bool isNew)
	{
		var statusString = status.Id.ToString(); 

		if (!isNew)
		{
			foreach (var ops in Operations)
				BXRoleManager.RemoveRoleFromOperations(ops.OperationName, BXSaleModuleConfiguration.Id, statusString);
		}
		
		foreach (var role in AccessEdit.State)
		{
			int id = int.Parse(role.Key);
			var r = Roles.Find(x => x.RoleId == id);
			
			var allowed = 
				Operations
				.Where(x => role.Value[x.OperationId.ToString()] == BXOperationsEditOperationState.Allowed)
				.Select(x => x.OperationName)
				.ToArray();

			if (allowed.Length > 0)
				BXRoleManager.AddRolesToOperations(new[] { r.RoleName }, allowed.ToArray(), BXSaleModuleConfiguration.Id, statusString);
		}
	}

	protected void Page_Init(object sender, EventArgs e)
	{
		PrepareAccessState();
	}
	protected void Page_Load(object sender, EventArgs e)
	{
		Languages = BXLanguage.GetList(null, null);

		if (StatusId > 0)
			MasterTitle = string.Format(GetMessage("EditPageTitle"), StatusId);
		else
			MasterTitle = GetMessage("CreatePageTitle");

		Title = MasterTitle;

		currentUserCanModify = this.BXUser.IsCanOperate(BXSaleModule.Operations.AdminManagement);
		if (!currentUserCanModify)
			BXAuthentication.AuthenticationRequired();

		

		AddSeparator.Visible =
			AddAction.Visible =
			DeleteSeparator.Visible =
			DeleteAction.Visible = StatusId > 0 && currentUserCanModify && StatusId != BXSaleModuleConfiguration.InitialOrderStatusId;

		BXTabControl1.ShowSaveButton = BXTabControl1.ShowApplyButton = currentUserCanModify;

		if (StatusId > 0)
		{

			BindStatus(StatusId);

			if (!IsPostBack)
			{
				txtCode.Text = status.TextEncoder.Decode(status.XmlId);
				txtSort.Text = status.Sort.ToString();
				IsFinal.Checked = status.IsFinal;
			}
		}
	}
	protected void Page_PreRender(object sender, EventArgs e)
	{
		LoadAccessState();
	}
	protected void BXTabControl1_Command(object sender, BXTabControlCommandEventArgs e)
	{
		switch (e.CommandName.ToLower())
		{
			case "save":
				if (ValidateFields())
				{
					SaveStatus(false);
				}
				break;
			case "apply":
				if (ValidateFields())
				{
					SaveStatus(true);
				}
				break;
			case "cancel":
				Response.Redirect("SaleOrderStatusList.aspx");
				break;
			default:
				break;
		}
	}
	protected void BXContextMenuToolbar1_CommandClick(object sender, CommandEventArgs e)
	{
		switch (e.CommandName.ToLower())
		{
			case "getlist":
				Response.Redirect("SaleOrderStatusList.aspx");
				break;
			case "new":
				Response.Redirect("SaleOrderStatusEdit.aspx");
				break;
			case "delete":
				if (currentUserCanModify)
				{
					try
					{
						var stat = BXSaleOrderStatus.GetById(StatusId);
						stat.Delete();

						Response.Redirect("SaleOrderStatusList.aspx");
					}
					catch (Exception ex)
					{
						errorMessage.AddErrorMessage(ex.Message);
					}
				}
				else
				{
					errorMessage.AddErrorMessage(GetMessage("Exception.YouDontHaveRightsToPerformThisOperation"));
				}
				break;
			default:
				break;
		}
	}
}