﻿using Bitrix.UI;
using Bitrix.Security;
using System.Security;
using Bitrix.Services.Text;
using System;
using System.Linq;
using Bitrix.Modules;
using System.Web.UI.WebControls;
using System.Web.UI;
using Bitrix.DataLayer;
using System.Collections.Generic;
using Bitrix.Services;

namespace Bitrix.Sale.AdminPages
{
	public partial class OrderView : BXAdminPage
	{
		#region Messages
		int showMessage;

		bool HasErrors { get { return showMessage == -1; } }

		void PrepareResultMessage()
		{
			SuccessMessage.Visible = showMessage > 0;
		}
		void ShowErrorHtml(string html)
		{
			showMessage = -1;
			ErrorMessage.AddErrorMessage(html);
		}
		void ShowErrorText(string text)
		{
			ShowErrorHtml(Encode(text));
		}
		void ShowError(Exception ex)
		{
			showMessage = -1;
			ErrorMessage.AddErrorMessage(Encode(ex.Message));
		}
		void ShowOk()
		{
			if (showMessage == 0)
				showMessage = 1;
		}
		#endregion

		public bool CanManage { get; set; }
		public BXSaleOrder Item { get; set; }
		public BXSaleProductCollection Products { get; set; }
		public BXSaleOrderStatusCollection Statuses { get; set; }
		protected override string BackUrl
		{
			get
			{
				return base.BackUrl ?? "SaleOrderList.aspx";
			}
		}

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			

			var idString = Request.QueryString["id"];
		
			int id = 0;
			if (idString == null || !int.TryParse(idString, out id) || id <= 0)
				GoBack(); // не удалось найти такой объект
		 
			Item = BXSaleOrder.GetList(
				new BXFilter(
					new BXFilterItem(BXSaleOrder.Fields.Id, BXSqlFilterOperators.Equal, id)
				),
				null, 
				new BXSelectAdd(
					BXSaleOrder.Fields.User,
					BXSaleOrder.Fields.Site,
					BXSaleOrder.Fields.Currency,
					BXSaleOrder.Fields.CustomerType,
					BXSaleOrder.Fields.Status,

					BXSaleOrder.Fields.StatusSetEmployee.UserId,
					BXSaleOrder.Fields.StatusSetEmployee.UserName,

					BXSaleOrder.Fields.CancelledEmployee.UserId,
					BXSaleOrder.Fields.CancelledEmployee.UserName,

					BXSaleOrder.Fields.PaidEmployee.UserId,
					BXSaleOrder.Fields.PaidEmployee.UserName,

					BXSaleOrder.Fields.AllowShippingEmployee.UserId,
					BXSaleOrder.Fields.AllowShippingEmployee.UserName,

					BXSaleOrder.Fields.PaymentSystem.Name,
					BXSaleOrder.Fields.PaymentSystem.SiteId
				),
				new BXQueryParams(new BXPagingOptions(0, 1)),
				BXTextEncoder.EmptyTextEncoder
			)
			.FirstOrDefault();
			
			if (Item == null)
				GoBack();

			if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderView, BXSaleModuleConfiguration.Id, Item.StatusId))
				BXAuthentication.AuthenticationRequired();

			CanManage = BXUser.IsCanOperate(BXSaleModule.Operations.OrderManage, BXSaleModuleConfiguration.Id, Item.StatusId);

			NewStatus.Items.Clear();
			if (BXUser.IsCanOperate(BXSaleModule.Operations.OrderExitStatus, BXSaleModuleConfiguration.Id, Item.StatusId))
			{
				Statuses = BXSaleOrderStatus.GetList(
					null,
					new BXOrderBy(new BXOrderByPair(BXSaleOrderStatus.Fields.Sort, BXOrderByDirection.Asc)),
					null,
					null,
					BXTextEncoder.EmptyTextEncoder
				);

				foreach (var s in Statuses)
				{
					if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderEnterStatus, BXSaleModuleConfiguration.Id, s.Id))
						continue;
					if (s.Id == Item.StatusId)
						continue;

					NewStatus.Items.Add(new ListItem(
						string.Format("[{0}] {1}", s.XmlId, s.LocalizedName),
						s.Id.ToString()
					));
				}
			}

			var rc = Item.ReasonCancelled;

			ReasonCancelled.Items.Clear();
			ReasonCancelled.Items.AddRange
			(
				(
					from BXSaleOrderCancelReason r in Enum.GetValues(typeof(BXSaleOrderCancelReason))
					select new ListItem(BXSaleOrderCancelReasonLocalization.GetLocalizedName(r), r.ToString()) { Selected = (r == rc) }
				)
				.ToArray()
			);

			PaymentDocument.Text = Item.PaymentDocument;
			PaymentDocumentDate.Date = Item.PaymentDocumentDateUtc != DateTime.MinValue ? Item.PaymentDocumentDate : DateTime.MinValue;
			ShippingDocument.Text = Item.ShippingDocument;
			ShippingDocumentDate.Date = Item.ShippingDocumentDateUtc != DateTime.MinValue ? Item.ShippingDocumentDate : DateTime.MinValue;

			EmployeeComments.Text = Item.EmployeeComments;


		}
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);

			Title = GetMessageRaw("PageTitle");
			MasterTitle = string.Format(GetMessageRaw("MasterTitle"), Item.Id, Item.Created.ToString("g"));

			PrepareResultMessage();

			EditButton.Visible = CanManage && Item != null;
			EditButton.Href = string.Concat("SaleOrderEdit.aspx?id=", Item.Id.ToString());
			DeleteButton.Visible = CanManage && Item != null;

			Products = BXSaleProduct.GetList(
				new BXFilter(
					new BXFilterItem(BXSaleProduct.Fields.OrderId, BXSqlFilterOperators.Equal, Item.Id)
				),
				new BXOrderBy(
					new BXOrderByPair(BXSaleProduct.Fields.Id, BXOrderByDirection.Asc)
				),
				null,
				null,
				BXTextEncoder.EmptyTextEncoder
			);
		}

		protected void ActionBar_CommandClick(object sender, CommandEventArgs e)
		{
			if (e.CommandName != "delete")
				return;

			if (Item == null)
				return;

			if (!CanManage)
			{
				ShowErrorText(GetMessageRaw("Error.InsufficientRightsDelete"));
				return;
			}

			bool success = false;
			try
			{
				Item.Delete();
				success = true;
			}
			catch (BXEventException ex)
			{
				foreach (string s in ex.Messages)
					ShowErrorText(s);
			}
			catch
			{
				ErrorMessage.AddErrorMessage(GetMessageRaw("Error.Delete"));
			}

			if (success)
				GoBack();
		}
		protected void Order_SetStatus(object sender, EventArgs e)
		{
			int id;
			if (
				string.IsNullOrEmpty(NewStatus.SelectedValue) 
				|| !int.TryParse(NewStatus.SelectedValue, out id)
				|| !BXUser.IsCanOperate(BXSaleModule.Operations.OrderExitStatus, BXSaleModuleConfiguration.Id, Item.StatusId)
				|| !BXUser.IsCanOperate(BXSaleModule.Operations.OrderEnterStatus, BXSaleModuleConfiguration.Id, id)
				|| Statuses == null
				|| !Statuses.Exists(x => x.Id == id)
				|| id == Item.StatusId
			)
				return;
			
			Item.StatusId = id;
			Item.StatusSetDate = DateTime.Now;
			Item.StatusSetEmployeeId = BXIdentity.Current.Id;
			Item.Save();
			if (BXUser.IsCanOperate(BXSaleModule.Operations.OrderView, BXSaleModuleConfiguration.Id, id))
				Reload(TabControl.TabIndex, null, null);
			else 
				GoBack();
		}
		protected void Order_SetCancelled(object sender, EventArgs e)
		{
			if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderCancel, BXSaleModuleConfiguration.Id, Item.StatusId))
				return;
			if (Item.Cancelled != (Request.Form[UniqueID + "$CancelledFlag"] == "Y"))
				return;

			Item.Cancelled = !Item.Cancelled;
			Item.CancelledDate = DateTime.Now;
			Item.CancelledEmployeeId = BXIdentity.Current.Id;
			if (Item.Cancelled)
			{
				try
				{
					Item.ReasonCancelled = (BXSaleOrderCancelReason)Enum.Parse(typeof(BXSaleOrderCancelReason), ReasonCancelled.SelectedValue);
				}
				catch
				{
				}				 
			}
			
			Item.Save();
			Reload(TabControl.TabIndex, null, null);
		}
		protected void Order_SetPaid(object sender, EventArgs e)
		{
			if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderPay, BXSaleModuleConfiguration.Id, Item.StatusId))
				return;
			if (Item.Paid != (Request.Form[UniqueID + "$PaidFlag"] == "Y"))
				return;

			Item.Paid = !Item.Paid;
			Item.PaidDate = DateTime.Now;
			Item.PaidEmployeeId = BXIdentity.Current.Id;
			if (Item.Paid)
			{
				Item.PaymentDocument = PaymentDocument.Text;
				Item.PaymentDocumentDate = PaymentDocumentDate.Date;
			}
			Item.Save();
			Reload(TabControl.TabIndex, null, null);
		}
		protected void Order_SetAllowShipping(object sender, EventArgs e)
		{
			if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderAllowShipping, BXSaleModuleConfiguration.Id, Item.StatusId))
				return;
			if (Item.AllowShipping != (Request.Form[UniqueID + "$AllowShippingFlag"] == "Y"))
				return;

			Item.AllowShipping = !Item.AllowShipping;
			Item.AllowShippingDate = DateTime.Now;
			Item.AllowShippingEmployeeId = BXIdentity.Current.Id;
			if (Item.AllowShipping)
			{
				Item.ShippingDocument = ShippingDocument.Text;
				Item.ShippingDocumentDate = ShippingDocumentDate.Date;
			}
			Item.Save();
			Reload(TabControl.TabIndex, null, null);
		}
		protected void Order_SetEmployeeComments(object sender, EventArgs e)
		{
			if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderManage, BXSaleModuleConfiguration.Id, Item.StatusId))
				return;
			
			Item.EmployeeComments = EmployeeComments.Text;
			Item.Save();
			Reload(TabControl.TabIndex, null, null);
		}

		protected void Order_CheckPayment(object sender, EventArgs e)
		{
			if (!BXUser.IsCanOperate(BXSaleModule.Operations.OrderPay, BXSaleModuleConfiguration.Id, Item.StatusId))
				return;
			
			if (!BXSalePaymentManager.IsPaymentCheckSupported(Item.Id))
				return;

			try
			{
				BXSalePaymentManager.ProcessPaymentCheck(Item.Id);				
			}
			catch(Exception ex)
			{
				BXLogService.LogAll(ex, BXLogMessageType.Error, Page.AppRelativeVirtualPath);
				ErrorMessage.AddErrorText(GetMessageRaw("Error.CheckPayment"));
				return;
			}
			Reload(TabControl.TabIndex, null, null);
		}

		protected List<CustomProperty> GetCustomProperties()
		{
			var result = new List<CustomProperty>();
			var fields = BXSaleOrderField.GetCustomerFields(Item.CustomerTypeId, null);
			var values = BXSaleOrderValueManager.GetValues(Item.Id, fields, null);

			foreach (var v in values.Values)
			{
				var f = fields[v.Name]; 
				if (f != null)
				{
					var view = f.CustomType.CreatePublicView();
					view.Init(v, f.GetCustomFieldAdapter());
					result.Add(new CustomProperty 
					{
						TitleHtml = f.Name,
						ValueHtml = view.GetHtml("<br/>"),
						Sort = f.Sort
					});
				}
				else 
				{
					result.Add(new CustomProperty 
					{
						TitleHtml = Encode(v.UserLikeName),
						ValueHtml = string.Join("<br/>", v.Values.ConvertAll(x => Encode(x.ToString())).ToArray()),
						Sort = int.MaxValue
					});
				}
			}
			result.Sort((x, y) => x.Sort - y.Sort);
			return result;
		}
	
		protected class CustomProperty
		{
			public string TitleHtml { get; set; }
			public string ValueHtml { get; set; }
			public int Sort { get; set; }
		}
	}
}