﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using Bitrix.Sale;
using Bitrix.Security;
using Bitrix.DataLayer;
using Bitrix;
using Bitrix.Services.Text;
using System.Web.Script.Serialization;
using System.Text;
using System.Security.Policy;
using Bitrix.Services;
using System.Collections.Specialized;

public partial class bitrix_admin_SalePaymentSystemEdit : BXAdminPage
{
    protected override void OnPreInit(EventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnPreInit(e);
    }

    protected override void OnInit(EventArgs e)
    {
        string idStr = Request.QueryString["id"];
        if (string.IsNullOrEmpty(idStr) || !int.TryParse(idStr, out entityId))
            this.entityId = 0;

        SiteList.Items.Add(new ListItem(GetMessageRaw("Option.SelectSite"), string.Empty));
        BXSiteCollection siteColl = BXSite.GetList(null, null, null, null, BXTextEncoder.EmptyTextEncoder);
        foreach (BXSite site in siteColl)
            SiteList.Items.Add(new ListItem(string.Concat("[", site.Id, "] ", site.Name), site.Id));

        TryLoadEntity();
        SiteValidator.Enabled = this.editorMode == SalePaymentSystemEditorMode.Creation;
        base.OnInit(e);
    }

    protected override void OnLoad(EventArgs e)
    {
        //string siteId = !IsPostBack ? this.entity.SiteId : SiteList.SelectedValue;
        //if (!string.IsNullOrEmpty(siteId))
        //{
        //    BXSaleCustomerTypeCollection customerTypes = BXSaleCustomerType.GetAllForSite(siteId);
        //    foreach (BXSaleCustomerType customerType in customerTypes)
        //    {
        //        BXTabControlTab tab = new BXTabControlTab();
        //        tab.Text = customerType.TextEncoder.Decode(customerType.Name);
        //        TabControl.Tabs.Add(tab);
        //    }
        //}

        MasterTitle = Page.Title = EditorMode == SalePaymentSystemEditorMode.Modification ? string.Format(GetMessage("PageTitle.Modify"), this.entity != null ? HttpUtility.HtmlEncode(this.entity.Name) : EntityId.ToString()) : GetMessage("PageTitle.Create");
        if(IsPostBack && !string.IsNullOrEmpty(SiteList.SelectedValue))
            this.entity.SiteId = SiteList.SelectedValue;
        base.OnLoad(e);
    }

    protected override void OnPreRender(EventArgs e)
    {
        if (EditorMode != SalePaymentSystemEditorMode.Modification)
            AddButton.Visible = DeleteButton.Visible = false;

        if (EditorError != SalePaymentSystemEditorError.None)
        {
            errorMessage.AddErrorMessage(this.errorText);
            if (EditorMode == SalePaymentSystemEditorMode.Modification && EditorError == SalePaymentSystemEditorError.IsNotFound)
                TabControl.Visible = false;

            AddButton.Visible = DeleteButton.Visible = false;
        }

        BXPage.Scripts.RequireUtils();
        foreach(BXSalePaymentControllerInfo info in ControllerInfos)
            foreach(string path in info.GetClientSideFilePaths())
                BXPage.RegisterScriptInclude(path);

        ClientScript.RegisterOnSubmitStatement(
            GetType(),
            "SaveData",
            "Bitrix.SalePaymentSystemEditor.getInstance().save();");

        base.OnPreRender(e);
    }

    private int entityId = 0;
    protected int EntityId
    {
        get { return this.entityId; }
    }

    private BXSalePaymentSystem entity = null;
    public BXSalePaymentSystem Entity
    {
        get { return this.entity; }
    }

    private BXSite entitySite = null;
    public BXSite EntitySite
    {
        get { return this.entitySite ?? (this.entitySite = BXSite.GetById(this.entity.SiteId)); }
    }

    public string TargetSiteId
    {
        get
        {
            if(IsPostBack && !string.IsNullOrEmpty(SiteList.SelectedValue))
                return SiteList.SelectedValue;

            return this.entity != null && !string.IsNullOrEmpty(this.entity.SiteId) ? this.entity.SiteId : BXSite.DefaultSite.Id;
        }
    }

	public string PrepareBackUrl()
	{
		Uri url = BXSefUrlManager.CurrentUrl;
		NameValueCollection qp = HttpUtility.ParseQueryString(url.Query);
		qp["siteId"] = TargetSiteId;
		return string.Concat(url.AbsolutePath, "?", qp.ToString());
	}

    private void TryLoadEntity()
    {
        if (EntityId <= 0)
        {
            this.editorMode = SalePaymentSystemEditorMode.Creation;
            this.entity = new BXSalePaymentSystem();
            this.entity.SiteId = !string.IsNullOrEmpty(Request["siteId"]) ? Request["siteId"] : BXSite.DefaultSite.Id;
        }
        else
        {
            this.editorMode = SalePaymentSystemEditorMode.Modification;
            BXSalePaymentSystemCollection col = BXSalePaymentSystem.GetList(
                new BXFilter(new BXFilterItem(BXSalePaymentSystem.Fields.Id, BXSqlFilterOperators.Equal, EntityId)),
                null,
                null,
                new BXQueryParams(new BXPagingOptions(0, 1)));

            if ((this.entity = col.Count > 0 ? col[0] : null) == null)
            {
                this.errorText = string.Format(GetMessageRaw("Error.UnableToFindPaymentSystem"), EntityId);
                this.editorError = SalePaymentSystemEditorError.IsNotFound;
                return;
            }
        }

        if (IsPostBack)
            return;

        ActiveChkBx.Checked = this.entity.Active;

        ListItem siteItem = SiteList.Items.FindByValue(this.entity.SiteId);
        if (siteItem != null)
            siteItem.Selected = true;

        NameTbx.Text = this.entity.Name;
        DescriptionTbx.Text = this.entity.Description;
        SortTbx.Text = this.entity.Sort.ToString();
        XmlIdTbx.Text = this.entity.XmlId;
    }

    private void TrySaveEntity()
    {
        if (this.entity == null)
            return;

        try
        {
            if(!this.entity.IsNew)
            {
                BXSalePaymentSystemConfigCollection prevConfigs =  BXSalePaymentSystemConfig.GetList(
                new BXFilter(new BXFilterItem(BXSalePaymentSystemConfig.Fields.PaymentSystemId, BXSqlFilterOperators.Equal, this.entity.Id)),
                null
                );

                foreach(BXSalePaymentSystemConfig config in prevConfigs)
                    config.Delete();
            }

            this.entity.Active = ActiveChkBx.Checked;
            if(this.editorMode == SalePaymentSystemEditorMode.Creation)
                this.entity.SiteId = SiteList.SelectedValue;
            this.entity.Name = NameTbx.Text;
            this.entity.Description = DescriptionTbx.Text;
            int i;
            this.entity.Sort = int.TryParse(SortTbx.Text, out i) ? i : 100;
            this.entity.XmlId = XmlIdTbx.Text;

            this.entity.Save();
            this.entityId = this.entity.Id;

            JavaScriptSerializer serializer = new JavaScriptSerializer();
            serializer.RegisterConverters(
                new JavaScriptConverter[] 
                { 
                    new BXSalePaymentSystemConfigJsonConv(),
                    new BXSalePaymentControllerSettingsJsonConv()
                }
                );

            if(!string.IsNullOrEmpty(EditorData.Value))
            {
                BXSalePaymentSystemConfig[] configs = serializer.Deserialize<BXSalePaymentSystemConfig[]>(EditorData.Value);
                if(configs != null && configs.Length > 0)
                    foreach(BXSalePaymentSystemConfig config in configs)
                    {
                        config.PaymentSystemId = this.entityId;
                        config.Create();
                    }
            }
        }
        catch (Exception exc)
        {
            this.errorText = exc.Message;
            this.editorError = EditorMode == SalePaymentSystemEditorMode.Creation ? SalePaymentSystemEditorError.Creation : SalePaymentSystemEditorError.Modification;
        }
    }

    private string errorText = string.Empty;
    private SalePaymentSystemEditorError editorError = SalePaymentSystemEditorError.None;
    public SalePaymentSystemEditorError EditorError
    {
        get { return this.editorError; }
    }

    private SalePaymentSystemEditorMode editorMode = SalePaymentSystemEditorMode.Creation;
    public SalePaymentSystemEditorMode EditorMode
    {
        get { return this.editorMode; }
    }

    protected override string BackUrl
    {
        get { return !string.IsNullOrEmpty(base.BackUrl) ? base.BackUrl : "SalePaymentSystemList.aspx"; }
    }

    protected void OnCommand(object sender, BXTabControlCommandEventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            return;

        switch (e.CommandName)
        {
            case "save":
                {
                    if (IsValid && EditorError == SalePaymentSystemEditorError.None)
                    {
                        TrySaveEntity();
                        if (EditorError == SalePaymentSystemEditorError.None)
                            GoBack();
                    }
                }
                break;
            case "apply":
                {
                    if (IsValid && EditorError == SalePaymentSystemEditorError.None)
                    {
                        TrySaveEntity();
                        if (EditorError == SalePaymentSystemEditorError.None)
                            Response.Redirect(string.Format("SalePaymentSystemEdit.aspx?id={0}&tabindex={1}", EntityId.ToString(), TabControl.SelectedIndex));
                    }
                }
                break;
            case "cancel":
                GoBack();
                break;
        }
    }

    protected void OnToolBarButtonClick(object sender, CommandEventArgs e)
    {
        if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            return;

        if (e.CommandName == "delete")
        {
            try
            {
                if (this.entity != null)
                    this.entity.Delete();

                GoBack();
            }
            catch (Exception ex)
            {
                this.errorText = ex.Message;
                this.editorError = SalePaymentSystemEditorError.Deletion;
            }
        }
    }

    protected void OnSiteListValidate(object source, ServerValidateEventArgs args)
    {
        args.IsValid = !string.IsNullOrEmpty(SiteList.SelectedValue);
    }

    private BXSaleCustomerTypeCollection customerTypes = null;
    public BXSaleCustomerTypeCollection CustomerTypes
    {
        get { return this.customerTypes ?? (this.customerTypes = BXSaleCustomerType.GetAllForSite(TargetSiteId)); }
    }

    private string customerTypeJson = null;
    public string CustomerTypeJson 
    {
        get
        {
            if(this.customerTypeJson != null)
                return this.customerTypeJson;

            if(CustomerTypes.Count == 0)
                return this.customerTypeJson = "[]";

            JavaScriptSerializer serializer = new JavaScriptSerializer();
            serializer.RegisterConverters(new JavaScriptConverter[] { new BXSaleCustomerTypeJsonConv() });
            return this.customerTypeJson = serializer.Serialize(CustomerTypes);
        }
    }

    private string paymentSysConfigJson = null;
    public string PaymentSysConfigJson 
    {
        get
        {
            if(this.paymentSysConfigJson != null)
                return this.paymentSysConfigJson;

            if(EntityId <= 0)
                return this.paymentSysConfigJson = "[]";

            BXSalePaymentSystemConfigCollection c =  BXSalePaymentSystemConfig.GetList(
                new BXFilter(new BXFilterItem(BXSalePaymentSystemConfig.Fields.PaymentSystemId, BXSqlFilterOperators.Equal, EntityId)),
                null
                );

            if(c.Count == 0)
                return this.paymentSysConfigJson = "[]";

            JavaScriptSerializer serializer = new JavaScriptSerializer();
            serializer.RegisterConverters(
                new JavaScriptConverter[] 
                { 
                    new BXSalePaymentSystemConfigJsonConv(),
                    new BXSalePaymentControllerSettingsJsonConv()
                }
                );
            return this.paymentSysConfigJson = serializer.Serialize(c);
        }
    }

    private string paymentControllerInfoJson = null;
    public string PaymentControllerInfoJson 
    {
        get
        {
            if(this.paymentControllerInfoJson != null)
                return this.paymentControllerInfoJson;

            if(ControllerInfos.Count == 0)
                return this.paymentControllerInfoJson = "[]";

            JavaScriptSerializer serializer = new JavaScriptSerializer();
            serializer.RegisterConverters(new JavaScriptConverter[] { new BXSalePaymentControllerInfoJsonConv() });
            return this.paymentControllerInfoJson = serializer.Serialize(ControllerInfos);
        }
    }

    private string saleOrderFieldJson = null;
    public string SaleOrderFieldJson
    {
        get 
        {
            if(this.saleOrderFieldJson != null)
                return this.saleOrderFieldJson;

            if(CustomerTypes.Count == 0)
                return this.saleOrderFieldJson = "[]";

            JavaScriptSerializer serializer = new JavaScriptSerializer();
            serializer.RegisterConverters(new JavaScriptConverter[] { new BXSaleOrderFieldJsonConv() });

            StringBuilder sb = new StringBuilder();
            sb.AppendLine("if(typeof(Bitrix.SaleOrderField)==\"undefined\"){ Bitrix.SaleOrderField = {}; }");
            foreach(BXSaleCustomerType type in CustomerTypes)
            {
                BXSaleOrderFieldCollection c = BXSaleOrderField.GetCustomerFields(type.Id, null);
                sb.AppendFormat("Bitrix.SaleOrderField[{0}] = {1};", type.Id.ToString(), c.Count > 0 ? serializer.Serialize(c) : "[]").AppendLine();
            }

            return this.saleOrderFieldJson = sb.ToString();
        }
    }

    private IList<BXSalePaymentControllerInfo> controllerInfos = null;
    public IList<BXSalePaymentControllerInfo> ControllerInfos
    {
        get { return  this.controllerInfos ?? (this.controllerInfos = BXSalePaymentManager.GetControllerInfos());}
    }
}

public enum SalePaymentSystemEditorMode
{
    Creation = 1,
    Modification = 2
}

public enum SalePaymentSystemEditorError
{
    None = 0,
    IsNotFound = 1,
    Creation = 3,
    Modification = 4,
    Deletion = 5
}
