﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using Bitrix.Sale;
using Bitrix.Security;
using Bitrix;
using Bitrix.Services.Text;
using Bitrix.DataLayer;

public partial class bitrix_admin_SalePaymentSystemList : BXAdminPage
{
    protected override void OnPreInit(EventArgs e)
    {
        if (!BXUser.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            BXAuthentication.AuthenticationRequired();

        base.OnPreInit(e);
    }

    protected override void OnInit(EventArgs e)
    {
        BXSiteCollection siteColl = BXSite.GetList(null, null, null, null, BXTextEncoder.EmptyTextEncoder);
        foreach (BXSite site in siteColl)
            SiteFilter.Values.Add(new ListItem(string.Concat("[", site.Id, "] ", site.Name), site.Id));

        MasterTitle = Page.Title = GetMessage("PageTitle");
        base.OnInit(e);
    }

    protected void ItemGrid_Select(object sender, BXSelectEventArgs e)
    {
        BXFilter f = new BXFilter(ItemFilter.CurrentFilter, BXSalePaymentSystem.Fields);
        BXOrderBy o = null;
        if (string.IsNullOrEmpty(e.SortExpression))
            o = new BXOrderBy(
                new BXOrderByPair(BXSalePaymentSystem.Fields.Sort, BXOrderByDirection.Asc),
                new BXOrderByPair(BXSalePaymentSystem.Fields.Id, BXOrderByDirection.Asc)
            );
        else
            o = new BXOrderBy(BXSalePaymentSystem.Fields, e.SortExpression);

        BXSalePaymentSystemCollection c = BXSalePaymentSystem.GetList(
           f,
           o,
           null,
           new BXQueryParams(e.PagingOptions),
           BXTextEncoder.EmptyTextEncoder
           );

        List<SalePaymentSystemWrapper> items = new List<SalePaymentSystemWrapper>();
        foreach(BXSalePaymentSystem entity in c)
            items.Add(new SalePaymentSystemWrapper(entity, this));

        e.Data = items;
    }

    protected void ItemGrid_SelectCount(object sender, BXSelectCountEventArgs e)
    {
		e.Count = BXSalePaymentSystem.Count(new BXFilter(ItemFilter.CurrentFilter, BXSalePaymentSystem.Fields));
    }

    protected void ItemGrid_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType != DataControlRowType.DataRow)
            return;

        BXGridViewRow r = (BXGridViewRow)e.Row;
        r.UserData.Add("ID", ((SalePaymentSystemWrapper)r.DataItem).Id);
    }

    protected void ItemGrid_Delete(object sender, BXDeleteEventArgs e)
	{
        if (!BXUser.IsCanOperate(BXSaleModule.Operations.AdminManagement))
            return;

        BXSalePaymentSystemCollection c = null;
        try
        {
            BXFilter f = null;
            if(e.Keys != null)
                f = new BXFilter(new BXFilterItem(BXSalePaymentSystem.Fields.Id, BXSqlFilterOperators.Equal, e.Keys["ID"]));
            c = BXSalePaymentSystem.GetList(f, null);

        }
        catch (Exception ex)
        {
            ErrorMessage.AddErrorMessage(ex.Message);
            return;
        }

        bool aboutError = false;
        foreach (BXSalePaymentSystem entity in c)
        {
            try
            {
                entity.Delete();
                e.DeletedCount++;
            }
            catch(Exception exc)
            {
                ErrorMessage.AddErrorMessage(exc.Message);

                if (!aboutError)
                    aboutError = true;
            }
        }

        if(aboutError)
            ErrorMessage.AddErrorMessage(GetMessage("SalePaymentSystem.DeleteFailure"));
	}
}

public class SalePaymentSystemWrapper
{
    private BXSalePaymentSystem entity = null;
    private BXAdminPage page = null;
    public SalePaymentSystemWrapper(BXSalePaymentSystem entity, BXAdminPage page)
    {
        if(entity == null)
            throw new ArgumentNullException("entity");

        this.entity = entity;

        if(page == null)
            throw new ArgumentNullException("page");

        this.page = page;
    }

    public int Id
    {
        get { return this.entity.Id; }
    }

    public string Name
    {
        get { return HttpUtility.HtmlEncode(this.entity.Name); }
    }

    public string SiteId
    {
        get { return HttpUtility.HtmlEncode(this.entity.SiteId); }
    }

    public bool Active
    {
        get { return this.entity.Active; }
    }

    public int Sort
    {
        get { return this.entity.Sort; }
    }
   
    public string DateOfCreation
    {
        get { return this.entity.CreatedUtc.ToLocalTime().ToString("d"); }
    }

    public string DateOfLastUpdate
    {
        get { return this.entity.LastUpdatedUtc.ToLocalTime().ToString("d"); }
    }
}
