﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Web.UI.WebControls;
using Bitrix.Catalog;
using Bitrix.DataLayer;
using Bitrix.IBlock;
using Bitrix.Services.Text;
using Bitrix.UI;
using System.Web;
using Bitrix.Currency;
using Bitrix.Services.Js;
using Bitrix.Modules;

namespace Bitrix.Sale.AdminPages
{

	public partial class ProductSearch : BXAdminPage
	{
		bool sectionFilterLoaded;
		List<BXCatalog> Catalogs { get; set; }
		BXCatalogPriceType PriceType { get; set; }
		BXCurrency Currency { get; set; }

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			PriceType = BXCatalogPriceType.BasePriceType;
			Currency = BXCurrency.GetByAlphaCode(Request.QueryString["currency"]);

			Catalogs = BXCatalog.GetList(
				new BXFilter(
					new BXFilterItem(BXCatalog.Fields.IBlock.Active, BXSqlFilterOperators.Equal, "Y"),
					new BXFilterItem(BXCatalog.Fields.IBlock.Sites.SiteId, BXSqlFilterOperators.Equal, Request.QueryString["site"])
				),
				new BXOrderBy(
					new BXOrderByPair(BXCatalog.Fields.IBlock.Name, BXOrderByDirection.Asc)
				),
				new BXSelectAdd(
					BXCatalog.Fields.IBlock.ID,
					BXCatalog.Fields.IBlock.Name
				),
				null, 
				BXTextEncoder.EmptyTextEncoder
			)
			.Where(
				x => 
					x.IBlock != null 
					&& (
						BXUser.IsCanOperate(BXIBlock.Operations.IBlockPublicRead, "iblock", x.IBlockId)
						|| BXUser.IsCanOperate(BXIBlock.Operations.IBlockAdminRead, "iblock", x.IBlockId)
					)
			)
			.ToList();

			CatalogFilter.Items.Clear();
			CatalogFilter.Items.AddRange(
				Catalogs
				.ConvertAll(x => new ListItem(x.IBlock.Name, x.Id.ToString()))
				.ToArray()
			);

			//GridView.Load += (sender, ev) => { LoadSectionFilter(); }; // hack to make the section filter work
		}
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);
			BXPage.Scripts.RequireUtils();
		}

		protected void GridView_Select(object sender, Bitrix.UI.BXSelectEventArgs e)
		{
			bool hasCatalog = false;
			foreach (var item in AdminFilter.CurrentFilter)
			{
				if (string.Equals(item.filterName, "CatalogId", StringComparison.OrdinalIgnoreCase) && item.filterValue is int && Catalogs.Exists(x => x.Id == (int)item.filterValue))
				{
					hasCatalog = true;
					break;
				}
			}

			if (!hasCatalog)
			{
				e.Data = new List<Wrapper>();
				return;
			}
			
			var filter = BuildFilter();
			var orderBy = new BXOrderBy(BXCatalogItem.Fields, e.SortExpression);
			var select = new BXSelect(
				BXCatalogItem.Fields.IBlockElement,
				BXCatalogItem.Fields.IBlockElement.IBlock.XmlId
			);
			var queryParams = new BXQueryParams(e.PagingOptions);

			e.Data = BXCatalogItem.GetList(
				filter,
				orderBy,
				select,
				queryParams,
				BXTextEncoder.EmptyTextEncoder
			)
			.ConvertAll(x => new Wrapper(x, this));

		}
		protected void GridView_SelectCount(object sender, BXSelectCountEventArgs e)
		{
			bool hasCatalog = false;
			foreach (var item in AdminFilter.CurrentFilter)
			{
				if (string.Equals(item.filterName, "CatalogId", StringComparison.OrdinalIgnoreCase) 
					&& item.filterOperator == BXSqlFilterOperators.Equal
					&& item.filterValue is int 
					&& Catalogs.Exists(x => x.Id == (int)item.filterValue))
				{
					hasCatalog = true;
					break;
				}
			}
			
			if (!hasCatalog)
			{
				e.Count = 0;
				return;
			}

			var filter = BuildFilter();
			e.Count = BXCatalogItem.Count(filter);
		}

		private void LoadSectionFilter()
		{
			//if (sectionFilterLoaded)
			//    return;
			//sectionFilterLoaded = true;

			//SectionFilter.Items.Clear();
			//int id;
			//BXCatalog catalog;
			//if (
			//    !string.IsNullOrEmpty(CatalogFilter.SelectedValue)
			//    && int.TryParse(CatalogFilter.SelectedValue, out id)
			//    && (catalog = Catalogs.Find(x => x.Id == id)) != null
			//)
			//{
			//    SectionFilter.Items.Add(new ListItem("Верхний уровень", "0"));
			//    SectionFilter.Items.AddRange(
			//        BXIBlockSection.GetList(
			//            new BXFilter(
			//                new BXFilterItem(BXIBlockSection.Fields.IBlock.ID, BXSqlFilterOperators.Equal, catalog.IBlockId)
			//            ),
			//            new BXOrderBy(new BXOrderByPair(BXIBlockSection.Fields.LeftMargin, BXOrderByDirection.Asc)),
			//            new BXSelect(
			//                BXIBlockSection.Fields.ID,
			//                BXIBlockSection.Fields.Name,
			//                BXIBlockSection.Fields.DepthLevel
			//            ),
			//            null,
			//            BXTextEncoder.EmptyTextEncoder
			//        )
			//        .Select(x => new ListItem(BXStringUtility.Clone(". . ", x.DepthLevel) + x.Name, x.Id.ToString()))
			//        .ToArray()
			//    );

			//    var values = Request.Form.GetValues(SectionFilter.UniqueID);
			//    if (values != null && values.Length > 0)
			//    {
			//        foreach (ListItem li in SectionFilter.Items)
			//            li.Selected = Array.IndexOf(values, li.Value) >= 0;
			//    }
			//}
		}
		private BXFilter BuildFilter()
		{
			BXFilterOr sectionFilter = null;
			
			var mainFormFilter = new BXFormFilter(AdminFilter.CurrentFilter);
			mainFormFilter.RemoveAll(x => 
			{ 
				if (!x.filterName.StartsWith(":"))
					return false;
				
				sectionFilter = sectionFilter ?? new BXFilterOr();
				
				//if (x.filterName == ":InRootSection")
				//	sectionFilter.Add(new BXFilterItem(BXCatalogItem.Fields.IBlockElement.InRootSection, x.filterOperator, x.filterValue));
				//else if (x.filterName == ":SectionId")
				//	sectionFilter.Add(new BXFilterItem(BXCatalogItem.Fields.IBlockElement.Sections.Section.ID, x.filterOperator, x.filterValue));
				
				return true;
			});

			var filter = new BXFilter(mainFormFilter, BXCatalogItem.Fields);
			if (sectionFilter != null)
				filter.Add(sectionFilter);
			
			return filter;
		}

		protected void Catalog_OnBuildFilter(object sender, BXCustomAdminFilter.BuildFilterEventArgs e)
		{
			int id;
			if (!string.IsNullOrEmpty(CatalogFilter.SelectedValue) && int.TryParse(CatalogFilter.SelectedValue, out id))
				e.Result.Add(new BXFormFilterItem("CatalogId", id, BXSqlFilterOperators.Equal));
		}
		protected void Catalog_OnLoadState(object sender, BXCustomAdminFilter.StateEventArgs e)
		{
			if (e.State.Contains("id"))
			{
				try
				{
					CatalogFilter.SelectedValue = (string)e.State["id"];
				}
				catch
				{
				}
			}
			LoadSectionFilter();
		}
		protected void Catalog_OnSaveState(object sender, BXCustomAdminFilter.StateEventArgs e)
		{
			e.State["id"] = CatalogFilter.SelectedValue ?? "";			
		}
		protected void Catalog_OnReset(object sender, EventArgs e)
		{
			CatalogFilter.SelectedIndex = 0;
		}

		//protected void Section_OnBuildFilter(object sender, BXCustomAdminFilter.BuildFilterEventArgs e)
		//{
		//    bool inRoot = false;
		//    var ids = new List<int>();
		//    foreach (ListItem li in SectionFilter.Items)
		//    {
		//        if (!li.Selected || string.IsNullOrEmpty(li.Value))
		//            continue;
				
		//        int id;
		//        if (int.TryParse(li.Value, out id))
		//        {
		//            if (id == 0)
		//                inRoot = true;
		//            else if (!ids.Contains(id))
		//                ids.Add(id);
		//        }
		//    }

		//    if (inRoot)
		//        e.Result.Add(new BXFormFilterItem(":InRootSection", true, BXSqlFilterOperators.Equal));
			
		//    if (ids.Count == 1)
		//        e.Result.Add(new BXFormFilterItem(":SectionId", ids[0], BXSqlFilterOperators.Equal));
		//    else if (ids.Count > 1)
		//        e.Result.Add(new BXFormFilterItem(":SectionId", ids, BXSqlFilterOperators.In));
		//}
		//protected void Section_OnLoadState(object sender, BXCustomAdminFilter.StateEventArgs e)
		//{
		//    List<string> ids;
		//    if (!e.State.Contains("ids") || (ids = e.State["ids"] as List<string>) == null)
		//        return;

		//    foreach (ListItem li in SectionFilter.Items)
		//        li.Selected = ids.Contains(li.Value);
		//}
		//protected void Section_OnSaveState(object sender, BXCustomAdminFilter.StateEventArgs e)
		//{
		//    e.State["ids"] = (from ListItem li in SectionFilter.Items where li.Selected select li.Value).ToList();
		//}
		//protected void Section_OnReset(object sender, EventArgs e)
		//{
		//    foreach (ListItem li in SectionFilter.Items)
		//        li.Selected = false;
		//}

		public class Wrapper
		{
			BXCatalogItem item;
			ProductSearch page;
			BXCatalogPrice price;
			decimal? priceValue;
			BXCurrency priceCurrency;

			public Wrapper(BXCatalogItem item, ProductSearch page)
			{
				this.item = item;
				this.page = page;
			}

			public string Id 
			{
				get
				{
					return item.Id.ToString();
				}
			}

			public string Active 
			{
				get
				{
					return item.IBlockElement.Active ? page.GetMessageRaw("Kernel.Yes") : page.GetMessageRaw("Kernel.No");
				}
			}

			public string Name
			{
				get
				{
					return HttpUtility.HtmlEncode(item.IBlockElement.Name ?? "");
				}
			}

			public string Price
			{
				get
				{
					GetPrice();
					return HttpUtility.HtmlEncode(priceCurrency.MoneyToString(priceValue.Value));
				}
			}

			private void GetPrice()
			{
				if (priceValue != null)
					return;

				BXCatalogPrice catalogPrice;
				if (!BXCatalogPrice.TryGetPrice(item.Id, page.PriceType, BXTextEncoder.EmptyTextEncoder, new[] { BXCatalogPrice.Fields.Currency }, out catalogPrice))
				{
					priceValue = 0m;
					priceCurrency = page.Currency;					
					return;
				}

				price = catalogPrice;
				priceValue = catalogPrice.Value;
				priceCurrency = catalogPrice.Currency;						
			}

			public string ProductInfo
			{
				get
				{
					GetPrice();
					var priceVal = BXCurrency.ConvertMoney(priceValue.Value, priceCurrency, page.Currency);
					var json = new Dictionary<string, object>
					{
						{ "id", item.Id },
						{ "code", !string.IsNullOrEmpty(item.IBlockElement.XmlId) ? item.IBlockElement.XmlId : item.IBlockElementId.ToString() },
						{ "catalogcode", !string.IsNullOrEmpty(item.IBlockElement.IBlock.XmlId) ? item.IBlockElement.IBlock.XmlId : item.IBlockElement.IBlockId.ToString() },
						{ "name", item.IBlockElement.Name ?? "" },
						{ "url", ResolveUrl() },
						{ "price", priceVal.ToString("F2") },
						{ "vat", (item.VATRate != null ? item.VATRate.Value : 0m).ToString("N") },
						{ "vatincluded", price != null ? price.VATIncluded : true },
						{ "weight", item.Weight.ToString("N") }
					};
					return BXJSUtility.BuildJSON(json);
				}
			}

			private string ResolveUrl()
			{
				var cmd = new BXCommand("Bitrix.Search.ProvideUrl");
				cmd.AddP("moduleId", "iblock");
				cmd.AddP("itemGroup", item.IBlockElement.IBlockId.ToString());
				cmd.AddP("itemId", item.IBlockElementId.ToString());
				cmd.SiteId = page.Request.QueryString["site"];
				cmd.Send();

				foreach (var r in cmd.CommandResultDictionary.Values)
				{
					if (r.CommandResult != BXCommandResultType.Ok || r.Result == null)
						continue;
					var url = new Uri(page.Request.Url, r.Result.ToString());
					return url.PathAndQuery;
				}
				
				return "";
			}
		}
	}
}