﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using Bitrix.DataTypes;
using System.Collections.Specialized;

namespace Bitrix.Sale.UI
{
	public partial class OrderMappingEditor : BXControl, IPostBackDataHandler
	{
		// PROPERTIES
		protected BXParamsBag<BXSaleOrderFieldMapping> mappings = new BXParamsBag<BXSaleOrderFieldMapping>();
		public List<KeyValuePair<string, string>> Fields { get; private set; }
		public int CustomerTypeId { get; set; }

		// CONSTRUCTOR
		public OrderMappingEditor()
		{
			Fields = new List<KeyValuePair<string, string>>();
		}
		
		// METHODS
		public void LoadMappings(IDictionary<string, BXSaleOrderFieldMapping> mappings)
		{
			this.mappings.Clear();
			if (mappings == null)
				return;
			this.mappings.Assign(mappings);
		}

		public void SaveMappings(IDictionary<string, BXSaleOrderFieldMapping> mappings)
		{
			if (mappings == null)
				throw new ArgumentNullException("mappings");
			foreach (var p in this.mappings)
				mappings[p.Key] = new BXSaleOrderFieldMapping { Type = p.Value.Type, Value = p.Value.Value };
		}

		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);
			BXPage.Scripts.RequireCore(Page);
		}

		#region IPostBackDataHandler Members
		bool IPostBackDataHandler.LoadPostData(string postDataKey, NameValueCollection postCollection)
		{
			mappings.Clear();
			foreach (var f in Fields)
			{
				var type = postCollection[string.Concat(postDataKey, "$f$", f.Key, "$type")];
				var mappingType = BXSaleOrderFieldMappingType.DirectValue;
				if (type == "user")
					mappingType = BXSaleOrderFieldMappingType.UserField;
				else if (type == "user_property")
					mappingType = BXSaleOrderFieldMappingType.UserProperty;
				else if (type == "order")
					mappingType = BXSaleOrderFieldMappingType.OrderField;
				else if (type == "order_property")
					mappingType = BXSaleOrderFieldMappingType.OrderProperty;

				var value = postCollection[string.Concat(postDataKey, "$f$", f.Key, mappingType != BXSaleOrderFieldMappingType.DirectValue ? "$value1" : "$value2")];
				mappings.Add(
					f.Key, 
					new BXSaleOrderFieldMapping
					{
						Type = mappingType,
						Value = value ?? ""
					}
				);
			}
			return false;
		}
		void IPostBackDataHandler.RaisePostDataChangedEvent()
		{			
		}
		#endregion
	}
}