﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using Bitrix.Components;
using Bitrix.Security;
using System.Collections.Generic;
using Bitrix.Modules;
using Bitrix.UI;
using Bitrix.Sale.SaleCart;
using Bitrix.DataLayer;
using Bitrix.DataTypes;
using System.Web;
using Bitrix.Services.Text;

namespace Bitrix.Sale.Components
{
	public partial class SaleCartLineComponent : BXComponent
	{
		int curUserId = -1;

		//Id текущего пользователя, у анонимов - 0
		public int CurrentUserId
		{
			get { return curUserId != -1 ? curUserId : (curUserId = 
				(BXPrincipal.Current != null && BXPrincipal.Current.Identity.IsAuthenticated ? ((BXIdentity)BXPrincipal.Current.Identity).Id : 0)); }
		}

		BXSaleCart GetCart(BXFilterItem[] items)
		{
			var carts = BXSaleCart.GetList(
				new BXFilter(items), 
				new BXOrderBy(new BXOrderByPair(BXSaleCart.Fields.Id, BXOrderByDirection.Desc)));
			if (carts.Count > 0)
				return carts[0];
			return null;
		}


		BXSaleCart cart;

		protected void Page_Load(object sender, EventArgs e)
		{

			FillCart();

			IncludeComponentTemplate();
		}

		void FillCart()
		{

			var cartGuid = Request.Cookies.Get(BXSaleCart.ExternalStorageUIDKey);

			BXSaleCart userCart = null; // корзина пользователя
			BXSaleCart cookieCart = null; // корзина в куках

			List<BXFilterItem> items = new List<BXFilterItem>();

			if (CurrentUserId > 0)
			{
				userCart = BXSaleCart.GetByUserId(CurrentUserId);
			}

			if (cartGuid != null && !String.IsNullOrEmpty(cartGuid.Value))
			{
				cookieCart = BXSaleCart.GetByGuid(cartGuid.Value);
			}

			// если есть корзина пользователя и корзина анонима, то нужно очистить корзину анонима, переложив все товары из нее
			// в корзину пользователя, а корзину анонима затем удалить, и поменять информацию в куки

			if (cookieCart != null && (userCart == null || userCart.UniqueId != cookieCart.UniqueId) && CurrentUserId >0)
			{
				var id = BXSaleCart.Merge(cookieCart.Id,userCart == null ? -1 : userCart.Id, CurrentUserId);
				if (id > 0)
				{
					cart = BXSaleCart.GetById(id);
					Response.SetCookie(new HttpCookie(BXSaleCart.ExternalStorageUIDKey, cart.UniqueId));
				}
			}
			else
			{
				if (cookieCart!=null && cookieCart.UserId > 0 && CurrentUserId == 0) //в куки анонима лежит корзина зарегистрированного пользователя
				{												// сбрасываем информацию о корзине
					Response.Cookies[BXSaleCart.ExternalStorageUIDKey].Expires = DateTime.Now.AddDays(-1);
					cart = null;
				}
				else
					cart = userCart ?? cookieCart;
			}

			
			if (cart != null)
			{
				var cartItems = BXSaleCartItemManager.GetByCart(cart, new List<string>(), CurrentUserId, BXSite.Current.Id);
				var count = 0;
				foreach ( var item in cartItems )
				{
					if (!item.HasProductInfo)
						continue;
					if ((item.QuantityInStock != 0 || item.CanBuyMoreThenExists) && !item.IsPutAside)
						count++;
				}
				QuantityInCart = count;
			}
		}

		public int QuantityInCart { get; set; }

		string cartUrl;

		public string CartUrl
		{
			get
			{
				if (cartUrl != null)
					return cartUrl;

				var replace = new BXParamsBag<object>();
				replace.Add("UserId", CurrentUserId);
				return (cartUrl = ResolveTemplateUrl(Parameters.GetString("CartUrlTemplate", "cart.aspx"), replace));
			}
		}

		protected override void PreLoadComponentDefinition()
		{
			base.Title = GetMessage("Component.Title");
			base.Description = GetMessage("Component.Description");
			base.Icon = "images/sale_cart_line.gif";

			Group = new BXComponentGroup("Store", GetMessage("Group"), 100, BXComponentGroup.Store);

			ParamsDefinition.Add(
				"CartUrlTemplate",
				new BXParamText(
					GetMessageRaw("Param.CartUrlTemplate"),
					"cart.aspx",
					BXCategory.UrlSettings
					)
				);

		}

	
	}
}
