﻿using System;
using System.Data;
using System.Linq;
using System.Configuration;
using System.Collections;
using Bitrix.Components;
using Bitrix.Security;
using System.Collections.Generic;
using Bitrix.Modules;
using Bitrix.UI;
using Bitrix.Sale.SaleCart;
using Bitrix.DataLayer;
using Bitrix.DataTypes;
using System.Web;
using Bitrix.Services.Text;
using Bitrix.Currency;

namespace Bitrix.Sale.Components
{
	public partial class SaleCartComponent : BXComponent
	{
		int curUserId = -1;

		//Id текущего пользователя, у анонимов - 0
		public int CurrentUserId
		{
			get { return curUserId != -1 ? curUserId : (curUserId = 
				(BXPrincipal.Current != null && BXPrincipal.Current.Identity.IsAuthenticated ? ((BXIdentity)BXPrincipal.Current.Identity).Id : 0)); }
		}

		public string OrderFormUrl
		{
			get { return Parameters.GetString("OrderFormUrl", ""); }
		}

		BXSaleCart GetCart(BXFilterItem[] items)
		{
			var carts = BXSaleCart.GetList(
				new BXFilter(items), 
				new BXOrderBy(new BXOrderByPair(BXSaleCart.Fields.Id, BXOrderByDirection.Desc)));
			if (carts.Count > 0)
				return carts[0];
			return null;
		}

		List<BXSaleCartProduct> readyProducts;

		public List<BXSaleCartProduct> ReadyProducts
		{
			get { return readyProducts; }
		}

		List<BXSaleCartProduct> putAsideProducts;

		public List<BXSaleCartProduct> PutAsideProducts
		{
			get { return putAsideProducts; }
		}

		List<BXSaleCartProduct> unAvailableProducts;

		public List<BXSaleCartProduct> UnAvailableProducts
		{
			get { return unAvailableProducts; }
		}

		public bool ShowVAT
		{
			get
			{
				return Parameters.GetBool("ShowVAT", false);
			}
			set
			{
				Parameters["ShowVAT"] = value.ToString();
			}
		}

		BXSaleCart cart;

		protected void Page_Load(object sender, EventArgs e)
		{


			FillProducts("");

			IncludeComponentTemplate();
		}

		void FillProducts(string newCouponString)
		{
			readyProducts = new List<BXSaleCartProduct>();
			putAsideProducts = new List<BXSaleCartProduct>();
			unAvailableProducts = new List<BXSaleCartProduct>();

			BXPublicPage bitrixPage = Page as BXPublicPage;
			if (bitrixPage != null && !IsComponentDesignMode)
			{
				if (Parameters.GetBool("SetPageTitle", true))
					bitrixPage.BXTitle = GetMessageRaw("PageTitle");
			}

			var cartGuid = Request.Cookies.Get(BXSaleCart.ExternalStorageUIDKey);

			BXSaleCart userCart = null; // корзина пользователя
			BXSaleCart cookieCart = null; // корзина в куках

			List<BXFilterItem> items = new List<BXFilterItem>();

			if (CurrentUserId > 0)
			{
				userCart = BXSaleCart.GetByUserId(CurrentUserId);
			}

			if (cartGuid != null && !String.IsNullOrEmpty(cartGuid.Value))
			{
				cookieCart = BXSaleCart.GetByGuid(cartGuid.Value);
			}

			// если есть корзина пользователя и корзина анонима, то нужно очистить корзину анонима, переложив все товары из нее
			// в корзину пользователя, а корзину анонима затем удалить, и поменять информацию в куки

			if (cookieCart != null && (userCart == null || userCart.UniqueId != cookieCart.UniqueId) && CurrentUserId >0)
			{
				var id = BXSaleCart.Merge(cookieCart.Id, userCart == null ? -1 : userCart.Id, CurrentUserId);
				if (id > 0)
				{
					cart = BXSaleCart.GetById(id);
					Response.SetCookie(new HttpCookie(BXSaleCart.ExternalStorageUIDKey, cart.UniqueId));
				}
			}
			else
			{
				if (cookieCart!=null && cookieCart.UserId > 0 && CurrentUserId == 0) //в куки анонима лежит корзина зарегистрированного пользователя
				{												// сбрасываем информацию о корзине
					Response.Cookies[BXSaleCart.ExternalStorageUIDKey].Expires = DateTime.Now.AddDays(-1);
					cart = null;
				}
				else
					cart = userCart ?? cookieCart;
			}

			if (cart != null)
			{
				foreach (var item in BXSaleCartItemManager.GetByCart(cart, DisplayItemProperties, curUserId, BXSite.Current.Id))
				{
					if (!item.HasProductInfo)
						continue;

					if (item.QuantityInStock == 0 && !item.CanBuyMoreThenExists)
						unAvailableProducts.Add(item);
					else
						if (!item.IsPutAside){
							readyProducts.Add(item);
						}
						else
							putAsideProducts.Add(item);
				}
			}

			if (!BXStringUtility.IsNullOrTrimEmpty(newCouponString))
			{
				var coupon = BXSaleDiscountCoupon.FindCoupon(newCouponString);
				if(coupon!=null)
				{
					var link = BXCouponToCart.GetList(
						new BXFilter(
							new BXFilterItem(BXCouponToCart.Fields.CartId, BXSqlFilterOperators.Equal, cart.Id),
							new BXFilterItem(BXCouponToCart.Fields.CouponId, BXSqlFilterOperators.Equal, coupon.Id)
						),
						null,
						null,
						null
					)
					.FirstOrDefault();

					if (link != null)
					{
						BXCouponToCart.Delete(
							new BXFilter(
								new BXFilterItem(BXCouponToCart.Fields.CartId, BXSqlFilterOperators.Equal, cart.Id),
								new BXFilterItem(BXCouponToCart.Fields.CouponId, BXSqlFilterOperators.Equal, coupon.Id)
							)
						);
					}
						
					link = new BXCouponToCart { CartId = cart.Id, CouponString = newCouponString, Active = true, CouponId = coupon.Id, DiscountId = coupon.DiscountId };
					link.Save();
				}
			}
			var cartCoupons = new List<String>();
			if(cart!=null)
				cartCoupons = cart.Coupons.FindAll(
					x => x.Active = true).ConvertAll<string>(x => x.CouponString);

			FillDiscountValues(readyProducts, cartCoupons);
			foreach (var item in readyProducts)
				AddOwnProperties(item);
		}

		void FillDiscountValues(List<BXSaleCartProduct> prods, List<string> coupons)
		{
			if (prods == null || prods.Count == 0)
				return;

			var discountInfo = BXSaleDiscountManager.FindSuitableDiscounts(
				new BXSaleFindDiscountInfo(CurrentUserId, BXPrincipal.Current.GetAllRoleIds(true),
					prods, BXSite.Current.Id, BXCurrency.NationalCurrencyId, coupons));

			BXSaleDiscountFindResultInfo info = null; 

			if(discountInfo!=null)
				foreach (var prod in prods)
				{

					if (discountInfo.Discounts.TryGetValue(prod.ElementId, out info))
					{

						if (prod.DiscountValueNationalCurrency <= info.Value)
						{
							prod.DiscountValueNationalCurrency = info.Value;

							if(info.Discounts.Count>0)
								prod.DiscountName = info.Discounts[0].Name;

							prod.DiscountedPriceDisplayHtml = BXCurrency.NationalCurrency.MoneyToString(
								BXCurrency.ConvertMoney(
									prod.Price, prod.PriceCurrency, BXCurrency.NationalCurrency) - prod.DiscountValueNationalCurrency);
						}
					}

				}

			discountInfo.Discounts.TryGetValue(0,out fullOrderDiscount);


		}
		BXSaleDiscountFindResultInfo fullOrderDiscount;

		public BXSaleDiscountFindResultInfo FullOrderDiscount {
			get { return fullOrderDiscount; }
			set { fullOrderDiscount = value; }
		}

		void AddOwnProperties(BXSaleCartProduct item)
		{
			var propPrice = new BXSaleCartProductProperty();
			var cost = item.GetTotalCostNationalCurrency() - item.DiscountValueNationalCurrency;// -item.DiscountValueNationalCurrency;
			//if (cost < 0)
			//	cost = 0;

			propPrice.DisplayHtml = BXCurrency.NationalCurrency.MoneyToString(cost);
			propPrice.Name = "ItemPrice";
			propPrice.Title = GetMessage("ItemProperty.Price");
			propPrice.Value = cost;
			item.Properties["ItemPrice"]= propPrice;

			var propDiscount = new BXSaleCartProductProperty();
			propDiscount.DisplayHtml = BXCurrency.NationalCurrency.MoneyToString(item.DiscountValueNationalCurrency);
			propDiscount.Name = "DiscountValue";
			propDiscount.Title = GetMessage("ItemProperty.DiscountValue");
			propDiscount.Value = item.DiscountValueNationalCurrency;
			item.Properties["ItemDiscountValue"] = propDiscount;

			var propQuantity = new BXSaleCartProductProperty();
			propQuantity.DisplayHtml = item.QuantityInCart.ToString();
			propQuantity.Name = "ItemQuantity";
			propQuantity.Title = GetMessage("ItemProperty.Quantity");
			propQuantity.Value = item.QuantityInCart;
			item.Properties["ItemQuantity"]= propQuantity;

			var propName = new BXSaleCartProductProperty();
			propName.DisplayHtml = String.Format(@"<a href=""{0}"">{1}</a>",item.DetailUrl,HttpUtility.HtmlEncode(item.Name));
			propName.Name = "ItemName";
			propName.Title = GetMessage("ItemProperty.Name");
			propName.Value = item.Name;
			item.Properties["ItemName"] = propName;

			var propImageUrl = new BXSaleCartProductProperty();
			propImageUrl.DisplayHtml = String.Format(@"<img src=""{0}""/>", item.ImageUrl);
			propImageUrl.Name = "ItemImageUrl";
			propImageUrl.Title = GetMessage("ItemProperty.ImageUrl");
			propImageUrl.Value = item.ImageUrl;
			item.Properties["ItemImageUrl"] = propImageUrl;

			var propWeight = new BXSaleCartProductProperty();
			propWeight.DisplayHtml = item.Weight.ToString()+" "+GetMessage("Label.Gramm");
			propWeight.Name = "ItemWeight";
			propWeight.Title = GetMessage("ItemProperty.Weight");
			propWeight.Value = item.Weight;
			item.Properties["ItemWeight"] = propWeight;
		}

		public void UpdateQuantity( BXParamsBag<object> commandParameters, List<string> errors)
		{
			var cartItems = commandParameters.Get<Dictionary<int, int>>("cartItems");
			if (cartItems == null)
				return;
			// пересчитываем количество только у готовых к заказу
			int quantity = 0;
			foreach (var prod in ReadyProducts)
				if (cartItems.TryGetValue(prod.CartItemId, out quantity))// нужно обновить количество
				{
					if (prod.CanBuyMoreThenExists || prod.QuantityInStock > quantity)
						continue;

					if (prod.QuantityInStock == 0)
						cartItems.Remove(prod.CartItemId); // если товара нет, ничего не делаем, иначе потеряем реальное желаемое пользователем количество

					if (!prod.CanBuyMoreThenExists && quantity > prod.QuantityInStock) // если хотят установить кол-во большее чем есть, и контролируем остаток
					{
						errors.Add(String.Format(GetMessage("Error.SetQuantityForItemFailed"), prod.Name, prod.QuantityInStock));
					}
				}

			if (cartItems.Count > 0)
				BXSaleCartItemManager.UpdateQuantityInCart(cart.Id, cartItems, CurrentUserId);
		}

		public override bool ProcessCommand(string commandName, BXParamsBag<object> commandParameters, List<string> commandErrors)
		{
			if (cart == null)
			{
				commandErrors.Add(GetMessage("Error.CartNotFound"));
				return false;
			}
			if (commandName.Equals("UpdateQuantity",StringComparison.OrdinalIgnoreCase))
			{
				UpdateQuantity(commandParameters, commandErrors);

				//if (commandErrors.Count > 0)
				//{
					FillProducts(commandParameters.GetString("newCouponString",""));
					return commandErrors.Count == 0;
				//}
			}
			else if (commandName.Equals("Shelve", StringComparison.OrdinalIgnoreCase) 
				|| commandName.Equals("UnShelve", StringComparison.OrdinalIgnoreCase)
				|| commandName.Equals("Delete", StringComparison.OrdinalIgnoreCase)
				|| commandName.Equals("Notify", StringComparison.OrdinalIgnoreCase)
				|| commandName.Equals("UnNotify", StringComparison.OrdinalIgnoreCase)
				)
			{
				int itemId = 0;
				var paramId = commandParameters["cartItemId"];

				if (paramId == null || !Int32.TryParse(paramId.ToString(),out itemId))
				{
					commandErrors.Add(GetMessage("Error.CartItemIdNotFound"));
					return false;
				}

				BXSaleCartItem item = BXSaleCartItem.GetById(itemId);
				if (item == null || item.CartId != cart.Id)
				{
					commandErrors.Add(GetMessage("Error.WrongCartItemSupplied"));
					return false;
				}
				if (commandName.Equals("Delete", StringComparison.OrdinalIgnoreCase))
				{
					item.Delete();
					return true;
				}
				if (commandName.Equals("Notify", StringComparison.OrdinalIgnoreCase) || commandName.Equals("UnNotify", StringComparison.OrdinalIgnoreCase))
				{
					item.NotifyWhenInStock = commandName.Equals("Notify", StringComparison.OrdinalIgnoreCase);
				}
				else if (commandName.Equals("Shelve", StringComparison.OrdinalIgnoreCase) )
				{
					item.IsPutAside = true;
				}
				else if (commandName.Equals("UnShelve", StringComparison.OrdinalIgnoreCase))
				{
					var product = BXSaleCartItemManager.GetProduct(item, cart.ResolverId, new List<string>(), CurrentUserId);
					if (product == null)
					{
						commandErrors.Add(GetMessage("ProductInfoNotFound"));
						return false;
					}
					if (product.QuantityInStock <= product.QuantityInCart && !product.CanBuyMoreThenExists)
						item.Quantity = product.QuantityInStock;
					item.IsPutAside = false;
				}
				item.Save();
			}
			else if (commandName.Equals("FormOrder", StringComparison.OrdinalIgnoreCase))
			{
				UpdateQuantity(commandParameters, commandErrors);
				if (commandErrors.Count > 0)
					return false;
				FillProducts("");
				foreach (var prod in ReadyProducts)
				{
					if (!prod.CanBuyMoreThenExists && prod.QuantityInCart > prod.QuantityInStock) // если хотят установить кол-во большее чем есть, и контролируем остаток
					{
						commandErrors.Add(String.Format(GetMessage("Error.UnableToOrderItem"), prod.Name, prod.QuantityInStock));
					}
				}
				if (commandErrors.Count > 0)
					return false;
			}

			return true;
		}

		protected override void PreLoadComponentDefinition()
		{
			base.Title = GetMessage("Component.Title");
			base.Description = GetMessage("Component.Description");
			base.Icon = "images/sale_cart.gif";

			Group = new BXComponentGroup("Store", GetMessage("Group"), 100, BXComponentGroup.Store);

			BXCategory fields = new BXCategory(GetMessageRaw("FieldSettings"), "fields", 200);

			ParamsDefinition.Add(
				"DisplayProperties",
				new BXParamDoubleList(
					GetMessageRaw("Param.DisplayProperties"),
					"",
					BXCategory.Main
					)
				);

			ParamsDefinition.Add(
				"ShowVAT",
				new BXParamYesNo(
					GetMessageRaw("Param.ShowVAT"),
					false,
					BXCategory.Main
					)
				);

			ParamsDefinition.Add(
				"SetPageTitle",
				new BXParamYesNo(
					GetMessageRaw("Param.SetPageTitle"),
					true,
					BXCategory.Main
					)
				);

			ParamsDefinition.Add(
				"OrderFormUrl",
				new BXParamText(
					GetMessageRaw("Param.OrderFormUrl"),
					"~/orderform.aspx",
					BXCategory.UrlSettings
				)
			);



		}

		List<string> defaultProps = new List<string>{"ItemName","ItemPrice","ItemQuantity","ItemWeight"};

		List<string> displayProperties;

		public List<string> DisplayItemProperties
		{
			get 
			{
				return displayProperties ?? (displayProperties = Parameters.GetListString("DisplayProperties", defaultProps));
			}
		}

		Dictionary<string,BXSaleCartPropertyInfo> availableProperties;

		public IDictionary<string,BXSaleCartPropertyInfo> AvailableItemProperties
		{
			get 
			{ 
				if (availableProperties != null )
					return availableProperties;

				availableProperties = new Dictionary<string, BXSaleCartPropertyInfo>();
				foreach (var prop in BXSaleCartItemManager.GetAvailableProductProperties())
					availableProperties[prop.Name] = prop;

				availableProperties["ItemPrice"] = new BXSaleCartPropertyInfo("self", "ItemPrice", GetMessage("ItemProperty.Price"), GetMessage("ItemProperty.Price"));
				availableProperties["ItemName"] = new BXSaleCartPropertyInfo("self", "ItemName", GetMessage("ItemProperty.Name"), GetMessage("ItemProperty.Name"));
				availableProperties["ItemQuantity"] = new BXSaleCartPropertyInfo("self", "ItemQuantity", GetMessage("ItemProperty.Quantity"), GetMessage("ItemProperty.Quantity"));
				availableProperties["ItemImageUrl"] = new BXSaleCartPropertyInfo("self", "ItemImageUrl", GetMessage("ItemProperty.ImageUrl"), GetMessage("ItemProperty.ImageUrl"));
				availableProperties["ItemWeight"] = new BXSaleCartPropertyInfo("self", "ItemWeight", GetMessage("ItemProperty.Weight"), GetMessage("ItemProperty.Weight"));
				availableProperties["ItemDiscountValue"] = new BXSaleCartPropertyInfo("self", "ItemDiscountValue", GetMessage("ItemProperty.DiscountValue"), GetMessage("ItemProperty.DiscountValue"));
				return availableProperties;
			}
		}

		protected override void LoadComponentDefinition()
		{
			foreach (var item in AvailableItemProperties)
			{
				ParamsDefinition["DisplayProperties"].Values.Add(new BXParamValue(item.Value.ParamLabel, item.Value.Name));
			}
			if (String.IsNullOrEmpty(Parameters["DisplayProperties"]))
			{
				Parameters["DisplayProperties"] = BXStringUtility.ListToCsv(defaultProps);
			}
		}
	}
}
