﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using System.Text;
using Bitrix.Security;
using System.Web.Security;
using Bitrix.Components;
using Bitrix.DataLayer;
using Bitrix.Services.Text;
using System.Xml;
using System.Globalization;
using Bitrix.DataTypes;
using System.Security.Cryptography;
using Bitrix.Configuration;
using Bitrix.Services;
using System.IO;
using System.Web.Hosting;

namespace Bitrix.Sale.Components
{
	public partial class SaleExport1CComponent : BXComponent
	{
		private MD5 md5;
		static string uploadPath = null;

		public static string UploadPath
		{
			get { return uploadPath ?? (uploadPath = VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine(BXConfigurationUtility.Options.UploadFolderPath, "1c_orders"))); }
		}

		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);
			if (BXConfigurationUtility.IsDesignMode || IsComponentDesignMode || Request.QueryString["mode"] == null)
			{
				IncludeComponentTemplate();
				return;
			}

			try
			{
				switch (Request.QueryString["mode"])
				{
					case "checkauth":
						ExchangeAuth();
						break;
					case "init":
						ExchangeInit();
						break;
					case "query":
						ExchangeQuery();
						break;
					case "file":
						ExchangeFile();
						break;
					case "success":
						ExchangeSuccess();
						break;
					default:
						throw new Exception(string.Format("Режим '{0}' не поддерживается", Request.QueryString["mode"]));
				}
			}
			catch (Exception ex)
			{
				if (!(ex is BXSecurityException))
					BXLogService.LogAll(ex, 0, BXLogMessageType.Critical, "bitrix:sale.export.1c");

				Page.Response.ContentType = "text/plain";
				Page.Response.ContentEncoding = Encoding.GetEncoding(1251);
				Page.Response.Write("failure\n");
				Page.Response.Write(ex.Message);
			}

			Page.Response.End();
		}

		protected override void OnUnload(EventArgs e)
		{
			base.OnUnload(e);
			if (md5 != null)
				md5.Clear();
		}

		private string Md5(string input)
		{
			if (md5 == null)
				md5 = MD5.Create();

			var bytes = md5.ComputeHash(Encoding.Unicode.GetBytes(input));
			var s = new StringBuilder(bytes.Length * 2);
			for (int i = 0; i < bytes.Length; i++)
				s.Append(bytes[i].ToString("x2"));

			var r = s.ToString();
			s.Length = 0;

			return r;
		}

		private void ExchangeInit()
		{
			Page.Response.ContentType = "text/plain";
			Page.Response.ContentEncoding = Encoding.GetEncoding(1251);
			Page.Response.Write("zip=no\n");
			Page.Response.Write("file_limit=0\n");
		}

		private void ExchangeQuery()
		{
			var identity = BXIdentity.Current;
			if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.OrderExchange))
				throw new BXSecurityException(string.Format("У пользователя '{0}' отсутствует разрешение на выполнение обмена заказами", identity.Name));

			Page.Response.ContentType = "text/xml";
			Page.Response.ContentEncoding = Encoding.GetEncoding(1251);

			var filter = new BXFilter();

			if (Parameters.GetBool("ExportPaidOrders"))
				filter.Add(new BXFilterItem(BXSaleOrder.Fields.Paid, BXSqlFilterOperators.Equal, true));

			if (Parameters.GetBool("ExportAllowShippingOrders"))
				filter.Add(new BXFilterItem(BXSaleOrder.Fields.AllowShipping, BXSqlFilterOperators.Equal, true));

			var sites = Parameters.GetListString("Sites", null);
			if (sites != null && sites.Count > 0)
				filter.Add(new BXFilterItem(BXSaleOrder.Fields.SiteId, BXSqlFilterOperators.In, sites));

			var statuses = Parameters.GetListInt("Statuses", null);
			if (statuses != null && statuses.Count > 0)
				filter.Add(new BXFilterItem(BXSaleOrder.Fields.StatusId, BXSqlFilterOperators.In, statuses));

			var lastUpdate = BXOptionManager.GetOption(BXSaleModuleConfiguration.Id, "Last1CExportC@" + Md5(Parent.TemplateControl.AppRelativeVirtualPath), DateTime.MinValue);
			if (lastUpdate != DateTime.MinValue)
				filter.Add(new BXFilterItem(BXSaleOrder.Fields.LastUpdatedUtc, BXSqlFilterOperators.GreaterOrEqual, lastUpdate));
			BXOptionManager.SetOption(BXSaleModuleConfiguration.Id, "Last1CExport@" + Md5(Parent.TemplateControl.AppRelativeVirtualPath), DateTime.UtcNow);


			var settings = new XmlWriterSettings
			{
				ConformanceLevel = ConformanceLevel.Document,
				CheckCharacters = false,
				Indent = true,
				IndentChars = "\t",
				Encoding = Encoding.GetEncoding(1251)
			};
			using (var xml = XmlWriter.Create(Response.Output, settings))
			{
				xml.WriteStartDocument();
				BXSaleOrderExport.WriteCml(xml, filter, Parameters.GetString("CurrencyReplace"));
				xml.WriteEndDocument();
			}
		}

		private void ExchangeSuccess()
		{
			var identity = BXIdentity.Current;
			if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.OrderExchange))
				throw new BXSecurityException(string.Format("У пользователя '{0}' отсутствует разрешение на выполнение обмена заказами", identity.Name));


			var id = BXIdentity.Current;
			if (id.Name != BXOptionManager.GetOptionString(BXSaleModuleConfiguration.Id, "1CExportLogin@" + Md5(Parent.TemplateControl.AppRelativeVirtualPath), null)
			|| id.ProviderName != BXOptionManager.GetOptionString(BXSaleModuleConfiguration.Id, "1CExportProvider@" + Md5(Parent.TemplateControl.AppRelativeVirtualPath), null))
				throw new Exception("Wrong user session");

			BXOptionManager.SetOption(
				BXSaleModuleConfiguration.Id,
				"Last1CExportC@" + Md5(Parent.TemplateControl.AppRelativeVirtualPath),
				BXOptionManager.GetOption(BXSaleModuleConfiguration.Id, "Last1CExport@" + Md5(Parent.TemplateControl.AppRelativeVirtualPath), DateTime.UtcNow)
			);

			Page.Response.ContentEncoding = Encoding.GetEncoding(1251);
			Page.Response.ContentType = "text/plain";
			Page.Response.Write("success");
		}

		private void ExchangeFile()
		{
			var identity = BXIdentity.Current;
			if (!BXPrincipal.Current.IsCanOperate(BXSaleModule.Operations.OrderExchange))
				throw new BXSecurityException(string.Format("У пользователя '{0}' отсутствует разрешение на выполнение обмена заказами", identity.Name));


			var fileName = Request.QueryString["filename"];
			if (string.IsNullOrEmpty(fileName))
				throw new InvalidOperationException("Не задано имя файла");

			List<string> errors = new List<string>();

			var fi = new FileInfo(HostingEnvironment.MapPath(VirtualPathUtility.Combine(UploadPath, fileName)));
			fi.Directory.Create();

			try
			{
				foreach (var f in fi.Directory.GetFiles())
				{
					try
					{
						f.Delete();
					}
					catch
					{
					}
				}
			}
			catch
			{
			}
			
		
			using (var fs = new FileStream(fi.FullName, FileMode.Create, FileAccess.ReadWrite, FileShare.ReadWrite))
			{
				var buf = new byte[Math.Min(Request.InputStream.Length, 4096)];
				int read;
				while ((read = Request.InputStream.Read(buf, 0, buf.Length)) > 0)
					fs.Write(buf, 0, read);
				fs.Flush();
				fs.Position = 0;
							
				using (var r = XmlReader.Create(fs))
				{
					r.MoveToContent();

					int i = Parameters.GetInt("StatusOnDelivery");
					BXSaleOrderImport.ProcessCml(r, BXSefUrlManager.CurrentUrl, i > 0 ? (int?)i : null, errors);
				}
			}

			Page.Response.ContentEncoding = Encoding.GetEncoding(1251);
			Page.Response.ContentType = "text/plain";
			Page.Response.Write("success");
			foreach (var e in errors)
			{
				Page.Response.Write('\n');
				Page.Response.Write(e);
			}
		}

		private void ExchangeAuth()
		{
			var header = Page.Request.Headers["Authorization"];
			if (string.IsNullOrEmpty(header) || !header.StartsWith("Basic "))
				throw new Exception("Нет аутентификационных данных");

			string value = header.Substring("Basic".Length).Trim();
			value = Encoding.GetEncoding(1251).GetString(Convert.FromBase64String(value));
			int i = value.IndexOf(':');
			var login = value.Remove(i);
			var password = value.Substring(i + 1);
			string provider;
			if (!BXAuthentication.Authenticate(login, password, out provider))
				throw new Exception("Неверное имя пользователя или пароль");

			var cookie = BXAuthentication.GetAuthCookie(login, provider, false);
			Page.Response.ContentType = "text/plain";
			Page.Response.Write("success\n");
			Page.Response.Write(cookie.Name);
			Page.Response.Write("\n");
			Page.Response.Write(cookie.Value);

			BXOptionManager.SetOption(BXSaleModuleConfiguration.Id, "1CExportLogin@" + Md5(Parent.TemplateControl.AppRelativeVirtualPath), login);
			BXOptionManager.SetOption(BXSaleModuleConfiguration.Id, "1CExportProvider@" + Md5(Parent.TemplateControl.AppRelativeVirtualPath), provider);
		}

		protected override void PreLoadComponentDefinition()
		{
			var main = BXCategory.Main;

			Title = GetMessageRaw("Title");
			Description = GetMessageRaw("Description");
			Group = new BXComponentGroup("Utility", GetMessageRaw("Group"), 100, BXComponentGroup.Store);

			ParamsDefinition["Sites"] = new BXParamMultiSelection(GetMessageRaw("Param.Sites"), "", main);
			ParamsDefinition["ExportPaidOrders"] = new BXParamYesNo(GetMessageRaw("Param.ExportPaidOrders"), false, main);
			ParamsDefinition["ExportAllowShippingOrders"] = new BXParamYesNo(GetMessageRaw("Param.ExportAllowShippingOrders"), false, main);
			ParamsDefinition["ExportFinalOrders"] = new BXParamYesNo(GetMessageRaw("Param.ExportFinalOrders"), false, main);
			ParamsDefinition["StatusOnDelivery"] = new BXParamSingleSelection(GetMessageRaw("Param.StatusOnDelivery"), "", main);
			ParamsDefinition["Statuses"] = new BXParamMultiSelection(GetMessageRaw("Param.Statuses"), "", main);
			ParamsDefinition["CurrencyReplace"] = new BXParamText(GetMessageRaw("Param.CurrencyReplace"), "", main);
		}

		protected override void LoadComponentDefinition()
		{
			// Sites
			var sites = BXSite.GetAllSitesReadOnly();
			var siteIds = Parameters.GetListString("Sites", null);
			if (siteIds != null && siteIds.Count != 0)
			{
				siteIds = siteIds.FindAll(x => sites.Any(y => string.Equals(y.TextEncoder.Decode(y.Id), x, StringComparison.OrdinalIgnoreCase)));
				Parameters["Sites"] = BXStringUtility.ListToCsv(siteIds);
			}

			var siteValues = (from s in sites where s.Active select new BXParamValue(s.TextEncoder.Decode(s.Name), s.TextEncoder.Decode(s.Id))).ToList();
			siteValues.Sort((a, b) =>
			{
				if (siteIds != null)
				{
					int ia = siteIds.Exists(x => string.Equals(a.Value, x, StringComparison.OrdinalIgnoreCase)) ? 0 : 1;
					int ib = siteIds.Exists(x => string.Equals(b.Value, x, StringComparison.OrdinalIgnoreCase)) ? 0 : 1;
					if (ia != ib)
						return ia - ib;
				}
				return string.Compare(a.Title, b.Title);
			});
			ParamsDefinition["Sites"].Values = siteValues;

			// Statuses
			var statuses = BXSaleOrderStatus.GetList(
				null,
				new BXOrderBy(
					new BXOrderByPair(BXSaleOrderStatus.Fields.Sort, BXOrderByDirection.Asc),
					new BXOrderByPair(BXSaleOrderStatus.Fields.Id, BXOrderByDirection.Asc)
				),
				null,
				null,
				BXTextEncoder.EmptyTextEncoder
			);
			var statusIds = Parameters.GetListInt("Statuses", null);
			if (statusIds != null && statusIds.Count != 0)
			{
				statusIds = statusIds.FindAll(x => statuses.Exists(y => y.Id == x));
				Parameters["Statuses"] = BXStringUtility.ListToCsv(statusIds);
			}

			statuses.Sort((a, b) =>
			{
				if (statusIds != null)
				{
					int ia = statusIds.Contains(a.Id) ? 0 : 1;
					int ib = statusIds.Contains(b.Id) ? 0 : 1;
					if (ia != ib)
						return ia - ib;
				}
				int i = a.Sort - b.Sort;
				if (i != 0)
					return i;
				return a.Id - b.Id;
			});
			ParamsDefinition["Statuses"].Values = statuses.ConvertAll(s => new BXParamValue(string.Format("[{0}] {1}", s.XmlId, s.LocalizedName), s.Id.ToString()));

			ParamsDefinition["StatusOnDelivery"].Values.Clear();
			ParamsDefinition["StatusOnDelivery"].Values.Add(new BXParamValue(GetMessageRaw("Option.DontSet"), ""));
			ParamsDefinition["StatusOnDelivery"].Values.AddRange(statuses.ConvertAll(s => new BXParamValue(string.Format("[{0}] {1}", s.XmlId, s.LocalizedName), s.Id.ToString())));
		}
	}
	public class SaleExport1CTemplate : BXComponentTemplate<SaleExport1CComponent>
	{
	}
}
