﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using Bitrix.Components;
using Bitrix.Security;
using System.Collections.Generic;
using Bitrix.Modules;
using Bitrix.UI;
using Bitrix.Sale.SaleCart;
using Bitrix.DataLayer;
using Bitrix.DataTypes;
using System.Web;
using Bitrix.Services.Text;
using Bitrix.Sale;
using Bitrix.Currency;
using System.Web.Security;
using System.Text;
using Bitrix.Components.Editor;
using Bitrix.Services;
using Bitrix.Configuration;

namespace Bitrix.Sale.Components
{
	public partial class SaleOrderFormComponent : BXComponent
	{
		int curUserId = -1;

		SaleOrderFormComponentError error;

		public SaleOrderFormComponentError Error
		{
			get { return error; }
		}

		public string GetErrorMessage()
		{

			string result=GetMessage("Error." + error.ToString());
			switch (error)
			{
				case SaleOrderFormComponentError.UserRegistrationFailed:
					result += String.Format(
						GetMessageRaw("Message.AuthorizeOrRegister"),
						Encode(AttachBackUrl(LoginPath)),
						Encode(AttachBackUrl(RegisterPath))
					);
					break;
				case SaleOrderFormComponentError.DuplicateEmail:
					result += String.Format(
						GetMessageRaw("Message.AuthorizeOrSetOtherEmail"),
						Encode(AttachBackUrl(LoginPath))
					);
					break;
			}

			return result;
		}

		string AttachBackUrl(string url)
		{
			var index = url.IndexOf('?');
			string uriString;
			if (index != -1)
			{
				var qParams = HttpUtility.ParseQueryString(url.Substring(index + 1));
				qParams["back_url"] = HttpUtility.UrlEncode(BXSefUrlManager.CurrentUrl.ToString());
				uriString = url.Substring(0, index) + "?" + qParams.ToString();
			}
			else
				uriString = url + "?back_url=" + HttpUtility.UrlEncode(BXSefUrlManager.CurrentUrl.ToString());

			return uriString;
		}

		public enum SaleOrderFormComponentError
		{
			None=0,
			UserRegistrationFailed,
			DuplicateEmail,
			OrderIsEmpty,
			InvalidEmail
		}

		//Id текущего пользователя, у анонимов - 0
		public int CurrentUserId
		{
			get
			{
				return curUserId != -1 ? curUserId : (curUserId =
					(BXPrincipal.Current != null && BXPrincipal.Current.Identity.IsAuthenticated ? ((BXIdentity)BXPrincipal.Current.Identity).Id : 0));
			}
		}
		BXUser curUser;
		BXUser CurrentUser
		{
			get
			{
				if (CurrentUserId <= 0)
					return null;

				return curUser ?? (curUser = BXUser.GetById(CurrentUserId));
			}
		}

		BXSaleCart GetCart(BXFilterItem[] items)
		{
			var carts = BXSaleCart.GetList(
				new BXFilter(items),
				new BXOrderBy(new BXOrderByPair(BXSaleCart.Fields.Id, BXOrderByDirection.Desc)));
			if (carts.Count > 0)
				return carts[0];
			return null;
		}

		List<BXSaleCartProduct> products;

		public List<BXSaleCartProduct> Products
		{
			get { return products; }
		}

        public BXCurrency Currency
        {
            get { return BXCurrency.NationalCurrency; }
        }

        public decimal GetTotalWeight()
        {
            if(products == null || products.Count == 0)
                return 0m;

            decimal r = 0m;
            for(int i = 0; i < products.Count; i++)
                r += products[i].Weight;

            return r;
        }

        public decimal GetTotalCost()
        {
            if(products == null || products.Count == 0)
                return 0m;

            decimal r = 0m;
            for(int i = 0; i < products.Count; i++)
                r += products[i].PriceCurrency.ConvertMoney( products[i].GetTotalCost(), BXCurrency.NationalCurrency.Id);

			

            return r;
        }

		public decimal GetTotalDiscount()
		{
			if(products == null || products.Count == 0)
                return 0m;

			decimal r = 0;
			for (int i = 0; i < products.Count; i++)
				r += BXCurrency.ConvertMoney(products[i].DiscountValueNationalCurrency, BXCurrency.NationalCurrency, Currency);

			if (FullOrderDiscount != null && GetTotalCost() - r - BXCurrency.ConvertMoney(FullOrderDiscount.Value, BXCurrency.NationalCurrency, Currency) > 0)
				r +=  BXCurrency.ConvertMoney(FullOrderDiscount.Value, BXCurrency.NationalCurrency, Currency);

			return r;
		}

        public BXSaleDeliveryCargoType CargoType
        {
            get { return BXSaleDeliveryCargoType.Unknown; }
        }


        public int SourceCountryNumericCode
        {
            get { return BXSale.Current.CountryNumericCode; }
        }

        public string SourcePostalCode
        {
            get { return BXSale.Current.PostalCode; }
        }
		string successMessage;
		public string SuccessMessageHtml
		{
			get 
			{
				if (successMessage != null)
					return successMessage;

				if (order == null)
					return successMessage = String.Empty;

				var sb = new StringBuilder();
				sb.Append("<b>").Append(GetMessage("Message.OrderIsCreated")).Append("</b><br>");
				sb.AppendFormat(GetMessage("Message.Order"), order.Id, order.Created.ToString("g")).Append("<br><br>");
				sb.AppendFormat(
					GetMessageRaw("Message.ViewOrders"),
					Encode(OrderListPath)
				);
				if (RegisterNewUsers && user != null && SendNewUserCredentials)
					sb.Append("<br><br>").AppendFormat(GetMessage("Message.CredentialsWereSent"), user.Email); 

				return successMessage = sb.ToString();
			}
		}


		BXSaleCart cart;

		protected void Page_Load(object sender, EventArgs e)
		{
			if (ShowSuccessMessage)
			{
				successMessage = Session["SuccessMessage"] != null ? Session["SuccessMessage"].ToString() : String.Empty;
				Session["SuccessMessage"] = null;
				Session["ShowSuccessMessage"] = null;
			}

			if (CurrentUserId == 0 && !RegisterNewUsers)//пользователь не авторизован и не регистрируем автоматом
			{
				Response.Redirect(AttachBackUrl(RegisterPath));
			}

			products = new List<BXSaleCartProduct>();

			var cartGuid = Request.Cookies.Get(BXSaleCart.ExternalStorageUIDKey);

			BXSaleCart userCart = null; // корзина пользователя
			BXSaleCart cookieCart = null; // корзина в куках

			List<BXFilterItem> items = new List<BXFilterItem>();

			if (CurrentUserId > 0)
			{
				userCart = BXSaleCart.GetByUserId(CurrentUserId);
			}

			if (cartGuid != null && !String.IsNullOrEmpty(cartGuid.Value))
			{
				cookieCart = BXSaleCart.GetByGuid(cartGuid.Value);
			}

			// если есть корзина пользователя и корзина анонима, то нужно очистить корзину анонима, переложив все товары из нее
			// в корзину пользователя, а корзину анонима затем удалить, и поменять информацию в куки

			if (cookieCart != null && (userCart == null || userCart.UniqueId != cookieCart.UniqueId) && CurrentUserId > 0)
			{
				var id = BXSaleCart.Merge(cookieCart.Id, userCart == null ? -1 : userCart.Id, CurrentUserId);
				if (id > 0)
				{
					cart = BXSaleCart.GetById(id);
					Response.SetCookie(new HttpCookie(BXSaleCart.ExternalStorageUIDKey, cart.UniqueId));
				}
			}
			else
			{
				if (cookieCart != null && cookieCart.UserId > 0 && CurrentUserId == 0) //в куки анонима лежит корзина зарегистрированного пользователя
				{												// сбрасываем информацию о корзине
					Response.Cookies[BXSaleCart.ExternalStorageUIDKey].Expires = DateTime.Now.AddDays(-1);
					cart = null;
				}
				else
					cart = userCart ?? cookieCart;
			}

			if (cart != null)
			{
				foreach (var item in BXSaleCartItemManager.GetByCart(cart, new List<String>(), curUserId, BXSite.Current.Id))
				{
					if (!item.HasProductInfo)
						continue;

					if (!item.IsPutAside && (item.CanBuyMoreThenExists || item.QuantityInCart <= item.QuantityInStock))
						products.Add(item);

				}
			}



			if (products.Count == 0 && !ShowSuccessMessage)
			{
				error = SaleOrderFormComponentError.OrderIsEmpty;
			}
			var cartCoupons = new List<String>();
			if(cart!=null)
				cartCoupons = cart.Coupons.FindAll(
					x => x.Active == true).ConvertAll<string>(x => x.CouponString);
				FillDiscountValues(products, cartCoupons);

			IncludeComponentTemplate();
		}

		void FillDiscountValues(List<BXSaleCartProduct> prods, List<string> coupons)
		{
			appliedDiscountsWithCoupons = new List<int>();
			if (prods == null || prods.Count == 0)
				return;

			var discountInfo = BXSaleDiscountManager.FindSuitableDiscounts(
				new BXSaleFindDiscountInfo(CurrentUserId, BXPrincipal.Current.GetAllRoleIds(true),
					prods, BXSite.Current.Id, BXCurrency.NationalCurrencyId, coupons));

			BXSaleDiscountFindResultInfo info = null;

			if (discountInfo != null)
				foreach (var prod in prods)
				{

					if (discountInfo.Discounts.TryGetValue(prod.ElementId, out info))
					{

						if (prod.DiscountValueNationalCurrency <= info.Value)
						{
							var couponDiscounts = info.Discounts.FindAll(x => x.UseCoupons);

							appliedDiscountsWithCoupons.AddRange(couponDiscounts.ConvertAll<int>(x=>x.Id));

							prod.DiscountValueNationalCurrency = info.Value;
							if (info.Discounts.Count > 0)
							{
								prod.DiscountName = info.Discounts[0].Name;
								var coupon = cart.Coupons.Find(x=>x.DiscountId == info.Discounts[0].Id && x.Active);
								if (coupon != null)
									prod.CouponId = coupon.CouponId;

							}

							prod.DiscountedPriceDisplayHtml = BXCurrency.NationalCurrency.MoneyToString(
								BXCurrency.ConvertMoney(
									prod.Price, prod.PriceCurrency, BXCurrency.NationalCurrency) - prod.DiscountValueNationalCurrency);
						}
					}

				}
			discountInfo.Discounts.TryGetValue(0, out fullOrderDiscount);

		}
		BXSaleDiscountFindResultInfo fullOrderDiscount;

		public BXSaleDiscountFindResultInfo FullOrderDiscount
		{
			get { return fullOrderDiscount; }
			set { fullOrderDiscount = value; }
		}

		List<int> appliedDiscountsWithCoupons;

		BXUser user;

		public bool RegisterUserIfNeed(SaleOrderFormOrderInfo info)
		{
			if (CurrentUserId == 0)//  считаем пользователя анонимом и регистрируем его
			{
				var emailField = info.Fields.Find(x => x.UseAsEmail == true);
				var emailValue = info.Values.GetString(emailField.Code);
				var index = emailValue.IndexOf('@');
				if (index == -1)
				{
					error = SaleOrderFormComponentError.InvalidEmail;
					return false;
				}
				var login = emailValue.Remove(emailValue.IndexOf('@')).Trim();
				if (String.IsNullOrEmpty(login))
				{
					error = SaleOrderFormComponentError.InvalidEmail;
					return false;
				}

				user = new BXUser();

				user.ProviderName = BXUserManager.Provider.Name;
				user.UserName = login;
				user.IsApproved = true;

				string password = null;

				for(var i=0; i<5; i++)
				{
					if((password = BXUser.GetCorrectPassword(BXUserManager.GetProvider("self") as BXSqlMembershipProvider)) != null)
						break;
				}
				if ( password == null )
				{
					error = SaleOrderFormComponentError.UserRegistrationFailed;
					return false;
				}
				user.Password = password;
				user.Email = emailValue;
				int k = 0;
				int j = 0;
				bool userSaved = false;
				while ( k < 10 )
					try
					{
						user.Save();
						userSaved = true;
						break;
					}
					catch (MembershipCreateUserException ex)
					{
						switch (ex.StatusCode)
						{
							case MembershipCreateStatus.DuplicateUserName:
								user.UserName = login + (j++).ToString();
								break;
							case MembershipCreateStatus.InvalidPassword:
								user.Password = password = BXUser.GetCorrectPassword(BXUserManager.GetProvider("self") as BXSqlMembershipProvider);
								k++;
								break;
							case MembershipCreateStatus.DuplicateEmail:
								error = SaleOrderFormComponentError.DuplicateEmail;
								return false;
							default:
								k++;
								break; 
						}
					}
					catch (Exception ex1)
					{

					}
				if (userSaved)
				{
					string providerName;
					if (BXAuthentication.Authenticate(user.UserName, password, out providerName))
					{
						BXAuthentication.SetAuthCookie(user.UserName, user.ProviderName, false);
						curUserId = user.UserId;
						if (SendNewUserCredentials)
							SendUserInfoByEmail(user, password);
					}
					if (cart != null && cart.UserId == 0)
					{
						cart.UserId = user.UserId;
						cart.Save();
					}
				}
				else
					error = SaleOrderFormComponentError.UserRegistrationFailed;

				return userSaved;

			}
			return true;
		}

		void SendUserInfoByEmail(BXUser user, string password)
		{
			BXCommand cmd = new BXCommand("Bitrix.Sale.SaleAutoRegisterMessage");
			cmd.Parameters["EMAILTO"] = user.Email;
			cmd.Parameters["LOGIN"] = user.UserName;
			cmd.Parameters["PASSWORD"] = password;
			cmd.Send();

		}

		void SendNewOrderInfoByEmail(SaleOrderFormOrderInfo info, BXSaleOrder order, List<BXSaleProduct> products, BXUser user)
		{
			BXCommand cmd = new BXCommand("Bitrix.Sale.SaleNewOrderMessage");
			var emailField = info.Fields.Find(x => x.UseAsEmail);
			if (emailField == null || emailField.Values.Count ==0)
				return;

			cmd.Parameters["EMAILTO"] = info.Values.GetString(emailField.Name);
			cmd.Parameters["ORDER_ID"] = order.Id;
			cmd.Parameters["PRICE"] = order.Currency.MoneyToString(order.Sum);

			var list = products.ConvertAll<string>(x => x.Name + " (" + x.Quantity + ")").ToArray();
			cmd.Parameters["ORDER_LIST"] = String.Join("\n", list);
			cmd.Parameters["USERNAME"] = user.GetDisplayName();

			cmd.Send();
		}
		
		BXSaleOrder order;

		public bool MakeOrder(SaleOrderFormOrderInfo info, ICollection<string> errors)
		{
			if (cart == null || products.Count == 0)
			{
				errors.Add(GetMessageRaw("Error.NoElements"));
				return false;
			}
			
			if (RegisterNewUsers && CurrentUserId== 0 && !RegisterUserIfNeed(info))
			{
				errors.Add(GetErrorMessage());
				return false;
			}

			using (var tran = new BXSqlTransaction())
			{
				List<BXSaleProduct> oProducts = new List<BXSaleProduct>();
				
				decimal orderSum = 0;
				foreach (var p in Products)
				{
					orderSum += p.GetTotalCostNationalCurrency() - p.DiscountValueNationalCurrency;
				}

				orderSum = orderSum - (FullOrderDiscount != null ? FullOrderDiscount.Value : 0);

				order = new BXSaleOrder();
				order.CurrencyId = Currency.Id;

				order.CustomerTypeId = info.CustomerTypeId;
				order.UserId = CurrentUserId;
				order.StatusId = BXSaleModuleConfiguration.InitialOrderStatusId;
                order.DeliveryCost = info.DeliveryCost;
                order.DeliveryVariant = info.DeliveryVariant;
				order.Sum = orderSum + order.DeliveryCost;
				order.SiteId = BXSite.Current.Id;
                order.PaymentSystemId = info.PaymentSystemId;
				order.CustomerComments = info.CustomerComments;

				List<int> appliedCoupons = new List<int>();

				if (FullOrderDiscount != null && FullOrderDiscount.Discounts.Count > 0)
				{
					if(cart!=null){
						var coupon = cart.Coupons.Find(x => x.DiscountId == FullOrderDiscount.Discounts[0].Id && x.Active);
						if (coupon != null)
						{
							order.CouponId = coupon.CouponId;
							appliedCoupons.Add(coupon.CouponId);
						}
					}
					order.DiscountName = FullOrderDiscount.Discounts[0].Name;
					order.DiscountValue = BXCurrency.ConvertMoney(FullOrderDiscount.Discounts[0].Value, BXCurrency.NationalCurrency, Currency);
				}
				order.Save(tran.Connection, tran);
				

				List<int> prodIds = new List<int>();
				BXSaleOrderValueManager.SetValues(order.Id, info.Fields, info.Values, tran);
				
				foreach (var p in Products)
					oProducts.Add(BXSaleProduct.CreateFromSaleCartProduct(p, order.Id, cart.ResolverId, order.Currency.Id));
							
				foreach (var op in oProducts)
					op.Save(tran.Connection, tran);

				foreach (var p in products)
					prodIds.Add(p.CartItemId);

				BXSaleCart.RemoveItems(cart.Id, prodIds, tran);

				info.DoSave(new SaleOrderFormContext 
				{ 
					UserId = CurrentUserId,
					Transaction = tran
				});
				List<string> bErrors = new List<string>();
				if (!BXSaleCartItemManager.UpdateBalance(
						Products.ConvertAll<BXSaleBalanceInfo>(
							x => new BXSaleBalanceInfo()
							{
								ItemId = x.ElementId,
								Quantity = x.QuantityInCart*(-1)
							}).ToArray(),
						cart.ResolverId,
						new BXSqlTransactionHelper(tran),
						bErrors))
				{
					tran.Rollback();
					foreach (var error in bErrors)
						errors.Add(error);
					errors.Add(GetMessage("Error.YouCanReorder"));
					return false;
				}
				appliedCoupons.AddRange(Products.ConvertAll<int>(x => x.CouponId).FindAll(x => x != 0));
				BXSaleDiscountCoupon.DeactivateCoupons(new BXSqlTransactionHelper(tran), appliedCoupons, CurrentUserId);

				tran.Commit();



				SendNewOrderInfoByEmail(info, order, oProducts, CurrentUser ?? user);
			}

			if (user != null) // пользователь был создан и авторизован
			{
				
				Session["EmailSent"] = SendNewUserCredentials.ToString();
				Session["SuccessMessage"] = SuccessMessageHtml;
				Session["ShowSuccessMessage"] = "y";
				Response.Redirect(BXSefUrlManager.CurrentUrl.ToString());
			}
			return true;
		}

		bool showSuccessMessage;
		bool emailWasSent;

		public bool ShowSuccessMessage
		{
			get
			{
				if (Session["ShowSuccessMessage"] != null)
					return showSuccessMessage = true;
				return showSuccessMessage;
			}
			set
			{
				showSuccessMessage = value;
			}
		}

		public string OrderListPath
		{
			get { return Parameters.GetString("OrderListPath", ""); }
			set { Parameters["OrderListPath"] = value; }
		}

		public string RegisterPath
		{
			get { return Parameters.GetString("RegisterPath", "register.aspx?auth_page=register"); }
			set { Parameters["RegisterPath"] = value; }
		}

		public string LoginPath
		{
			get { return Parameters.GetString("LoginPath", "register.aspx"); }
			set { Parameters["LoginPath"] = value; }
		}

		public bool RegisterNewUsers
		{
			get { return Parameters.GetBool("RegisterNewUsers", true); }
			set { Parameters["RegisterNewUsers"] = value.ToString(); }
		}

		public bool SendNewUserCredentials
		{
			get { return Parameters.GetBool("SendNewUserCredentials", true); }
			set { Parameters["SendNewUserCredentials"] = value.ToString(); }
		}

		public bool SendNewOrderEmail
		{
			get { return Parameters.GetBool("SendNewOrderEmail", true); }
			set { Parameters["SendNewOrderEmail"] = value.ToString(); }
		}

		BXSaleCustomerTypeCollection cTypes;

		protected BXSaleCustomerTypeCollection CustomerTypes
		{
			get { return cTypes ?? (cTypes = BXSaleCustomerType.GetAllForSite(BXSite.Current.Id)); }
		}

		public List<string> CustomProperties
		{
			get
			{
				var result = new List<string>();
				if (order == null)
					return result;
				var fields = BXSaleOrderField.GetCustomerFields(order.CustomerTypeId, null);
				var values = BXSaleOrderValueManager.GetValues(order.Id, fields, null);

				foreach (var v in values.Values)
				{
					var f = fields[v.Name];
					if (f != null)
						result.Add(f.Code);
				}
				return result;
			}
		}

		protected override void PreLoadComponentDefinition()
		{
			base.Title = GetMessage("Component.Title");
			base.Description = GetMessage("Component.Description");
			base.Icon = "images/sale_form.gif";
			base.Group = new BXComponentGroup("Store", GetMessage("Group"), 100, BXComponentGroup.Store);

			ParamsDefinition.Add(
				"RegisterNewUsers",
				new BXParamYesNo(
					GetMessageRaw("Param.RegisterNewUsers"),
					true,
					BXCategory.Main,
					new ParamClientSideActionGroupViewSwitch(ClientID, "RegisterNewUsers", "On", "Off")
					)
				);

			ParamsDefinition.Add(
				"SendNewUserCredentials",
				new BXParamYesNo(
					GetMessageRaw("Param.SendNewUserCredentials"),
					true,
					BXCategory.Main,
					new ParamClientSideActionGroupViewMember(ClientID, "SendNewUserCredentials", new string[] {"On"})
					)
				);

			ParamsDefinition.Add(
				"SendNewOrderEmail",
				new BXParamYesNo(
					GetMessageRaw("Param.SendNewOrderEmail"),
					true,
					BXCategory.Main
					)
				);

			foreach (var customerType in CustomerTypes)
			{
				ParamsDefinition.Add(
					"DisplayCustomProps_" + customerType.Id.ToString(),
					new BXParamDoubleList(
						String.Format(GetMessage("Param.DisplayProps"), customerType.TextEncoder.Decode(customerType.Name)),
						"",
						BXCategory.DetailSettings
						)
					);
			}

			ParamsDefinition.Add(
				"LoginPath",
				new BXParamText(
					GetMessageRaw("Param.LoginPath"),
					"login.aspx",
					BXCategory.UrlSettings
					)
				);

			ParamsDefinition.Add(
				"RegisterPath",
				new BXParamText(
					GetMessageRaw("Param.RegisterPath"),
					"login.aspx?auth_page=register",
					BXCategory.UrlSettings
					)
				);

			ParamsDefinition.Add(
				"OrderListPath",
				new BXParamText(
					GetMessageRaw("Param.OrderListPath"),
					"orders.aspx",
					BXCategory.UrlSettings
					)
				);
		}

		protected override void LoadComponentDefinition()
		{
			foreach (var customerType in CustomerTypes)
			{
				ParamsDefinition["DisplayCustomProps_" + customerType.Id.ToString()]
					.Values.AddRange(
					BXSaleOrderField.GetCustomerFields(customerType.Id, null).ConvertAll<BXParamValue>(
						x => new BXParamValue(x.Name, x.Code.ToString())
						));
			}
		}
	}

    public class SaleOrderFormDeliveryInfo
    {
        public int DestinationCountryNumericCode { get; set; }
        public string DestinationPostalCode { get; set; }
        public string DestinationLocationName { get; set; }
        public BXSaleDeliveryCargoType CargoType { get; set; }
        public decimal Weight { get; set; }
        public decimal Cost { get; set; }
        public decimal DeliveryCost { get; set; }
    }
    public class SaleOrderFormPaymentInfo
    {
        public int CustomerTypeId { get; set; }
    }
	public class SaleOrderFormOrderInfo
	{
		public int CustomerTypeId { get; set; }
		public BXSaleOrderFieldCollection Fields { get; set; }
		public BXCustomPropertyCollection Values { get; set; }
		public event Action<SaleOrderFormContext> Save;

 		public decimal DeliveryCost { get; set; }
        public string DeliveryVariant { get; set; }
        public int PaymentSystemId { get; set; }
		public string CustomerComments { get; set; }


		internal void DoSave(SaleOrderFormContext context)
		{
			if (Save != null)
				Save(context);
		}
	}

	public class SaleOrderFormContext 
	{
		public int UserId { get; internal set; }
		public BXSqlTransaction Transaction { get; internal set; }
	}

	public class SaleOrderFormTemplate : BXComponentTemplate<SaleOrderFormComponent>
	{
	}
}
