﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using Bitrix.Components;
using Bitrix.Security;
using System.Collections.Generic;
using Bitrix.Modules;
using Bitrix.UI;
using Bitrix.Sale.SaleCart;
using Bitrix.DataLayer;
using Bitrix.DataTypes;
using System.Web;
using Bitrix.Services.Text;
using Bitrix.Sale;
using Bitrix.Services;
using Bitrix.Currency;

namespace Bitrix.Sale.Components
{
	public enum SaleOrderListComponentError
	{
		None = 0,
		NotExecuted,
		Unauthorized,
		InvalidPage,
		OrderNotFound
	}

	public enum SaleOrderListComponentMode
	{
		Active = 0,
		All,
		Finished,
		Cancelled
	}

	public partial class SaleOrderListComponent : BXComponent
	{
		SaleOrderListComponentError error = SaleOrderListComponentError.None;

		public string GetErrorMessage()
		{
			if (error == SaleOrderListComponentError.None)
				return string.Empty;

			return GetMessage("Error." + error.ToString());
		}

		public SaleOrderListComponentError Error
		{
			get { return error; }
		}

		int curUserId = -1;

		public string OrderDetailUrlTemplate
		{
			get { return Parameters.Get("OrderDetailUrlTemplate", ""); }
			set { Parameters["OrderDetailUrlTemplate"] = value; }
		}

		public string CancelOrderUrlTemplate
		{
			get { return Parameters.Get("CancelOrderUrlTemplate", ""); }
			set { Parameters["CancelOrderUrlTemplate"] = value; }
		}

		public string CopyOrderUrlTemplate
		{
			get { return Parameters.Get("CopyOrderUrlTemplate", ""); }
			set { Parameters["CopyOrderUrlTemplate"] = value; }
		}

		public string ActionVariable
		{
			get { return Parameters.GetString("ActionVariable", "action"); }
			set { Parameters["ActionVariable"] = value; }
		}

		public string PagingIndexTemplate
		{
			get { return Parameters.GetString("PagingIndexTemplate", ""); }
			set { Parameters["PagingIndexTemplate"] = value; }
		}

		public string PagingPageTemplate
		{
			get { return Parameters.GetString("PagingPageTemplate", ""); }
			set { Parameters["PagingPageTemplate"] = value; }
		}

		public SaleOrderListComponentMode Mode
		{
			get { return Parameters.GetEnum<SaleOrderListComponentMode>("Mode", SaleOrderListComponentMode.Active); }
			set { Parameters["Mode"] = value.ToString(); }
		}

		//Id текущего пользователя, у анонимов - 0
		public int CurrentUserId
		{
			get { return curUserId != -1 ? curUserId : (curUserId = 
				(BXPrincipal.Current != null && BXPrincipal.Current.Identity.IsAuthenticated ? ((BXIdentity)BXPrincipal.Current.Identity).Id : 0)); }
		}

		List<SaleOrderWrapper> orders;

		public List<SaleOrderWrapper> Orders
		{
			get { return orders ?? ( orders = new List<SaleOrderWrapper>()); }
		}

		string cartUrl;
		public string CartUrl
		{
			get 
			{
				if (cartUrl != null)
					return cartUrl;
				var replace = new BXParamsBag<object>();
				replace.Add("UserId", CurrentUserId);
				return cartUrl = ResolveTemplateUrl(Parameters.GetString("CartUrlTemplate", "cart.aspx"), replace);
			}
			set { cartUrl = value; }
		}

		protected void Page_Load(object sender, EventArgs e)
		{
			if ( CurrentUserId == 0 )
			{
				error  = SaleOrderListComponentError.Unauthorized;
				IncludeComponentTemplate();
				return;
			}
			// копирование заказа
			var action = Parameters.GetString("Action", "");
			if (action.Equals("copy", StringComparison.OrdinalIgnoreCase))
			{
				var orderId = Parameters.GetInt("OrderId", 0);
				if (orderId > 0)
				{
					var order = BXSaleOrder.GetById(orderId);
					if (order == null)
					{
						error = SaleOrderListComponentError.OrderNotFound;
					}
					else 
					if (order.UserId != CurrentUserId)
					{
						error = SaleOrderListComponentError.Unauthorized;
					}
					if (error == SaleOrderListComponentError.None)
						BXSaleCart.PutOrderIntoCart(order.Id);
					try
					{
						Response.Redirect(CartUrl);
					}
					catch { }
				}
			}

			var modeParam = Request.QueryString["mode"] != null ? "mode=" + HttpUtility.UrlEncode(Request.QueryString["mode"]) : "";

			if(!string.IsNullOrEmpty(modeParam))
			{
				PagingIndexTemplate += (PagingIndexTemplate.IndexOf("?")> 0 || !EnableSef ? "&":"?")+ modeParam;

				PagingPageTemplate += (PagingPageTemplate.IndexOf("?") > 0 || !EnableSef ? "&" : "?") + modeParam;
			}

			var ordersFilter = new BXFilter();
			ordersFilter.Add(new BXFilterItem(BXSaleOrder.Fields.UserId, BXSqlFilterOperators.Equal, CurrentUserId));
			ordersFilter.Add(new BXFilterItem(BXSaleOrder.Fields.SiteId, BXSqlFilterOperators.Equal, BXSite.Current.Id));


			switch (Mode)
			{
				case SaleOrderListComponentMode.Active:
					ordersFilter.Add(new BXFilterItem(BXSaleOrder.Fields.Cancelled, BXSqlFilterOperators.Equal, false));
					ordersFilter.Add(new BXFilterItem(BXSaleOrder.Fields.Status.IsFinal, BXSqlFilterOperators.NotEqual, true));
					break;
				case SaleOrderListComponentMode.Cancelled:
					ordersFilter.Add(new BXFilterItem(BXSaleOrder.Fields.Cancelled, BXSqlFilterOperators.Equal, true));
					break;
				case SaleOrderListComponentMode.All:
					break;
				case SaleOrderListComponentMode.Finished:
					ordersFilter.Add(new BXFilterItem(BXSaleOrder.Fields.Cancelled, BXSqlFilterOperators.Equal, false));
					ordersFilter.Add(new BXFilterItem(BXSaleOrder.Fields.Status.IsFinal, BXSqlFilterOperators.Equal, true));
					break;
			}
			var replace = new BXParamsBag<object>();
			bool legal;
			BXPagingParams pagingParams = PreparePagingParams();
			BXQueryParams qParams = PreparePaging(
				pagingParams,
				delegate()
				{
					return BXSaleOrder.Count(ordersFilter);
				},
				replace,
				out legal
			);
			if (!legal)
			{
				error = SaleOrderListComponentError.InvalidPage;
				IncludeComponentTemplate();
				return;
			}

			var ordersCol = BXSaleOrder.GetList(
				ordersFilter,
				new BXOrderBy(new BXOrderByPair(BXSaleOrder.Fields.CreatedUtc, BXOrderByDirection.Desc)),
				new BXSelectAdd(BXSaleOrder.Fields.Status, BXSaleOrder.Fields.Currency),qParams);

			replace.Clear();
			replace.Add("UserId", CurrentUserId);
			var queryParams = HttpUtility.ParseQueryString(BXSefUrlManager.CurrentUrl.Query);
			queryParams.Remove(ActionVariable);
			queryParams.Remove("id");
			queryParams.Add("id","orderId");
			queryParams.Add(ActionVariable,"cancel");

			foreach (var order in ordersCol)
			{
				replace.Add("OrderId", order.Id);
				var detailUrl = ResolveTemplateUrl(OrderDetailUrlTemplate, replace);
				var cancelUrl = ResolveTemplateUrl(CancelOrderUrlTemplate, replace);
				var copyUrl = ResolveTemplateUrl(CopyOrderUrlTemplate, replace);
				var wrapper = new SaleOrderWrapper(order, detailUrl, cancelUrl, copyUrl, this);
				var items = BXSaleProduct.GetList
					(new BXFilter(new BXFilterItem(BXSaleProduct.Fields.OrderId, BXSqlFilterOperators.Equal, order.Id)),
					new BXOrderBy(new BXOrderByPair(BXSaleProduct.Fields.Name, BXOrderByDirection.Asc)));
				foreach (var item in items)
					wrapper.Products.Add(item);

				Orders.Add(wrapper);
			}

			replace.Add("UserId", CurrentUserId);
			cartUrl = ResolveTemplateUrl(Parameters.GetString("CartUrlTemplate", "cart.aspx"), replace);

			BXPublicPage bitrixPage = Page as BXPublicPage;
			if (bitrixPage != null && !IsComponentDesignMode)
			{
				if ( Parameters.GetBool("SetPageTitle", true))
					bitrixPage.BXTitle = GetMessageRaw("PageTitle");
			}

			IncludeComponentTemplate();
		}

		protected override void PreLoadComponentDefinition()
		{
			base.Title = GetMessage("Component.Title");
			base.Description = GetMessage("Component.Description");
			base.Icon = "images/sale_order_list.gif";

			Group = new BXComponentGroup("Store", GetMessage("Group"), 100, BXComponentGroup.Store);

			BXCategory urlCategory = BXCategory.UrlSettings,
				mainCategory = BXCategory.Main,
				sefCategory = BXCategory.Sef;

			ParamsDefinition.Add(
				"SetPageTitle",
				new BXParamYesNo(
					GetMessageRaw("Param.SetPageTitle"),
					true,
					mainCategory
					)
				);

			ParamsDefinition.Add(
				"OrderDetailUrlTemplate",
				new BXParamText(
					GetMessageRaw("Param.OrderDetailUrlTemplate"),
					"orders.aspx?orderId=#OrderId#&action=detail",
					urlCategory
					)
				);

			ParamsDefinition.Add(
				"CancelOrderUrlTemplate",
				new BXParamText(
					GetMessageRaw("Param.CancelOrderUrlTemplate"),
					"orders.aspx?orderId=#OrderId#&action=cancel",
					urlCategory
					)
				);

			ParamsDefinition.Add(
				"CopyOrderUrlTemplate",
				new BXParamText(
					GetMessageRaw("Param.CopyOrderUrlTemplate"),
					"orders.aspx?orderId=#OrderId#&action=copy",
					urlCategory
					)
				);

			ParamsDefinition.Add(
				"CartUrlTemplate",
				new BXParamText(
					GetMessageRaw("Param.CartUrlTemplate"),
					"cart.aspx",
					BXCategory.UrlSettings
					)
				);

			BXParametersDefinition.SetPaging(ParamsDefinition, ClientID, BXParametersDefinition.PagingParamsOptions.DisableLoad | BXParametersDefinition.PagingParamsOptions.DisableShowAll);
			BXParametersDefinition.SetPagingUrl(ParamsDefinition, ClientID, BXParametersDefinition.PagingParamsOptions.DisableLoad | BXParametersDefinition.PagingParamsOptions.DisableShowAll);

		}

		protected override void LoadComponentDefinition()
		{
			BXParametersDefinition.SetPaging(ParamsDefinition, ClientID, BXParametersDefinition.PagingParamsOptions.DisableShowAll);
			BXParametersDefinition.SetPagingUrl(ParamsDefinition, ClientID,  BXParametersDefinition.PagingParamsOptions.DisableShowAll);
		}

		public BXCurrency Currency
		{
			get { return BXCurrency.NationalCurrency; }
		}
	}

	public class SaleOrderWrapper
	{
		BXSaleOrder order;
		List<BXSaleProduct> products;
		SaleOrderListComponent component;


		public SaleOrderWrapper(BXSaleOrder order, string detailUrl, string cancelUrl, string copyUrl, SaleOrderListComponent component)
		{
			if ( order == null )
				throw new ArgumentNullException("Order");

			if (component == null)
				throw new ArgumentNullException("Component");

			this.order = order;
			DetailUrl = detailUrl;
			CancelUrl = cancelUrl;
			CopyUrl = copyUrl;
			this.component = component;
		}

		public BXSaleOrder Order
		{
			get { return order; }
			set
			{
				if (value == null)
					throw new ArgumentNullException("Order");
				order = value;
			}
		}

		public string StatusClassName
		{
			get { return order.Status != null ? order.Status.XmlId.ToLower() : "default"; }
		}

		public SaleOrderListComponent Component
		{
			get { return component; }
			set { component = value; }
		}
		
		public string DetailUrl {get; set; }
		public string CancelUrl { get; set; }
		public string CopyUrl { get; set; }

		public List<BXSaleProduct> Products
		{
			get { return products ?? (products = new List<BXSaleProduct>()); }
		}

		string localizedStatusName;

		public string LocalizedStatusName
		{
			get 
			{
				if (localizedStatusName != null)
					return localizedStatusName;

				string name = string.Empty;
				if( order.Status!=null )
				{
					name =order.Status.GetLocalizedName(Bitrix.Services.BXLoc.CurrentLocale);
				}
				
				return localizedStatusName = String.IsNullOrEmpty(name) ? component.GetMessage("LocNotFound") : name;
			}
		}
	}
}
