﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using Bitrix.Components;
using Bitrix.DataLayer;
using Bitrix.Security;
using Bitrix.Services;
using Bitrix.Configuration;
using System.Web.UI.HtmlControls;
using System.Collections.Specialized;
using Bitrix.Services.Text;

namespace Bitrix.Sale.Components
{
	public partial class SaleOrderPaymentReceiveComponent : BXComponent
	{
		private bool renderHandler;
		private IBXSalePaymentResponseController controller;
		private BXSalePaymentSystemConfig paymentConfig;
		private Control paymentController;
	

		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);
			paymentConfig = BXSalePaymentSystemConfig.GetById(Parameters.GetInt("PaymentConfig"));

			try
			{
				if (IsComponentDesignMode || BXConfigurationUtility.IsDesignMode)
					return;

				if (paymentConfig != null && !string.IsNullOrEmpty(paymentConfig.ResponseControllerFileVirtualPath))
					paymentController = LoadControl(paymentConfig.ResponseControllerFileVirtualPath);
				
				if (this.paymentController == null)
					return;
				
				controller = paymentController as IBXSalePaymentResponseController;
				if (controller == null)
					return;

				var page = Page as BXPublicPage;
				if (page != null)
					page.EnableCsrfValidation = false;
				Controls.Add(paymentController);

				controller.InitializeData(paymentConfig);
			}
			catch
			{

			}
		}
		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);
							
			if (controller != null)
			{
				controller.Execute();
				using (var h = new HtmlTextWriter(Response.Output))
					paymentController.RenderControl(h);
				Response.End();	
				return;
			}
			
			IncludeComponentTemplate();
		}
		
		protected override void PreLoadComponentDefinition()
		{
			base.Title = GetMessage("Title");
			base.Description = GetMessage("Description");
			base.Icon = "images/sale_order_payment.gif";
			base.Group = new BXComponentGroup("OrderCheckout", GetMessage("Group"), 100, BXComponentGroup.Store);

			ParamsDefinition["CustomerTypeId"] = new BXParamSingleSelection(GetMessageRaw("Param.CustomerTypeId"), "", BXCategory.Main);
			ParamsDefinition["PaymentConfig"] = new BXParamSingleSelection(GetMessageRaw("Param.PaymentConfig"), "", BXCategory.Main);
		}

		protected override void LoadComponentDefinition()
		{
			base.LoadComponentDefinition();

			var param = ParamsDefinition["CustomerTypeId"];
			param.Values =
				BXSaleCustomerType.GetAllForSite(DesignerSite)
				.ConvertAll<BXParamValue>(x => new BXParamValue(x.TextEncoder.Decode(x.Name), x.Id.ToString()));
			param.RefreshOnDirty = true;

			var cid = Parameters.GetInt("CustomerTypeId");

			param = ParamsDefinition["PaymentConfig"];
			param.Values = BXSalePaymentSystemConfig.GetList(
				new BXFilter(
					cid != 0
					? new BXFilterItem(BXSalePaymentSystemConfig.Fields.CustomerTypeId, BXSqlFilterOperators.Equal, cid)
					: new BXFilterItem(BXSalePaymentSystemConfig.Fields.CustomerType.Site.ID, BXSqlFilterOperators.Equal, DesignerSite)
				),
				new BXOrderBy(new BXOrderByPair(BXSalePaymentSystemConfig.Fields.Name, BXOrderByDirection.Asc)),
				new BXSelect(
					BXSalePaymentSystemConfig.Fields.Id,
					BXSalePaymentSystemConfig.Fields.Name
				),
				null,
				BXTextEncoder.EmptyTextEncoder
			)
			.ConvertAll(x => new BXParamValue(x.Name, x.Id.ToString()));
		}
	}

	public partial class SaleOrderPaymentReceiveTemplate : BXComponentTemplate<SaleOrderPaymentReceiveComponent>
	{

	}
}
