﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.UI;
using Bitrix.Components;
using Bitrix.DataLayer;
using Bitrix.Security;
using Bitrix.Services;
using Bitrix.Configuration;
using System.Web.UI.HtmlControls;
using System.Collections.Specialized;

namespace Bitrix.Sale.Components
{
    /// <summary>
    /// Компонент "Оплата Заказа"
    /// </summary>
    public partial class SaleOrderPaymentComponent : BXComponent
    {
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (CurrentUserId <= 0)
                this.error = SaleOrderPaymentComponentError.Unauthorized;
            else if (OrderId <= 0)
                this.error = SaleOrderPaymentComponentError.OrderIdNotFound;
            else
            {

                BXSaleOrderCollection col = BXSaleOrder.GetList(
                    new BXFilter(
                        new BXFilterItem(BXSaleOrder.Fields.Id, BXSqlFilterOperators.Equal, OrderId),
                        new BXFilterItem(BXSaleOrder.Fields.SiteId, BXSqlFilterOperators.Equal, BXSite.Current.Id),
                        new BXFilterItem(BXSaleOrder.Fields.UserId, BXSqlFilterOperators.Equal, CurrentUserId)
                        ),
                    null,
                    null,
                    new BXQueryParams(new BXPagingOptions(0, 1))
                    );

                if ((this.order = col.Count > 0 ? col[0] : null) == null)
                    this.error = SaleOrderPaymentComponentError.OrderNotFound;
                else if (this.order.PaymentSystemId <= 0)
                    this.error = SaleOrderPaymentComponentError.PaymentSystemNotFound;
                else if ((this.paymentConfig = BXSalePaymentManager.GetPaymentSystemConfig(this.order)) == null)
                    this.error = SaleOrderPaymentComponentError.PaymentSystemNotAvailable;
                else if (string.IsNullOrEmpty(this.paymentConfig.ControllerFileVirtualPath))
                    this.error = SaleOrderPaymentComponentError.ControllerLoadingFailed;
                else
                {
                    try
                    {
                        this.paymentController = LoadControl(this.paymentConfig.ControllerFileVirtualPath);
                    }
                    catch
                    {
                    }

                    if (this.paymentController == null)
                        this.error = SaleOrderPaymentComponentError.ControllerLoadingFailed;
                    else
                    {
                        IBXSalePaymentController controller = this.paymentController as IBXSalePaymentController;
                        if (controller != null)
                            controller.InitializeData(this.order, this.paymentConfig);
                    }
                }
            }
            IncludeComponentTemplate();
        }

        private SaleOrderPaymentComponentError error = SaleOrderPaymentComponentError.None;
        public SaleOrderPaymentComponentError ComponentError
        {
            get { return this.error; }
        }

        private string errorMsg = string.Empty;
        public string[] GetErrorMessages()
        {
            if (this.error == SaleOrderPaymentComponentError.None)
                return new string[0];

            List<string> list = new List<string>();
            if ((this.error & SaleOrderPaymentComponentError.Unauthorized) != 0)
                list.Add(GetMessage("Error.Unauthorized"));
            if ((this.error & SaleOrderPaymentComponentError.OrderNotFound) != 0)
                list.Add(string.Format(GetMessage("Error.OrderNotFound"), OrderId));
            if ((this.error & SaleOrderPaymentComponentError.OrderIdNotFound) != 0)
                list.Add(string.Format(GetMessage("Error.OrderIdNotFound"), OrderId));
            if ((this.error & SaleOrderPaymentComponentError.PaymentSystemNotFound) != 0)
                list.Add(string.Format(GetMessage("Error.PaymentSystemNotFound"), OrderId));
            if ((this.error & SaleOrderPaymentComponentError.PaymentSystemNotAvailable) != 0)
                list.Add(string.Format(GetMessage("Error.PaymentSystemNotAvailable"), PaymentConfig != null ? PaymentConfig.Name : "<?>"));
            if ((this.error & SaleOrderPaymentComponentError.ControllerLoadingFailed) != 0)
                list.Add(string.Format(GetMessage("Error.ControllerLoadingFailed"), PaymentConfig != null ? PaymentConfig.Name : "<?>"));
            if ((this.error & SaleOrderPaymentComponentError.General) != 0)
                list.Add(HttpUtility.HtmlEncode(this.errorMsg));

            return list.ToArray();
        }

        private int CurrentUserId
        {
            get
            {
                BXIdentity identity = (BXIdentity)BXPrincipal.Current.Identity;
                return identity != null ? identity.Id : 0;
            }
        }

        private BXUser user = null;
        public BXUser CurrentUser
        {
            get
            {
                if (CurrentUserId <= 0)
                    return null;
                return user ?? (user = BXUser.GetById(CurrentUserId));
            }
        }

        private int? orderId = null;
        public int OrderId
        {
            get { return (this.orderId ?? (this.orderId = Parameters.GetInt("OrderId", 0))).Value; }
            set { Parameters["OrderId"] = (orderId = value > 0 ? value : 0).ToString(); }
        }

        private BXSaleOrder order = null;
        public BXSaleOrder Order
        {
            get { return this.order; }
        }

        private BXSalePaymentSystemConfig paymentConfig = null;
        public BXSalePaymentSystemConfig PaymentConfig
        {
            get { return this.paymentConfig; }
        }

        private Control paymentController = null;
        public Control PaymentController
        {
            get { return this.paymentController; }
        }

        public Control LoadPaymentControllerStub()
        {
            string stubFileVirtualPath = string.Empty;
            if (this.paymentConfig != null && !string.IsNullOrEmpty(this.paymentConfig.ControllerStubFileVirtualPath))
                stubFileVirtualPath = this.paymentConfig.ControllerStubFileVirtualPath;

            if (string.IsNullOrEmpty(stubFileVirtualPath))
                stubFileVirtualPath = "~/bitrix/controls/Sale/payment/error.ascx";

            Control c = LoadControl(stubFileVirtualPath);

            IBXSalePaymentController controller = c as IBXSalePaymentController;
            if (controller != null)
                controller.InitializeData(this.order, this.paymentConfig);

            return c;
        }

        protected override void PreLoadComponentDefinition()
        {
            base.Title = GetMessage("Title");
            base.Description = GetMessage("Description");
            base.Icon = "images/sale_order_payment.gif";
            base.Group = new BXComponentGroup("OrderCheckout", GetMessage("Group"), 100, BXComponentGroup.Store);

            ParamsDefinition.Add(
                "OrderId",
                new BXParamText(
                    GetMessageRaw("Param.OrderId"),
                    "<%$Request:orderId %>",
                    BXCategory.Main
                    )
                );
        }

        public string DisplayPaymentFormParamName
        {
            get { return "displayPaymentForm"; }
        }

        public bool DisplayPaymentForm
        {
            get { return string.Equals(Request[DisplayPaymentFormParamName], "Y", StringComparison.OrdinalIgnoreCase); }
        }

        public string GetDisplayFormUrl()
        {
            Uri url = BXSefUrlManager.CurrentUrl;
            NameValueCollection qsParams = HttpUtility.ParseQueryString(url.Query);
            qsParams[DisplayPaymentFormParamName] = "Y";
            return HttpUtility.HtmlAttributeEncode(string.Concat(url.AbsolutePath, "?", qsParams.ToString()));
        }
    }

    public partial class SaleOrderPaymentTemplate : BXComponentTemplate<SaleOrderPaymentComponent>
    {
        protected override void Render(HtmlTextWriter writer)
        {
            if (Component.ComponentError != SaleOrderPaymentComponentError.None)
            {
                if (IsComponentDesignMode)
                {
                    writer.Write(BXLoc.GetMessage(Component, "YouHaveToAdjustTheComponent"));
                    return;
                }
                else if (BXConfigurationUtility.IsDesignMode && Bitrix.UI.TemplateRequisite.GetCurrentPublicPanelVisiblity(Component.Page))
                {
                    foreach (string msg in Component.GetErrorMessages())
                        writer.Write("<div>" + msg + "</div>");
                    return;
                }
            }

            bool openInNewWin = Component.PaymentConfig != null && Component.PaymentConfig.ControllerInfo != null ? Component.PaymentConfig.ControllerInfo.OpenInNewWindow : false;

            if (openInNewWin)
            {
                if (Component.DisplayPaymentForm)
                {
                    Response.Buffer = true;
                    writer.Flush();
                    Response.Clear();
                    if (Component.ComponentError == SaleOrderPaymentComponentError.None)
                        Component.PaymentController.RenderControl(writer);
                    else
                    {
                        Control stub = Component.LoadPaymentControllerStub();
                        if (stub != null)
                            stub.RenderControl(writer);
                    }
                    Response.End();
                }
                else
                {
                    HtmlAnchor anchor = new HtmlAnchor();
                    anchor.HRef = Component.GetDisplayFormUrl();
                    anchor.InnerText = GetMessageRaw("Link.Pay");
                    anchor.Title = GetMessageRaw("Tooltip.Pay");
                    anchor.Target = "_blank";
                    anchor.RenderControl(writer);
                }
            }
            else
            {
                if (Component.ComponentError == SaleOrderPaymentComponentError.None)
                    Component.PaymentController.RenderControl(writer);
                else
                {
                    Control stub = Component.LoadPaymentControllerStub();
                    if (stub != null)
                        stub.RenderControl(writer);
                }
            }
        }
    }

    public enum SaleOrderPaymentComponentError
    {
        None = 0x0,
        General = 0x1,
        Unauthorized = 0x2,
        OrderIdNotFound = 0x4,
        OrderNotFound = 0x8,
        PaymentSystemNotFound = 0x10,
        PaymentSystemNotAvailable = 0x20,
        ControllerLoadingFailed = 0x40
    }
}
