﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using Bitrix.Components;
using Bitrix.Security;
using System.Collections.Generic;
using Bitrix.Modules;
using Bitrix.UI;
using Bitrix.Sale.SaleCart;
using Bitrix.DataLayer;
using Bitrix.DataTypes;
using System.Web;
using Bitrix.Services.Text;
using Bitrix.Sale;
using Bitrix.Services;
using Bitrix.Components.Editor;

namespace Bitrix.Sale.Components
{
	public partial class SaleOrderComponent : BXComponent
	{
		int curUserId = -1;

		//Id текущего пользователя, у анонимов - 0
		public int CurrentUserId
		{
			get { return curUserId != -1 ? curUserId : (curUserId = 
				(BXPrincipal.Current != null && BXPrincipal.Current.Identity.IsAuthenticated ? ((BXIdentity)BXPrincipal.Current.Identity).Id : 0)); }
		}

		private string ActionVariable
		{
			get
			{
				return Parameters.GetString("ActionVariable", "");
			}
			set
			{
				Parameters["ActionVariable"] = value;
			}
		}

		private string PageVariable
		{
			get
			{
				return Parameters.GetString("PageVariable", "");
			}
			set
			{
				Parameters["PageVariable"] = value;
			}
		}

		private string OrderVariable
		{
			get
			{
				return Parameters.GetString("OrderVariable", "");
			}
			set
			{
				Parameters["OrderVariable"] = value;
			}
		}

		public bool EnableSef
		{
			get
			{
				return Parameters.Get("EnableSef", false);
			}
		}

		public string SefFolder
		{
			get { return Parameters.Get<string>("SEFFolder", String.Empty); }
		}

		protected void Page_Load(object sender, EventArgs e)
		{
			var page = EnableSef ? PrepareSefMode() : PrepareNormalMode();

			IncludeComponentTemplate(page);
		}


		protected override void PreLoadComponentDefinition()
		{
			base.Title = GetMessage("Component.Title");
			base.Description = GetMessage("Component.Description");
			base.Icon = "images/sale_form.gif";

			Group = new BXComponentGroup("Store", GetMessage("Group"), 100, BXComponentGroup.Store);

			BXCategory urlCategory = BXCategory.UrlSettings,
				mainCategory = BXCategory.Main,
				sefCategory = BXCategory.Sef;

			ParamsDefinition.Add(
				"SetPageTitle",
				new BXParamYesNo(
					GetMessageRaw("Param.SetPageTitle"),
					true,
					mainCategory
					)
				);

			ParamsDefinition.Add(
				"EnableSef",
				new BXParamYesNo(
					GetMessageRaw("Param.EnableSef"),
					false,
					sefCategory,
					new ParamClientSideActionGroupViewSwitch(ClientID, "EnableSef", "On", "Off")
					)
				);

			ParamsDefinition.Add(
				"SefFolder",
				new BXParamText(
					GetMessageRaw("Param.SefFolder"),
					"",
					sefCategory,
					new ParamClientSideActionGroupViewMember(ClientID, "SefFolder", new string[] { "On" })
					)
				);

			ParamsDefinition.Add(
				"OrderListTemplate",
				new BXParamText(
					GetMessageRaw("Param.OrderListTemplate"),
					"",
					sefCategory,
					new ParamClientSideActionGroupViewMember(ClientID, "OrderListTemplate", new string[] { "On" })
					)
				);

			ParamsDefinition.Add(
				"OrderListPageTemplate",
				new BXParamText(
					GetMessageRaw("Param.OrderListPageTemplate"),
					"/page-#i:PageId#/",
					sefCategory,
					new ParamClientSideActionGroupViewMember(ClientID, "OrderListPageTemplate", new string[] { "On" })
					)
				);

			ParamsDefinition.Add(
				"OrderDetailTemplate",
				new BXParamText(
					GetMessageRaw("Param.OrderDetailTemplate"),
					"/#i:OrderId#/",
					sefCategory,
					new ParamClientSideActionGroupViewMember(ClientID, "OrderDetailTemplate", new string[] { "On" })
					)
				);

			ParamsDefinition.Add(
				"CancelOrderTemplate",
				new BXParamText(
					GetMessageRaw("Param.CancelOrderTemplate"),
					"/#i:OrderId#/cancel/",
					sefCategory,
					new ParamClientSideActionGroupViewMember(ClientID, "CancelOrderTemplate", new string[] { "On" })
					)
				);

			ParamsDefinition.Add(
				"CopyOrderTemplate",
				new BXParamText(
					GetMessageRaw("Param.CopyOrderTemplate"),
					"/#i:OrderId#/copy/",
					sefCategory,
					new ParamClientSideActionGroupViewMember(ClientID, "CopyOrderTemplate", new string[] { "On" })
					)
				);

			ParamsDefinition.Add(
				"ActionVariable",
				new BXParamText(
					GetMessageRaw("Param.ActionVariable"),
					"action",
					sefCategory,
					new ParamClientSideActionGroupViewMember(ClientID, "ActionVariable", new string[] { "Off" })
					)
				);

			ParamsDefinition.Add(
				"OrderVariable",
				new BXParamText(
					GetMessageRaw("Param.OrderVariable"),
					"order",
					sefCategory,
					new ParamClientSideActionGroupViewMember(ClientID, "OrderVariable", new string[] { "Off" })
					)
				);

			ParamsDefinition.Add(
				"PageVariable",
				new BXParamText(
					GetMessageRaw("Param.PageVariable"),
					"page",
					sefCategory,
					new ParamClientSideActionGroupViewMember(ClientID, "PageVariable", new string[] { "Off" })
					)
				);

			ParamsDefinition.Add(
				"CartUrlTemplate",
				new BXParamText(
					GetMessageRaw("Param.CartUrlTemplate"),
					"cart.aspx",
					BXCategory.UrlSettings
					)
				);

			foreach (var customerType in CustomerTypes)
			{
				ParamsDefinition.Add(
					"DisplayCustomProps_" + customerType.Code.ToString(),
					new BXParamDoubleList(
						String.Format(GetMessage("Param.DisplayProps"), customerType.TextEncoder.Decode(customerType.Name)),
						"",
						BXCategory.DetailSettings
						)
					);
			}

			BXParametersDefinition.SetPaging(ParamsDefinition, ClientID, 
				BXParametersDefinition.PagingParamsOptions.DisableLoad | BXParametersDefinition.PagingParamsOptions.DisableShowAll);
		}

		BXSaleCustomerTypeCollection cTypes;

		protected BXSaleCustomerTypeCollection CustomerTypes
		{
			get { return cTypes ?? (cTypes = BXSaleCustomerType.GetAllForSite(DesignerSite)); }
		}

		protected override void LoadComponentDefinition()
		{
			BXParametersDefinition.SetPaging(ParamsDefinition, ClientID, 
				BXParametersDefinition.PagingParamsOptions.DisablePreLoad | BXParametersDefinition.PagingParamsOptions.DisableShowAll);

			foreach (var customerType in CustomerTypes)
			{
				ParamsDefinition["DisplayCustomProps_" + customerType.Code.ToString()]
					.Values.AddRange(
					BXSaleOrderField.GetCustomerFields(customerType.Id, null).ConvertAll<BXParamValue>(
						x => new BXParamValue(x.Name, x.Code.ToString())
						));
			}
		}


		private string PrepareSefMode()
		{
			string sefFolder = Parameters.GetString("SEFFolder", "");
			ComponentCache["IndexUrlTemplate"] = CombineLink(sefFolder, "");
			ComponentCache["OrderDetailUrlTemplate"] = CombineLink(sefFolder, Parameters.GetString("OrderDetailTemplate"));
			ComponentCache["CancelOrderUrlTemplate"] = CombineLink(sefFolder, Parameters.GetString("CancelOrderTemplate"));
			ComponentCache["OrderListPageUrlTemplate"] = CombineLink(sefFolder, Parameters.GetString("OrderListPageTemplate"));
			ComponentCache["OrderListUrlTemplate"] = CombineLink(sefFolder, Parameters.GetString("OrderListTemplate"));
			ComponentCache["CopyOrderUrlTemplate"] = CombineLink(sefFolder, Parameters.GetString("CopyOrderTemplate"));
			string page = PrepareSefPage();

			return page;

		}
		private BXParamsBag<string> GetSefMap()
		{
			BXParamsBag<string> sefMap = new BXParamsBag<string>();
			sefMap.Add("detail", Parameters.GetString("OrderDetailTemplate", ""));
			sefMap.Add("list.page", Parameters.GetString("OrderListPageTemplate", ""));
			sefMap.Add("list.cancel", Parameters.GetString("CancelOrderTemplate"));
			sefMap.Add("list.copy", Parameters.GetString("CopyOrderTemplate"));
			return sefMap;
		}
		private string PrepareSefPage()
		{
			BXParamsBag<string> sefMap = GetSefMap();
			string code = BXSefUrlUtility.MapVariable(Parameters.GetString("SEFFolder", ""), sefMap, ComponentCache, "list", null, null);
			string page;
			string action;
			int i = code.IndexOf('.');
			if (i == -1)
			{
				page = code;
				action = string.Empty;
			}
			else
			{
				page = code.Remove(i);
				action = code.Substring(i + 1);
			}

			switch (page)
			{
				case "list":
					switch (action)
					{
						case "cancel":
							ComponentCache["Action"] = "cancel";
							return "cancel";
						case "copy":
							ComponentCache["Action"] = "copy";
							break;
					}
					break;
			}

			return page;
		}

		private string PrepareNormalMode()
		{
			UriBuilder uri = new UriBuilder(BXSefUrlManager.CurrentUrl);
			uri.Query = null;

			string actionVar = UrlEncode(ActionVariable);
			string pageVar = UrlEncode(PageVariable);
			string filePath = Request.FilePath;
			string orderVar = UrlEncode(OrderVariable);

			ComponentCache["IndexUrlTemplate"] = uri.ToString();
			ComponentCache["CancelOrderUrlTemplate"] = String.Format("{0}?{1}=#OrderId#&{2}=cancel", filePath, orderVar, actionVar);
			ComponentCache["CopyOrderUrlTemplate"] = String.Format("{0}?{1}=#OrderId#&{2}=copy", filePath, orderVar, actionVar);
			ComponentCache["OrderDetailUrlTemplate"] = String.Format("{0}?{1}=#OrderId#&{2}=detail", filePath, orderVar, actionVar);
			ComponentCache["OrderListPageUrlTemplate"] = String.Format("{0}?{1}=#PageId#", filePath, pageVar);
			ComponentCache["OrderListUrlTemplate"] = filePath;

			return PrepareNormalPage();
		}
		private string PrepareNormalPage()
		{

			int? pageId = ReadQueryInt(PageVariable);
			int? orderId = ReadQueryInt(OrderVariable);
			string action = (Request.QueryString[ActionVariable] ?? string.Empty).ToLowerInvariant();

			ComponentCache["PageId"] = pageId.ToString();
			switch(action)
			{
				case "detail":
					ComponentCache["OrderId"] = orderId.HasValue ? orderId.Value : 0;
					return "detail";
				case "cancel":
					ComponentCache["Action"] ="cancel";
					ComponentCache["OrderId"] = orderId.HasValue ? orderId.Value : 0;
					return "cancel";
				case "copy":
					ComponentCache["Action"] = "copy";
					ComponentCache["OrderId"] = orderId.HasValue ? orderId.Value : 0;
					break;
			}
			if (action.Equals("detail", StringComparison.Ordinal))
				return "detail";

			return "list";
		}

		private int? ReadQueryInt(string param)
		{
			string[] values = Request.QueryString.GetValues(param);
			if (values == null || values.Length == 0)
				return null;
			int i;
			if (!int.TryParse(values[0], out i))
				return null;
			return i;
		}
	}
}
