﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.Sale;
using System.Globalization;
using Bitrix.Currency;
using Bitrix.Services;
using System.Collections.Specialized;
using System.IO;
using System.Text;


/// <summary>
/// Обработчик оплаты заказа "Assist"
/// </summary>
public partial class BXAssistPaymentController : UserControl, IBXSalePaymentController
{
    #region IBXSalePaymentController Members
    public string DisplayName
    {
        get { return BXSaleModule.GetMessage("AssistPaymentController.DisplayName", false); }
    }

    private BXSaleOrder order = null;
    private BXSalePaymentSystemConfig conf = null;
    private BXSalePaymentControllerSettings settings = null;
    public void InitializeData(BXSaleOrder order, BXSalePaymentSystemConfig conf)
    {
        if(order == null)
            throw new ArgumentNullException("order");

        this.order = order;
        this.conf = conf;
        if (conf != null)
        {
            this.settings = conf.ControllerSettings;
            this.settings.Order = order;
        }
        this.renderingMode = string.Equals(HttpUtility.ParseQueryString(BXSefUrlManager.CurrentUrl.Query)["assistPayForm"], "Y", StringComparison.OrdinalIgnoreCase) ? ControllerRenderingMode.Form : ControllerRenderingMode.Frame;
    }
    #endregion

	public int OrderId
    {
        get { return this.order.Id; }
    }
    internal enum ControllerRenderingMode
    {
        Frame = 1,
        Form = 2
    }

    private ControllerRenderingMode renderingMode = ControllerRenderingMode.Frame;
    internal ControllerRenderingMode RenderingMode
    {
        get { return this.renderingMode; }
    }

    public string GetFormRenderingUrl()
    {
        Uri url = BXSefUrlManager.CurrentUrl;
        NameValueCollection qsParams = HttpUtility.ParseQueryString(url.Query);
        qsParams["assistPayForm"] = "Y";
        return HttpUtility.HtmlAttributeEncode(string.Concat(url.AbsolutePath, "?", qsParams.ToString()));
    }

    public string GetHtmlEncStrParam(string name, string defaultValue)
    {
        if (this.settings == null)
            return defaultValue;

        return HttpUtility.HtmlEncode(this.settings.GetString(name, defaultValue));
    }

    public string GetHtmlAttrEncStrParam(string name)
    {
        return GetHtmlAttrEncStrParam(name, string.Empty);
    }

    public string GetHtmlAttrEncStrParam(string name, string defaultValue)
    {
        if (this.settings == null)
            return defaultValue;

        return HttpUtility.HtmlEncode(this.settings.GetString(name, defaultValue));
    }

    public string GetStrParam(string name)
    {
        return GetStrParam(name, string.Empty);
    }

    public string GetStrParam(string name, string defaultValue)
    {
        if (this.settings == null)
            return defaultValue;

        return this.settings.GetString(name, defaultValue);
    }

    public int GetInt32Param(string name)
    {
        return GetInt32Param(name, 0);
    }

    public int GetInt32Param(string name, int defaultValue)
    {
        if (this.settings == null)
            return defaultValue;

        return this.settings.GetInt32(name, defaultValue);
    }

    private BXCurrency currency = null;
    public BXCurrency Currency
    {
        get 
        {
            if(this.currency != null)
                return this.currency;

            int currencyId = this.settings.GetInt32("currency", 0);
            return this.currency = currencyId > 0 ? BXCurrency.GetById(currencyId) : BXCurrency.NationalCurrency;
        }
    }

    public string CurrencySymbol
    {
        get { return Currency != null ? Currency.GetSymbol() : string.Empty; }
    }

    public string CurrencyAlphaCode
    {
        get 
        {
            if(Currency == null || string.Equals(Currency.Iso4217AlphaCode, "RUB", StringComparison.Ordinal))
                return "RUR";

            return Currency.Iso4217AlphaCode;
        }
    }

    public string FormatMoney(decimal value, bool addCurrencySymbol)
    {
        return Currency != null ? Currency.MoneyToString(value, CultureInfo.CurrentCulture, addCurrencySymbol) : value.ToString("F2");
    }

    public string OrderTitle
    {
        get 
        {
            DateTime d = this.settings.GetDateTime("created", DateTime.MinValue);
            return string.Format(
                "Счёт №{0} от {1}", 
                OrderId.ToString(), 
                d != DateTime.MinValue ? HttpUtility.HtmlEncode(d.ToString("d.mm.yyyy")) : "-"
                );
        }
    }

    /// <summary>
    /// Итоговая сумма заказа
    /// </summary>
    public decimal OrderTotal
    {
        get { return this.settings != null ? this.settings.GetDecimal("sum", 0m) : 0m; }
    }

    public override void RenderControl(HtmlTextWriter writer)
    {
        if(this.renderingMode == ControllerRenderingMode.Frame)
        {
            base.RenderControl(writer);
            return;
        }

        Response.ContentEncoding = Encoding.GetEncoding("Windows-1251");
        Response.Charset="windows-1251";
        writer.Flush();
        Response.Clear();
        base.RenderControl(writer);
        Response.End();
    }
}
