﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.Sale;
using Bitrix.UI;

/// <summary>
/// Обработчик оплаты заказа "Квитанция Сбербанка России"
/// </summary>
public partial class BXSberbankPaymentController : BXControl, IBXSalePaymentController
{
    #region IBXSalePaymentController Members
    public string DisplayName
    {
        get { return BXSaleModule.GetMessage("SberbankPaymentController.DisplayName", false); }
    }

    private BXSaleOrder order = null;
    private BXSalePaymentSystemConfig conf = null;
    private BXSalePaymentControllerSettings settings = null;
    public void InitializeData(BXSaleOrder order, BXSalePaymentSystemConfig conf)
    {
        if(order == null)
            throw new ArgumentNullException("order");

        this.order = order;
        this.conf = conf;
        if(conf != null)
        {
            this.settings = conf.ControllerSettings;
            this.settings.Order = order;
        }
    }
    #endregion

    public int OrderId
    {
        get { return this.order.Id; }
    }

    private readonly static string subsForEmpty = "-";

    public int GetInt32Param(string name)
    {
        return GetInt32Param(name, 0);
    }

    public int GetInt32Param(string name, int defaultValue)
    {
        if (this.settings == null)
            return defaultValue;

        return this.settings.GetInt32(name, defaultValue);
    }

    /// <summary>
    /// Дата создания заказа
    /// </summary>
    public string OrderCreationDate
    {
        get 
        { 
            if(this.settings == null)
                return subsForEmpty;

            DateTime d = this.settings.GetDateTime("orderCreationDate", DateTime.MinValue);

            return d != DateTime.MinValue ? HttpUtility.HtmlEncode(d.ToString("d.mm.yyyy")) : subsForEmpty;
        }
    }

    /// <summary>
    /// Итоговая сумма заказа, целая часть (рубли)
    /// </summary>
    public string OrderTotalIntegerPart
    {
        get 
        {
            if(this.settings == null)
                return subsForEmpty;

            decimal m = this.settings.GetDecimal("sum", 0m);
            return m != 0m ? HttpUtility.HtmlEncode(Convert.ToInt32(decimal.Floor(m)).ToString()) : subsForEmpty; 
        }
    }

    /// <summary>
    /// Итоговая сумма заказа, сотые части (копейки)
    /// </summary>
    public string OrderTotalCentPart
    {
        get 
        {
            if(this.settings == null)
                return subsForEmpty;

            decimal m = this.settings.GetDecimal("sum", 0m);
            return this.order != null ? HttpUtility.HtmlEncode(Convert.ToInt32(decimal.Floor((m - decimal.Floor(m)) * 100)).ToString()) : subsForEmpty; 
        }
    }

    private string GetHtmlEncodeStringParam(string name, string defaultValue)
    {
       if(this.settings == null)
           return subsForEmpty;

        return HttpUtility.HtmlEncode(this.settings.GetString(name, defaultValue));
    }

    /// <summary>
    /// ФИО клиента
    /// </summary>
    public string CustomerFullName
    {
        get { return GetHtmlEncodeStringParam("customerFullName", string.Empty); }
    }

    /// <summary>
    /// Адрес клиента
    /// </summary>
    public string CustomerAddress 
    {
        get { return GetHtmlEncodeStringParam("customerAddress", string.Empty); }
    }

    /// <summary>
    /// Наименование магазина
    /// </summary>
    public string CompanyName
    {
        get { return GetHtmlEncodeStringParam("companyName", string.Empty); }
    }

    /// <summary>
    /// ИНН магазина
    /// </summary>
    public string INN
    {
        get { return GetHtmlEncodeStringParam("inn", string.Empty); }
    }

    /// <summary>
    /// КПП магазина
    /// </summary>
    public string KPP
    {
        get { return GetHtmlEncodeStringParam("kpp", string.Empty); }
    }

    /// <summary>
    /// Расчетный счёт магазина
    /// </summary>
    public string SettlementAccount
    {
        get { return GetHtmlEncodeStringParam("settlementAccount", string.Empty); }
    }

    /// <summary>
    /// Название банка магазина
    /// </summary>
    public string BankName
    {
        get { return GetHtmlEncodeStringParam("bankName", string.Empty); }
    }

    /// <summary>
    /// БИК банка магазина
    /// </summary>
    public string BIC
    {
        get { return GetHtmlEncodeStringParam("bic", string.Empty); }
    }

    /// <summary>
    /// Корсчёт банка магазина
    /// </summary>
    public string CorrespondentAccount
    {
        get { return GetHtmlEncodeStringParam("correspondentAccount", string.Empty); }
    }
}
