﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.Sale;
using System.Globalization;
using Bitrix.Currency;
using Bitrix.Services;
using System.Collections.Specialized;
using System.IO;
using System.Text;
using System.Xml;
using System.Security.Cryptography;
using Bitrix.Services.Text;

namespace Bitrix.Sale
{
	public partial class BXYandexMoneyPaymentReceiveController : UserControl, IBXSalePaymentResponseController
	{
		protected BXSalePaymentSystemConfig conf;
		protected BXSalePaymentControllerSettings settings;
		protected BXSaleOrder order;
		protected int code;
		protected string action;
		protected string shopId;
		protected string invoiceId;
		protected string techMessage;
		//protected BXSalePaymentControllerSettings settings;


		#region IBXSalePaymentResponseController Members

		public void InitializeData(BXSalePaymentSystemConfig conf)
		{
			this.conf = conf;
			settings = conf.ControllerSettings;
		}

		public void Execute()
		{
			action = Request.Form["action"];
			shopId = Request.Form["shopId"];
			invoiceId = Request.Form["invoiceId"];

			order = GetOrder();
			if (order == null)
			{
				code = 200;
				techMessage = "Заказ не найден";
				return;
			}
			settings.Order = order;

			if (!CheckMd5())
			{
				code = 1;
				techMessage = "Ошибка авторизации";
			}

			if (settings.GetString("shopId", null) != shopId)
			{
				code = 200;
				techMessage = "Неверный ИД магазина";
				return;
			}

			if (action == "Check")
				Check();
			else if (action == "PaymentSuccess")
				PaymentSuccess();
			else 
			{
				code = 200;
				techMessage = "Действие не поддерживается";
			}			
		}

		private BXSaleOrder GetOrder()
		{
			string customerNumber = Request.Form["customerNumber"];
			int id;
			if (string.IsNullOrEmpty(customerNumber) || !int.TryParse(customerNumber, out id))
				return null;

			var order = BXSaleOrder.GetById(id, BXTextEncoder.EmptyTextEncoder);
			if (order == null || order.PaymentSystemId != conf.PaymentSystemId)
				return null;
			return order;
		}

		private bool CheckMd5()
		{
			using (var md5 = MD5.Create())
			{
				var val = string.Join(
					";",
					new[] 
					{
						Request.Form["orderIsPaid"],
						Request.Form["orderSumAmount"],
						Request.Form["orderSumCurrencyPaycash"],
						Request.Form["orderSumBankPaycash"],
						Request.Form["shopId"],
						Request.Form["invoiceId"],
						Request.Form["customerNumber"],
						settings.GetString("shopPassword", "")
					}
				);

				var bytes = md5.ComputeHash(Encoding.GetEncoding(1251).GetBytes(val));
				var s = new StringBuilder(bytes.Length * 2);
				foreach (var b in bytes)
					s.Append(b.ToString("X2"));

				return string.Equals(s.ToString(), Request.Form["md5"], StringComparison.Ordinal);
			}
		}

		private void Check()
		{
			decimal sum;
			var orderSumAmount = Request.Form["orderSumAmount"];
			
			if (string.IsNullOrEmpty(orderSumAmount) || !decimal.TryParse(orderSumAmount, NumberStyles.Currency, CultureInfo.InvariantCulture, out sum) || order.Sum != sum)
			{
				code = 200;
				techMessage = string.Format("Неверная сумма заказа");
				return;
			}

			code = 0;
		}

		private void PaymentSuccess()
		{
			var response = new BXYandexMoneyPrcResponse(Request);
			response.Process();

			if (response.Sum != order.Sum || !response.IsOrderPaid)
			{
				code = 200;
				techMessage = "Неверная сумма заказа";
				return;
			}
			
			try 
			{
				order.Paid = true;
				order.SumPaid = response.Sum;
				order.PaymentSystemResponse = response;
				order.Save();
			}
			catch(Exception ex)
			{
				BXLogService.LogAll(ex, BXLogMessageType.Critical, "Обработка запроса от Яндекс.Деньги");
				code = 1000;
				techMessage = "Ошибка оплаты заказа";
				return;
			}

			code = 0;
		}

		#endregion

		protected override void Render(HtmlTextWriter writer)
		{
			Response.ContentType = "text/xml";
			Response.Charset = "windows-1251";
			Response.ContentEncoding = Encoding.GetEncoding(1251);
			
			var s = new XmlWriterSettings { Encoding = Response.ContentEncoding };
			using (var xml = XmlWriter.Create(Response.OutputStream, s))
			{
				xml.WriteStartDocument();
				xml.WriteStartElement("response");
				xml.WriteAttributeString("performedDatetime", DateTime.UtcNow.ToString(@"yyyy-MM-ddTHH\:mm\:sszzz"));
				
				xml.WriteStartElement("result");
				xml.WriteAttributeString("code", code.ToString());
				xml.WriteAttributeString("action", action ?? "");
				xml.WriteAttributeString("shopId", shopId ?? "");
				xml.WriteAttributeString("invoiceId", invoiceId ?? "");
				if (code > 0)
					xml.WriteAttributeString("techMessage", techMessage ?? "");
				xml.WriteEndElement();
				
				xml.WriteEndElement();
			}
		}
	}
}
