﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Bitrix.IBlock;
using System.Text;
using Bitrix.UI;

using System.Collections.Generic;
using Bitrix.Security;
using Bitrix.Modules;
using Bitrix;
using Bitrix.IO;
using Bitrix.Services.Text;
using Bitrix.DataLayer;
using System.Text.RegularExpressions;
using Bitrix.Services;

public partial class bitrix_admin_SecurityFilter : BXAdminPage
{
	protected BXSiteCollection Sites;

	List<BXSecurityFilterMask> masks;
	protected List<BXSecurityFilterMask> Masks
	{
		get
		{
			if (masks != null)
				return masks;
			masks = BXSecurityModuleConfiguration.GetFilterMasks();
			masks.Sort(delegate(BXSecurityFilterMask a, BXSecurityFilterMask b)
			{
				return string.Compare(a.SiteId, b.SiteId);
			});
			masks.Add(new BXSecurityFilterMask(0, "", ""));
			return masks;
		}
	}

    protected bool IsExclusionsSet
    {
        get { return (Masks.Count > 1); }
    }

	protected bool IsSecurityEnabled
	{
		get
		{ return BXSecurityModuleConfiguration.IsSecurityEnabled; }
	}

	protected void Page_Init(object sender, EventArgs e)
	{
		if (!BXUser.IsCanOperate(BXSecurityModuleOperations.AdminManagement))
			BXAuthentication.AuthenticationRequired();

		Sites = BXSite.GetList(new BXFilter(), null, null, null, BXTextEncoder.EmptyTextEncoder);
		switch (BXSecurityModuleConfiguration.FilterReaction)
		{
			case BXSecurityActiveReactionSettings.MakeDataSafe: rblSettings.SelectedValue = "safe"; break;
			case BXSecurityActiveReactionSettings.LeaveDangerousData: rblSettings.SelectedValue = "leave"; break;
			case BXSecurityActiveReactionSettings.ClearDangerousData: rblSettings.SelectedValue = "clear"; break;
		}

		btnSecurity.Text = GetMessageRaw(BXSecurityModuleConfiguration.IsSecurityEnabled ? "ButtonText.DisableSecurity" : "ButtonText.EnableSecurity");
		btnSecurity.CommandName = BXSecurityModuleConfiguration.IsSecurityEnabled ? "disable" : "enable";

		if (string.IsNullOrEmpty(BackUrl))
			BXTabControl1.ShowCancelButton = false;

		rblSettings.Items[0].Text += "<a href=\"#filter_remark_1\" style=\"vertical-align: super; text-decoration: none;\"><span class=\"required\">1</span></a>";
		MasterTitle = GetMessageRaw("MasterTitle");
	}

	protected void BXTabControl1_Command(object sender, BXTabControlCommandEventArgs e)
	{
		if (e.CommandName != "cancel")
		{
			List<BXSecurityFilterMask> saveList = new List<BXSecurityFilterMask>();
			string[] masks = Request.Form.GetValues("txtmask");
			string[] ids = Request.Form.GetValues("hfregexid");
			string[] sites = Request.Form.GetValues("selsite");
			for (int i = 0; i < masks.Length; i++)
			{
				int id;
				if (i >= ids.Length || !int.TryParse(ids[i], out id))
					id = 0;
				saveList.Add(new BXSecurityFilterMask(id, sites[i], masks[i]));
			}
			BXSecurityModuleConfiguration.SetFilterMasks(saveList);


			switch (rblSettings.SelectedValue)
			{
				case "safe": BXSecurityModuleConfiguration.FilterReaction = BXSecurityActiveReactionSettings.MakeDataSafe; break;
				case "clear": BXSecurityModuleConfiguration.FilterReaction = BXSecurityActiveReactionSettings.ClearDangerousData; break;
				case "leave": BXSecurityModuleConfiguration.FilterReaction = BXSecurityActiveReactionSettings.LeaveDangerousData; break;
			}

			BXSecurityModuleConfiguration.LogAttacks = cbLogAttacks.Checked;

			if (e.CommandName == "apply")
				RedirectToTab(BXTabControl1.SelectedIndex);
		}

		Response.Redirect(BackUrl ?? BXSefUrlManager.CurrentUrl.ToString());
	}

	protected void ManageSecurity_Command(object sender, CommandEventArgs e)
	{
		BXSecurityModuleConfiguration.IsSecurityEnabled = (e.CommandName == "enable");
		Response.Redirect(BXSefUrlManager.CurrentUrl.ToString());
	}
}
